// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   all3075_demon.c   -   Demon zum Steuern von ALL3075/3076 Netzwerksteckdosen ber TCP/IP (ALL4500/5000)        **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2011 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************

// **********   Eintrge in der Konfigurations-Datenbank, die vom Demon ausgewertet werden   **********
//
// /control/powerline/master/comport"										Serielle Schnittstelle, an der das PL-Mastermodul hngt (normalerweise /dev/ttyS4)
// /control/powerline/master/baudrate"									Baudrate dieser Schnittstelle (normalerweise 4800)
// /control/powerline/server/udpport"										UDP-Port fr Steuerkommandos (54009)
// /control/powerline/server/username										zu bergebender Username
// /control/powerline/server/password										zu bergebendes Pawort
// /control/powerline/max_retries												Anzahl Versuche, einen Slave zu erreichen
// /control/powerline/max_slaves"												Anzahl zu verwaltender Slaves
// /control/powerline/reload_counter"										Zhler, um festzustellen ob ein Neuladen der Config ntig ist
// Powerline fliegt raus, wenn die Umstellung fertig ist

// /control/all3075/server/loglevel											Loglevel 
// /control/all3075/server/max_retries									Wiederholungen
// /control/all3075/server/max_slaves										Anzahl Slaves
// /control/all3075/server/readdelay										Verzgerung zwischen Polls
// /control/all3075/server/reload_counter								Config-Zhler
// /control/all3075/slaveXX/enabled								 			Enabled = 1
// /control/all3075/slaveXX/type									 			1: ALL3075, andere gibts momentan noch nicht
// /control/all3075/slaveXX/name												Name
// /control/all3075/slaveXX/comment											Blatext
// /control/all3075/slaveXX/ip_address									IP der Steckdose
// /control/all3075/slaveXX/user												Webserver Username
// /control/all3075/slaveXX/password										Webserver Pawort
// /control/all3075/slaveXX/mapfrom_actor								logischer Aktor, der die Dose kontrolliert
// /control/actors/actorXX/value"												gibt den zu schaltenden Wert; wird mit gepolltem Wert upgedated



#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
// #include <sys/socket.h>
// #include <arpa/inet.h>
#include <string.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <termios.h>
#include <time.h>
#include <sys/time.h>
// #include <sys/ipc.h>
#include <sys/shm.h>
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "sqldb.h"
#include "sensor_demon.h"
#include "shm_use.h"
#include "all3075_demon.h"
#include "xml_lib.h"
#include "logging.h"



// *****   Globale Variablen   *****

unsigned short int		cache_max_slaves;
signed char						cache_slave_enabled[ALL3075_MAX_SLAVES];
signed short int			cache_slave_type[ALL3075_MAX_SLAVES];
char									cache_slave_ip[ALL3075_MAX_SLAVES][16];
char									cache_slave_user[ALL3075_MAX_SLAVES][20];
char									cache_slave_password[ALL3075_MAX_SLAVES][20];
signed long int				cache_slave_mapfrom_actor[ALL3075_MAX_SLAVES];
signed long int				cache_slave_value[ALL3075_MAX_SLAVES];
signed long int				cache_slave_mapto_sensor[ALL3075_MAX_SLAVES];
signed long int				all3075_reload_counter = -1;			// Merker fr Reload-Counter
signed long int				read_delay;												// Verzgerung zwischen selbstttigen Reads
signed long int				all3075_max_retries = 5;
int										shID;
struct s_shm_actordata*	shm_actor_r_data;								// Aktordaten R im Shared-RAM
struct s_shm_actordata*	shm_actor_w_data;								// Aktordaten W im Shared-RAM
signed long int				shadow_w_data[MAX_LOGICAL_ACTORS];// Kopie der Aktordaten-Integers
unsigned long int			last_read_timestamp = 0;					// Zeit des letzten Reads
signed long int				zyklischer_sklave = 0;						// Nummer fr Zyklische Abfrage




// *****   get_ms_now()   -   aktuelle Zeit in ms ermitteln   *****

long get_ms_now(void)
	{
	long rw;
	const long msdivider = CLOCKS_PER_SEC / 1000;					// Teiler um von Clockticks aus ms zu kommen
  struct timeval tv;
	time_t curtime;
	struct tm *myTm;


	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln 
  curtime = tv.tv_sec;																	// Lores-Zeit in s reusziehen
	myTm = localtime(&curtime);														// ...und berfhren in tm-Struktur
	rw = (myTm->tm_hour * 3600 + myTm->tm_min * 60 + myTm->tm_sec) * 1000 + tv.tv_usec / msdivider;
	return(rw);
	}



// *****   all3075_schalte2()   -   Schalten von Slaves mit Wiederholung im Fehlerfall   *****

signed long int all3075_schalte2(unsigned short int dosennummer,unsigned short int schaltwunsch,unsigned short int tries)
	{
	signed long int rw = -1;
	signed short int t;
	char ep[333];

	for(t = 0;t < tries;t++)
		{
		rw = all3075_schalte(dosennummer,schaltwunsch);
		if(rw >= 0) goto ende;
		sprintf(ep,"all3075_schalte2: Retry for IP slave %d:%d (RW:%ld)",dosennummer,schaltwunsch,rw);
		log_append(4,ep);																		// Meldung ins Logfile schreiben
		usleep(150000);																			// 150 ms warten
		}

ende:
	return(rw);
	}



// *****   all3075_query2()   -   Abfragen von Slaves mit Wiederholung im Fehlerfall   *****

signed long int all3075_query2(unsigned short int dosennummer,unsigned short int tries)
	{
	signed long int rw = -1;
	signed short int t;
	char ep[333];

	for(t = 0;t < tries;t++)
		{
		rw = all3075_query(dosennummer);
		if(rw >= 0) goto ende;
		sprintf(ep,"all3075_query2: Retry for IP slave %d (RW:%ld)",dosennummer,rw);
		log_append(4,ep);																		// Meldung ins Logfile schreiben
		usleep(150000);																			// 150 ms warten
		}

ende:
	return(rw);
	}



// *****   all3075_schalte() - Schalten von Slaves   *****

signed long int all3075_schalte(unsigned short int dosennummer,unsigned short int schaltwunsch)
	{
	signed long int rw = -2;
	char ep[512];
	char pw_string[80];																		// Passwort/Username
	char dosen_url[150];																	// URL der Steckdose inkl. evtl. Passwort
	char kommando[256];																		// Befehlspuffer
	char sw;																							// ASCII-Zeichen fr Schaltwunsch


	sw = '0';
	if (schaltwunsch > 0) sw = '1';

	pw_string[0] = 0;																			// kein Passwort
	if((strlen(cache_slave_user[dosennummer]) > 0) && (strlen(cache_slave_password[dosennummer]) > 0))
		{
		sprintf(pw_string,"%s:%s@",cache_slave_user[dosennummer],cache_slave_password[dosennummer]);
		}

	sprintf(dosen_url,"http://%s%s/r?b=1&r=0&s=%c&mp=%d",pw_string,cache_slave_ip[dosennummer],sw,rand());
	sprintf(kommando,"wget \"%s\" --output-document=\"/dev/null\" --quiet",dosen_url);
	// printf("SKommando: \"%s\"\n",kommando);
	system(kommando);

	rw = all3075_query(dosennummer);											// Status zurckholen
	sprintf(ep,"all3075_schalte(%d,%d): %s RW=%ld",dosennummer,schaltwunsch,cache_slave_ip[dosennummer],rw);
	log_append(4,ep);																			// Meldung ins Logfile schreiben
	return(rw);
	}



// *****   all3075_query() - Ermitteln des Schaltzustands von Slaves   *****

signed long int all3075_query(unsigned short int dosennummer)
	{
	signed long int rw = -2;
	char ep[512];
	char pw_string[80];																		// Passwort/Username
	char dosen_url[150];																	// URL der Steckdose inkl. evtl. Passwort
	char kommando[256];																		// Befehlspuffer
	char xmlbla[100];																			// Puffer fr XML-Daten


	pw_string[0] = 0;																			// kein Passwort
	if((strlen(cache_slave_user[dosennummer]) > 0) && (strlen(cache_slave_password[dosennummer]) > 0))
		{
		sprintf(pw_string,"%s:%s@",cache_slave_user[dosennummer],cache_slave_password[dosennummer]);
		}

	sprintf(dosen_url,"http://%s%s/xml?mp=%d",pw_string,cache_slave_ip[dosennummer],rand());
	
	system("rm /tmp/all3075.xml 2>/dev/null");						// XML-Datei prophylaktisch lschen
	sprintf(kommando,"wget \"%s\" --output-document=\"/tmp/all3075.xml\" --quiet",dosen_url);
	// printf("QKommando: \"%s\"\n",kommando);
	system(kommando);																			// Status-XML-File mit wget holen

	rw = read_xml("/tmp/all3075.xml","t0",xmlbla);
	if (rw < 0) goto ende;																// File nicht vorhanden (Dose hat nicht geantwortet)

	rw = atoi(xmlbla);																		// Status aus dem XML-File extrahieren
	system("rm /tmp/all3075.xml 2>/dev/null");						// und dieses anschlieend lschen

ende:
	sprintf(ep,"all3075_query(%d): %ld",dosennummer,rw);
	log_append(4,ep);																			// Meldung ins Logfile schreiben
	return(rw);
	}



// *****   cache_setzedefault()   -   Mappingtabelle lschen   *****

void cache_setzedefault(void)
	{
	unsigned short int t;

	for(t = 0;t < ALL3075_MAX_SLAVES;t++)
		{
		cache_slave_enabled[t] = 0;
		cache_slave_type[t] = 0;
		cache_slave_ip[t][0] = 0;
		cache_slave_user[t][0] = 0;
		cache_slave_password[t][0] = 0;
		cache_slave_mapfrom_actor[t] = -1;
		cache_slave_value[t] = -1;
		cache_slave_mapto_sensor[t] = -1;
		}

	cache_max_slaves = 0;
	}


// **********   Demon-Configfile einlesen   **********

void all3075_demon_readconfig(void)
	{
	char ep[1024];
	unsigned short int t;
	unsigned short int sklaven = 0;

	all3075_reload_counter = sqldb_read_long("/control/all3075/server/reload_counter");

	log_level = sqldb_read_long("/control/all3075/server/loglevel");
	cache_max_slaves = sqldb_read_long("/control/all3075/server/max_slaves");
	read_delay = sqldb_read_long("/control/all3075/server/readdelay");

	for(t = 0;t < cache_max_slaves;t++)
		{
		sprintf(ep,"/control/all3075/slave%d/enabled",t);
		cache_slave_enabled[t] = sqldb_read_long(ep);
		if (cache_slave_enabled[t] > 0)
			{
			sklaven++;
			sprintf(ep,"/control/all3075/slave%d/type",t);
			cache_slave_type[t] = sqldb_read_long(ep);
			sprintf(ep,"/control/all3075/slave%d/mapfrom_actor",t);
			cache_slave_mapfrom_actor[t] = sqldb_read_long(ep);
			sprintf(ep,"/control/all3075/slave%d/mapto_sensor",t);
			cache_slave_mapto_sensor[t] = sqldb_read_long(ep);
			sprintf(ep,"/control/all3075/slave%d/ip_address",t);
			sqldb_read_string(ep,cache_slave_ip[t]);
			sprintf(ep,"/control/all3075/slave%d/name",t);
			sqldb_read_string(ep,cache_slave_user[t]);
			sprintf(ep,"/control/all3075/slave%d/password",t);
			sqldb_read_string(ep,cache_slave_password[t]);
			}
		}

	sprintf(ep,"Global ALL3075 Configuration read. Loglevel:%d, Readdelay:%ld, Slaves:%d",log_level,read_delay,sklaven);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	}



// *****   Hauptschleife   *****

int main(int argc,char **argv)
	{
	unsigned long					ti = 0;													// Zhler, geht bei jedem Durchlauf um 1 hoch
	char									ep[1024];												// Puffer fr Logfile-Meldungen
	signed long int				t;
	signed long int				i;


	// auf existierendes Shared Memory fr Demonstatus zugreifen
	if (use_my_demonstatus_shm(5) != 1) goto errorende;		// MY_DEMON_ID = 5: all3075_demon

	// Status von Demon in SHM eintragen
	my_demonstatus_data[my_demon_id].demon_id = my_demon_id;														// 0 ist Default, hier wird die eigene ID eingetragen
	sprintf(my_demonstatus_data[my_demon_id].name_string,"all3075_demon");							// Name
	sprintf(my_demonstatus_data[my_demon_id].version_string,"%s",ALL3075_DEMON_VERSION);// Versionsnummer
	my_demonstatus_data[my_demon_id].last_boot = time(0);																// Timestamp(UTC) des Starts
	my_demonstatus_data[my_demon_id].last_activity = time(0);														// Timestamp(UTC) der letzten Aktion
	my_demonstatus_data[my_demon_id].ti = ti;																						// Counter
	sprintf(my_demonstatus_data[my_demon_id].debug_message,"Start");										// letzte Debug-Meldung
	my_demonstatus_data[my_demon_id].error_message[0] = 0;															// letzte Fehlermeldung
	my_demonstatus_data[my_demon_id].status_long = 0;																		// numerischer Status
	sprintf(my_demonstatus_data[my_demon_id].status_string,"startup");									// String-Status

	// Logging-System initialisieren
	strcpy (log_filename,"/www/reports/all3075_demon.log"); // Logfilenamen setzen
	log_level = sqldb_read_long("/control/all3075/server/loglevel"); // Debuglevel setzen
	// log_kill();																						// Logfile loeschen
	sprintf(ep,"----------   all3075_demon V %s started (debuglevel=%d)   ----------",ALL3075_DEMON_VERSION,log_level);
	log_append(1,ep);																			// Startmeldung ins Logfile schreiben



	// auf existierendes Shared Memory fr Aktordaten Read zugreifen
	shID = shmget(SHM_ID_ACTOR_R_DATA,SHM_ACTOR_R_DATA_SIZE,0666);
	if (shID >= 0)
		{
		shm_actor_r_data = shmat(shID,0,0);
		if (shm_actor_r_data == (struct s_shm_actordata *)-1)
			{
			printf("Problem beim Zugriff auf ACTOR_R_DATA shared memory !\n");
			goto ende;
			}
		else
			{
			// printf("SHM_Actor_r_data OK.\n");
			}
		}
	else
		{
		// shmget lief schief
		printf("shmget fr SHM_Actor_r_data war erfolglos ! (shm_create nicht aufgerufen ?)\n");
		goto ende;
    }

	// auf existierendes Shared Memory fr Aktordaten Write zugreifen
	shID = shmget(SHM_ID_ACTOR_W_DATA,SHM_ACTOR_W_DATA_SIZE,0666);
	if (shID >= 0)
		{
		shm_actor_w_data = shmat(shID,0,0);
		if (shm_actor_w_data == (struct s_shm_actordata *)-1)
			{
			printf("Problem beim Zugriff auf ACTOR_W_DATA shared memory !\n");
			goto ende;
			}
		else
			{
			// printf("SHM_Actor_w_data OK.\n");
			}
		}
	else
		{
		// shmget lief schief
		printf("shmget fr SHM_Actor_w_data war erfolglos ! (shm_create nicht aufgerufen ?)\n");
		goto ende;
    }


	// Shadow-Write auf ngltige Werte setzen
	for (t = 0;t < MAX_LOGICAL_ACTORS;t++)
		{
		shadow_w_data[t] = -99;
		}


	// Einstellungen fr ALL3975 Demon
	cache_setzedefault();
	all3075_demon_readconfig();

	// Start loggen
	sprintf(ep,"**********   all3075 demon V %s started   **********",ALL3075_DEMON_VERSION);
	log_append(1,ep);
	printf("all3075 demon V %s started.\n",ALL3075_DEMON_VERSION);


	last_read_timestamp = get_ms_now();										// now() merken
	update_demondebug("running");													// letzte Debug-Meldung
hauptschleife:
	update_demonstatus(ti,1,"main loop");									// Status von Demon in SHM aktualisieren
	ti++;																									// Schleifenzhler erhhen
	usleep(50000);																				// 1/4 Sekunde Rechenzeit freigeben

	if (all3075_reload_counter != sqldb_read_long("/control/all3075/server/reload_counter"))
		{
		update_demonstatus(ti,2,"reload");									// Status von Demon in SHM aktualisieren
		all3075_demon_readconfig();
		sprintf(ep,"Global ALL3075 Configuration re-read.");
		log_append(1,ep);																			// Meldung ins Logfile schreiben
		goto hauptschleife;
		}



	// -----   Aktor-Werte auf Vernderungen prfen   -----
	for(t = 0;t < ALL3075_MAX_SLAVES;t++)
		{
		if (cache_slave_enabled[t] > 0)
			{
			if (cache_slave_mapfrom_actor[t] >= 0)
				{
				if ((shm_actor_w_data[cache_slave_mapfrom_actor[t]].value_long != shadow_w_data[t])
				||  (shm_actor_w_data[cache_slave_mapfrom_actor[t]].value_long != shm_actor_r_data[cache_slave_mapfrom_actor[t]].value_long))
					{
					// Slave aktiv, korrekt gemappt und Aktor-W-Wert hat sich gendert: wir mssen Schalten !
					// printf("ALL3075: Aktor %ld will slave %ld auf %ld setzen. (%s)\n",cache_slave_mapfrom_actor[t],t,shm_actor_w_data[cache_slave_mapfrom_actor[t]].value_long,cache_slave_ip[t]);
					update_demonstatus(ti,10,"switch");						// Status von Demon in SHM aktualisieren
					sprintf(ep,"W %ld:%ld",t,shm_actor_w_data[cache_slave_mapfrom_actor[t]].value_long);
					update_demondebug(ep);
					i = all3075_schalte2(t,shm_actor_w_data[cache_slave_mapfrom_actor[t]].value_long,all3075_max_retries);
					if (i >= 0)
						{
						shadow_w_data[t] = i;
						shm_actor_w_data[cache_slave_mapfrom_actor[t]].value_long = shadow_w_data[t];
						shm_actor_r_data[cache_slave_mapfrom_actor[t]].value_long = shadow_w_data[t];
						}
					else
						{
						shm_actor_r_data[cache_slave_mapfrom_actor[t]].value_long = -204800000;
						}
					}
				}
			}
		}



	// -----   Zyklisches Lesen der Aktoren   -----
	for(t = 0;t < ALL3075_MAX_SLAVES;t++)
		{
		if(cache_slave_enabled[t] > 0) goto beginn_zyklus;
		}
	update_demonstatus(ti,20,"no slaves");								// Status von Demon in SHM aktualisieren
	update_demondebug("no slaves");												// Status von Demon in SHM aktualisieren
	sleep(3);																							// dann knnen wir auch lnger schlafen
	goto ende_zyklus;
beginn_zyklus:
	update_demonstatus(ti,30,"verify");										// Status von Demon in SHM aktualisieren
	if (last_read_timestamp > get_ms_now()) last_read_timestamp = 0; // berlauf um Mitternacht kompensieren
	if (get_ms_now() > last_read_timestamp + read_delay * 1000)
		{
		// printf("**READ %ld** delay=%ld\n",last_read_timestamp,read_delay);
suche_zyklus:
		last_read_timestamp = get_ms_now();
		zyklischer_sklave++;
		if(zyklischer_sklave >= ALL3075_MAX_SLAVES) zyklischer_sklave = 0;
		if (cache_slave_enabled[zyklischer_sklave] <= 0) goto suche_zyklus;

		i = all3075_query2(zyklischer_sklave,all3075_max_retries);
		if (i >= 0)
			{
			shm_actor_r_data[cache_slave_mapfrom_actor[zyklischer_sklave]].value_long = i;
			sprintf(ep,"R %ld:%ld",zyklischer_sklave,i);
			update_demondebug(ep);
			}
		else
			{
			shm_actor_r_data[cache_slave_mapfrom_actor[zyklischer_sklave]].value_long = -204800000;
			sprintf(ep,"R %ld:-",zyklischer_sklave);
			update_demondebug(ep);
			}

		}
ende_zyklus: ;

	goto hauptschleife;

ende:
	return(0);

errorende:
	return(-1);
	}
