// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   clock_demon.c   -   Demon zum Anzeigen der Uhrzeit und Schalten des Weckrelais                                **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2009 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************

// **********   I/O Registerbelegunh   **********
//
// Alle I2C-Ports sind ausschlielich Ausgnge, Zurcklesen ist nicht mglich
// 


#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "sqldb.h"
#include "at91gpio.h"
#include "all_i2c.h"
#include "i2c_chips.h"
#include "logging.h"



// **********   Globale Variablen   **********

signed long int ti = 0;																	// System-Timer
unsigned char weckzeit1_enabled = 0;										// Weckzeit 1 Aktiv
unsigned char weckzeit1_sekunde;												// Weckzeit 1 Sekunde
unsigned char weckzeit1_minute;													// Weckzeit 1 Minute
unsigned char weckzeit1_stunde;													// Weckzeit 1 Stunde
unsigned char weckzeit2_enabled = 0;										// Weckzeit 2 Aktiv
unsigned char weckzeit2_sekunde;												// Weckzeit 2 Sekunde
unsigned char weckzeit2_minute;													// Weckzeit 2 Minute
unsigned char weckzeit2_stunde;													// Weckzeit 2 Stunde
unsigned char uhrzeit_sekunde;													// Uhr Sekunde
unsigned char uhrzeit_minute;														// Uhr Minute
unsigned char uhrzeit_stunde;														// Uhr Stunde
unsigned char last_sekunde = 80;												// letzte Sekunde





// *****   setze_troete(0/1)   -   Trte ein- und ausschalten   *****

void setze_troete(unsigned char schaltbefehl)
	{
	gpio_port_init_output(PORT_B,27);											// Trtenrelais-Ausgang PB27

	if(schaltbefehl > 0)
		{
		gpio_port_write(PORT_B,27,1);												// H
		}
	else
		{
		gpio_port_write(PORT_B,27,0);												// L
		}
	}



// *****   demon_readconfig   -   Konfigurationsdaten lesen   *****

void demon_readconfig(void)
	{
	weckzeit1_enabled = sqldb_read_long("/clock/alarm1_enabled");
	weckzeit1_stunde = sqldb_read_long("/clock/alarm1_hour");
	weckzeit1_minute = sqldb_read_long("/clock/alarm1_minute");
	weckzeit1_sekunde = sqldb_read_long("/clock/alarm1_second");
	weckzeit2_enabled = sqldb_read_long("/clock/alarm2_enabled");
	weckzeit2_stunde = sqldb_read_long("/clock/alarm2_hour");
	weckzeit2_minute = sqldb_read_long("/clock/alarm2_minute");
	weckzeit2_sekunde = sqldb_read_long("/clock/alarm2_second");
	printf("Weckzeit1: %02d:%02d:%02d, enabled: %d\n",weckzeit1_stunde,weckzeit1_minute,weckzeit1_sekunde,weckzeit1_enabled);
	printf("Weckzeit2: %02d:%02d:%02d, enabled: %d\n",weckzeit2_stunde,weckzeit2_minute,weckzeit2_sekunde,weckzeit2_enabled);
	}



// *****   led_showuhrzeit() - Uhrzeit im groen Display anzeigen   *****

void led_showuhrzeit(unsigned char stunde,unsigned char minute,unsigned char sekunde)
	{
	unsigned char ziffern[11];
	unsigned char zehner;
	unsigned char einer;
	unsigned short int speed = 2;
	

	ziffern[ 0] =  63;
	ziffern[ 1] =   6;
	ziffern[ 2] =  91;
	ziffern[ 3] =  79;
	ziffern[ 4] = 102;
	ziffern[ 5] = 109;
	ziffern[ 6] = 124;
	ziffern[ 7] =   7;
	ziffern[ 8] = 127;
	ziffern[ 9] = 111;
	ziffern[10] =   0;


	zehner = sekunde / 10;
	einer = sekunde - 10 * zehner;
	simplified_pcf8574_write(16,0,0,speed,6,ziffern[zehner]); // int_output[6] = ziffern[zehner];
	simplified_pcf8574_write(16,0,0,speed,7,ziffern[einer]); // int_output[7] = ziffern[einer];
	simplified_pcf8574_write(17,0,0,speed,6,ziffern[zehner]); // int_output[6] = ziffern[zehner];
	simplified_pcf8574_write(17,0,0,speed,7,ziffern[einer]); // int_output[7] = ziffern[einer];
	simplified_pcf8574_write(18,0,0,speed,6,ziffern[zehner]); // int_output[6] = ziffern[zehner];
	simplified_pcf8574_write(18,0,0,speed,7,ziffern[einer]); // int_output[7] = ziffern[einer];

	zehner = minute / 10;
	einer = minute - 10 * zehner;
	simplified_pcf8574_write(16,0,0,speed,4,ziffern[zehner]); // int_output[4] = ziffern[zehner];
	simplified_pcf8574_write(16,0,0,speed,5,ziffern[einer]); // int_output[5] = ziffern[einer];
	simplified_pcf8574_write(17,0,0,speed,4,ziffern[zehner]); // int_output[4] = ziffern[zehner];
	simplified_pcf8574_write(17,0,0,speed,5,ziffern[einer]); // int_output[5] = ziffern[einer];
	simplified_pcf8574_write(18,0,0,speed,4,ziffern[zehner]); // int_output[4] = ziffern[zehner];
	simplified_pcf8574_write(18,0,0,speed,5,ziffern[einer]); // int_output[5] = ziffern[einer];

	zehner = stunde / 10;
	einer = stunde - 10 * zehner;
	simplified_pcf8574_write(16,0,0,speed,2,ziffern[zehner]); // int_output[2] = ziffern[zehner];
	simplified_pcf8574_write(16,0,0,speed,3,ziffern[einer]); // int_output[3] = ziffern[einer];
	simplified_pcf8574_write(17,0,0,speed,2,ziffern[zehner]); // int_output[2] = ziffern[zehner];
	simplified_pcf8574_write(17,0,0,speed,3,ziffern[einer]); // int_output[3] = ziffern[einer];
	simplified_pcf8574_write(18,0,0,speed,2,ziffern[zehner]); // int_output[2] = ziffern[zehner];
	simplified_pcf8574_write(18,0,0,speed,3,ziffern[einer]); // int_output[3] = ziffern[einer];
	}







// **********   Demon   **********

int main(int argc,char **argv)
	{
	char ep[80];
	time_t startzeit;
	time_t now;
	struct tm *myTm;
	unsigned char muell = 0;
	// signed short int t;
	signed short int troetencounter = 0;


	gpio_devmem_open();																		// GPIOs oeffnen
	setze_troete(0);																			// Trte ausschalten
	time(&startzeit);																			// Systemzeit in Sekunden seit 01.01.1970 holen

	strcpy(log_filename,"/www/reports/clock.log");
	log_level = 3;																				// Debuglevel setzen
	sprintf(ep,"----------   clock_demon started (debuglevel=%d)   ----------",log_level);
	log_append(1,ep);																			// Startmeldung ins Logfile schreiben


	i2c_ruhe(16);																					// Ruhe am Bus (Initialisierung auf Standardwerte)
	i2c_ruhe(17);																					// Ruhe am Bus (Initialisierung auf Standardwerte)
	i2c_ruhe(18);																					// Ruhe am Bus (Initialisierung auf Standardwerte)

	demon_readconfig();																		// Einstellungen fuer den Clock-Demon lesen
	// printf("TimeInterval=%ld\r\n",zeitintervall);




hauptschleife:
	sleep(0);																							// ein bichen Zeit vertrdeln
	ti++;																									// Sys-Clock hochzhlen
	// schreibe_datenfile();																	// XML-Datei erzeugen

	time(&now);																						// Systemzeit in Sekunden seit 01.01.1970 holen
	myTm = localtime(&now);																// berfhren in tm-Struktur
	uhrzeit_sekunde = myTm->tm_sec;												// Sekunde
	uhrzeit_minute = myTm->tm_min;												// Minute
	uhrzeit_stunde = myTm->tm_hour;												// Stunde
	if(uhrzeit_sekunde == last_sekunde) goto hauptschleife;
	last_sekunde = uhrzeit_sekunde;

	muell++;

	if (troetencounter > 0) troetencounter--;
	if (troetencounter <= 0) setze_troete(0);							// Trte nach Timeout ausschalten
	led_showuhrzeit(uhrzeit_stunde,uhrzeit_minute,uhrzeit_sekunde);

	if(weckzeit1_enabled > 0)
		{
		if(weckzeit1_stunde == uhrzeit_stunde)
			{
			if(weckzeit1_minute == uhrzeit_minute)
				{
				if(weckzeit1_sekunde == uhrzeit_sekunde)
					{
					setze_troete(1);																		// Trte einschalten
					troetencounter = 15;																// fr 15 Sekunden
					}
				}
			}
		}

	if(weckzeit2_enabled > 0)
		{
		if(weckzeit2_stunde == uhrzeit_stunde)
			{
			if(weckzeit2_minute == uhrzeit_minute)
				{
				if(weckzeit2_sekunde == uhrzeit_sekunde)
					{
					setze_troete(1);																		// Trte einschalten
					troetencounter = 15;																// fr 15 Sekunden
					}
				}
			}
		}

	/*
	setze_troete(1);
	sleep(1);
	setze_troete(0);
	sleep(1);
	*/
	
	goto hauptschleife;																		// ... und wieder dasselbe Spiel von vorne


// ende:
	sprintf(ep,"----------   clock_demon terminated   ----------");
	log_append(1,ep);																			// Endmeldung ins Logfile schreiben
	return(0);
	}

