// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   cv_demon.c   -   Demon zum verarbeiten der Kontrollspannungseingnge (Poti-Eingnge) f. ALL3651               **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2011 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************

// **********   Eintrge in der Konfigurations-Datenbank, die vom Demon ausgewertet werden   **********
//
// /control/cv/enabled																	Muss berhaupt etwas getan werden ?
// /control/cv/reloadcounter														wird bei nderungen der config hochgezhlt
// /control/cv/cv0/type																	0: inaktiv, 1: Copy to Color gem. Target, 2: Switch ab Level in Treshold
// /control/cv/cv0/target																0:keines,1:r0,2:g0,3:b0,4:w0,5:r1,6:g1,7:b1,8:w1,9:alle Ch0,10:alle Ch1,11:Alle Ch0+1
// /control/cv/cv0/treshold															Schaltschwelle, ab der geschaltet werden soll
// /control/cv/cv1/type																	0: inaktiv, 1: Copy to Color gem. Target, 2: Switch ab Level in Treshold
// /control/cv/cv1/target																0:keines,1:r0,2:g0,3:b0,4:w0,5:r1,6:g1,7:b1,8:w1,9:alle Ch0,10:alle Ch1,11:Alle Ch0+1
// /control/cv/cv1/treshold															Schaltschwelle, ab der geschaltet werden soll
// /control/cv/cv2/type																	0: inaktiv, 1: Copy to Color gem. Target, 2: Switch ab Level in Treshold
// /control/cv/cv2/target																0:keines,1:r0,2:g0,3:b0,4:w0,5:r1,6:g1,7:b1,8:w1,9:alle Ch0,10:alle Ch1,11:Alle Ch0+1
// /control/cv/cv2/treshold															Schaltschwelle, ab der geschaltet werden soll
// /control/cv/cv3/type																	0: inaktiv, 1: Copy to Color gem. Target, 2: Switch ab Level in Treshold
// /control/cv/cv3/target																0:keines,1:r0,2:g0,3:b0,4:w0,5:r1,6:g1,7:b1,8:w1,9:alle Ch0,10:alle Ch1,11:Alle Ch0+1
// /control/cv/cv3/treshold															Schaltschwelle, ab der geschaltet werden soll
// /control/cv/cv4/type																	0: inaktiv, 1: Copy to Color gem. Target, 2: Switch ab Level in Treshold
// /control/cv/cv4/target																0:keines,1:r0,2:g0,3:b0,4:w0,5:r1,6:g1,7:b1,8:w1,9:alle Ch0,10:alle Ch1,11:Alle Ch0+1
// /control/cv/cv4/treshold															Schaltschwelle, ab der geschaltet werden soll
// /control/cv/cv5/type																	0: inaktiv, 1: Copy to Color gem. Target, 2: Switch ab Level in Treshold
// /control/cv/cv5/target																0:keines,1:r0,2:g0,3:b0,4:w0,5:r1,6:g1,7:b1,8:w1,9:alle Ch0,10:alle Ch1,11:Alle Ch0+1
// /control/cv/cv5/treshold															Schaltschwelle, ab der geschaltet werden soll


#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <time.h>
#include <sys/time.h>
#include <sys/shm.h>
#include "at91gpio.h"
#include "all_i2c.h"
#include "i2c_chips.h"
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "cv_demon.h"
#include "sqldb.h"
#include "logging.h"
#include "shm_use.h"



// *****   Globale Variablen   *****

#define CONFIG_DATABASE_FILE "/etc/allnetenv/config.s3db"
#define VAR_DATABASE_FILE "/tmp/var.s3db"


unsigned long int	ti = 0;																// Zhler, geht bei jedem Durchlauf um 1 hoch
signed short int	programm_nummer = -1;									// Nummer des gerade aktiven Programms
signed short int	programm_az = -1;											// gerade abgearbeitete Zeilennummer
signed short int	programm_status = -1;									// Status (-2:muss geladen werden, -1:angehalten, 1: in Delay innerhalb der Zeile, 0:Zeile muss bearbeitet werden)
signed short int	programm_delaycounter = 0;						// Delayzhler innerhalb der Zeile
signed short int	programm_zeilennummer = -1;						// aktuelle Zeilennummer
signed short int	programm_max_steps = 16;							// Anzahl zulssiger Zeilennummern
signed short int	fernbedienung = -1;										// Typ der Fernbedienung (-1: Keine, 0: alte kleine FB mit altem Tiny85,
																												//                        1: PortaLuce FB mit neuem Tiny85,
																												//                        2: alte kleine FB mit neuem Tiny85)
signed short int	old_tiny_major = 0;										// IR-Empfnger Major ID
signed short int	old_tiny_minor = 0;										// IR-Empfnger Minor ID

struct programmzeile
	{
	signed short int 		cmdtoken;													// 0:NOP, 1:RGB_Set, 2:Restart 3:End
	signed short int		r0;
	signed short int		g0;
	signed short int		b0;
	signed short int		w0;
	signed short int		dim0;
	signed short int		r1;
	signed short int		g1;
	signed short int		b1;
	signed short int		w1;
	signed short int		dim1;
	signed short int		delay;
	signed short int		target;
	signed short int		switchtype;
	} pz;
struct programmzeile programm_zeile[101];




// *****   sql_read_rgbvalues()   -   Daten aus der tmpDB holen   *****

signed short int sql_read_rgbvalues(sqlite3 *ptr,unsigned char *r0,unsigned char *g0,unsigned char *b0,unsigned char *w0,unsigned char *d0,unsigned char *r1,unsigned char *g1,unsigned char *b1,unsigned char *w1,unsigned char *d1)
	{
	signed short int rw = 0;
	sqlite3_stmt *vm;
	char *sql_query;
	signed short int i = 0;
	char zs[96];
	char sql[80] = "SELECT r0,g0,b0,w0,d0,r1,g1,b1,w1,d1 from rgboutput where id = 0;";


	//printf("cv_demon%ld: read Start\n",ti);
beginn:
	//printf("SQL:\"%s\"\n",sql);
	sql_query = sqlite3_mprintf("%s",sql);								// Vorbereitung des SQL-Querys
	
	// Umwandlung des SQl Querys in Byte-code
	rw = sqlite3_prepare(ptr,sql_query,strlen(sql_query),&vm,NULL);
	if (rw != SQLITE_OK)
		{
		// printf("(Error %d bei \"%s\")\n",rw,sql);
		// printf("SQLITE Read lock problem !!!!!\n");
		sqlite3_finalize(vm);																// Loeschen des Statements
		sqlite3_free(sql_query);														// freigeben des Bytecodes
		usleep(25000);																			// 25 ms warten
		goto beginn;
		}

	// Ausgabe des Datensatzes
	while(sqlite3_step(vm) != SQLITE_DONE)
		{
		i++;
		if(sqlite3_column_text(vm,0) == NULL)
			{
			// printf("cv_demon: sql_read_rgbvalues: NULL Pointer als Result bei \"%s\" - alte Werte werden beibehalten !\n",sql);
			rw = -1;
			goto final;
			}
		// printf("%s\n",sqlite3_column_text(vm,0));						// Inhalt des Datensatzes ausgeben
		strcpy(zs,(char *)sqlite3_column_text(vm,0));				// String aus dem DB-Recordset kopieren
		// printf("RW0:\"%s\"\n",zs);
		*r0 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,1));				// String aus dem DB-Recordset kopieren
		*g0 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,2));				// String aus dem DB-Recordset kopieren
		*b0 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,3));				// String aus dem DB-Recordset kopieren
		*w0 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,4));				// String aus dem DB-Recordset kopieren
		*d0 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,5));				// String aus dem DB-Recordset kopieren
		*r1 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,6));				// String aus dem DB-Recordset kopieren
		*g1 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,7));				// String aus dem DB-Recordset kopieren
		*b1 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,8));				// String aus dem DB-Recordset kopieren
		*w1 = atol(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,9));				// String aus dem DB-Recordset kopieren
		*d1 = atol(zs);
		rw = 1;
		}
	if(i < 1)																							// wenn kein Datensatz, -1 zurckgeben
		{
		printf("cv_demon: sql_read_rgbvalues: kein Datensatz - alte Werte werden beibehalten !\n");
		rw = 0;
		}

final:
	sqlite3_finalize(vm);																	// Loeschen des Statements
	sqlite3_free(sql_query);															// freigeben des Bytecodes
	// printf("cv_demon: read End\n");
	return(rw);
	}



// *****   sql_write_rgbvalues   -   Schreiben der RGB-Output-Werte in die tmpDB   *****

void sql_write_rgbvalues(sqlite3 *ptr,unsigned char r0,unsigned char g0,unsigned char b0,unsigned char w0,unsigned char d0,unsigned char r1,unsigned char g1,unsigned char b1,unsigned char w1,unsigned char d1)
	{
	char sql[255];

	// printf("rgb_demon: write Start\n");
	sprintf(sql,"UPDATE rgboutput SET r0=%d,g0=%d,b0=%d,w0=%d,d0=%d,r1=%d,g1=%d,b1=%d,w1=%d,d1=%d WHERE id = 0;",r0,g0,b0,w0,d0,r1,g1,b1,w1,d1);
	// printf("SQL: \"%s\"\n",sql);
	do_sqlyy(ptr,sql,0);
	// printf("rgb_demon: write End\n");
	}



// *****   sql_read_potivalues   -   Lesen der Poti-Spannungen aus der tmpDB   *****

signed short int sql_read_potivalues(sqlite3 *ptr,unsigned char *p0,unsigned char *p1,unsigned char *p2,unsigned char *p3,unsigned char *p4,unsigned char *p5)
	{
	sqlite3_stmt *vm;
	char *sql_query;
	signed short int i = 0;
	char zs[96];
	signed short int rw;
	char sql[80] = "SELECT [p0],[p1],[p2],[p3],[p4],[p5] from [potis] where [id]=0;";


	// printf("SQL:\"%s\"\n",sql);
	sql_query = sqlite3_mprintf("%s",sql);								// Vorbereitung des SQL-Querys
	
	// Umwandlung des SQl Querys in Byte-code
	rw = sqlite3_prepare(ptr,sql_query,strlen(sql_query),&vm,NULL);
	if (rw != SQLITE_OK)
		{
		// printf("(Error %d bei \"%s\")\n",rw,sql);
		// printf("SQLITE Read lock problem !!!!!\n");
		sqlite3_finalize(vm);																// Loeschen des Statements
		sqlite3_free(sql_query);														// freigeben des Bytecodes
		rw = -2;																						// Rckmeldung: "DB-Fehler"
		goto ende;
		}

	// Ausgabe des Datensatzes
	while(sqlite3_step(vm) != SQLITE_DONE)
		{
		i++;
		if(sqlite3_column_text(vm,0) == NULL)
			{
			// printf("sql_read_potivalues: NULL Pointer als Result bei \"%s\" - alte Werte werden beibehalten !\n",sql);
			rw = -1;
			goto final;
			}
		// printf("%s\n",sqlite3_column_text(vm,0));						// Inhalt des Datensatzes ausgeben
		strcpy(zs,(char *)sqlite3_column_text(vm,0));				// String aus dem DB-Recordset kopieren
		*p0 = atoi(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,1));				// String aus dem DB-Recordset kopieren
		*p1 = atoi(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,2));				// String aus dem DB-Recordset kopieren
		*p2 = atoi(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,3));				// String aus dem DB-Recordset kopieren
		*p3 = atoi(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,4));				// String aus dem DB-Recordset kopieren
		*p4 = atoi(zs);
		strcpy(zs,(char *)sqlite3_column_text(vm,5));				// String aus dem DB-Recordset kopieren
		*p5 = atoi(zs);
		// printf("P0:%d\n",*p0);
		rw = 1;
		}
	if(i < 1)																							// wenn kein Datensatz, -1 zurckgeben
		{
		// printf("sql_read_potivalues: kein Datensatz - alte Werte werden beibehalten !\n");
		rw = -0;
		}

final:
	sqlite3_finalize(vm);																	// Loeschen des Statements
	sqlite3_free(sql_query);															// freigeben des Bytecodes

ende: return(rw);
	}



// *****   Hauptschleife   *****

int main(int argc,char **argv)
	{
	unsigned char 	old_poti0 = 0;
	unsigned char 	old_poti1 = 0;
	unsigned char 	old_poti2 = 0;
	unsigned char 	old_poti3 = 0;
	unsigned char 	old_poti4 = 0;
	unsigned char 	old_poti5 = 0;
	unsigned char		t = 0;
	signed long int	i;
	unsigned long		az_potis = 0;													// nderungszhler Potis
	char ep[1024];																				// Puffer fr Logfile-Meldungen


	// auf existierendes Shared Memory fr Demonstatus zugreifen
	if (use_my_demonstatus_shm(8) != 1) goto errorende;		// MY_DEMON_ID = 8: cv_demon

	// Status von Demon in SHM eintragen
	my_demonstatus_data[my_demon_id].demon_id = my_demon_id;														// 0 ist Default, hier wird die eigene ID eingetragen
	sprintf(my_demonstatus_data[my_demon_id].name_string,"cv_demon");										// Name
	sprintf(my_demonstatus_data[my_demon_id].version_string,"%s",CVDEMON_VERSION);			// Versionsnummer
	my_demonstatus_data[my_demon_id].last_boot = time(0);																// Timestamp(UTC) des Starts
	my_demonstatus_data[my_demon_id].last_activity = time(0);														// Timestamp(UTC) der letzten Aktion
	my_demonstatus_data[my_demon_id].ti = ti;																						// Counter
	sprintf(my_demonstatus_data[my_demon_id].debug_message,"Start");										// letzte Debug-Meldung
	my_demonstatus_data[my_demon_id].error_message[0] = 0;															// letzte Fehlermeldung
	my_demonstatus_data[my_demon_id].status_long = 0;																		// numerischer Status
	sprintf(my_demonstatus_data[my_demon_id].status_string,"startup");									// String-Status

	// Logging-System initialisieren
	strcpy (log_filename,"/www/reports/cv_demon.log");		// Logfilenamen setzen
	log_level = 4;																				// Debuglevel setzen
	log_kill();																						// Logfile loeschen
	sprintf(ep,"----------   cv_demon V %s started (debuglevel=%d)   ----------",CVDEMON_VERSION,log_level);
	log_append(1,ep);																			// Startmeldung ins Logfile schreiben

	sqlite3_open(VAR_DATABASE_FILE,&sql_var_handle);			// SQLITE3-Datenbank ffnen


	// Einstellungen fr die Fernbedienung
	fernbedienung = sqldb_read_long("/control/rgb/remotecontrol/type");

	// Start loggen
	printf("cv_demon V %s started (debuglevel=%d)\n",CVDEMON_VERSION,log_level);


	// ----- Poti-Werte aus der tmpDB lesen -----
	// ... und gelesene Werte als alte Werte merken
	sql_read_potivalues(sql_var_handle,&old_poti0,&old_poti1,&old_poti2,&old_poti3,&old_poti4,&old_poti5);


	update_demondebug("running");													// letzte Debug-Meldung
hauptschleife:
	update_demonstatus(ti,1,"main loop");									// Status von Demon in SHM aktualisieren
	i = 0;
	usleep(10000);																				// 10 ms warten
	ti++;																									// Schleifenzhler erhhen
	// printf("ti=%ld\n",ti);

	// ----- Poti-Werte aus der tmpDB lesen -----
	sql_read_potivalues(sql_var_handle,&t_poti0,&t_poti1,&t_poti2,&t_poti3,&t_poti4,&t_poti5);

	// feststellen, ob sich etwas gendert hat
	az_potis = 0;
	if (old_poti0			!= t_poti0		) az_potis++;
	if (old_poti1			!= t_poti1		) az_potis++;
	if (old_poti2			!= t_poti2		) az_potis++;	
	if (old_poti3			!= t_poti3		) az_potis++;
	if (old_poti4			!= t_poti4		) az_potis++;
	if (old_poti5			!= t_poti5		) az_potis++;	
	if (az_potis == 0) goto hauptschleife;								// keine nderung, dann brauchen wir nichts zu tun
	// printf("CV0:%d, CV1:%d, CV2:%d, CV3:%d, CV4:%d, CV5:%d\n",t_poti0,t_poti1,t_poti2,t_poti3,t_poti4,t_poti5);
	sprintf(ep,"CV0:%d,CV1:%d,CV2:%d,CV3:%d,CV4:%d,CV5:%d",t_poti0,t_poti1,t_poti2,t_poti3,t_poti4,t_poti5);
	update_demondebug(ep);																// letzte Debug-Meldung

	// ----- RGB-Werte aus der tmpDB lesen -----
reread:
	t = sql_read_rgbvalues(sql_var_handle,&t_r0,&t_g0,&t_b0,&t_w0,&t_dim0,&t_r1,&t_g1,&t_b1,&t_w1,&t_dim1);
	// printf("R0:%d,G0:%d,B0:%d,W0:%d,  R1:%d,G1:%d,B1:%d,W1:%d\n",t_r0,t_g0,t_b0,t_w0,t_r1,t_g1,t_b1,t_w1);
	if (t != 1)
		{
		i++;
		if (i > 10)
			{
			printf("cv_demon: kann keine RGB-Werte aus der tmp-DB lesen (Errorcode: %d)\n",t);
			i = 0;
			}
		sleep(20000);																				// 20 ms warten
		goto reread;																				// und es dann wieder probieren
		}

	// ----- Wenn an den Potis gedreht wurde, dann die RGB-Werte nach Potivorgaben setzen -----

	if (az_potis > 0)
		{
		t_r0 = t_poti0;																			// RGB-Farben nach Potis setzen
		t_g0 = t_poti1;
		t_b0 = t_poti2;
		t_r1 = t_poti3;
		t_g1 = t_poti4;
		t_b1 = t_poti5;
		sql_write_rgbvalues(sql_var_handle,t_r0,t_g0,t_b0,t_w0,t_dim0,t_r1,t_g1,t_b1,t_w1,t_dim1);
		}



	// Alte Werte merken
	old_poti0 = t_poti0;	
	old_poti1 = t_poti1;	
	old_poti2 = t_poti2;	
	old_poti3 = t_poti3;	
	old_poti4 = t_poti4;	
	old_poti5 = t_poti5;	

	goto hauptschleife;

// ende:
	sqlite3_close(sql_var_handle);												// Datenbank schlieen
	return(0);

errorende:
	return(-1);
	}
