// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   d0bus_demon.c   -   Demon zum Empfang von D0-Bus Daten ber IR-Auslesekopf                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2011 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************

// **********   Eintrge in der Konfigurations-Datenbank, die vom Demon ausgewertet werden   **********
//
// X = Portnummer, die in der Kommandozeile bergeben wurde (0...3)
// 
// /control/d0bus/portX/loglevel												Log-Level
// /control/d0bus/portX/reload_counter									Zhler, um festzustellen ob ein Neuladen der Config ntig ist
// /control/d0bus/portX/enabled													Port Aktiv (0: Nein, 1: Ja)
// /control/d0bus/portX/typ															Zhlertyp (0: undefiniert, 1: EMH EHZH)
// /control/d0bus/portX/comport													Serielle Schnittstelle, an der das Zhlerinterface hngt (z.B. "/dev/ttyS1")
// /control/d0bus/portX/baudrate												normalerweise 9600
// /control/d0bus/portX/name														Benennung des Zhlers
// /control/d0bus/portX/comment													Blatext
// /control/d0bus/portX/voltage0												Spannung Phase 0
// /control/d0bus/portX/voltage1												Spannung Phase 1
// /control/d0bus/portX/voltage2												Spannung Phase 2
//
//
// Eintrge in der tmp-DB, die vom Demon upgedated werden. (Anlegen durch den powermeter_demon)
//
// D0_numberX_watt0																			Wattzahl 0
// D0_numberX_watt1																			Wattzahl 1
// D0_numberX_watt2																			Wattzahl 2
// D0_numberX_kwh0																			Zhlerstand Zhler 0
// D0_numberX_kwh1																			Zhlerstand Zhler 1
// D0_numberX_kwh2																			Zhlerstand Zhler 2
// D0_numberX_ti																				laufende Nummer (-2 wenn noch nie was war)
// D0_numberX_timestamp																	Zeitpunkt des letzten Daten-Updates



#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <termios.h>
#include <time.h>
#include <sys/time.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "sqldb.h"
#include "sensor_demon.h"
#include "shm_use.h"
#include "d0bus_demon.h"
#include "logging.h"



// *****   Globale Variablen   *****

unsigned short int		d0bus_portnumber;									// Portnummer (0...3)
unsigned long int			d0bus_meter_reloadcounter;				// Reloadcounter fr Config-nderungen
unsigned long int			last_read_timestamp = 0;					// Zeit des letzten Reads
signed char						d0bus_meter_enabled;
signed char						d0bus_meter_typ;
char									d0bus_meter_comport[20];
unsigned long int			d0bus_meter_baudrate;
double								d0bus_meter_value_kwh[3];
double								d0bus_meter_value_watt[3];
double								d0bus_meter_voltage[3];
char									d0bus_meter_name[80];
char									d0bus_meter_comment[80];
int										d0bus_meter_fd;
sqlite3								*sql_var_handle;									// Handle fr die DB


#define VAR_DATABASE_FILE "/tmp/var.s3db"



// *****   rs232_open("/dev/...")   -   seriellen Port ffnen   *****

int rs232_open(char device[],unsigned long int baudrate)
	{
	int fd = 0;
	struct termios my_termios;

	printf("device=%s\n",device);

	fd = open(device,O_RDWR | O_NOCTTY | O_NDELAY);
	if (fd < 0)
		{
		printf("open error %d %s\n", fd, strerror(fd));
		}
	else
		{
		// printf("fd is %d\n", fd);
		tcgetattr(fd, &my_termios);
		// printf("old cflag=%08x\n", my_termios.c_cflag);
		// printf("old oflag=%08x\n", my_termios.c_oflag);
		// printf("old iflag=%08x\n", my_termios.c_iflag);
		// printf("old lflag=%08x\n", my_termios.c_lflag);
		// printf("old line=%02x\n", my_termios.c_line);
		tcflush(fd, TCIFLUSH);

		if (baudrate == 9600)																// 9600: 9600, 8, N, 1
			{
			my_termios.c_cflag = B9600 | CS8  |CREAD | CLOCAL | HUPCL;
			goto baud_set;
			}
		if (baudrate == 9601)																// 9601: 9600, 7, E, 1
			{
			my_termios.c_cflag = B9600 | CS7  | PARENB | CREAD | CLOCAL | HUPCL;
			goto baud_set;
			}
		printf("Invalid Baudrate %ld specified, exiting !\n",baudrate);
		exit(-1);

baud_set:
		my_termios.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // Raw processing, kein Echo !
		my_termios.c_iflag &= ~(IXON | IXOFF | IXANY); // kein Handshaking

		cfsetospeed(&my_termios, B9600);
		tcsetattr(fd, TCSANOW, &my_termios);
 
		// printf("new cflag=%08x\n", my_termios.c_cflag);
		// printf("new oflag=%08x\n", my_termios.c_oflag);
		// printf("new iflag=%08x\n", my_termios.c_iflag);
		// printf("new lflag=%08x\n", my_termios.c_lflag);
		// printf("new line=%02x\n", my_termios.c_line);
		}
	return fd;
	}



// *****   rs232_write - String seriell ausgeben   *****

int rs232_write(int fd,char* psOutput)
	{
	int iOut;
	if (fd < 1)
		{
		printf(" port is not open\n");
		return -1;
		}

	iOut = write(fd,psOutput,strlen(psOutput));
	if (iOut < 0)
		{
		printf("write error %d %s\n",iOut,strerror(iOut));
		}
	else
		{
		printf("wrote %d chars: %s\n", iOut, psOutput);
		}
	return iOut;
	}



// *****   rs232_read - ein Zeichen seriell einlesen   *****

signed short int rs232_read(int fd)
	{
	char tp[2];																						// temporrer Mini-Puffer
	signed short int rw = -1;
	signed long int l;


	l = read(fd,tp,1);																		// Nachschauen, ob ein Zeichen angekommen ist
	if (l > 0)
		{
		if (l > 1) printf("[L=%ld]",l);
		rw = tp[0];
		}
	else
		{
		rw = -1;
		}

	return(rw);
	}



// *****   get_ms_now()   -   aktuelle Zeit in ms ermitteln   *****

long get_ms_now(void)
	{
	long rw;
	const long msdivider = CLOCKS_PER_SEC / 1000;					// Teiler um von Clockticks aus ms zu kommen
  struct timeval tv;
	time_t curtime;
	struct tm *myTm;


	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln 
  curtime = tv.tv_sec;																	// Lores-Zeit in s reusziehen
	myTm = localtime(&curtime);														// ...und berfhren in tm-Struktur
	rw = (myTm->tm_hour * 3600 + myTm->tm_min * 60 + myTm->tm_sec) * 1000 + tv.tv_usec / msdivider;
	return(rw);
	}



// **********   Demon-Configfile einlesen   **********

void d0bus_demon_readconfig(unsigned char nummer)
	{
	char ep[1024];


	sprintf(ep,"/control/d0bus/port%d/loglevel",nummer);
	log_level = sqldb_read_long(ep);
	sprintf(ep,"/control/d0bus/port%d/reload_counter",nummer);
	d0bus_meter_reloadcounter = sqldb_read_long(ep);

	sprintf(ep,"/control/d0bus/port%d/enabled",nummer);
	d0bus_meter_enabled = sqldb_read_long(ep);
	sprintf(ep,"/control/d0bus/port%d/typ",nummer);
	d0bus_meter_typ = sqldb_read_long(ep);
	sprintf(ep,"/control/d0bus/port%d/baudrate",nummer);
	d0bus_meter_baudrate = sqldb_read_long(ep);
	sprintf(ep,"/control/d0bus/port%d/comport",nummer);
	sqldb_read_string(ep,d0bus_meter_comport);
	sprintf(ep,"/control/d0bus/port%d/name",nummer);
	sqldb_read_string(ep,d0bus_meter_name);
	sprintf(ep,"/control/d0bus/port%d/comment",nummer);
	sqldb_read_string(ep,d0bus_meter_comment);

	d0bus_meter_value_kwh[0] = 0;
	d0bus_meter_value_kwh[1] = 0;
	d0bus_meter_value_kwh[2] = 0;
	d0bus_meter_value_watt[0] = 0;
	d0bus_meter_value_watt[1] = 0;
	d0bus_meter_value_watt[2] = 0;

	sprintf(ep,"/control/d0bus/port%d/voltage0",nummer);
	d0bus_meter_voltage[0] = sqldb_read_float(ep);
	sprintf(ep,"/control/d0bus/port%d/voltage1",nummer);
	d0bus_meter_voltage[1] = sqldb_read_float(ep);
	sprintf(ep,"/control/d0bus/port%d/voltage2",nummer);
	d0bus_meter_voltage[2] = sqldb_read_float(ep);

	if(d0bus_meter_voltage[0] < 10) d0bus_meter_voltage[0] = 230;
	if(d0bus_meter_voltage[1] < 10) d0bus_meter_voltage[1] = 231;
	if(d0bus_meter_voltage[2] < 10) d0bus_meter_voltage[2] = 232;

	sprintf(ep,"Global D0Bus[%d] Configuration read. Loglevel:%d, Enabled:%d, Comport:\"%s\", %ld Baud, Typ %d",nummer,log_level,d0bus_meter_enabled,d0bus_meter_comport,d0bus_meter_baudrate,d0bus_meter_typ);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	printf("%s\n",ep);
	}



// *****   sql_update_d0werte   -   Schreiben der D0-Zhlerstnde und Wattzahlen in die tmp-DB   *****

void sql_update_d0werte(sqlite3 *ptr,unsigned char nummer,double watt0,double watt1,double watt2,double kwh0,double kwh1,double kwh2)
	{
	char sql[255];
	static unsigned long int local_ti = 0;
	// time_t timer;
	// struct tm *tblock;
	// char zeitstring[40];
	double wattsumme;
	double kwhsumme;


	wattsumme = watt0 + watt1 + watt2;
	kwhsumme = kwh0 + kwh1 + kwh2;

/*
	sprintf(sql,"UPDATE [d0] set [value]='%1.3f' where [tag]='D0_number%d_watt0';",watt0,nummer);
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"UPDATE [d0] set [value]='%1.3f' where [tag]='D0_number%d_watt1';",watt1,nummer);
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"UPDATE [d0] set [value]='%1.3f' where [tag]='D0_number%d_watt2';",watt2,nummer);
	do_sqlyy(ptr,sql,0);

	sprintf(sql,"UPDATE [d0] set [value]='%1.4f' where [tag]='D0_number%d_kwh0';",kwh0,nummer);
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"UPDATE [d0] set [value]='%1.4f' where [tag]='D0_number%d_kwh1';",kwh1,nummer);
	do_sqlyy(ptr,sql,0);
	sprintf(sql,"UPDATE [d0] set [value]='%1.4f' where [tag]='D0_number%d_kwh2';",kwh2,nummer);
	do_sqlyy(ptr,sql,0);

	sprintf(sql,"UPDATE [d0] set [value]='%ld' where [tag]='D0_number%d_ti';",local_ti,nummer);
	do_sqlyy(ptr,sql,0);
*/

	local_ti++;

/*
	// TODO: Timestamp updaten (Ingo fragen: UTC oder Localtime ?) momentan Localtime.
	timer = time(NULL);
	tblock = localtime(&timer);
	sprintf(zeitstring,"%04d-%02d-%02d %02d:%02d:%02d",tblock->tm_year+1900,tblock->tm_mon + 1,tblock->tm_mday,tblock->tm_hour,tblock->tm_min,tblock->tm_sec);
	sprintf(sql,"UPDATE [d0] set [value]='%s' where [tag]='D0_number%d_timestamp';",zeitstring,nummer);
	do_sqlyy(ptr,sql,0);
*/

	// auch direkt die Measurements-Table updaten
	sprintf(sql,"UPDATE [measurements] set [dkwh%d]='%1.4f',[w%d]='%1.1f',[d%dwatt0]='%1.1f',[d%dwatt1]='%1.1f',[d%dwatt2]='%1.1f',[a%d]='%1.3f' where [id]='0';",
					nummer,kwhsumme,nummer + 8,wattsumme,nummer,watt0,nummer,watt1,nummer,watt2,
					nummer + 8,watt0 / d0bus_meter_voltage[0] + watt1 / d0bus_meter_voltage[1] + watt2 / d0bus_meter_voltage[2]);
	do_sqlyy(ptr,sql,0);
	}



// *****   print_usage() - Hinweise zur Kommandozeile ausgeben   *****

void print_usage()
	{
	printf("USAGE: d0bus_demon portnumber (portnumber: 0...3)\n");
	exit(0);
	}



// *****   d0bus_read_emh_ehz - Zhler von EMH (NZR) auslesen   *****

void d0bus_read_emh_ehz(int fd)
	{
	signed long int				t;
	signed short int 			a;
	unsigned char					il;
	unsigned char					ih;
	unsigned char					iz;
	char									lru[10] = {0,0,0,0,0,0,0,0,0,0};// last received Unsinn
	char									d0puffer[4097];									// Puffer fr D0-Bus-Daten
	signed short int			d0puffer_laenge = 4096;					// max. Lnge bevor Reset
	signed short int			d0pointer;											// Zeiger in den Puffer
	signed short int			d0status;												// D0-Schnittstellenstatus
	double								z0;
	double								z1;
	double								z2;
	double								watt;
	char									ep[1024];												// Text fr Logging

	// signed short int 			z;
	// char									hd_ascii[32];


reset_input:
	last_read_timestamp = get_ms_now();										// now() merken
	il = 0;
	ih = 0;
	iz = 255;
	d0pointer = 0;																				// Puffer lschen
	d0status = 0;																					// Status auf Scan setzen

	// d0status Werte:
	// 0: Suche Anfang
	// 1: Anfang gefunden, Bytes werden gelesen
	// 2: Ende gefunden

pollschleife:
	a = rs232_read(d0bus_meter_fd);												// Nachschauen, ob ein Zeichen angekommen ist
	// printf("l=%d\n",l);
	if (a != -1)
		{
		// printf("%c",a);																	// Ja, dann in den Empfangspuffer schreiben
/*
		if(iz >= 16)
			{
			iz = 0;
			printf("%s\n",hd_ascii);
			for (t = 0; t <= 16;t++)
				{
				hd_ascii[t] = 0;
				}
			if (ih < 16) printf("0");
			printf("%x",ih);
			if (il < 16) printf("0");
			printf("%x: ",il); 
			}
		z = a;																							// Zeichen nehmen
		if (a < 32) z = '.';																// wenn np-char < 32, "." schreiben
		if ((a >= 128) && (a < 160)) z = '.';								// wenn np-cher 129 ... 160, dann auch "." schreiben
		hd_ascii[iz] = z;																		// Zeichen merken
		if (a < 16) printf("0");
		printf("%x ",a);
		fflush(stdout);
*/
		il++; if (il == 0) ih++;														// Adrezhler erhhen
		iz++;

		for(t = 9;t >= 0;t--)
			{
			lru[t + 1] = lru[t];
			}
		lru[0] = a;
		}
	else
		{
		goto weiter;																				// Kein Zeichen angekommen
		}

	if (d0status == 1)
		{
		if(d0pointer > d0puffer_laenge)
			{
			sprintf(ep,"--- input buffer overflow - resetting ---");
			log_append(3,ep);
			goto reset_input;
			}
		d0pointer++;																				// Pufferzeiger erhhen
		d0puffer[d0pointer] = a;														// Zeichen in den Puffer schreiben
		}

	if ((lru[0] == 0x01) && (lru[1] == 0x01) && (lru[2] == 0x01) && (lru[3] == 0x01) && (lru[4] == 0x1b) && (lru[5] == 0x1b) && (lru[6] == 0x1b) && (lru[7] == 0x1b))
		{
		iz = 255;
		il = 0;
		ih = 0;
		sprintf(ep,"\n--- Blockanfang [1b 1b 1b 1b 01 01 01 01] ---");
		log_append(4,ep);
		// printf("%s\n",ep);
		d0status = 1;																				// Status auf "Lese Bytes" setzen
		d0pointer = 0;																			// Zeiger auf Anfang des Puffers verbiegen
		goto weiter;
		}

	if ((lru[3] == 0x1a) && (lru[4] == 0x1b) && (lru[5] == 0x1b) && (lru[6] == 0x1b) && (lru[7] == 0x1b))
		{
		iz = 255;
		il = 0;
		ih = 0;
		sprintf(ep,"\n--- Blockende [1b 1b 1b 1b 1a XX YY ZZ] x=%d, y=%d, z=%d ---",lru[2],lru[1],lru[0]);
		log_append(4,ep);
		// printf("%s\n",ep);
		d0status = 2;																				// Status auf "Fertig" setzen
		goto weiter;
		}




weiter: ;
	if (d0status != 2) goto pollschleife;

	// Zhlerstand 0 an Bytes 008C...0090,
	// Zhlerstand 1 an Bytes 00A1...00A5,
	// Zhlerstand 2 an Bytes 00B6...00BA,
	// und die aktuelle Leistung in W an Bytes 00CB...00CE.
	// 308 Datenbytes mssen es sein.

	sprintf(ep,"Block enthielt %d Datenbytes.",d0pointer);
	log_append(4,ep);
	d0status = 0;

	if (d0pointer > 250)
		{
		z0  = (double)d0puffer[0x090 + 1];
		z0 += (double)d0puffer[0x08F + 1] * 256.0;
		z0 += (double)d0puffer[0x08E + 1] * 256.0 * 256.0;
		z0 += (double)d0puffer[0x08D + 1] * 256.0 * 256.0 * 256.0;
		z0 += (double)d0puffer[0x08C + 1] * 256.0 * 256.0 * 256.0 * 256.0;
		z1  = (double)d0puffer[0x0A5 + 1];
		z1 += (double)d0puffer[0x0A4 + 1] * 256.0;
		z1 += (double)d0puffer[0x0A3 + 1] * 256.0 * 256.0;
		z1 += (double)d0puffer[0x0A2 + 1] * 256.0 * 256.0 * 256.0;
		z1 += (double)d0puffer[0x0A1 + 1] * 256.0 * 256.0 * 256.0 * 256.0;
		z2  = (double)d0puffer[0x0BA + 1];
		z2 += (double)d0puffer[0x0B9 + 1] * 256.0;
		z2 += (double)d0puffer[0x0B8 + 1] * 256.0 * 256.0;
		z2 += (double)d0puffer[0x0B7 + 1] * 256.0 * 256.0 * 256.0;
		z2 += (double)d0puffer[0x0B6 + 1] * 256.0 * 256.0 * 256.0 * 256.0;
		watt  = (double)d0puffer[0x0CE + 1];
		watt += (double)d0puffer[0x0CD + 1] * 256.0;
		watt += (double)d0puffer[0x0CC + 1] * 256.0 * 256.0;
		watt += (double)d0puffer[0x0CB + 1] * 256.0 * 256.0 * 256.0;
		sprintf(ep,"W puffer: %d %d %d %d",d0puffer[0x0CB + 1],d0puffer[0x0CC + 1],d0puffer[0x0CD + 1],d0puffer[0x0CE + 1]);
		log_append(4,ep);
		z0 /= 10000;
		z1 /= 10000;
		z2 /= 10000;
		watt /= 10;
		sprintf(ep,"(D0:%d, TI:%ld) Zhler: %1.4f KWh, Watt: %1.1f",d0bus_portnumber,ti,z0,watt);
		log_append(4,ep);
		// printf("%s\n",ep);
		sql_update_d0werte(sql_var_handle,d0bus_portnumber,watt,0,0,z0,0,0);
		}
	else
		{
		sprintf(ep,"Falsche Lnge (%d Bytes), Paket wird ignoriert.",d0pointer);
		log_append(3,ep);
		}
	}



// *****   d0bus_read_easymeter_q3d_ruecklaufsperre - Zhler von Easymeter (Q3D mit Rcklaufsperre) auslesen   *****

void d0bus_read_easymeter_q3d_ruecklaufsperre(int fd)
	{
	signed short int 			a;
	char									d0puffer[128];									// Puffer fr D0-Bus-Daten
	signed short int			d0puffer_laenge = 125;					// max. Lnge bevor Reset
	signed short int			d0pointer;											// Zeiger in den Puffer
	double								kwh0 = 0;
	double								watt0 = 0;
	double								watt1 = 0;
	double								watt2 = 0;
	double								wattsumme = 0;
	signed char						flag_start;											// Merker, ob Start erkannt wurde
	signed char						flag_kwh0 = 0;
	signed char						flag_watt0 = 0;
	signed char						flag_watt1 = 0;
	signed char						flag_watt2 = 0;
	signed char						flag_wattsumme = 0;
	signed char						flag_error = 0;
	char									*p;
	char									ep[1024];												// String fr Debug-Messages


reset_input:
	flag_kwh0 = 0;
	flag_watt0 = 0;
	flag_watt1 = 0;
	flag_watt2 = 0;
	flag_wattsumme = 0;
	flag_start = 0;
	flag_error = 0;
	kwh0 = 0;
	watt0 = 0;
	watt1 = 0;
	watt2 = 0;
	wattsumme = 0;

reset_input2:
	last_read_timestamp = get_ms_now();										// now() merken
	d0pointer = 0;																				// Puffer lschen


pollschleife:
	a = rs232_read(d0bus_meter_fd);												// Nachschauen, ob ein Zeichen angekommen ist
	// printf("l=%d\n",l);
	if (a != -1)
		{
		// printf("%c",a);																	// Ja, dann in den Empfangspuffer schreiben
		if (a == 13) a = 10;																// Sollte ein CR daherkommen, konvertieren wir es in LF
		if (a == 10) a = 0;																	// wenn LF erkannt wurde, ein Endezeichen draus machen
		d0puffer[d0pointer] = a;														// Empfangspuffer mit dem Zeichen befllen
		d0pointer++;																				// und den Pointer erhhen
		if(d0pointer > d0puffer_laenge)
			{
			sprintf(ep,"--- input buffer overflow - resetting ---");
			log_append(3,ep);
			goto reset_input;
			}
		if (a != 0) goto pollschleife;											// auf weitere Zeichen warten

		if (d0puffer[0] == 0) goto reset_input2;						// leere Zeilen ignorieren
		sprintf(ep,"IN: %s",d0puffer);											// Anzeige der eingegangenen Zeichen
		log_append(4,ep);

		if ((d0puffer[0] == '/') && (d0puffer[1] == 'E') && (d0puffer[2] == 'S') && (d0puffer[3] == 'Y'))
			{
			sprintf(ep,"--- Blockanfang ---");								// START
			log_append(4,ep);
			flag_start = 1;
			goto reset_input2;
			}
		if (d0puffer[0] == '!')															// ENDE
			{
			sprintf(ep,"--- Blockende ---");
			log_append(4,ep);
			flag_start = 2;
			goto speichern;
			}
		if (strstr(d0puffer,"1-0:1.8.0*255(") != 0)					// KWH0
			{
			// printf("-- kWh0 gefunden\n");
			if (strstr(d0puffer,"*kWh)") != NULL)
				{
				// printf("-- kWh0 gltiges Ende gefunden\n");
				p = strchr(d0puffer,'(') + 1;
				// printf("Zahlenkette: \"%s\"\n",p);
				kwh0 = atof(p);
				// printf("Zahl:: %1.7f\n",kwh0);
				flag_kwh0 = 1;				
				}
			}
		if (strstr(d0puffer,"1-0:21.7.255*255(") != 0)			// WATT0
			{
			// printf("-- Watt0 gefunden\n");
			if (strstr(d0puffer,"*W)") != NULL)
				{
				// printf("-- gltiges Ende gefunden\n");
				p = strchr(d0puffer,'(') + 1;
				// printf("Zahlenkette: \"%s\"\n",p);
				watt0 = atof(p);
				// printf("Zahl:: %1.2f\n",watt0);
				flag_watt0 = 1;				
				}
			}
		if (strstr(d0puffer,"1-0:41.7.255*255(") != 0)			// WATT1
			{
			// printf("-- Watt1 gefunden\n");
			if (strstr(d0puffer,"*W)") != NULL)
				{
				// printf("-- gltiges Ende gefunden\n");
				p = strchr(d0puffer,'(') + 1;
				// printf("Zahlenkette: \"%s\"\n",p);
				watt1 = atof(p);
				// printf("Zahl:: %1.2f\n",watt1);
				flag_watt1 = 1;				
				}
			}
		if (strstr(d0puffer,"1-0:61.7.255*255(") != 0)			// WATT2
			{
			// printf("-- Watt2 gefunden\n");
			if (strstr(d0puffer,"*W)") != NULL)
				{
				// printf("-- gltiges Ende gefunden\n");
				p = strchr(d0puffer,'(') + 1;
				// printf("Zahlenkette: \"%s\"\n",p);
				watt2 = atof(p);
				// printf("Zahl:: %1.2f\n",watt2);
				flag_watt2 = 1;				
				}
			}

		}
	goto reset_input2;

speichern:
	flag_error = 0;
	if (flag_kwh0 == 0) flag_error++;
	if (flag_watt0 == 0) flag_error++;
	if (flag_watt1 == 0) flag_error++;
	if (flag_watt2 == 0) flag_error++;
	// if (flag_wattsumme == 0) flag_error++;

	if (flag_error != 0)
		{
		if (flag_kwh0 == 0)
			{
			sprintf(ep,"*** kWh0 fehlt");
			log_append(3,ep);
			}
		if (flag_watt0 == 0)
			{
			sprintf(ep,"*** watt0 fehlt");
			log_append(3,ep);
			}
		if (flag_watt1 == 0)
			{
			sprintf(ep,"*** watt1 fehlt");
			log_append(3,ep);
			}
		if (flag_watt2 == 0)
			{
			sprintf(ep,"*** watt2 fehlt");
			log_append(3,ep);
			}
		// if (flag_wattsumme == 0) printf("*** wattsumme fehlt\n");
		}
	else
		{
		sprintf(ep,"(D0:%d, TI:%ld) Zhler0: %1.4f KWh, Watt0: %1.1f, Watt1: %1.1f, Watt2: %1.1f",d0bus_portnumber,ti,kwh0,watt0,watt1,watt2);
		log_append(4,ep);
		sql_update_d0werte(sql_var_handle,d0bus_portnumber,watt0,watt1,watt2,kwh0,0,0);
		}
	}



// *****   Hauptschleife   *****

int main(int argc,char **argv)
	{
	char									ep[1024];												// Puffer fr Logfile-Meldungen


	// Portnummer ermitteln
 	if (argc != 2)
		{
mist_ende:
		print_usage();
		}
		d0bus_portnumber = atoi(argv[1]);
		if (d0bus_portnumber < 0)
			{
			printf("Port number needs to be in the range of 0...3 !\n");
			goto mist_ende;
			}


	// Logging-System initialisieren
	sprintf(log_filename,"/www/reports/d0bus_demon%d.log",d0bus_portnumber); // Logfilenamen setzen
	log_level = 3;																				// Debuglevel setzen

	sqlite3_open(VAR_DATABASE_FILE,&sql_var_handle);			// SQLITE3-Datenbank ffnen

// restart:
	d0bus_demon_readconfig(d0bus_portnumber);


	// Start loggen
	sprintf(ep,"----------   d0bus_demon V %s started (debuglevel=%d, portnumber=%d, Type:%d, Voltages:%1.1fV,%1.1fV,%1.1fV)   ----------",D0BUS_DEMON_VERSION,log_level,d0bus_portnumber,d0bus_meter_typ,d0bus_meter_voltage[0],d0bus_meter_voltage[1],d0bus_meter_voltage[2]);
	log_append(1,ep);																			// Startmeldung ins Logfile schreiben
	printf(ep,"d0bus_demon V %s started (debuglevel=%d, portnumber=%d, Type:%d, Voltages:%1.1fV,%1.1fV,%1.1fV)\n",D0BUS_DEMON_VERSION,log_level,d0bus_portnumber,d0bus_meter_typ,d0bus_meter_voltage[0],d0bus_meter_voltage[1],d0bus_meter_voltage[2]);


	if (d0bus_meter_typ == 1)															// EMH
		{
		d0bus_meter_fd = rs232_open(d0bus_meter_comport, /*d0bus_meter_baudrate*/ 9600); // 9600N81
		}
	if (d0bus_meter_typ == 2)															// Easymeter Q3D mit Rcklaufsperre
		{
		d0bus_meter_fd = rs232_open(d0bus_meter_comport, /*d0bus_meter_baudrate*/ 9601); // 9600E71
		}
	// printf("d0bus_meter_fd[%d]=%d\n",d0bus_portnumber,d0bus_meter_fd);

	// w232(d0bus_meter_fd[0],"Hier mal was Eingeben:\n");



hauptschleife:
	ti++;																									// Schleifenzhler erhhen
	// usleep(10000);																				// 10 ms Rechenzeit freigeben

	sprintf(ep,"/control/d0bus/port%d/reload_counter",d0bus_portnumber);
	if (d0bus_meter_reloadcounter != sqldb_read_long(ep))
		{
		d0bus_demon_readconfig(d0bus_portnumber);
		sprintf(ep,"Global D0 Configuration re-read.");
		log_append(1,ep);																		// Meldung ins Logfile schreiben
		goto hauptschleife;
		}

	if (d0bus_meter_enabled == 0)													// Wenn inaktiv, einfach 5 Sek. trdeln und dann zum Anfang
		{
		sleep(5);
		goto hauptschleife;
		}

	if (d0bus_meter_typ == 1)															// EMH
		{
		d0bus_read_emh_ehz(d0bus_meter_fd);
		goto hauptschleife;
		}

	if (d0bus_meter_typ == 2)															// Easymeter Q3D mit Rcklaufsperre (Einrichtungszhler)
		{
		d0bus_read_easymeter_q3d_ruecklaufsperre(d0bus_meter_fd);
		goto hauptschleife;
		}

	goto hauptschleife;



// ende:
	sqlite3_close(sql_var_handle);												// tmp-Datenbank schlieen
	return(0);
	}
