// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   download_demon.c   -   Demon zum automatischen Saugen von XML-Files via HTTP                                  **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2009 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************

// **********   Eintrge in der Konfigurations-Datenbank, die vom Demon ausgewertet werden   **********
//
// /control/powerline/master/comport"										Serielle Schnittstelle, an der das PL-Mastermodul hngt (normalerweise /dev/ttyS4)
// /control/powerline/master/baudrate"									Baudrate dieser Schnittstelle (normalerweise 4800)
// /control/powerline/server/udpport"										UDP-Port fr Steuerkommandos (54009)
// /control/powerline/server/username										zu bergebender Username
// /control/powerline/server/password										zu bergebendes Pawort
// /control/powerline/max_retries												Anzahl Versuche, einen Slave zu erreichen
// /control/powerline/max_slaves"												Anzahl zu verwaltender Slaves
// /control/powerline/reload_counter"										Zhler, um festzustellen ob ein Neuladen der Config ntig ist
// /control/actors/actorXX/value"												gibt den zu schaltenden Wert; wird mit gepolltem Wert upgedated



#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "sqldb.h"
#include "download_demon.h"
#include "logging.h"
#include "xml_lib.h"



// *****   Globale Variablen   *****

#define CONFIG_DATABASE_FILE "/etc/allnetenv/config.s3db"

unsigned short int		cache_max_files;
signed char						cache_file_enabled[DOWNLOAD_MAX_FILES];
char									cache_file_url[DOWNLOAD_MAX_FILES][150];
char									cache_file_local_filename[DOWNLOAD_MAX_FILES][150];
signed long int				cache_file_poll_interval[DOWNLOAD_MAX_FILES];
signed long int				download_reload_counter;
unsigned long int			last_read_timestamp = 0;					// Zeit des letzten Reads
signed long int				zyklischer_sklave = 0;						// Nummer fr Zyklische Abfrage
signed long int				cache_file_nextdownload[DOWNLOAD_MAX_FILES];



// *****   Demon-Config lschen   *****

void dldemon_setzedefault(void)
	{
	unsigned short int t;

	for(t = 0;t < DOWNLOAD_MAX_FILES;t++)
		{
		cache_file_enabled[t] = 0;
		cache_file_url[t][0] = 0;
		cache_file_local_filename[t][0] = 0;
		cache_file_poll_interval[t] = 60;
		cache_file_nextdownload[t] = 0;
		}
	}



// *****   Demon-Configfile einlesen   *****

void dldemon_readconfig(void)
	{
	char ep[1024];
	unsigned short int t;
	unsigned short int files = 0;


	download_reload_counter = sqldb_read_long("/control/sensors_remote/reload_counter");

	log_level = sqldb_read_long("/control/sensors_remote/loglevel");
	cache_max_files = sqldb_read_long("/control/sensors_remote/max_pages");

	for(t = 0;t < cache_max_files;t++)
		{
		sprintf(ep,"/control/sensors_remote/page%d/enabled",t);
		cache_file_enabled[t] = sqldb_read_long(ep);
		if (cache_file_enabled[t] > 0)
			{
			files++;
			cache_file_nextdownload[t] = 0;
			sprintf(ep,"/control/sensors_remote/page%d/web_url",t);
			sqldb_read_string(ep,cache_file_url[t]);
			sprintf(ep,"/control/sensors_remote/page%d/local_filename",t);
			sqldb_read_string(ep,cache_file_local_filename[t]);
			sprintf(ep,"/control/sensors_remote/page%d/poll_interval",t);
			cache_file_poll_interval[t] = sqldb_read_long(ep);
			}
		}

	sprintf(ep,"Global download_demon Configuration read. Loglevel:%d, Files:%d",log_level,files);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	}



// *****   Hauptschleife   *****

int main(int argc,char **argv)
	{
	unsigned long					ti = 0;													// Zhler, geht bei jedem Durchlauf um 1 hoch
	char									ep[1024];												// Puffer fr Logfile-Meldungen
	signed long int				t;
	struct timeval 				tv;															// Uhrzeit-Kram
	signed long int				now;
	char									kommando[500];									// Puffer fr WGET-Befehl
	char									puffer[66000];									// Puffer fr das Einlesen des Files
	signed long int				filelaenge;											// Lnge des Files



	// Einstellungen fr lokalen Demon
	dldemon_setzedefault();
	dldemon_readconfig();

	// Logging-System initialisieren
	strcpy (log_filename,"/www/reports/download_demon.log"); // Logfilenamen setzen
	sprintf(ep,"----------   download_demon V %s started (debuglevel=%d)   ----------",DOWNLOADDEMON_VERSION,log_level);
	log_append(1,ep);																			// Startmeldung ins Logfile schreiben

	printf("download_demon V %s started (debuglevel=%d)\n",DOWNLOADDEMON_VERSION,log_level);


hauptschleife:
	ti++;																									// Schleifenzhler erhhen
	usleep(200000);																				// 1/5 Sekunde Rechenzeit freigeben

	if (download_reload_counter != sqldb_read_long("/control/sensors_remote/reload_counter"))
		{
		dldemon_readconfig();
		sprintf(ep,"Global Download Configuration re-read.");
		log_append(1,ep);																			// Meldung ins Logfile schreiben
		goto hauptschleife;
		}


	// -----   Zyklisches Holen der XML-Files   -----
	for(t = 0;t < DOWNLOAD_MAX_FILES;t++)
		{
		if(cache_file_enabled[t] > 0)
			{
			gettimeofday(&tv,NULL);														// Hires-Zeit ermitteln
			now = tv.tv_sec;																	// now() merken
			// sprintf(ep,"File %ld: next:%ld, now:%ld",t,cache_file_nextdownload[t],now);
			// log_append(4,ep);
			if (cache_file_nextdownload[t] <= now)
				{
				// sprintf(kommando,"wget \"%s\" --output-document=\"/tmp/download.tmp\" --quiet",cache_file_url[t]);
				sprintf(kommando,"curl -s -o \"/tmp/download.tmp\" \"%s\"",cache_file_url[t]);
				sprintf(ep,"Kommando1: \"%s\"",kommando);
				log_append(4,ep);
				system(kommando);

				filelaenge = readfileinstring("/tmp/download.tmp",puffer);
				// sprintf(ep,"File hat %ld Bytes",filelaenge);
				// log_append(4,ep);

				sprintf(kommando,"mv /tmp/download.tmp %s",cache_file_local_filename[t]);
				// sprintf(ep,"Kommando2: \"%s\"",kommando);
				// log_append(4,ep);
				system(kommando);

				sprintf(ep,"\"%s\" --> \"%s\": (%ld Bytes)",cache_file_url[t],cache_file_local_filename[t],filelaenge);
				log_append(4,ep);

				gettimeofday(&tv,NULL);													// Hires-Zeit ermitteln
				now = tv.tv_sec;																// now() merken
				cache_file_nextdownload[t] = now + cache_file_poll_interval[t];
				}
			}
		}

	goto hauptschleife;

// ende:
	return(0);
	}
