// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   lcd_demon.c   -   Demon zum Anzeigen von Strings im ALL5000 LCD                                               **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2009 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************

// **********   Eintrge in der Konfigurations-Datenbank, die vom Demon ausgewertet werden   **********
//
// /sys/lcd/udp_rx_port																	UDP-Portnummer, auf die gelauscht wird
// /sys/lcd/udp_rx_username															zu bergebender Username
// /sys/lcd/udp_rx_password															zu bergebendes Pawort


#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h> 
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/select.h>
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "sqldb.h"
#include "lcm.h"


// **********   Globale Variablen   **********

signed long int ti = 0;																	// System-Timer
unsigned long int s0_ticks_per_kwh;											// Anzahl Impulse pro KWh (meist 1000, aber auch 100 gibt es)
unsigned long int s0_tickcount;													// Anzahl Impulse seit Programmstart
double s0_kwh_count;																		// Stromverbrauch in KWh seit letztem Reset
unsigned char uhrzeit_sekunde;													// Uhr Sekunde
unsigned char uhrzeit_minute;														// Uhr Minute
unsigned char uhrzeit_stunde;														// Uhr Stunde
unsigned char last_sekunde = 80;												// letzte Sekunde
unsigned short int udp_rx_port = 54321;									// TCP Socket fr Client/Server Verbindung
char						udp_rx_username[1024] = "";							// Username fr UDP-Verbindung
char						udp_rx_password[1024] = "";							// Passwort fr UDP-Verbindung



// *****   demon_readconfig   -   Konfigurationsdaten lesen   *****

void demon_readconfig(void)
	{
	udp_rx_port = sqldb_read_long("/control/lcd/server/udp_rx_port");
	sqldb_read_string("/control/lcd/server/udp_rx_username",udp_rx_username);
	sqldb_read_string("/control/lcd/server/udp_rx_password",udp_rx_password);
	}



// **********   Demon   **********

int main(int argc,char **argv)
	{
	int											udp_socket;
	struct sockaddr_in			udp_control1;
	struct sockaddr_in			udp_control2;
	char										udp_puffer[30001];
	unsigned int						udp_laenge;
	int											udp_received = 0;
	char										udp_received_username[21];
	char										udp_received_password[21];
	char										c;
	signed short int				t;
  fd_set									read_fds;
	int											fd = 0;
	char										ret;
	// int							read_count;
	// char						buf[RESULT_LENGTH];
	// int							i;
	// char						title[STR_MAX_LENGTH*2];
	// int							title_length;
	char										headline[33] = "** ALL5000 **";
	signed short int				all5000 = 0;


	// LCD ist nur beim ALL5000 eingebaut; also beenden wenn ein anderes Gert erkannt wurde
	all5000 = sqldb_read_long("/sys/hardware/numeric_model");
	if (all5000 != 5000) goto endeende;


	demon_readconfig();																		// Einstellungen fr lokalen Demon

	// Create the UDP socket
	if ((udp_socket = socket(PF_INET,SOCK_DGRAM,IPPROTO_UDP)) < 0) // UDP Socket anlegen
		{
		printf("ERROR: Es konnte kein neuer UDP-Socket angelegt werden.");
		goto ende;
		}
	if (fcntl(udp_socket,F_SETFL,O_NONBLOCK) < 0)					// UDP-Socket auf Non-Blocking setzen
		{
		printf("ERROR: Socket konnte nicht auf non-blocking geschaltet werden.");
		goto ende;
		}

	// Construct the server sockaddr_in structure
	memset(&udp_control1,0,sizeof(udp_control1));					// Clear struct
	udp_control1.sin_family = AF_INET;										// Internet/IP
	udp_control1.sin_addr.s_addr = htonl(INADDR_ANY);			// Any IP address
	udp_control1.sin_port = htons(udp_rx_port);						// server port

	// Bind the socket
	udp_laenge = sizeof(udp_control1);
	if (bind(udp_socket,(struct sockaddr *) &udp_control1,udp_laenge) < 0)
		{
		printf("ERROR: Failed to bind server socket");
		goto ende;
		}


	FD_ZERO(&read_fds);
 	fd = init_lcm("/dev/ttyS2");
	if(fd < 0) return(-1);
  FD_SET(fd,&read_fds);

	// ***   Clear LCM display   ***
	writE(fd,Clear_display,2);
	ret = read_lcm(fd,RESULT_LENGTH);
	if(ret != RESULT_OK) reset_lcm(fd);
  
// ***   Set cursor back to 0,0   ***
	writE(fd,Return_home,2);
	ret = read_lcm(fd,RESULT_LENGTH);
	if(ret != RESULT_OK) reset_lcm(fd);

// ***   Turn on the LCM display and hide cursor   ***
	writE(fd,Display_ON_NOCURSOR,2);
	ret = read_lcm(fd,RESULT_LENGTH);
	if(ret != RESULT_OK) reset_lcm(fd);

	usleep(timing_val);

// ***   Output title text at 0,0   ***
	lcm_writetext(fd,headline,strlen(headline));

	/* memset((void *)title,0,STR_MAX_LENGTH * 2);
	for (i = 0;i < STR_MAX_LENGTH;i++)
		{
		if(!headline[i]) break;
		title[i * 2] = 0x12;
		title[i * 2 + 1] = headline[i];
		}
	title_length = i;
	writE(fd,title,title_length * 2);
	usleep(timing_val);
	read_count = read(fd,buf,RESULT_LENGTH);
	if(read_count>0)
		{
		// printf("Write2 command return[%d]: %02X\n",read_count,buf[0]);
		}
	*/


	printf("lcd_demon started. Listening to UDP port %d, Username:\"%s\", Password:\"%s\"\n",udp_rx_port,udp_rx_username,udp_rx_password);



hauptschleife:
	usleep(200000);																				// 200 ms warten
	// Receive a message from the client
	memset(&udp_puffer,0,64);															// Puffer lschen
	udp_laenge = sizeof(udp_control2);
	if ((udp_received = recvfrom(udp_socket,udp_puffer,64,0,(struct sockaddr *) &udp_control2,&udp_laenge)) >= 0)
		{
		// fprintf(stderr,"%d Bytes angekommen von: %s, Daten=\"%s\"\n",udp_received,inet_ntoa(udp_control2.sin_addr),udp_puffer + 40);
		if (udp_received != 58)
			{
			printf("lcd_demon: Falsche Daten von %s: Lnge nicht 58 Bytes sondern %d Bytes, Paket wird verworfen.\n",inet_ntoa(udp_control2.sin_addr),udp_received);
			
			goto ende_udp;
			}

		for (t = 0;t < 20 - 1;t++)
			{
			udp_received_username[t] = 0;
			c = udp_puffer[t];
			if (c == 0) goto weiter_user;
			udp_received_username[t] = c;
			}
weiter_user:

		for (t = 0;t < 20 - 1;t++)
			{
			udp_received_password[t] = 0;
			c = udp_puffer[20 + t];
			if (c == 0) goto weiter_pw;
			udp_received_password[t] = c;
			}
weiter_pw:

		if(strcmp(udp_received_username,udp_rx_username) == 0)
			{
			if(strcmp(udp_received_password,udp_rx_password) == 0)
				{
				t = udp_puffer[40] - '0';
				// printf("lcd_demon: UDP cmd=%d, string:\"%s\"\n",t,udp_puffer + 41);
				if((t == 0) || (t == 1))
					{
					// printf("Zeile %d, String:\"%s\"\n",t,udp_puffer + 41);
					lcm_selectline(fd,t);
					lcm_writetext(fd,udp_puffer + 41,16);
					goto oki;
					}
				printf("ERROR: falsche Zeilennummer\n");
oki:		;
				}
			else
				{
				printf("lcd_demon: Passwort falsch.\n");
				}
			}
		else
			{
			printf("lcd_demon: Username falsch.\n");
			}

ende_udp: ;
		}
	goto hauptschleife;																		// ... und wieder dasselbe Spiel von vorne

ende:
	close(fd);
endeende:
	return(0);
	}

