// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   lcd_write.c   -   Commandlinetool, um Strings im LCD des ALL5000 anzuzeigen                                   **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2009 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************


#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <fcntl.h>
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "sqldb.h"


// *****   udp_send() - Daten senden   *****

void udp_send(char zielip[],unsigned short int portnummer,unsigned char *daten,unsigned short int laenge)
	{
	int							udp_socket;
	struct					sockaddr_in steuerstruct;

	// printf("udp_send an %s:%d, %d Bytes.\n",zielip,portnummer,laenge);

	if ((udp_socket = socket(PF_INET,SOCK_DGRAM,IPPROTO_UDP)) >= 0) // UDP-Socket anlegen
		{
		// Construct the server sockaddr_in structure
		memset(&steuerstruct,0,sizeof(steuerstruct));				// Struct auf 0 setzen
		steuerstruct.sin_family = AF_INET;									// Internet/IP
		steuerstruct.sin_addr.s_addr = inet_addr(zielip);		// IP-Addresse
		steuerstruct.sin_port = htons(portnummer);					// UDP-Portnummer

		sendto(udp_socket,daten,laenge,0,(struct sockaddr *) &steuerstruct,sizeof(steuerstruct));
		close(udp_socket);
		// printf("OK.\n");
		}
	}



void print_usage()
	{
	printf("USAGE: lcd_write line_number \"text\"\n");
	exit(0);
	}



// *****   Testprogramm   *****

int main(int argc,char **argv)
	{
	unsigned short int			lcd_zeile;
	char										lcd_bla[17];
	unsigned char						k[58];
	unsigned short int			udp_rx_port = 54321;					// TCP Socket fr Client/Server Verbindung
	char										udp_rx_username[1024] = "";		// Username fr UDP-Verbindung
	char										udp_rx_password[1024] = "";		// Passwort fr UDP-Verbindung
	unsigned char						j;
	unsigned char						c;
	unsigned char						l;
	char										bla[17];
	unsigned char						t;
	signed short int				all5000 = 0;


 	if (argc != 3)
		{
		print_usage();
		}


	lcd_zeile = atoi(argv[1]);
 	strcpy(lcd_bla,argv[2]);

	// LCD ist nur beim ALL5000 eingebaut; also beenden wenn ein anderes Gert erkannt wurde
	all5000 = sqldb_read_long("/sys/hardware/numeric_model");
	if (all5000 != 5000) goto ende;			

	udp_rx_port = sqldb_read_long("/control/lcd/server/udp_rx_port");
	sqldb_read_string("/control/lcd/server/username",udp_rx_username);
	sqldb_read_string("/control/lcd/server/password",udp_rx_password);

	for(t = 0;t < 16;t++)
		{
		bla[t] = 32;
		}
	bla[16] = 0;

	for(t = 0;t < 16;t++)
		{
		if(lcd_bla[t] == 0) goto blafertig;
		bla[t] = lcd_bla[t];
		}
blafertig: ;

	// printf("lcd_write (%d:\"%s\"), Port:%d, User:\"%s\", Pass:\"%s\"\n",lcd_zeile,bla,udp_rx_port,udp_rx_username,udp_rx_password);


						
	memset(k,0,58);

	l = 0;
	for(j = 0;j < 20;j++)
		{
		c = udp_rx_username[j];
		if (c == 0) goto skip1;
		k[l++] = c;
		}
skip1:
		l = 20;
		for(j = 0;j < 20;j++)
			{
			c = udp_rx_password[j];
			if (c == 0) goto skip2;
			k[l++] = c;
			}
skip2:
		k[40] = lcd_zeile + '0';
		k[41] = bla[ 0];
		k[42] = bla[ 1];
		k[43] = bla[ 2];
		k[44] = bla[ 3];
		k[45] = bla[ 4];
		k[46] = bla[ 5];
		k[47] = bla[ 6];
		k[48] = bla[ 7];
		k[49] = bla[ 8];
		k[50] = bla[ 9];
		k[51] = bla[10];
		k[52] = bla[11];
		k[53] = bla[12];
		k[54] = bla[13];
		k[55] = bla[14];
		k[56] = bla[15];
		k[57] = 0;

		udp_send("127.0.0.1",udp_rx_port,k,58);

ende:
	return(0);
	}

