// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   powerline_demon.c   -   Demon zum Steuern von Narrowband-Powerline-Slaves ber Echelon-Module (ALL5000)       **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2009 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************

// **********   Eintrge in der Konfigurations-Datenbank, die vom Demon ausgewertet werden   **********
//
// /control/powerline/server/comport"										Serielle Schnittstelle, an der das PL-Mastermodul hngt (normalerweise /dev/ttyS4)
// /control/powerline/server/baudrate"									Baudrate dieser Schnittstelle (normalerweise 4800)
// /control/powerline/server/udpport"										UDP-Port fr Steuerkommandos (54009)
// /control/powerline/server/username										zu bergebender Username
// /control/powerline/server/password										zu bergebendes Pawort
// /control/powerline/max_retries												Anzahl Versuche, einen Slave zu erreichen
// /control/powerline/max_slaves"												Anzahl zu verwaltender Slaves
// /control/powerline/reload_counter"										Zhler, um festzustellen ob ein Neuladen der Config ntig ist
// /control/actors/actorXX/value"												gibt den zu schaltenden Wert; wird mit gepolltem Wert upgedated



#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <termios.h>
#include <time.h>
#include <sys/time.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
// #include "at91gpio.h"
// #include "all_i2c.h"
// #include "i2c_chips.h"
#include "sqldb.h"
#include "sensor_demon.h"
#include "shm_use.h"
#include "powerline_demon.h"
#include "logging.h"



// *****   Globale Variablen   *****

unsigned short int		cache_max_slaves;
signed char						cache_slave_enabled[POWERLINE_MAX_SLAVES];
signed char						cache_slave_use_buttons[POWERLINE_MAX_SLAVES];
signed char						cache_slave_typ[POWERLINE_MAX_SLAVES];
char									cache_slave_lon_id[POWERLINE_MAX_SLAVES][13];
signed long int				cache_slave_mapfrom_actor[POWERLINE_MAX_SLAVES][8];
signed long int				cache_slave_value[POWERLINE_MAX_SLAVES][8];
char									cache_slave_name[POWERLINE_MAX_SLAVES][80];
char									cache_slave_comment[POWERLINE_MAX_SLAVES][80];
signed long int				powerline_reload_counter = -1;		// Merker fr Reload-Counter
signed long int				powerline_baudrate;								// immer 4800
char									powerline_comport[150];						// Schnittstelle des Master-Moduls
signed long int				powerline_modus;									// Betriebsmodus (0: Demon aktiv, 1: Bridge, Demon deaktiviert)
signed long int				powerline_listen_to_broadcasts;		// Broadcasts bercksichtigen ?
signed long int				read_delay;												// Verzgerung zwischen selbstttigen Reads
signed long int				powerline_max_retries = 5;
int										shID;
struct s_shm_actordata*	shm_actor_r_data;								// Aktordaten R im Shared-RAM
struct s_shm_actordata*	shm_actor_w_data;								// Aktordaten W im Shared-RAM
signed long int				shadow_w_data[MAX_LOGICAL_ACTORS];// Kopie der Aktordaten-Integers
unsigned long int			last_read_timestamp = 0;					// Zeit des letzten Reads
signed long int				zyklischer_sklave = 0;						// Nummer fr Zyklische Abfrage
int										global_fd = 0;										// file descriptor fr RS232-Port
char									servicepin_slave_lon_id[13] = "000000000000"; // LON-ID des Slaves, der zuletzt seine ID per Servicepin-Broadcast versandt hat


// *****   rs232_open("/dev/...")   -   seriellen Port ffnen   *****
// the baudrate parameter is ignored, it is always set to 4800

int rs232_open(char device[],unsigned long int baudrate)
	{
	int fd;
	struct termios options;


	memset((void *)&options,0,sizeof(options));

	fd = open(device,O_RDWR | O_NOCTTY |O_SYNC);
	if( fd < 0) goto ende;


	tcdrain(fd);

	// Comport-Parameter einstellen
  options.c_cflag = B4800 | CS8 | CLOCAL | CREAD;
  options.c_cflag &= ~(PARENB | CSTOPB);
  options.c_iflag |= (IGNPAR | ECHOCTL | ECHOKE);
  options.c_lflag &= ~(ICANON | OPOST| ECHO | ECHOE | ISIG | IEXTEN | ECHOK | ONLCR |IXON);
  options.c_cc[VMIN]  = 0;
  options.c_cc[VTIME]  = 0;
  tcsetattr(fd,TCSAFLUSH,&options);

ende:
	return(fd);
	}



// *****   get_ms_now()   -   aktuelle Zeit in ms ermitteln   *****

long get_ms_now(void)
	{
	long rw;
	const long msdivider = CLOCKS_PER_SEC / 1000;					// Teiler um von Clockticks aus ms zu kommen
  struct timeval tv;
	time_t curtime;
	struct tm *myTm;


	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln 
  curtime = tv.tv_sec;																	// Lores-Zeit in s reusziehen
	myTm = localtime(&curtime);														// ...und berfhren in tm-Struktur
	rw = (myTm->tm_hour * 3600 + myTm->tm_min * 60 + myTm->tm_sec) * 1000 + tv.tv_usec / msdivider;
	return(rw);
	}



// *****   powerline_schalte2()   -   Schalten von Slaves mit Wiederholung im Fehlerfall   *****

signed long int powerline_schalte2(unsigned short int dosennummer,unsigned short int kanal,unsigned short int schaltwunsch,unsigned short int tries,int fd)
	{
	signed long int rw = -1;
	signed short int t;
	char ep[333];


	sprintf(ep,"------------------- Schalte powerline slave %d[%d]:%d ---------------------------------",dosennummer,kanal,schaltwunsch);
	log_append(4,ep);																		// Meldung ins Logfile schreiben

	for(t = 0;t < tries;t++)
		{
		rw = powerline_schalte(dosennummer,kanal,schaltwunsch,fd);
		if(rw == schaltwunsch) goto ende;
		sprintf(ep,"powerline_schalte2: Try %d for powerline slave %d[%d]:%d (RW:%ld)",t,dosennummer,kanal,schaltwunsch,rw);
		log_append(4,ep);																		// Meldung ins Logfile schreiben
		usleep(150000);																			// 150 ms warten
		}

ende:
	sprintf(ep,"powerline_schalte2: powerline slave %d[%d]:%d (RW:%ld nach %d Retries)",dosennummer,kanal,schaltwunsch,rw,t);
	log_append(3,ep);																		// Meldung ins Logfile schreiben
	return(rw);
	}



// *****   powerline_query2()   -   Abfragen von Slaves mit Wiederholung im Fehlerfall   *****

signed long int powerline_query2(unsigned short int dosennummer,unsigned short int kanal,unsigned short int tries,int fd)
	{
	signed long int rw = -1;
	signed short int t;
	char ep[333];


	sprintf(ep,"------------------- Query powerline slave %d[%d] ---------------------------------",dosennummer,kanal);
	log_append(4,ep);																		// Meldung ins Logfile schreiben

	for(t = 0;t < tries;t++)
		{
		rw = powerline_query(dosennummer,kanal,fd);
		if(rw >= 0) goto ende;
		sprintf(ep,"powerline_query2: Try %d for powerline slave %d[%d] (RW:%ld)",t,dosennummer,kanal,rw);
		log_append(4,ep);																		// Meldung ins Logfile schreiben
		usleep(150000);																			// 150 ms warten
		}

ende:
	sprintf(ep,"powerline_query2: powerline slave %d[%d] (RW:%ld)",dosennummer,kanal,rw);
	log_append(4,ep);																		// Meldung ins Logfile schreiben
	return(rw);
	}



// *****   powerline_query_buttonflag2()   -   Abfragen von Slaves mit Wiederholung im Fehlerfall   *****

signed long int powerline_query_buttonflag2(unsigned short int dosennummer,unsigned short int kanal,unsigned short int tries,int fd)
	{
	signed long int rw = -1;
	signed short int t;
	char ep[333];

	if(cache_slave_use_buttons[dosennummer] < 1)
		{
		rw = 0;																							// Wenn Buttons ausgeschaltet sind, gleich 0 zurckliefern
		goto endeende;																			// ohne Logging fertig
		}

	for(t = 0;t < tries;t++)
		{
		rw = powerline_query_buttonflag(dosennummer,kanal,fd);
		if(rw >= 0) goto ende;
		sprintf(ep,"powerline_query_buttonflag2: Try %d for powerline slave %d[%d] (RW:%ld)",t,dosennummer,kanal,rw);
		log_append(4,ep);																		// Meldung ins Logfile schreiben
		usleep(150000);																			// 150 ms warten
		}

ende:
	sprintf(ep,"powerline_query_buttonflag2(%d[%d]): %ld",dosennummer,kanal,rw);
	log_append(4,ep);																			// Meldung ins Logfile schreiben
endeende: ;
	return(rw);
	}



// *****   powerline_query_devicetyp2()   -   Abfragen von Slaves mit Wiederholung im Fehlerfall   *****

signed long int powerline_query_devicetyp2(char dosenid[],unsigned short int tries,int fd)
	{
	signed long int rw = -1;
	signed short int t;
	char ep[333];

	for(t = 0;t < tries;t++)
		{
		rw = powerline_query_devicetyp(dosenid,fd);
		if(rw >= 0) goto ende;
		sprintf(ep,"powerline_query_devicetyp2: Retry for powerline slave with ID %s (RW:%ld)",dosenid,rw);
		log_append(4,ep);																		// Meldung ins Logfile schreiben
		usleep(150000);																			// 150 ms warten
		}

ende:
	return(rw);
	}



// *****   powerline_query_servicepin2()   -   Abfragen von Slave-Servicepinmessages mit Wiederholung im Fehlerfall   *****

signed long int powerline_query_servicepin2(unsigned short int tries,int fd)
	{
	signed long int rw = -1;
	signed short int t;
	char ep[333];

	for(t = 0;t < tries;t++)
		{
		rw = powerline_query_servicepin(fd);
		if(rw >= 0) goto ende;
		sprintf(ep,"powerline_query_servicepin2: Retry (RW:%ld)",rw);
		log_append(4,ep);																		// Meldung ins Logfile schreiben
		usleep(150000);																			// 150 ms warten
		}

ende:
	return(rw);
	}



// *****   powerline_schalte() - Schalten von Slaves   *****

signed long int powerline_schalte(unsigned short int dosennummer,unsigned short int kanal,unsigned short int schaltwunsch,int fd)
	{
	signed long int rw = -2;
	char ep[512];
	char sendepuffer[33 + 1];															// Puffer fr Sendestring
	char empfangspuffer[40] = "(ENNOANSWER)";							// Puffer fr Antwort
	unsigned char empfangspuffer_laenge;									// Anzahl Zeichen im Empfangspuffer
	char sw;																							// ASCII-Zeichen fr Schaltwunsch
	int t;
	char a;
	char l;
	char tp[2];																						// temporrer Mini-Puffer
	long startzeit;


	empfangspuffer_laenge = 0;														// Eingangspuffer lschen

flushloop:
	l = read(fd,tp,1);																		// Nachschauen, ob ein Zeichen angekommen ist
	if (l > 0)
		{
		a = tp[0];
		empfangspuffer[empfangspuffer_laenge++] = a;				// Ja, dann in den Empfangspuffer schreiben
		goto flushloop;
		}

	if (empfangspuffer_laenge > 0)
		{
		sprintf(ep,"%d Zeichen im Empfangspuffer ignoriert !",empfangspuffer_laenge);
		log_append(4,ep);																		// Meldung ins Logfile schreiben
		for(t = 0;t < 39;t++)
			{
			empfangspuffer[t] = '*';
			}
		empfangspuffer_laenge = 0;														// Eingangspuffer lschen
		}

	sw = '0';
	if (schaltwunsch > 0) sw = '1';

	// strcpy(sendepuffer,"[TS00FF        P00T0501A02FBB00]\r"); // Kommendo fr Toggeln
	sendepuffer[ 0] = '[';																// einleitende Klammer
	sendepuffer[ 1] = 'T';																// TS: Toggle Switch
	sendepuffer[ 2] = 'S';
	sendepuffer[ 3] = '0';																// 0x: Kanal 0x
	sendepuffer[ 4] = '0' + kanal;												// Kanalnummer
	sendepuffer[ 5] = '0';
	sendepuffer[ 6] = sw;																	// Schaltwunsch 2-stellig 00 oder 01
	sendepuffer[ 7] = 32;
	sendepuffer[ 8] = 32;
	sendepuffer[ 9] = 32;
	sendepuffer[10] = 32;
	sendepuffer[11] = 32;
	sendepuffer[12] = 32;
	sendepuffer[13] = 32;
	sendepuffer[14] = 32;
	sendepuffer[15] = 'P';
	sendepuffer[16] = '0';
	sendepuffer[17] = '0';
	sendepuffer[18] = 'T';
	sendepuffer[19] = cache_slave_lon_id[dosennummer][ 0];// LON-ID der gewnschten Steckdose
	sendepuffer[20] = cache_slave_lon_id[dosennummer][ 1];
	sendepuffer[21] = cache_slave_lon_id[dosennummer][ 2];
	sendepuffer[22] = cache_slave_lon_id[dosennummer][ 3];
	sendepuffer[23] = cache_slave_lon_id[dosennummer][ 4];
	sendepuffer[24] = cache_slave_lon_id[dosennummer][ 5];
	sendepuffer[25] = cache_slave_lon_id[dosennummer][ 6];
	sendepuffer[26] = cache_slave_lon_id[dosennummer][ 7];
	sendepuffer[27] = cache_slave_lon_id[dosennummer][ 8];
	sendepuffer[28] = cache_slave_lon_id[dosennummer][ 9];
	sendepuffer[29] = cache_slave_lon_id[dosennummer][10];
	sendepuffer[30] = cache_slave_lon_id[dosennummer][11];
	sendepuffer[31] = ']';
	sendepuffer[32] = 13;
	sendepuffer[33] = 0;

	// printf("Sendepuffer: \"%s\"\n",sendepuffer);
	for(t = 0;t < 33;t++)																	// Kommando seriell senden
		{
		tp[0] = sendepuffer[t];															// Zeichen aus unserem Kommandostring holen
		write(fd,tp,1);																			// Zeichen ber den seriellen Port senden
		usleep(6500);																				// 6.5 ms nach jedem Zeichen warten
		}


	startzeit = get_ms_now();															// Startzeit in ms ermitteln
	// printf("xx: Startzeit: %ld\n",startzeit);
looping:
	l = read(fd,tp,1);																		// Nachschauen, ob ein Zeichen angekommen ist
	if (l > 0)
		{
		a = tp[0];
		empfangspuffer[empfangspuffer_laenge++] = a;				// Ja, dann in den Empfangspuffer schreiben
		if (a == 13)																				// Ist es ein <CR> ?
			{
			empfangspuffer[empfangspuffer_laenge] = 0;				// Ja, dann ist die Antwort komplett
			goto looping_ende;																// und wir knnen sie zurckliefern
			}
		}

	// printf("xx: zeit: %ld, %d zeichen\n",get_ms_now(),empfangspuffer_laenge);
	if (get_ms_now() - startzeit < 650) goto looping;			// Timeout abprfen
	// printf("Timeout (ENNO_ANSWER   P00)\n");										// Timeout erreicht - "EN" fr "No Answer" zurckgeben
	rw = -1;																							// Rckwert -1 fr Error
	goto fertig;																					// und wir sind fertig - Retries sind Sache des Application Layers


looping_ende:
	// printf ("%s\n",empfangspuffer);												// Rckgabe der erhaltenen Meldung
	rw = 0;																								// Rckwert 0 fr "alles OK"
	if (empfangspuffer[1] == 'E')
		{
		rw = -1;																						// Rckwert -1 fr "Fehler"
		goto fertig;
		}
	if (empfangspuffer[1] != 'R')
		{
		rw = -2;																						// Rckwert -2 fr "1. Zeichen nicht R"
		goto fertig;
		}
	if (empfangspuffer[2] != 'S')
		{
		rw = -3;																						// Rckwert -3 fr "2. Zeichen nicht S"
		goto fertig;
		}
	if (empfangspuffer[6] > '0') rw = 1;									// 1 bei Status "Ein" zurckgeben

fertig:
	sendepuffer[32] = 0;
	if (empfangspuffer[empfangspuffer_laenge - 1] < 32) empfangspuffer[empfangspuffer_laenge - 1] = 0;
	sprintf(ep,"powerline_schalte: TX:\"%s\", RX:\"%s\"",sendepuffer,empfangspuffer);
	log_append(4,ep);																			// Meldung ins Logfile schreiben
	// sprintf(ep,"powerline_schalte(%d[%d],%d): %s",dosennummer,kanal,schaltwunsch,cache_slave_lon_id[dosennummer]);
	// log_append(4,ep);																			// Meldung ins Logfile schreiben
	return(rw);
	}



// *****   powerline_query() - Ermitteln des Schaltzustands von Slaves   *****

signed long int powerline_query(unsigned short int dosennummer,unsigned short int kanal,int fd)
	{
	signed long int rw = -2;
	char ep[512];
	char sendepuffer[33 + 1];															// Puffer fr Sendestring
	char empfangspuffer[40] = "(ENNOANSWER)";							// Puffer fr Antwort
	unsigned char empfangspuffer_laenge;									// Anzahl Zeichen im Empfangspuffer
	int t;
	char a;
	char l;
	char tp[2];																						// temporrer Mini-Puffer
	long startzeit;


	for(t = 0;t < 39;t++)
		{
		empfangspuffer[t] = '*';
		}
	empfangspuffer[39] = 0;
	empfangspuffer_laenge = 0;														// Eingangspuffer lschen

flushloop:
	l = read(fd,tp,1);																		// Nachschauen, ob ein Zeichen angekommen ist
	if (l > 0)
		{
		a = tp[0];
		empfangspuffer[empfangspuffer_laenge++] = a;				// Ja, dann in den Empfangspuffer schreiben
		goto flushloop;
		}

	if (empfangspuffer_laenge > 0)
		{
		sprintf(ep,"%d Zeichen im Empfangspuffer ignoriert !",empfangspuffer_laenge);
		log_append(4,ep);																		// Meldung ins Logfile schreiben
		for(t = 0;t < 39;t++)
			{
			empfangspuffer[t] = '*';
			}
		empfangspuffer_laenge = 0;														// Eingangspuffer lschen
		}

	// strcpy(sendepuffer,"[TO00          P00T0501A02FBB00]\r"); // Kommendo fr Statusabfrage
	sendepuffer[ 0] = '[';																// einleitende Klammer
	sendepuffer[ 1] = 'T';																// TO: Query Output
	sendepuffer[ 2] = 'O';
	sendepuffer[ 3] = '0';																// 0x: Kanal 0x
	sendepuffer[ 4] = '0' + kanal;												// Kanalnummer
	sendepuffer[ 5] = 32;
	sendepuffer[ 6] = 32;
	sendepuffer[ 7] = 32;
	sendepuffer[ 8] = 32;
	sendepuffer[ 9] = 32;
	sendepuffer[10] = 32;
	sendepuffer[11] = 32;
	sendepuffer[12] = 32;
	sendepuffer[13] = 32;
	sendepuffer[14] = 32;
	sendepuffer[15] = 'P';
	sendepuffer[16] = '0';
	sendepuffer[17] = '0';
	sendepuffer[18] = 'T';
	sendepuffer[19] = cache_slave_lon_id[dosennummer][ 0];// LON-ID der gewnschten Steckdose
	sendepuffer[20] = cache_slave_lon_id[dosennummer][ 1];
	sendepuffer[21] = cache_slave_lon_id[dosennummer][ 2];
	sendepuffer[22] = cache_slave_lon_id[dosennummer][ 3];
	sendepuffer[23] = cache_slave_lon_id[dosennummer][ 4];
	sendepuffer[24] = cache_slave_lon_id[dosennummer][ 5];
	sendepuffer[25] = cache_slave_lon_id[dosennummer][ 6];
	sendepuffer[26] = cache_slave_lon_id[dosennummer][ 7];
	sendepuffer[27] = cache_slave_lon_id[dosennummer][ 8];
	sendepuffer[28] = cache_slave_lon_id[dosennummer][ 9];
	sendepuffer[29] = cache_slave_lon_id[dosennummer][10];
	sendepuffer[30] = cache_slave_lon_id[dosennummer][11];
	sendepuffer[31] = ']';
	sendepuffer[32] = 13;
	sendepuffer[33] = 0;

	// printf("Sendepuffer: \"%s\"\n",sendepuffer);
	for(t = 0;t < 33;t++)																	// Kommando seriell senden
		{
		tp[0] = sendepuffer[t];															// Zeichen aus unserem Kommandostring holen
		write(fd,tp,1);																			// Zeichen ber den seriellen Port senden
		usleep(6500);																				// 6.5 ms nach jedem Zeichen warten
		}


	startzeit = get_ms_now();															// Startzeit in ms ermitteln
looping:
	l = read(fd,tp,1);																		// Nachschauen, ob ein Zeichen angekommen ist
	if (l > 0)
		{
		a = tp[0];
		empfangspuffer[empfangspuffer_laenge++] = a;				// Ja, dann in den Empfangspuffer schreiben
		if (a == 13)																				// Ist es ein <CR> ?
			{
			empfangspuffer[empfangspuffer_laenge] = 0;				// Ja, dann ist die Antwort komplett
			goto looping_ende;																// und wir knnen sie zurckliefern
			}
		}

	if (get_ms_now() - startzeit < 650) goto looping;			// Timeout abprfen
	// printf("(ENNO_ANSWER   P00)\n");										// Timeout erreicht - "EN" fr "No Answer" zurckgeben
	rw = -1;																							// Rckwert -1 fr Error
	goto fertig;																					// und wir sind fertig - Retries sind Sache des Application Layers


looping_ende:
	// printf ("Empfangspuffer: \"%s\"\n",empfangspuffer);		// Rckgabe der erhaltenen Meldung
	rw = 0;																								// Rckwert 0 fr "alles OK"
	if (empfangspuffer[1] == 'E')
		{
		rw = -1;																						// Rckwert -1 fr "Fehler"
		goto fertig;
		}
	if (empfangspuffer[1] != 'R')
		{
		rw = -2;																						// Rckwert -2 fr "Fehler"
		goto fertig;
		}
	if (empfangspuffer[2] != 'S')
		{
		rw = -3;																						// Rckwert -3 fr "Fehler"
		goto fertig;
		}
	if (empfangspuffer[6] > '0') rw = 1;									// 1 bei Status "Ein" zurckgeben

fertig:
	sendepuffer[32] = 0;
	if (empfangspuffer[empfangspuffer_laenge - 1] < 32) empfangspuffer[empfangspuffer_laenge - 1] = 0;
	sprintf(ep,"powerline_query: TX:\"%s\", RX:\"%s\"",sendepuffer,empfangspuffer);
	log_append(4,ep);																			// Meldung ins Logfile schreiben
	// sprintf(ep,"powerline_query(%d[%d]): %ld",dosennummer,kanal,rw);
	// log_append(4,ep);																			// Meldung ins Logfile schreiben
	return(rw);
	}



// *****   powerline_query_buttonflag() - Ermitteln des manuellen Knopf-Flags von Slaves   *****

signed long int powerline_query_buttonflag(unsigned short int dosennummer,unsigned short int kanal,int fd)
	{
	signed long int rw = 0;																// Default hier ist 0
	// char ep[512];
	char sendepuffer[33 + 1];															// Puffer fr Sendestring
	char empfangspuffer[40] = "(ENNOANSWER)";							// Puffer fr Antwort
	unsigned char empfangspuffer_laenge;									// Anzahl Zeichen im Empfangspuffer
	int t;
	char a;
	char l;
	char tp[2];																						// temporrer Mini-Puffer
	long startzeit;


	empfangspuffer_laenge = 0;														// Eingangspuffer lschen

	// strcpy(sendepuffer,"[TO00          P00T0501A02FBB00]\r"); // Kommendo fr Statusabfrage
	sendepuffer[ 0] = '[';																// einleitende Klammer
	sendepuffer[ 1] = 'T';																// TA03: Query Buttonflag
	sendepuffer[ 2] = 'A';
	sendepuffer[ 3] = '0';
	sendepuffer[ 4] = '3';
	sendepuffer[ 5] = '0';																// 0x: Kanal 0x
	sendepuffer[ 6] = '0' + kanal;												// Kanalnummer
	sendepuffer[ 7] = 32;
	sendepuffer[ 8] = 32;
	sendepuffer[ 9] = 32;
	sendepuffer[10] = 32;
	sendepuffer[11] = 32;
	sendepuffer[12] = 32;
	sendepuffer[13] = 32;
	sendepuffer[14] = 32;
	sendepuffer[15] = 'P';
	sendepuffer[16] = '0';
	sendepuffer[17] = '0';
	sendepuffer[18] = 'T';
	sendepuffer[19] = cache_slave_lon_id[dosennummer][ 0];// LON-ID der gewnschten Steckdose
	sendepuffer[20] = cache_slave_lon_id[dosennummer][ 1];
	sendepuffer[21] = cache_slave_lon_id[dosennummer][ 2];
	sendepuffer[22] = cache_slave_lon_id[dosennummer][ 3];
	sendepuffer[23] = cache_slave_lon_id[dosennummer][ 4];
	sendepuffer[24] = cache_slave_lon_id[dosennummer][ 5];
	sendepuffer[25] = cache_slave_lon_id[dosennummer][ 6];
	sendepuffer[26] = cache_slave_lon_id[dosennummer][ 7];
	sendepuffer[27] = cache_slave_lon_id[dosennummer][ 8];
	sendepuffer[28] = cache_slave_lon_id[dosennummer][ 9];
	sendepuffer[29] = cache_slave_lon_id[dosennummer][10];
	sendepuffer[30] = cache_slave_lon_id[dosennummer][11];
	sendepuffer[31] = ']';
	sendepuffer[32] = 13;
	sendepuffer[33] = 0;

	// sprintf(ep,"QBF_Sendepuffer: \"%s\"",sendepuffer);
	// log_append(4,ep);																			// Meldung ins Logfile schreiben

	for(t = 0;t < 33;t++)																	// Kommando seriell senden
		{
		tp[0] = sendepuffer[t];															// Zeichen aus unserem Kommandostring holen
		write(fd,tp,1);																			// Zeichen ber den seriellen Port senden
		usleep(6500);																				// 6.5 ms nach jedem Zeichen warten
		}


	startzeit = get_ms_now();															// Startzeit in ms ermitteln
looping:
	l = read(fd,tp,1);																		// Nachschauen, ob ein Zeichen angekommen ist
	if (l > 0)
		{
		a = tp[0];
		empfangspuffer[empfangspuffer_laenge++] = a;				// Ja, dann in den Empfangspuffer schreiben
		if (a == 13)																				// Ist es ein <CR> ?
			{
			empfangspuffer[empfangspuffer_laenge] = 0;				// Ja, dann ist die Antwort komplett
			goto looping_ende;																// und wir knnen sie zurckliefern
			}
		}

	if (get_ms_now() - startzeit < 400) goto looping;			// Timeout abprfen
	// printf("(ENNO_ANSWER   P00)\n");										// Timeout erreicht - "EN" fr "No Answer" zurckgeben
	rw = -1;																							// Rckwert -1 fr Error
	goto fertig;																					// und wir sind fertig - Retries sind Sache des Application Layers


looping_ende:
	// sprintf (ep,"QBF_Empangspuffer: \"%s\"",empfangspuffer);												// Rckgabe der erhaltenen Meldung
	// log_append(4,ep);																			// Meldung ins Logfile schreiben
	rw = 0;																								// Rckwert 0 fr "alles OK"
	if (empfangspuffer[1] == 'E')
		{
		rw = -1;																						// Rckwert -1 fr "Fehler"
		goto fertig;
		}
	if (empfangspuffer[1] != 'R')
		{
		rw = -2;																						// Rckwert -2 fr "1. Zeichen nicht R"
		goto fertig;
		}
	if (empfangspuffer[2] != 'A')
		{
		rw = -3;																						// Rckwert -3 fr "2. Zeichen nicht A"
		goto fertig;
		}
	if (empfangspuffer[10] > '0') rw = 1;									// 1 bei Status "Ein" zurckgeben

fertig:
	// sprintf(ep,"powerline_query_buttonflag(%d[%d]): %ld",dosennummer,kanal,rw);
	// log_append(4,ep);																			// Meldung ins Logfile schreiben
	return(rw);
	}



// *****   powerline_query_servicepin() - Ermitteln, ob irgendein Slave eine Servicepin-Message geschickt hat   *****

signed long int powerline_query_servicepin(int fd)
	{
	signed long int rw = -2;
	char ep[512];
	char sendepuffer[33 + 1];															// Puffer fr Sendestring
	char empfangspuffer[40] = "(ENNOANSWER)";							// Puffer fr Antwort
	unsigned char empfangspuffer_laenge;									// Anzahl Zeichen im Empfangspuffer
	int t;
	char a;
	char l;
	char tp[2];																						// temporrer Mini-Puffer
	long startzeit;
	char servicepin_slave_lonid_back[13];									// Rckwert


	empfangspuffer_laenge = 0;														// Eingangspuffer lschen

	// strcpy(sendepuffer,"[TO00          P00T0501A02FBB00]\r"); // Kommendo fr Statusabfrage
	sendepuffer[ 0] = '[';																// einleitende Klammer
	sendepuffer[ 1] = 'Q';																// QP: Query Servicepin
	sendepuffer[ 2] = 'P';
	sendepuffer[ 3] = 32;
	sendepuffer[ 4] = 32;
	sendepuffer[ 5] = 32;
	sendepuffer[ 6] = 32;
	sendepuffer[ 7] = 32;
	sendepuffer[ 8] = 32;
	sendepuffer[ 9] = 32;
	sendepuffer[10] = 32;
	sendepuffer[11] = 32;
	sendepuffer[12] = 32;
	sendepuffer[13] = 32;
	sendepuffer[14] = 32;
	sendepuffer[15] = 'P';
	sendepuffer[16] = '0';
	sendepuffer[17] = '0';
	sendepuffer[18] = 'T';
	sendepuffer[19] = '0'																	;// LON-ID der gewnschten Steckdose
	sendepuffer[20] = '1';
	sendepuffer[21] = '1';
	sendepuffer[22] = '1';
	sendepuffer[23] = '1';
	sendepuffer[24] = '1';
	sendepuffer[25] = '1';
	sendepuffer[26] = '1';
	sendepuffer[27] = '1';
	sendepuffer[28] = '1';
	sendepuffer[29] = '1';
	sendepuffer[30] = '1';
	sendepuffer[31] = ']';
	sendepuffer[32] = 13;
	sendepuffer[33] = 0;

	// printf("Sendepuffer: \"%s\"\n",sendepuffer);
	for(t = 0;t < 33;t++)																	// Kommando seriell senden
		{
		tp[0] = sendepuffer[t];															// Zeichen aus unserem Kommandostring holen
		write(fd,tp,1);																			// Zeichen ber den seriellen Port senden
		usleep(6500);																				// 6.5 ms nach jedem Zeichen warten
		}


	startzeit = get_ms_now();															// Startzeit in ms ermitteln
looping:
	l = read(fd,tp,1);																		// Nachschauen, ob ein Zeichen angekommen ist
	if (l > 0)
		{
		a = tp[0];
		empfangspuffer[empfangspuffer_laenge++] = a;				// Ja, dann in den Empfangspuffer schreiben
		if (a == 13)																				// Ist es ein <CR> ?
			{
			empfangspuffer[empfangspuffer_laenge] = 0;				// Ja, dann ist die Antwort komplett
			goto looping_ende;																// und wir knnen sie zurckliefern
			}
		}

	if (get_ms_now() - startzeit < 400) goto looping;			// Timeout abprfen
	// printf("(ENNO_ANSWER   P00)\n");										// Timeout erreicht - "EN" fr "No Answer" zurckgeben
	rw = -1;																							// Rckwert -1 fr Error
	goto fertig;																					// und wir sind fertig - Retries sind Sache des Application Layers


looping_ende:
	// printf ("QP Empfangspuffer: \"%s\"\n",empfangspuffer);		// Rckgabe der erhaltenen Meldung
	rw = 0;																								// Rckwert 0 fr "alles OK"
	if ((empfangspuffer[1] == 'P') && (empfangspuffer[2] == 'I'))
		{
		servicepin_slave_lonid_back[ 0] = empfangspuffer[ 3];
		servicepin_slave_lonid_back[ 1] = empfangspuffer[ 4];
		servicepin_slave_lonid_back[ 2] = empfangspuffer[ 5];
		servicepin_slave_lonid_back[ 3] = empfangspuffer[ 6];
		servicepin_slave_lonid_back[ 4] = empfangspuffer[ 7];
		servicepin_slave_lonid_back[ 5] = empfangspuffer[ 8];
		servicepin_slave_lonid_back[ 6] = empfangspuffer[ 9];
		servicepin_slave_lonid_back[ 7] = empfangspuffer[10];
		servicepin_slave_lonid_back[ 8] = empfangspuffer[11];
		servicepin_slave_lonid_back[ 9] = empfangspuffer[12];
		servicepin_slave_lonid_back[10] = empfangspuffer[13];
		servicepin_slave_lonid_back[11] = empfangspuffer[14];
		servicepin_slave_lonid_back[12] = 0;
		if(strcmp(servicepin_slave_lonid_back,servicepin_slave_lon_id) != 0)
			{
			// printf("New %s, old %s\n",servicepin_slave_lonid_back,servicepin_slave_lon_id);
			rw = 1;
			strcpy(servicepin_slave_lon_id,servicepin_slave_lonid_back);
			}
		else
			{
			// printf("No changes\n");
			rw = 0;																						// keine nderung gegenber dem letzten Mal
			}
		}
	if (empfangspuffer[1] == 'E') rw = -1;								// Rckwert -1 fr "Fehler"

fertig:
	sprintf(ep,"powerline_query_servicepin: %ld (%s)",rw,servicepin_slave_lonid_back);
	log_append(4,ep);																			// Meldung ins Logfile schreiben
	return(rw);
	}



// *****   powerline_query_devicetyp() - Ermitteln des Devicetyps eines Slaves   *****

signed long int powerline_query_devicetyp(char dosenid[],int fd)
	{
	signed long int rw = 0;																// Default hier ist 0
	char ep[512];
	char sendepuffer[33 + 1];															// Puffer fr Sendestring
	char empfangspuffer[40] = "(ENNOANSWER)";							// Puffer fr Antwort
	unsigned char empfangspuffer_laenge;									// Anzahl Zeichen im Empfangspuffer
	int t;
	char a;
	char l;
	char tp[2];																						// temporrer Mini-Puffer
	long startzeit;


	empfangspuffer_laenge = 0;														// Eingangspuffer lschen

	// strcpy(sendepuffer,"[TO00          P00T0501A02FBB00]\r"); // Kommendo fr Statusabfrage
	sendepuffer[ 0] = '[';																// einleitende Klammer
	sendepuffer[ 1] = 'T';																// Td: Query Devicetyp
	sendepuffer[ 2] = 'd';
	sendepuffer[ 3] = 32;
	sendepuffer[ 4] = 32;
	sendepuffer[ 5] = 32;
	sendepuffer[ 6] = 32;
	sendepuffer[ 7] = 32;
	sendepuffer[ 8] = 32;
	sendepuffer[ 9] = 32;
	sendepuffer[10] = 32;
	sendepuffer[11] = 32;
	sendepuffer[12] = 32;
	sendepuffer[13] = 32;
	sendepuffer[14] = 32;
	sendepuffer[15] = 'P';
	sendepuffer[16] = '0';
	sendepuffer[17] = '0';
	sendepuffer[18] = 'T';
	sendepuffer[19] = dosenid[ 0];// LON-ID der gewnschten Steckdose
	sendepuffer[20] = dosenid[ 1];
	sendepuffer[21] = dosenid[ 2];
	sendepuffer[22] = dosenid[ 3];
	sendepuffer[23] = dosenid[ 4];
	sendepuffer[24] = dosenid[ 5];
	sendepuffer[25] = dosenid[ 6];
	sendepuffer[26] = dosenid[ 7];
	sendepuffer[27] = dosenid[ 8];
	sendepuffer[28] = dosenid[ 9];
	sendepuffer[29] = dosenid[10];
	sendepuffer[30] = dosenid[11];
	sendepuffer[31] = ']';
	sendepuffer[32] = 13;
	sendepuffer[33] = 0;

	// printf("Sendepuffer: \"%s\"\n",sendepuffer);
	for(t = 0;t < 33;t++)																	// Kommando seriell senden
		{
		tp[0] = sendepuffer[t];															// Zeichen aus unserem Kommandostring holen
		write(fd,tp,1);																			// Zeichen ber den seriellen Port senden
		usleep(6500);																				// 6.5 ms nach jedem Zeichen warten
		}


	startzeit = get_ms_now();															// Startzeit in ms ermitteln
looping:
	l = read(fd,tp,1);																		// Nachschauen, ob ein Zeichen angekommen ist
	if (l > 0)
		{
		a = tp[0];
		empfangspuffer[empfangspuffer_laenge++] = a;				// Ja, dann in den Empfangspuffer schreiben
		if (a == 13)																				// Ist es ein <CR> ?
			{
			empfangspuffer[empfangspuffer_laenge] = 0;				// Ja, dann ist die Antwort komplett
			goto looping_ende;																// und wir knnen sie zurckliefern
			}
		}

	if (get_ms_now() - startzeit < 400) goto looping;			// Timeout abprfen
	// printf("(ENNO_ANSWER   P00)\n");										// Timeout erreicht - "EN" fr "No Answer" zurckgeben
	rw = -1;																							// Rckwert -1 fr Error
	goto fertig;																					// und wir sind fertig - Retries sind Sache des Application Layers


looping_ende:
	// printf ("%s\n",empfangspuffer);												// Rckgabe der erhaltenen Meldung
	rw = 0;																								// Rckwert 0 fr "alles OK"
	if (empfangspuffer[1] == 'E')
		{
		rw = -1;																						// Rckwert -1 fr "Fehler"
		goto fertig;
		}
	if (empfangspuffer[1] != 'R')
		{
		rw = -2;																						// Rckwert -2 fr "1. Zeichen nicht R"
		goto fertig;
		}
	if (empfangspuffer[2] != 'd')
		{
		rw = -3;																						// Rckwert -3 fr "2. Zeichen nicht d"
		goto fertig;
		}
	if (empfangspuffer[4] > '0') rw = empfangspuffer[4] - '0'; // 0...9 bei Statusmeldung zurckgeben

fertig:
	sprintf(ep,"powerline_query_devicetyp(%s): %ld",dosenid,rw);
	log_append(4,ep);																			// Meldung ins Logfile schreiben
	return(rw);
	}



// *****   cache_setzedefault()   -   Mappingtabelle lschen   *****

void cache_setzedefault(void)
	{
	unsigned short int t;
	unsigned short int i;

	for(t = 0;t < POWERLINE_MAX_SLAVES;t++)
		{
		cache_slave_enabled[t] = 0;
		cache_slave_use_buttons[t] = 0;
		cache_slave_typ[t] = 0;
		strcpy(cache_slave_lon_id[t],"000000000000");
		shm_actor_r_data[t].value_long = -204800000;
		shm_actor_r_data[t].value_float = -2048000.0;
		strcpy(shm_actor_r_data[t].value_string,"(unknown)");
		for (i = 0;i < 8;i++)
			{
			cache_slave_mapfrom_actor[t][i] = -1;
			cache_slave_value[t][i] = -204800000;
			}
		cache_slave_name[t][0] = 0;
		cache_slave_comment[t][0] = 0;
		}
	}


// **********   Demon-Configfile einlesen   **********

void pldemon_readconfig(void)
	{
	char ep[1024];
	unsigned short int t;
	// unsigned short int sklaven = 0;

	powerline_reload_counter = sqldb_read_long("/control/powerline/server/reload_counter");

	log_level = sqldb_read_long("/control/powerline/server/loglevel");
	cache_max_slaves = sqldb_read_long("/control/powerline/server/max_slaves");
	read_delay = sqldb_read_long("/control/powerline/server/readdelay");
	powerline_baudrate = sqldb_read_long("/control/powerline/server/baudrate");
	sqldb_read_string("/control/powerline/server/comport",powerline_comport);
	powerline_modus = sqldb_read_long("/control/powerline/server/modus");
	powerline_listen_to_broadcasts = sqldb_read_long("/control/powerline/server/listen_to_broadcasts");

	for(t = 0;t < cache_max_slaves;t++)
		{
		cache_slave_load(t);
		}

	sprintf(ep,"Global Powerline Configuration read. Loglevel:%d, Readdelay:%ld, Slaves:%d, Modus:%ld, BC:%ld, Comport:%s, Baudrate:%ld",log_level,read_delay,cache_max_slaves,powerline_modus,powerline_listen_to_broadcasts,powerline_comport,powerline_baudrate);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	}



// *****   cache_slave_load(nr)   -   Lesen eines Powerline-Slaves   *****

unsigned short int cache_slave_load(unsigned short int t)
	{
	char ep[1024];
	unsigned short int rw = 0;

	sprintf(ep,"/control/powerline/slave%d/enabled",t);
	cache_slave_enabled[t] = sqldb_read_long(ep);
	sprintf(ep,"/control/powerline/slave%d/use_buttons",t);
	cache_slave_use_buttons[t] = sqldb_read_long(ep);
	sprintf(ep,"/control/powerline/slave%d/type",t);
	cache_slave_typ[t] = sqldb_read_long(ep);
	if (cache_slave_enabled[t] > 0) rw = 1;
	sprintf(ep,"/control/powerline/slave%d/mapfrom_actor0",t);
	cache_slave_mapfrom_actor[t][0] = sqldb_read_long(ep);
	sprintf(ep,"/control/powerline/slave%d/mapfrom_actor1",t);
	cache_slave_mapfrom_actor[t][1] = sqldb_read_long(ep);
	sprintf(ep,"/control/powerline/slave%d/mapfrom_actor2",t);
	cache_slave_mapfrom_actor[t][2] = sqldb_read_long(ep);
	sprintf(ep,"/control/powerline/slave%d/mapfrom_actor3",t);
	cache_slave_mapfrom_actor[t][3] = sqldb_read_long(ep);
	sprintf(ep,"/control/powerline/slave%d/mapfrom_actor4",t);
	cache_slave_mapfrom_actor[t][4] = sqldb_read_long(ep);
	sprintf(ep,"/control/powerline/slave%d/mapfrom_actor5",t);
	cache_slave_mapfrom_actor[t][5] = sqldb_read_long(ep);
	sprintf(ep,"/control/powerline/slave%d/mapfrom_actor6",t);
	cache_slave_mapfrom_actor[t][6] = sqldb_read_long(ep);
	sprintf(ep,"/control/powerline/slave%d/mapfrom_actor7",t);
	cache_slave_mapfrom_actor[t][7] = sqldb_read_long(ep);
	sprintf(ep,"/control/powerline/slave%d/lon_id",t);
	sqldb_read_string(ep,cache_slave_lon_id[t]);
	sprintf(ep,"/control/powerline/slave%d/name",t);
	sqldb_read_string(ep,cache_slave_name[t]);
	sprintf(ep,"/control/powerline/slave%d/comment",t);
	sqldb_read_string(ep,cache_slave_comment[t]);

	return(rw);
	}



// *****   cache_slave_save(nr)   -   Schreiben eines Powerline-Slaves   *****

void cache_slave_save(unsigned short int t)
	{
	char ep[1024];

	sprintf(ep,"/control/powerline/slave%d/enabled",t);
	sqldb_write_long(ep,cache_slave_enabled[t]);
	sprintf(ep,"/control/powerline/slave%d/use_buttons",t);
	sqldb_write_long(ep,cache_slave_use_buttons[t]);
	sprintf(ep,"/control/powerline/slave%d/type",t);
	sqldb_write_long(ep,cache_slave_typ[t]);
	sprintf(ep,"/control/powerline/slave%d/mapfrom_actor0",t);
	sqldb_write_long(ep,cache_slave_mapfrom_actor[t][0]);
	sprintf(ep,"/control/powerline/slave%d/mapfrom_actor1",t);
	sqldb_write_long(ep,cache_slave_mapfrom_actor[t][1]);
	sprintf(ep,"/control/powerline/slave%d/mapfrom_actor2",t);
	sqldb_write_long(ep,cache_slave_mapfrom_actor[t][2]);
	sprintf(ep,"/control/powerline/slave%d/mapfrom_actor3",t);
	sqldb_write_long(ep,cache_slave_mapfrom_actor[t][3]);
	sprintf(ep,"/control/powerline/slave%d/mapfrom_actor4",t);
	sqldb_write_long(ep,cache_slave_mapfrom_actor[t][4]);
	sprintf(ep,"/control/powerline/slave%d/mapfrom_actor5",t);
	sqldb_write_long(ep,cache_slave_mapfrom_actor[t][5]);
	sprintf(ep,"/control/powerline/slave%d/mapfrom_actor6",t);
	sqldb_write_long(ep,cache_slave_mapfrom_actor[t][6]);
	sprintf(ep,"/control/powerline/slave%d/mapfrom_actor7",t);
	sqldb_write_long(ep,cache_slave_mapfrom_actor[t][7]);
	sprintf(ep,"/control/powerline/slave%d/lon_id",t);
	sqldb_write_string(ep,cache_slave_lon_id[t]);
	sprintf(ep,"/control/powerline/slave%d/name",t);
	sqldb_write_string(ep,cache_slave_name[t]);
	sprintf(ep,"/control/powerline/slave%d/comment",t);
	sqldb_write_string(ep,cache_slave_comment[t]);

	if(t > cache_max_slaves)
		{
		cache_max_slaves = t;
		sqldb_write_long("/control/powerline/server/max_slaves",cache_max_slaves);
		}
	}



// *****   verify_servicepin_messages()   -   Testen, ob Broadcasts angekommen sind   *****

void verify_servicepin_messages(void)
	{
	signed short int i;
	char ep[1024];
	signed short int t;
	signed short int j;
	signed short int firstfree = -1;

	if (powerline_listen_to_broadcasts > 0)
		{
		i = powerline_query_servicepin2(powerline_max_retries,global_fd);
		if(i > 0)
			{
			sprintf(ep,"Service Pin Broadcast from slave \"%s\" detected",servicepin_slave_lon_id);
			log_append(3,ep);																		// Meldung ins Logfile schreiben

			for(t = 0;t < POWERLINE_MAX_SLAVES;t++)
				{
				if (cache_slave_enabled[t] > 0)
					{
					if(strcmp(cache_slave_lon_id[t],servicepin_slave_lon_id) == 0)
						{
						sprintf(ep,"Slave \"%s\" already configured as #%d",servicepin_slave_lon_id,t);
						log_append(3,ep);															// Meldung ins Logfile schreiben
						goto qsp_ende;
						}
					}
				else
					{
					if(firstfree < 0) firstfree = t;
					}
				}

			sprintf(ep,"Slave \"%s\" not found; will try to add it at position %d...",servicepin_slave_lon_id,firstfree);
			log_append(3,ep);															// Meldung ins Logfile schreiben
			i = powerline_query_devicetyp2(servicepin_slave_lon_id,powerline_max_retries + 10,global_fd);
			if((i >= 1) && (i <= 3))
				{
				sprintf(ep,"Devicetyp: %d (wir gehen auch von %d Ports aus)...",i,i);
				log_append(3,ep);															// Meldung ins Logfile schreiben

				cache_slave_enabled[firstfree] = 1;
				cache_slave_use_buttons[firstfree] = 0;
				cache_slave_typ[firstfree] = i;
				strcpy(cache_slave_lon_id[firstfree],servicepin_slave_lon_id);
				shm_actor_r_data[firstfree].value_long = -204800000;
				shm_actor_r_data[firstfree].value_float = -2048000.0;
				strcpy(shm_actor_r_data[firstfree].value_string,"(unknown)");
				sprintf(ep,"Powerline Slave %d",firstfree);
				strcpy(cache_slave_name[firstfree],ep);
				strcpy(cache_slave_comment[firstfree],"Auto-Added by Servicepin Message Request");
				for (t = 0;t < 8;t++)
					{
					cache_slave_mapfrom_actor[firstfree][t] = -1;
					cache_slave_value[firstfree][t] = -204800000;
					if (t < i)
						{
						sprintf(ep,"%s Channel %d",cache_slave_name[firstfree],t);
						j = actor_add(ep,"auto-added",1,1,-1,-1,-1);	// Aktor neu anlegen
						cache_slave_mapfrom_actor[firstfree][t] = j;	// und Mapping eintragen
						actor_show(cache_slave_mapfrom_actor[firstfree][t]); // und diesen Aktor auch anzeigen
						}
					}

				cache_slave_save(firstfree);											// Speichern des neuen Slaves
				powerline_reload_counter = -42;										// invalidieren des Powerline-Reloadcounters, so erzwingen des Neuladens
				actor_increase_counter();													// invalidieren des Aktor-Reloadcounters
				sprintf(ep,"Slave \"%s\" saved at position %d (with %d ports)",servicepin_slave_lon_id,firstfree,i);
				log_append(3,ep);																	// Meldung ins Logfile schreiben
				}
				else
				{
				sprintf(ep,"Devicetyp: %d, da nur 1...3 zulssing sind, wird der Slave ignoriert.",i);
				log_append(3,ep);															// Meldung ins Logfile schreiben
				}

qsp_ende: ;
			}
		}
	}



// *****   actor_add   -   hinzufgen eines neuen logischen Aktors   *****

signed short int actor_add(char name[],char bla[],signed short int enabled,signed short int typ,signed short int bootdefault,signed short int einaus_sensor,signed short int watt_sensor)
	{
	char ep[512];
	signed long int t;
	signed long int i;
	signed long int max;


	// freie Aktornummer suchen
	for(t = 0;t < 1000;t++)
		{
		sprintf(ep,"/control/actors/actor%ld/enabled",t);
		i = sqldb_read_long(ep);
		if(i <= 0) goto schreibe;
		}
	t = -1;
	goto ende;

	// Aktor schreiben
schreibe:
	sprintf(ep,"/control/actors/actor%ld/enabled",t);
	sqldb_write_long(ep,enabled);
	sprintf(ep,"/control/actors/actor%ld/name",t);
	sqldb_write_string(ep,name);
	sprintf(ep,"/control/actors/actor%ld/type",t);
	sqldb_write_long(ep,typ);
	sprintf(ep,"/control/actors/actor%ld/bootdefault",t);
	sqldb_write_long(ep,bootdefault);
	sprintf(ep,"/control/actors/actor%ld/mapto_sensor",t);
	sqldb_write_long(ep,einaus_sensor);
	sprintf(ep,"/control/actors/actor%ld/wattmapto_sensor",t);
	sqldb_write_long(ep,watt_sensor);

	// Wenn die Nummer grer als das Maximum ist, dieses erhhen
	max = sqldb_read_long("/control/actors/max_actors");
	if(t >= max)
		{
		max = t + 1;
		sqldb_write_long("/control/actors/max_actors",max);
		}

ende:
	sprintf(ep,"neuer logischer Aktor %ld \"%s\" angelegt.",t,name);
	log_append(3,ep);															// Meldung ins Logfile schreiben
	return(t);
	}



// *****   actor_show()   -   logischen Aktor zu Anzeigeseite 1 hinzufgen   *****

void actor_show(signed short int nummer)
	{
	char ep[512];
	signed short int t;
	signed short int i;

	for(t = 0;t < 1000;t++)
		{
		sprintf(ep,"/control/actors_display/page0/actor%d",t);
		i = sqldb_read_long(ep);
		if(i == nummer) goto ende;													// bereits konfiguriert
		if(i < 0) goto gefunden;														// leere Nummer gefunden
		}
	goto ende;

gefunden:
	sqldb_write_long(ep,nummer);

ende:
	sprintf(ep,"Logischer Aktor %d wurde auf Anzeigeseite0 an Position %d eingefgt",nummer,t);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	}



// *****   actor_increase_counter()   -   Reload-Counter fr logische Aktoren erhhen   *****

void actor_increase_counter(void)
	{
	signed long int t;

	t = sqldb_read_long("/control/actors/reload_counter");
	t++;
	sqldb_write_long("/control/actors/reload_counter",t);
	}



// *****   Hauptschleife   *****

int main(int argc,char **argv)
	{
	unsigned long					ti = 0;													// Zhler, geht bei jedem Durchlauf um 1 hoch
	char									ep[1024];												// Puffer fr Logfile-Meldungen
	signed long int				t;
	signed long int				i;
	unsigned short int		j;


	// Logging-System initialisieren
	strcpy (log_filename,"/www/reports/powerline_demon.log"); // Logfilenamen setzen
	log_level = 3;																				// Debuglevel setzen
	sprintf(ep,"----------   powerline_demon V %s started (debuglevel=%d)   ----------",POWERLINE_DEMON_VERSION,log_level);
	log_append(1,ep);																			// Startmeldung ins Logfile schreiben



	// auf existierendes Shared Memory fr Aktordaten Read zugreifen
	shID = shmget(SHM_ID_ACTOR_R_DATA,SHM_ACTOR_R_DATA_SIZE,0666);
	if (shID >= 0)
		{
		shm_actor_r_data = shmat(shID,0,0);
		if (shm_actor_r_data == (struct s_shm_actordata *)-1)
			{
			printf("Problem beim Zugriff auf ACTOR_R_DATA shared memory !\n");
			goto ende;
			}
		else
			{
			// printf("SHM_Actor_r_data OK.\n");
			}
		}
	else
		{
		// shmget lief schief
		printf("shmget fr SHM_Actor_r_data war erfolglos ! (shm_create nicht aufgerufen ?)\n");
		goto ende;
    }

	// auf existierendes Shared Memory fr Aktordaten Write zugreifen
	shID = shmget(SHM_ID_ACTOR_W_DATA,SHM_ACTOR_W_DATA_SIZE,0666);
	if (shID >= 0)
		{
		shm_actor_w_data = shmat(shID,0,0);
		if (shm_actor_w_data == (struct s_shm_actordata *)-1)
			{
			printf("Problem beim Zugriff auf ACTOR_W_DATA shared memory !\n");
			goto ende;
			}
		else
			{
			// printf("SHM_Actor_w_data OK.\n");
			}
		}
	else
		{
		// shmget lief schief
		printf("shmget fr SHM_Actor_w_data war erfolglos ! (shm_create nicht aufgerufen ?)\n");
		goto ende;
    }


	// Shadow-Write auf ngltige Werte setzen
	for (t = 0;t < MAX_LOGICAL_ACTORS;t++)
		{
		shadow_w_data[t] = -99;
		}


	// Einstellungen fr lokalen Powerline-UDP-Demon
	cache_setzedefault();
	pldemon_readconfig();

	global_fd = rs232_open(powerline_comport,powerline_baudrate);
	if (global_fd <= 0)
		{
		sprintf (ep,"ERROR: RS232-Port %s konnte nicht geffnet werden !\r",powerline_comport);
		log_append(1,ep);																		// Meldung ins Logfile schreiben
		goto ende;
		}

	// Start loggen
	sprintf(ep,"**********   powerline demon V %s started   **********",POWERLINE_DEMON_VERSION);
	log_append(1,ep);
	printf("powerline demon V %s started. (modus=%ld,BC=%ld)\n",POWERLINE_DEMON_VERSION,powerline_modus,powerline_listen_to_broadcasts);


	last_read_timestamp = get_ms_now();										// now() merken
hauptschleife:
	ti++;																									// Schleifenzhler erhhen
	usleep(100000);																				// 100 ms Rechenzeit freigeben
	if (powerline_reload_counter != sqldb_read_long("/control/powerline/server/reload_counter"))
		{
		pldemon_readconfig();
		sprintf(ep,"Global Powerline Configuration re-read.");
		log_append(1,ep);																		// Meldung ins Logfile schreiben
		goto hauptschleife;
		}

	if(powerline_modus != 0)
		{
		for(t = 0;t < POWERLINE_MAX_SLAVES;t++)
			{
			if (cache_slave_enabled[t] > 0)
				{
				for(j = 0;j < cache_slave_typ[zyklischer_sklave];j++)
					{
					shm_actor_r_data[cache_slave_mapfrom_actor[zyklischer_sklave][j]].value_long = -204800000;
					}
				}
			}
		verify_servicepin_messages();
		sleep(5);																						// 5 Sekunden Rechenzeit freigeben
		goto hauptschleife;																	// im Bridge-Modus gibts fr den Demon nichts zu tun.
		}
	// -----   Aktor-Werte auf Vernderungen prfen   -----
	for(t = 0;t < POWERLINE_MAX_SLAVES;t++)
		{
		if (cache_slave_enabled[t] > 0)
			{
			for(j = 0;j < cache_slave_typ[t];j++)
				{
				if (cache_slave_mapfrom_actor[t][j] >= 0)
					{
					if (shm_actor_r_data[cache_slave_mapfrom_actor[t][j]].value_long > -2048000)
						{
						if ((shm_actor_w_data[cache_slave_mapfrom_actor[t][j]].value_long != shadow_w_data[cache_slave_mapfrom_actor[t][j]])
						||  (shm_actor_w_data[cache_slave_mapfrom_actor[t][j]].value_long != shm_actor_r_data[cache_slave_mapfrom_actor[t][j]].value_long))
							{
							// Slave aktiv, korrekt gemappt und Aktor-W-Wert hat sich gendert: wir mssen Schalten !
							sprintf(ep,"*** Aktor %ld will slave %ld[%d] auf %ld setzen. (%s) ***",cache_slave_mapfrom_actor[t][j],t,j,shm_actor_w_data[cache_slave_mapfrom_actor[t][j]].value_long,cache_slave_lon_id[t]);
							log_append(3,ep);																			// Meldung ins Logfile schreiben
							i = powerline_query_buttonflag2(zyklischer_sklave,j,powerline_max_retries,global_fd);
							if(i > 0)
								{
								// printf("*** Knopf gedrckt ***\n");
								sprintf(ep,"** Knopf an Powerline Slave %ld[%d] gedrckt - Istatatus %ld wird als Sollstatus bernommen.",t,j,shm_actor_r_data[cache_slave_mapfrom_actor[t][j]].value_long);
								log_append(3,ep);																			// Meldung ins Logfile schreiben
								shm_actor_w_data[cache_slave_mapfrom_actor[t][j]].value_long = shm_actor_r_data[cache_slave_mapfrom_actor[t][j]].value_long;
								shadow_w_data[cache_slave_mapfrom_actor[t][j]] = shm_actor_r_data[cache_slave_mapfrom_actor[t][j]].value_long;
								}
							i = powerline_schalte2(t,j,shm_actor_w_data[cache_slave_mapfrom_actor[t][j]].value_long,powerline_max_retries,global_fd);
							if (i >= 0)
								{
								sprintf(ep,"+++ Schaltvorgang OK +++");
								log_append(3,ep);																			// Meldung ins Logfile schreiben
								shadow_w_data[cache_slave_mapfrom_actor[t][j]] = i;
								shm_actor_w_data[cache_slave_mapfrom_actor[t][j]].value_long = shadow_w_data[cache_slave_mapfrom_actor[t][j]];
								shm_actor_r_data[cache_slave_mapfrom_actor[t][j]].value_long = shadow_w_data[cache_slave_mapfrom_actor[t][j]];
								}
							else
								{
								sprintf(ep,"--- Schaltvorgang nicht erfolgreich ---");
								log_append(3,ep);																			// Meldung ins Logfile schreiben
								shm_actor_r_data[cache_slave_mapfrom_actor[t][j]].value_long = -204800000;
								}

							}
						}
					else
						{
						// Slave aktiv und korrekt gemappt, aber Lesewert=-2048000: Wir brauchen nicht zu schalten, weil nicht erreichbar
						// sprintf(ep,"*** Slave %ld[%d] unerreichbar ***",t,j);
						// log_append(3,ep);																			// Meldung ins Logfile schreiben
						}
					}
				}
			}
		}



	// -----   Zyklisches Lesen der Aktoren   -----
	for(t = 0;t < POWERLINE_MAX_SLAVES;t++)
		{
		if(cache_slave_enabled[t] > 0) goto beginn_zyklus;
		}
	goto ende_zyklus;
beginn_zyklus:
	if (last_read_timestamp > get_ms_now()) last_read_timestamp = 0; // berlauf um Mitternacht kompensieren
	if (get_ms_now() > last_read_timestamp + read_delay * 1000)
		{
		// printf("**READ %ld** delay=%ld\n",last_read_timestamp,read_delay);
suche_zyklus:
		last_read_timestamp = get_ms_now();
		zyklischer_sklave++;
		if(zyklischer_sklave >= POWERLINE_MAX_SLAVES) zyklischer_sklave = 0;
		if (cache_slave_enabled[zyklischer_sklave] <= 0) goto suche_zyklus;
		// printf("cache_slave_typ[%ld]=%d\n",zyklischer_sklave,cache_slave_typ[zyklischer_sklave]);
		for(j = 0;j < cache_slave_typ[zyklischer_sklave];j++)
			{
			i = powerline_query2(zyklischer_sklave,j,powerline_max_retries,global_fd);
			if (i >= 0)
				{
				shm_actor_r_data[cache_slave_mapfrom_actor[zyklischer_sklave][j]].value_long = i;
				}
			else
				{
				shm_actor_r_data[cache_slave_mapfrom_actor[zyklischer_sklave][j]].value_long = -204800000;
				}
			}

		}
ende_zyklus: ;

	verify_servicepin_messages();
	goto hauptschleife;

ende:
	return(0);
	}
