// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   rgb_listener.c   -   Lauschen auf den UDP-Port des RGB-Demons und bei Eingang von Daten schreiben in tmpDB    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2011 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************

// **********   Eintrge in der Konfigurations-Datenbank, die vom Listener ausgewertet werden   **********
//
// /sys/network/udp_rx_port															UDP-Portnummer, auf die gelauscht wird
// /sys/network/udp_rx_username													zu bergebender Username
// /sys/network/udp_rx_password													zu bergebendes Pawort
// /sys/network/udp_rx_reloadcounter										wird beim Speichern erhht
// /sys/network/udp_rx_enabled													0: Nein, 1: Ja



#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <time.h>
#include <sys/time.h>
#include <sys/shm.h>
#include "i2c_chips.h"
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "rgb_listener.h"
#include "sqldb.h"
#include "logging.h"
#include "shm_use.h"



// *****   Globale Variablen   *****

#define CONFIG_DATABASE_FILE "/etc/allnetenv/config.s3db"
#define VAR_DATABASE_FILE "/tmp/var.s3db"



// *****   sql_write_rgbvalues   -   Schreiben der RGB-Output-Werte in die tmpDB   *****

void sql_write_rgbvalues(sqlite3 *ptr,unsigned char r0,unsigned char g0,unsigned char b0,unsigned char w0,unsigned char d0,unsigned char r1,unsigned char g1,unsigned char b1,unsigned char w1,unsigned char d1)
	{
	char sql[255];

	sprintf(sql,"UPDATE rgboutput SET r0=%d,g0=%d,b0=%d,w0=%d,d0=%d,r1=%d,g1=%d,b1=%d,w1=%d,d1=%d WHERE id = 0;",r0,g0,b0,w0,d0,r1,g1,b1,w1,d1);
	// printf("rgb_listener SQL: \"%s\"\n",sql);
	do_sqlyy(ptr,sql,0);
	}



// *****   Hauptschleife   *****

int main(int argc,char **argv)
	{
	unsigned long				ti = 0;														// Zhler, geht bei jedem Durchlauf um 1 hoch
	unsigned char				c;
	unsigned short int	udp_rx_port = 54321;							// TCP Socket fr Client/Server Verbindung
	char								udp_rx_username[1024] = "";				// Username fr UDP-Verbindung
	char								udp_rx_password[1024] = "";				// Passwort fr UDP-Verbindung
	unsigned short int	udp_rx_enabled = 0;								// 0: Aus, 1: An
	unsigned short int	udp_rx_reloadcounter = -1;				// Reloadcounter
	int									udp_socket = -1;
	struct sockaddr_in	udp_control1;
	struct sockaddr_in	udp_control2;
	char								udp_puffer[30001];
	unsigned int				udp_laenge;
	int									udp_received = 0;
	char								udp_received_username[21];
	char								udp_received_password[21];
	char								ep[1024];
	signed short int		t;


	// auf existierendes Shared Memory fr Demonstatus zugreifen
	if (use_my_demonstatus_shm(2) != 1) goto errorende;		// MY_DEMON_ID = 2: rgb_listener

	// Status von Demon in SHM eintragen
	my_demonstatus_data[my_demon_id].demon_id = my_demon_id;														// 0 ist Default, hier wird die eigene ID eingetragen
	sprintf(my_demonstatus_data[my_demon_id].name_string,"rgb_listener");								// Name
	sprintf(my_demonstatus_data[my_demon_id].version_string,"%s",RGBLISTENER_VERSION);	// Versionsnummer
	my_demonstatus_data[my_demon_id].last_boot = time(0);																// Timestamp(UTC) des Starts
	my_demonstatus_data[my_demon_id].last_activity = time(0);														// Timestamp(UTC) der letzten Aktion
	my_demonstatus_data[my_demon_id].ti = ti;																						// Counter
	sprintf(my_demonstatus_data[my_demon_id].debug_message,"Start");										// letzte Debug-Meldung
	my_demonstatus_data[my_demon_id].error_message[0] = 0;															// letzte Fehlermeldung
	my_demonstatus_data[my_demon_id].status_long = 0;																		// numerischer Status
	sprintf(my_demonstatus_data[my_demon_id].status_string,"startup");									// String-Status


	// Logging-System initialisieren
	strcpy (log_filename,"/www/reports/rgb_listener.log");// Logfilenamen setzen
	log_level = 4;																				// Debuglevel setzen
	log_kill();																						// Logfile loeschen
	sprintf(ep,"----------   rgb_listener V %s started (debuglevel=%d)   ----------",RGBLISTENER_VERSION,log_level);
	log_append(1,ep);																			// Startmeldung ins Logfile schreiben




	sqlite3_open(VAR_DATABASE_FILE,&sql_var_handle);			// SQLITE3-Datenbank ffnen

reload:
	update_demonstatus(ti,2,"reload");										// Status von Demon in SHM aktualisieren
	// Einstellungen fr lokalen RGB-Demon
	udp_rx_port = sqldb_read_long("/sys/network/udp_rx_port");
	sqldb_read_string("/sys/network/udp_rx_username",udp_rx_username);
	sqldb_read_string("/sys/network/udp_rx_password",udp_rx_password);
	udp_rx_enabled = sqldb_read_long("/sys/network/udp_rx_enabled");
	udp_rx_reloadcounter = sqldb_read_long("/sys/network/udp_rx_reloadcounter");

	sprintf(ep,"rgb_listener: UDP Port:%d, Username:\"%s\", Password:\"%s\", enabled:%d",udp_rx_port,udp_rx_username,udp_rx_password,udp_rx_enabled);
	log_append(1,ep);
	printf("%s\n",ep);

	// Destroy old UDP socket, if one exists
	if (udp_socket >= 0)
		{
		printf("rgb_listener: DESTROY\n");
		close(udp_socket);
		}

	// Create the UDP socket
	if ((udp_socket = socket(PF_INET,SOCK_DGRAM,IPPROTO_UDP)) < 0) // UDP Socket anlegen
		{
		printf("rgb_listener: ERROR: Es konnte kein neuer UDP-Socket angelegt werden.");
		log_append(1,"ERROR: Es konnte kein neuer UDP-Socket angelegt werden.");
		goto ende;
		}
	if (fcntl(udp_socket,F_SETFL,O_NONBLOCK) < 0)					// UDP-Socket auf Non-Blocking setzen
		{
		printf("rgb_listener: ERROR: Socket konnte nicht auf non-blocking geschaltet werden.");
		log_append(1,"ERROR: Socket konnte nicht auf non-blocking geschaltet werden.");
		goto ende;
		}

	// Construct the server sockaddr_in structure
	memset(&udp_control1,0,sizeof(udp_control1));					// Clear struct
	udp_control1.sin_family = AF_INET;										// Internet/IP
	udp_control1.sin_addr.s_addr = htonl(INADDR_ANY);			// Any IP address
	udp_control1.sin_port = htons(udp_rx_port);						// server port

	// Bind the socket
	udp_laenge = sizeof(udp_control1);
	if (bind(udp_socket,(struct sockaddr *) &udp_control1,udp_laenge) < 0)
		{
		printf("rgb_listener: ERROR: Failed to bind server socket");
		log_append(1,"ERROR: Failed to bind server socket");
		goto ende;
		}



	update_demondebug("running");													// Status von Demon in SHM aktualisieren
hauptschleife:
	update_demonstatus(ti,1,"main loop");									// Status von Demon in SHM aktualisieren

	usleep(10000);																				// 10 ms warten
	ti++;																									// Schleifenzhler erhhen
	// printf("ti=%ld\n",ti);

	if (udp_rx_reloadcounter != sqldb_read_long("/sys/network/udp_rx_reloadcounter")) goto reload;

	if (udp_rx_enabled == 0)
		{
		update_demonstatus(ti,1,"sleeping");								// Status von Demon in SHM aktualisieren
		update_demondebug("RX deactivated");								// Status von Demon in SHM aktualisieren
		sleep(2);
		goto hauptschleife;
		}


	// ----- Wenn via UDP Daten eingetrudelt sind, werden diese hier geprft und verarbeitet -----
	// Receive a message from the client
	memset(&udp_puffer,0,64);														// Puffer lschen
	udp_laenge = sizeof(udp_control2);
	if ((udp_received = recvfrom(udp_socket,udp_puffer,64,0,(struct sockaddr *) &udp_control2,&udp_laenge)) >= 0)
		{
		update_demonstatus(ti,2,"got data");									// Status von Demon in SHM aktualisieren
		sprintf(ep,"%d from %s",udp_received,inet_ntoa(udp_control2.sin_addr));
		update_demondebug(ep);																// Status von Demon in SHM aktualisieren
		sprintf(ep,"%d Bytes angekommen von: %s, Daten=\"%s\"",udp_received,inet_ntoa(udp_control2.sin_addr),udp_puffer);
		log_append(4,ep);

		if (udp_received != 50)
			{
			sprintf(ep,"rgb_listener: Falsche Daten von %s: Lnge nicht 50 Bytes sondern %d Bytes, Paket wird verworfen.",inet_ntoa(udp_control2.sin_addr),udp_received);
			printf("%s\n",ep);
			log_append(3,ep);
			sprintf(ep,"%d instead of 50",udp_received);
			update_demonerror(ep);														// Status von Demon in SHM aktualisieren
			
			goto ende_udp;
			}

		for (t = 0;t < 20 - 1;t++)
			{
			udp_received_username[t] = 0;
			c = udp_puffer[t];
			if (c == 0) goto weiter_user;
			udp_received_username[t] = c;
			}
weiter_user:

		for (t = 0;t < 20 - 1;t++)
			{
			udp_received_password[t] = 0;
			c = udp_puffer[20 + t];
			if (c == 0) goto weiter_pw;
			udp_received_password[t] = c;
			}
weiter_pw:

		if(strcmp(udp_received_username,udp_rx_username) == 0)
			{
			if(strcmp(udp_received_password,udp_rx_password) == 0)
				{
				// Daten bernehmen
				t_r0 = udp_puffer[40];
				t_g0 = udp_puffer[41];
				t_b0 = udp_puffer[42];
				t_w0 = udp_puffer[43];
				t_dim0 = udp_puffer[44];
				t_r1 = udp_puffer[45];
				t_g1 = udp_puffer[46];
				t_b1 = udp_puffer[47];
				t_w1 = udp_puffer[48];
				t_dim1 = udp_puffer[49];
				sprintf(ep,"(Control_Debug) UDP input: r0:%d,g0:%d,b0:%d,w0:%d, r1:%d,r2:%d,r3:%d,w1:%d Dim0:%d,Dim1:%d",t_r0,t_g0,t_b0,t_w0,t_r1,t_g1,t_b1,t_w1,t_dim0,t_dim1);
				log_append(4,ep);
				sprintf(ep,"UDP input: r0:%d,g0:%d,b0:%d,w0:%d, r1:%d,r2:%d,r3:%d,w1:%d Dim0:%d,Dim1:%d\n",t_r0,t_g0,t_b0,t_w0,t_r1,t_g1,t_b1,t_w1,t_dim0,t_dim1);
				update_demonstatus(ti,10,"data OK");						// Status von Demon in SHM aktualisieren
				update_demondebug("writing RGB");								// Status von Demon in SHM aktualisieren
				sql_write_rgbvalues(sql_var_handle,t_r0,t_g0,t_b0,t_w0,t_dim0,t_r1,t_g1,t_b1,t_w1,t_dim1);
				}
			else
				{
				printf("rgb_listener: Passwort falsch.\n");
				log_append(3,"Passwort falsch.");
				update_demonerror("wrong PW");									// Status von Demon in SHM aktualisieren
				}
			}
		else
			{
			printf("rgb_listener: Username falsch.\n");
			log_append(3,"Username falsch.");
			update_demonerror("wrong User");									// Status von Demon in SHM aktualisieren
			}

ende_udp: ;
		}


	// ----- Falls noch mehr UDP-Pakete da sind, diese hier alle einlesen und ignorieren -----
	t = 0;
	udp_laenge = sizeof(udp_control2);
udp_flush:
	if ((udp_received = recvfrom(udp_socket,udp_puffer,30000,0,(struct sockaddr *) &udp_control2,&udp_laenge)) >= 0)
		{
		t++;
		// fprintf(stderr,"%d Bytes angekommen von: %s, Daten=\"%s\"\n",udp_received,inet_ntoa(udp_control2.sin_addr),udp_puffer);
		goto udp_flush;
		}
	// if (t > 0) printf("RGB_demon: %d useless UDP packets trashed.\n",t);
	if (t > 0) update_demondebug("packets ignored");			// Status von Demon in SHM aktualisieren



	goto hauptschleife;


ende:
	sqlite3_close(sql_var_handle);												// Datenbank schlieen
errorende:
	return(0);
	}
