// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   s0count_demon.c   -   Demon zum Zhlen der Impuse eines S0-Bus Stromzhlers an einem ALL3690 Powermeter       **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2009 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************

// **********   I/O Registerbelegung   **********
//
// PB27    S0-Bus Eingang 0 (Pullup nach 3.3V, Kontakt schaltet nach Masse)
// PB26    S0-Bus Eingang 1 (Pullup nach 3.3V, Kontakt schaltet nach Masse)
// 

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "sqldb.h"
#include "at91gpio.h"
#include "all_i2c.h"
#include "i2c_chips.h"
#include "logging.h"



// **********   Globale Variablen   **********

#define								S0COUNT_DEMON_VERSION "1.05"
#define								VAR_DATABASE_FILE "/tmp/var.s3db"

unsigned long int			s0_tickcount[2];									// Anzahl Impulse seit Programmstart
sqlite3								*sql_var_handle;									// Handle fr die DB



// *****   demon_readconfig   -   Konfigurationsdaten lesen   *****

void demon_readconfig(void)
	{
	s0_tickcount[0] = sqldb_read_long("/powermeter/s0/counter");
	s0_tickcount[1] = sqldb_read_long("/powermeter/s1/counter");
	}



// **********   Demon   **********

int main(int argc,char **argv)
	{
	char ep[80];
	char s0_logfile[80] = "/www/reports/s0.log";					// Hier soll die Logdatei hin
	signed short int hwmodel = 3690;											// Hardware-ID

	hwmodel = sqldb_read_long("/sys/hardware/numeric_model");		// auf welcher Hardware sind wir ? 3690 oder 3691 ?

	// ALL3690/3691: Ersten GPIO fr S0-Counter 0 fr Input herrichten
	gpio_devmem_open();																		// GPIOs oeffnen
	gpio_port_init_output(PORT_B,27);											// PB27 ist der S0 Eingang 0
	gpio_port_write(PORT_B,27,0);													// L
	sleep(1);																							// 1 Sekunde warten
	gpio_port_write(PORT_B,27,1);													// H
	gpio_port_init_input(PORT_B,27);											// auf Eingang setzen

	if(hwmodel == 3691)
		{
		// ALL3691: Zweiten GPIO fr S0-Counter 1 fr Input herrichten
		gpio_port_init_output(PORT_B,26);											// PB26 ist der S0 Eingang 1
		gpio_port_write(PORT_B,26,0);													// L
		sleep(1);																							// 1 Sekunde warten
		gpio_port_write(PORT_B,26,1);													// H
		gpio_port_init_input(PORT_B,26);											// auf Eingang setzen
		}
	else
		{
		// ALL3691: Zweiten GPIO fr S0-Counter 1 fest auf H legen, um ihn zu deaktivieren
		gpio_port_init_output(PORT_B,26);											// PB26 ist der S0 Eingang 1
		gpio_port_write(PORT_B,26,1);													// H
		}

	strcpy(log_filename,s0_logfile);											// Pfade fuer Logfile setzen
	log_level = 3;																				// Debuglevel setzen
	sprintf(ep,"----------   s0count_demon V %s started (HWmodel=%d, debuglevel=%d, %ld Ticks0, %ld Ticks1)   ----------",S0COUNT_DEMON_VERSION,hwmodel,log_level,s0_tickcount[0],s0_tickcount[1]);
	log_append(1,ep);																			// Startmeldung ins Logfile schreiben

	demon_readconfig();																		// Einstellungen fuer den Clock-Demon lesen

	printf("S0count_demon V %s hwmodel=%d started.\n(%ld Ticks0, %ld Ticks1)\n",S0COUNT_DEMON_VERSION,hwmodel,s0_tickcount[0],s0_tickcount[1]);

	sqlite3_open(VAR_DATABASE_FILE,&sql_var_handle);			// SQLITE3-Datenbank ffnen

hauptschleife:
	// sleep(0);																					// ein bichen Zeit vertrdeln
	usleep(5000);
	if(gpio_port_read(PORT_B,27) == 0) goto hauptschleife; // Solange S0 0 L ist, erstmal abwarten
	if(gpio_port_read(PORT_B,26) == 0) goto hauptschleife; // Solange S0 1 L ist, erstmal abwarten
	usleep(5000);																					// danach 5 ms warten, um evtl. Kontaktprellen nicht mitzuzhlen

lauern:
	usleep(5000);																					// 5 ms warten
	if((gpio_port_read(PORT_B,27) > 0) && (gpio_port_read(PORT_B,26) > 0)) goto lauern; // darauf lauern, da einer der Ports auf L gezogen wird
	usleep(5000);																					// danach 5 ms warten, um evtl. Kontaktprellen nicht mitzuzhlen

	if(gpio_port_read(PORT_B,27) == 0) s0_tickcount[0]++;	// Tick an Port 0 zhlen
	if(gpio_port_read(PORT_B,26) == 0) s0_tickcount[1]++;	// Tick an Port 1 zhlen
	printf("Tick-Counts: 0:%ld, 1:%ld\n",s0_tickcount[0],s0_tickcount[1]);

	sprintf(ep,"UPDATE s0 SET [value]='%ld' where [tag]='S0_0';",s0_tickcount[0]);
	do_sqlyy(sql_var_handle,ep,0);
	sprintf(ep,"UPDATE s0 SET [value]='%ld' where [tag]='S0_1';",s0_tickcount[1]);
	do_sqlyy(sql_var_handle,ep,0);

	// sprintf(ep,"UPDATE [measurements] set [stick0]='%ld' where [id]='0';",s0_tickcount[0]);
	// do_sqlyy(sql_var_handle,ep,0);
	// sprintf(ep,"UPDATE [measurements] set [stick1]='%ld' where [id]='0';",s0_tickcount[1]);
	// do_sqlyy(sql_var_handle,ep,0);
	sprintf(ep,"UPDATE [measurements] set [stick0]='%ld',[stick1]='%ld' where [id]='0';",s0_tickcount[0],s0_tickcount[1]);
	do_sqlyy(sql_var_handle,ep,0);


	goto hauptschleife;																		// ... und wieder dasselbe Spiel von vorne


// ende:
	sprintf(ep,"----------   s0count_demon terminated   ----------");
	log_append(1,ep);																			// Endmeldung ins Logfile schreiben
	sqlite3_close(sql_var_handle);												// tmp-Datenbank schlieen
	return(0);
	}

