// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   stripe_demon.c   -   Demon zum Anzeigen von Daten auf 16 RGB-Stripes                                          **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2009 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************

// **********   I/O Registerbelegung   **********

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <fcntl.h>
#include <termios.h>
#include "xml_lib.h"
#include "at91gpio.h"
#include "all_i2c.h"
#include "i2c_chips.h"
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "sqldb.h"
#include "logging.h"



// **********   Globale Variablen   **********

typedef struct tagBITMAPFILEHEADER
	{
	unsigned short int	bfType;
	unsigned long int		bfSize;
	unsigned short int	bfReserved1;
	unsigned short int	bfReserved2;
	unsigned long int		bfOffBits;
	}
	BITMAPFILEHEADER, *PBITMAPFILEHEADER;

typedef struct tagBITMAPINFOHEADER
	{
	unsigned long int		biSize;
	signed long int			biWidth;
	signed long int			biHeight;
	unsigned short int	biPlanes;
	unsigned short int	biBitCount;
	unsigned long int		biCompression;
	unsigned long int		biSizeImage;
	signed long int			biXPelsPerMeter;
	signed long int			biYPelsPerMeter;
	unsigned long int		biClrUsed;
	unsigned long int		biClrImportant;
	}
	BITMAPINFOHEADER, *PBITMAPINFOHEADER;

#define LONG_STRIPE_X 5000
#define STRIPE_X 256
#define STRIPE_Y 32
#define DELAY_X 5000

signed long int ti = 0;																	// System-Timer
char input_bmp_filename[80] = "/tmp/bitmap.bmp";				// Input-Dateiname
unsigned char stripe_r[STRIPE_X][STRIPE_Y];							// Rot-Wert
unsigned char stripe_g[STRIPE_X][STRIPE_Y];							// Rot-Wert
unsigned char stripe_b[STRIPE_X][STRIPE_Y];							// Rot-Wert
unsigned char long_stripe_r[LONG_STRIPE_X][STRIPE_Y];		// Rot-Wert
unsigned char long_stripe_g[LONG_STRIPE_X][STRIPE_Y];		// Rot-Wert
unsigned char long_stripe_b[LONG_STRIPE_X][STRIPE_Y];		// Rot-Wert

unsigned char font_face[256][32][32];										// Fontdaten
unsigned char font_width[256];													// Breite des Zeichens
unsigned char font_height[256];													// Hhe des Zeichens



// *****   parallel_init()   -   initialisieren des Parallelports   *****

void parallel_init()
	{
	gpio_port_init_output(PORT_B,20);											// Bit 0
	gpio_port_init_output(PORT_B,21);											// Bit 1
	gpio_port_init_output(PORT_B,22);											// Bit 2
	gpio_port_init_output(PORT_B,23);											// Bit 3
	gpio_port_init_output(PORT_B,24);											// Bit 4
	gpio_port_init_output(PORT_B,25);											// Bit 5
	gpio_port_init_output(PORT_B,26);											// Bit 6
	gpio_port_init_output(PORT_B,27);											// Bit 7
	gpio_port_init_output(PORT_B,29);											// Strobe

	gpio_port_write(PORT_B,20,0);													// L
	gpio_port_write(PORT_B,21,0);													// L
	gpio_port_write(PORT_B,22,0);													// L
	gpio_port_write(PORT_B,23,0);													// L
	gpio_port_write(PORT_B,24,0);													// L
	gpio_port_write(PORT_B,25,0);													// L
	gpio_port_write(PORT_B,26,0);													// L
	gpio_port_write(PORT_B,27,0);													// L
	gpio_port_write(PORT_B,29,0);													// L
	}



// *****   prarallel_out(zeichen)   -   paralleles Ausgaben eines Zeichens   *****

void parallel_out(unsigned char z)
	{
	// printf("[%d:%c]",z,z); fflush(stdout);
	gpio_port_write(PORT_B,29,0);													// Strobe L
	if(z &  1)																						// Bit 0
		{
		gpio_port_write(PORT_B,20,1);												// H
		}
	else
		{
		gpio_port_write(PORT_B,20,0);												// L
		}

	if(z &  2)																						// Bit 1
		{
		gpio_port_write(PORT_B,21,1);												// H
		}
	else
		{
		gpio_port_write(PORT_B,21,0);												// L
		}

	if(z &  4)																						// Bit 2
		{
		gpio_port_write(PORT_B,22,1);												// H
		}
	else
		{
		gpio_port_write(PORT_B,22,0);												// L
		}

	if(z &  8)																						// Bit 3
		{
		gpio_port_write(PORT_B,23,1);												// H
		}
	else
		{
		gpio_port_write(PORT_B,23,0);												// L
		}

	if(z & 16)																						// Bit 4
		{
		gpio_port_write(PORT_B,24,1);												// H
		}
	else
		{
		gpio_port_write(PORT_B,24,0);												// L
		}

	if(z & 32)																						// Bit 5
		{
		gpio_port_write(PORT_B,25,1);												// H
		}
	else
		{
		gpio_port_write(PORT_B,25,0);												// L
		}

	if(z & 64)																						// Bit 6
		{
		gpio_port_write(PORT_B,26,1);												// H
		}
	else
		{
		gpio_port_write(PORT_B,26,0);												// L
		}

	if(z & 128)																						// Bit 7
		{
		gpio_port_write(PORT_B,27,1);												// H
		}
	else
		{
		gpio_port_write(PORT_B,27,0);												// L
		}

	gpio_port_write(PORT_B,29,1);												// Strobe H
	}



// *****   fpga_write(serial pointer,string,laenge)   -   Zeichenkette ans FPGA bergeben   *****
#define PARALLEL_PORT 1

void fpga_write(int ser_p,char string[],unsigned short int laenge)
	{
	unsigned short int t;
	unsigned char z;

// xx: parallel_out(z++); goto xx;


	if(PARALLEL_PORT == 0)
		{
		// klassiche serielle Ausgabe
		write(ser_p,string,laenge);
		}
	else
		{
		// parallele Ausgabe
		for(t = 0;t < laenge;t++)
			{
			z = string[t];
			// printf("%c",z); fflush(stdout);
			parallel_out(z);
			}
		}
	}

// *****   stripe_fill()   -   Bitmap mit einer festen Farbe vorbelegen   *****

void stripe_fill(unsigned char r,unsigned char g,unsigned char b)
	{
	unsigned long int x;
	unsigned long int y;

	for (x = 0;x < STRIPE_X;x++)
		{
		for (y = 0;y < STRIPE_Y;y++)
			{
			stripe_r[x][y] = r;
			stripe_g[x][y] = g;
			stripe_b[x][y] = b;
			}
		}
	}



// *****   long_stripe_fill()   -   Bitmap mit einer festen Farbe vorbelegen   *****

void long_stripe_fill(unsigned char r,unsigned char g,unsigned char b)
	{
	unsigned long int x;
	unsigned long int y;

	for (x = 0;x < LONG_STRIPE_X;x++)
		{
		for (y = 0;y < STRIPE_Y;y++)
			{
			long_stripe_r[x][y] = r;
			long_stripe_g[x][y] = g;
			long_stripe_b[x][y] = b;
			}
		}
	}



// *****   read_font()   -   Zeichensatz einlesen   *****

void read_font(unsigned short int nummer)
	{
	char								filename[90];
	unsigned short int	t;
	FILE								*f;
	unsigned short int	h1_bfType;
	unsigned long int		h1_bfSize;
	unsigned short int	h1_bfReserved1;
	unsigned short int	h1_bfReserved2;
	unsigned long int		h1_bfOffBits;
	unsigned long int		h2_biSize;
	signed long int			h2_biWidth;
	signed long int			h2_biHeight;
	unsigned short int	h2_biPlanes;
	unsigned short int	h2_biBitCount;
	unsigned long int		h2_biCompression;
	unsigned long int		h2_biSizeImage;
	signed long int			h2_biXPelsPerMeter;
	signed long int			h2_biYPelsPerMeter;
	unsigned long int		h2_biClrUsed;
	unsigned long int		h2_biClrImportant;
	unsigned char				r;
	unsigned char				g;
	unsigned char				b;
	signed short int		x;
	signed short int		y;
	unsigned short int	xcount;



	for(t = 0;t < 256;t++)
		{
		font_width[t] = 0;
		font_height[t] = 0;
		for(x = 0;x < 32;x++)
			{
			for(y = 0;y < 32;y++)
				{
				font_face[t][x][y] = 0;
				}
			}
		sprintf(filename,"/root/font%d/%d.bmp",nummer,t);
		f = fopen(filename,"rb");
		if(f != NULL)
			{
			fread(&h1_bfType,sizeof(h1_bfType),1,f);
			fread(&h1_bfSize,sizeof(h1_bfSize),1,f);
			fread(&h1_bfReserved1,sizeof(h1_bfReserved1),1,f);
			fread(&h1_bfReserved2,sizeof(h1_bfReserved2),1,f);
			fread(&h1_bfOffBits,sizeof(h1_bfOffBits),1,f);
			fread(&h2_biSize,sizeof(h2_biSize),1,f);
			fread(&h2_biWidth,sizeof(h2_biWidth),1,f);
			fread(&h2_biHeight,sizeof(h2_biHeight),1,f);
			fread(&h2_biPlanes,sizeof(h2_biPlanes),1,f);
			fread(&h2_biBitCount,sizeof(h2_biBitCount),1,f);
			fread(&h2_biCompression,sizeof(h2_biCompression),1,f);
			fread(&h2_biSizeImage,sizeof(h2_biSizeImage),1,f);
			fread(&h2_biXPelsPerMeter,sizeof(h2_biXPelsPerMeter),1,f);
			fread(&h2_biYPelsPerMeter,sizeof(h2_biYPelsPerMeter),1,f);
			fread(&h2_biClrUsed,sizeof(h2_biClrUsed),1,f);
			fread(&h2_biClrImportant,sizeof(h2_biClrImportant),1,f);

			// printf("CHAR\"%c\"%d: ",t,t);
			// printf("bfType=%d, ",h1_bfType);
			// printf("bfSize=%ld, ",h1_bfSize);
			// printf("bfOffBits=%ld, ",h1_bfOffBits);
			// printf("Width=%ld,",h2_biWidth);
			// printf("Height=%ld, ",h2_biHeight);
			// printf("Bits Depth=%d, ",h2_biBitCount);
			// printf("Compression=%ld\n",h2_biCompression);

			font_width[t] = h2_biWidth;
			font_height[t] = h2_biHeight;
			fseek(f,h1_bfOffBits,SEEK_SET);
			for(y = 0;y < h2_biHeight;y++)
				{
				xcount = 0;
				for(x = 0;x < h2_biWidth;x++)
					{
					fread(&b,1,1,f);
					fread(&g,1,1,f);
					fread(&r,1,1,f);
					if((r == 0) && (g == 0) && (b == 0))
						{
						font_face[t][x][y] = 1;
						}
					xcount += 3;
					}
crap:		if((xcount % 4) != 0)
					{
					xcount++;
					fread(&r,1,1,f);
					goto crap;
					}
				}
			fclose(f);
			}
		}
	}



// *****   save_bitmap_bmp()   -   Bitmap speichern   *****

void save_bitmap_bmp(char filename[])
	{
	FILE								*f;
	unsigned short int	h1_bfType;
	unsigned long int		h1_bfSize;
	unsigned short int	h1_bfReserved1;
	unsigned short int	h1_bfReserved2;
	unsigned long int		h1_bfOffBits;
	unsigned long int		h2_biSize;
	signed long int			h2_biWidth;
	signed long int			h2_biHeight;
	unsigned short int	h2_biPlanes;
	unsigned short int	h2_biBitCount;
	unsigned long int		h2_biCompression;
	unsigned long int		h2_biSizeImage;
	signed long int			h2_biXPelsPerMeter;
	signed long int			h2_biYPelsPerMeter;
	unsigned long int		h2_biClrUsed;
	unsigned long int		h2_biClrImportant;
	signed short int		x;
	signed short int		y;
	unsigned short int	xcount;
	unsigned char nulli = 0;




	h1_bfType = 19778;																		// BMP magic number "BM"
	h1_bfSize = 54 + STRIPE_X * STRIPE_Y * 3;							// Filesize (Padding habe ich mal unterschlagen)
	h1_bfReserved1 = 0;																		// immer 0
	h1_bfReserved2 = 0;																		// immer 0
	h1_bfOffBits = 54;																		// Lnge dieser Prambel
	h2_biSize = 40;																				// Konstante 40
	h2_biWidth = STRIPE_X;																// Bildbreite
	h2_biHeight = STRIPE_Y;																// Bildhhe
	h2_biPlanes = 1;																			// Anzahl Farbebenen
	h2_biBitCount = 24;																		// Farbtiefe
	h2_biCompression = 0;																	// 0 ist unkomprimiert
	h2_biSizeImage = 0;
	h2_biXPelsPerMeter = 0;
	h2_biYPelsPerMeter = 0;
	h2_biClrUsed = 0;
	h2_biClrImportant = 0;

	f = fopen(filename,"wb");
	if(f != NULL)
		{
		fwrite(&h1_bfType,sizeof(h1_bfType),1,f);
		fwrite(&h1_bfSize,sizeof(h1_bfSize),1,f);
		fwrite(&h1_bfReserved1,sizeof(h1_bfReserved1),1,f);
		fwrite(&h1_bfReserved2,sizeof(h1_bfReserved2),1,f);
		fwrite(&h1_bfOffBits,sizeof(h1_bfOffBits),1,f);
		fwrite(&h2_biSize,sizeof(h2_biSize),1,f);
		fwrite(&h2_biWidth,sizeof(h2_biWidth),1,f);
		fwrite(&h2_biHeight,sizeof(h2_biHeight),1,f);
		fwrite(&h2_biPlanes,sizeof(h2_biPlanes),1,f);
		fwrite(&h2_biBitCount,sizeof(h2_biBitCount),1,f);
		fwrite(&h2_biCompression,sizeof(h2_biCompression),1,f);
		fwrite(&h2_biSizeImage,sizeof(h2_biSizeImage),1,f);
		fwrite(&h2_biXPelsPerMeter,sizeof(h2_biXPelsPerMeter),1,f);
		fwrite(&h2_biYPelsPerMeter,sizeof(h2_biYPelsPerMeter),1,f);
		fwrite(&h2_biClrUsed,sizeof(h2_biClrUsed),1,f);
		fwrite(&h2_biClrImportant,sizeof(h2_biClrImportant),1,f);

		fseek(f,h1_bfOffBits,SEEK_SET);

		for(y = 0;y < h2_biHeight;y++)
			{
			xcount = 0;
			for(x = 0;x < h2_biWidth;x++)
				{
				fwrite(&stripe_b[x][y],1,1,f);
				fwrite(&stripe_g[x][y],1,1,f);
				fwrite(&stripe_r[x][y],1,1,f);
				xcount += 3;
				}
crap:	if((xcount % 4) != 0)
				{
				xcount++;
				fwrite(&nulli,1,1,f);
				goto crap;
				}
			}
		fclose(f);
		}
	}



// *****   hex2dez(h,l)   -   Hexadezimalzahl in dezimal umwandeln   *****

unsigned char hex2dez(unsigned char h, unsigned char l)
	{
	unsigned char rw = 0;
	unsigned char a[20] = "0123456789ABCDEF";
	unsigned char t;

	for(t = 0;t < 16;t++)
		{
		if(l == a[t]) rw += t;
		if(h == a[t]) rw += t * 16;
		}

	return(rw);
	}



// *****   rendere_string()   -   String in Bitmap schreiben   *****

signed short int rendere_string(signed short int x,signed short int y,char blatext[],unsigned char color_r,unsigned char color_g,unsigned char color_b)
	{
	signed short int rw = 0;
	signed short int xpos;
	signed short int xlauf;
	signed short int ylauf;
	signed short int t;
	unsigned char z;
	unsigned char h;
	unsigned char l;
	unsigned char r;
	unsigned char g;
	unsigned char b;


	r = color_r;
	g = color_g;
	b = color_b;

	xpos = x;
	for(t = 0;t < strlen(blatext);t++)
		{
		z = blatext[t];
		if (z == '#')
			{
			h = blatext[++t];
			l = blatext[++t];
			r = hex2dez(h,l);
			h = blatext[++t];
			l = blatext[++t];
			g = hex2dez(h,l);
			h = blatext[++t];
			l = blatext[++t];
			b = hex2dez(h,l);
			goto weiter;
			}
		if(font_width[z] == 0) z = 0; // nicht im Font vorhandene Zeichen durch 0 ersetzen
		// printf("writing \"%c\" (ASCII %d) at X=%d/Y=%d\n",z,z,xpos,y);
		for(xlauf = 0;xlauf < font_width[z];xlauf++)
			{
			for(ylauf = 0;ylauf < font_height[z];ylauf++)
				{
				if(font_face[z][xlauf][ylauf] != 0)
					{
					if((x != 0) && (xpos + xlauf > STRIPE_X)) goto ende;
					if(xpos + xlauf >= 0 - font_width[z])
						{
						if((xpos + xlauf < STRIPE_X) && (y + ylauf < STRIPE_Y) && (xpos + xlauf >= 0) && (y + ylauf >= 0))
							{
							stripe_r[xpos + xlauf][y + ylauf] = r;
							stripe_g[xpos + xlauf][y + ylauf] = g;
							stripe_b[xpos + xlauf][y + ylauf] = b;
							}
						}
					}
				}
			}
		xpos += font_width[z];
weiter: ;
		}
	rw = xpos;
ende:
	return(rw);
	}


// *****   rendere_long_string()   -   String in Bitmap schreiben   *****

signed short int rendere_long_string(signed short int x,signed short int y,char blatext[],unsigned char color_r,unsigned char color_g,unsigned char color_b)
	{
	signed short int rw = 0;
	signed short int xpos;
	signed short int xlauf;
	signed short int ylauf;
	signed short int t;
	unsigned char z;
	unsigned char h;
	unsigned char l;
	unsigned char r;
	unsigned char g;
	unsigned char b;


	r = color_r;
	g = color_g;
	b = color_b;

	xpos = x;
	for(t = 0;t < strlen(blatext);t++)
		{
		z = blatext[t];
		if (z == '#')
			{
			h = blatext[++t];
			l = blatext[++t];
			r = hex2dez(h,l);
			h = blatext[++t];
			l = blatext[++t];
			g = hex2dez(h,l);
			h = blatext[++t];
			l = blatext[++t];
			b = hex2dez(h,l);
			goto weiter;
			}
		if(font_width[z] == 0) z = 0; // nicht im Font vorhandene Zeichen durch 0 ersetzen
		// printf("writing \"%c\" (ASCII %d) at X=%d/Y=%d\n",z,z,xpos,y);
		for(xlauf = 0;xlauf < font_width[z];xlauf++)
			{
			for(ylauf = 0;ylauf < font_height[z];ylauf++)
				{
				if(font_face[z][xlauf][ylauf] != 0)
					{
					if((x != 0) && (xpos + xlauf > LONG_STRIPE_X)) goto ende;
					if(xpos + xlauf >= 0 - font_width[z])
						{
						if((xpos + xlauf < LONG_STRIPE_X) && (y + ylauf < STRIPE_Y) && (xpos + xlauf >= 0) && (y + ylauf >= 0))
							{
							long_stripe_r[xpos + xlauf][y + ylauf] = r;
							long_stripe_g[xpos + xlauf][y + ylauf] = g;
							long_stripe_b[xpos + xlauf][y + ylauf] = b;
							}
						}
					}
				}
			}
		xpos += font_width[z];
weiter: ;
		}
	rw = xpos;
ende:
	return(rw);
	}




// *****   open_bitmap()   -   Bild laden   *****

signed long int open_bitmap(char filename[])
	{
	FILE *f;
	signed long int rw = -99;

	unsigned short int	h1_bfType;
	unsigned long int		h1_bfSize;
	unsigned short int	h1_bfReserved1;
	unsigned short int	h1_bfReserved2;
	unsigned long int		h1_bfOffBits;
	unsigned long int		h2_biSize;
	signed long int			h2_biWidth;
	signed long int			h2_biHeight;
	unsigned short int	h2_biPlanes;
	unsigned short int	h2_biBitCount;
	unsigned long int		h2_biCompression;
	unsigned long int		h2_biSizeImage;
	signed long int			h2_biXPelsPerMeter;
	signed long int			h2_biYPelsPerMeter;
	unsigned long int		h2_biClrUsed;
	unsigned long int		h2_biClrImportant;
	unsigned char				r;
	unsigned char				g;
	unsigned char				b;
	signed short int		x;
	signed short int		y;
	unsigned long int		pos;



	f = fopen(filename,"rb");
	if (f != NULL)
		{
		//printf("sizeof(h1):%d\n",sizeof(h1));
		printf("ffne Bitmap File \"%s\"\n",filename);

		fread(&h1_bfType,sizeof(h1_bfType),1,f);
		fread(&h1_bfSize,sizeof(h1_bfSize),1,f);
		fread(&h1_bfReserved1,sizeof(h1_bfReserved1),1,f);
		fread(&h1_bfReserved2,sizeof(h1_bfReserved2),1,f);
		fread(&h1_bfOffBits,sizeof(h1_bfOffBits),1,f);

		fread(&h2_biSize,sizeof(h2_biSize),1,f);
		fread(&h2_biWidth,sizeof(h2_biWidth),1,f);
		fread(&h2_biHeight,sizeof(h2_biHeight),1,f);
		fread(&h2_biPlanes,sizeof(h2_biPlanes),1,f);
		fread(&h2_biBitCount,sizeof(h2_biBitCount),1,f);
		fread(&h2_biCompression,sizeof(h2_biCompression),1,f);
		fread(&h2_biSizeImage,sizeof(h2_biSizeImage),1,f);
		fread(&h2_biXPelsPerMeter,sizeof(h2_biXPelsPerMeter),1,f);
		fread(&h2_biYPelsPerMeter,sizeof(h2_biYPelsPerMeter),1,f);
		fread(&h2_biClrUsed,sizeof(h2_biClrUsed),1,f);
		fread(&h2_biClrImportant,sizeof(h2_biClrImportant),1,f);

		printf("bfType=%d\n",h1_bfType);
		printf("bfSize=%ld\n",h1_bfSize);
		printf("bfOffBits=%ld\n",h1_bfOffBits);

		printf("Width=%ld\n",h2_biWidth);
		printf("Height=%ld\n",h2_biHeight);
		printf("Bits Depth=%d\n",h2_biBitCount);
		printf("Compression=%ld\n",h2_biCompression);

		pos = h1_bfOffBits;
		fseek(f,pos,SEEK_SET);
		for(y = 0;y < STRIPE_Y;y++)
			{
			for(x = 0;x < h2_biWidth /*STRIPE_X*/ ;x++)
				{
				// fread(&c,1,1,f);
				fread(&b,1,1,f);
				fread(&g,1,1,f);
				fread(&r,1,1,f);
				stripe_r[x][y] = r;
				stripe_g[x][y] = g;
				stripe_b[x][y] = b;
				pos += 3;
				}
			}
		printf("pos=%ld\n",pos);
		fclose(f);

		f = fopen("out.txt","wb");
		for(y = STRIPE_Y - 1;y >= 0;y--)
			{
			for(x = 0;x < STRIPE_X;x++)
				{
				if(stripe_r[x][y]> 0)
					{
					fprintf(f,"R");
					}
				else
					{
					fprintf(f," ");
					}
				if(stripe_g[x][y]> 0)
					{
					fprintf(f,"G");
					}
				else
					{
					fprintf(f," ");
					}
				if(stripe_b[x][y]> 0)
					{
					fprintf(f,"B");
					}
				else
					{
					fprintf(f," ");
					}
				}
			fprintf(f,"\n");
			}
		fclose(f);

		rw = 1;
		}
	else
		{
		printf("Konnte File \"%s\" nicht ffnen.\n",filename);
		rw = -1;
		}

	return(rw);
	}




// *****   save_bitmap_txt(name)   -   Ausgabe der Bitmap als Textdatei   *****

void save_bitmap_txt(char filename[])
	{
	FILE *f;
	signed short int		x;
	signed short int		y;

	f = fopen(filename,"wb");
	for(y = STRIPE_Y - 1;y >= 0;y--)
		{
		for(x = 0;x < STRIPE_X;x++)
			{
			if(stripe_r[x][y]> 0)
				{
				fprintf(f,"R");
				}
			else
				{
				fprintf(f," ");
				}
			if(stripe_g[x][y]> 0)
				{
				fprintf(f,"G");
				}
			else
				{
				fprintf(f," ");
				}
			if(stripe_b[x][y]> 0)
				{
				fprintf(f,"B");
				}
			else
				{
				fprintf(f," ");
				}
			}
		fprintf(f,"\n");
		}
	fclose(f);
	}



// *****   save_bitmap_fpga(name)   -   Ausgabe der Bitmap als FPGA-Steuerdatei   *****
//
// #epd - enable direct pixel linear set mode
//        Start ab Koordinate x,y (x = ACCU3, y = 0)
//        Wenn der direct pixel linear set mode aktiv ist, knnen beliebig viele Pixel modifiziert werden mit:
//        rr gg bb \CR (ascii, mit oder ohne blanks)
//        Die Pixel werden Spaltenweise (y=min bis y=max) geschrieben. Nach jeder vollen Spalte wird die nchste Reihe selektiert und so weiter...
//        ENDE mit #dpx

void save_bitmap_fpga(char filename[])
	{
	FILE *f;
	signed short int		x;
	signed short int		y;

	f = fopen(filename,"wb");
	fprintf(f,"#clr\r");																	// Display lschen
	fprintf(f,"#ldc\r00000000\r");												// Akku 3 mit 0 vorbelexen (X = 0)
	fprintf(f,"#epd\r");																	// Direct Pixel Mode aktivieren

	for(x = 0;x < STRIPE_X;x++)
		{
		for(y = STRIPE_Y - 1;y >= 0;y--)
			{
			fprintf(f,"%02x%02x%02x\r",stripe_r[x][y],stripe_g[x][y],stripe_b[x][y]);
			}
		}
	fprintf(f,"#dpx\r");																	// Direct Pixel Mode beenden
	fprintf(f,"#mrf\r");																	// manueller Refresh
	fclose(f);
	}


// *****   rs232_open()   -   seriellen Port ffnen   *****

int rs232_open(void)
	{
	int fd;
	struct termios options;


	memset((void *)&options,0,sizeof(options));

	fd = open("/dev/ttyS1",O_RDWR | O_NOCTTY |O_SYNC);
	if( fd < 0) goto ende;


	tcdrain(fd);

	// Comport-Parameter einstellen
  options.c_cflag = B460800 | CS8 | CLOCAL | CREAD;
  // options.c_cflag &= ~(PARENB | CSTOPB);

	options.c_cflag &= ~PARENB;	// keine Parit?tbits
	// options.c_cflag &= ~CSTOPB;	// nur ein Stop-bit
	options.c_cflag |=  CSTOPB;	// 2 Stopbits
	

  options.c_iflag |= (IGNPAR | ECHOCTL | ECHOKE);
  options.c_lflag &= ~(ICANON | OPOST| ECHO | ECHOE | ISIG | IEXTEN | ECHOK | ONLCR |IXON);
  options.c_cc[VMIN]  = 0;
  options.c_cc[VTIME]  = 0;
  tcsetattr(fd,TCSAFLUSH,&options);

ende:
	return(fd);
	}



// *****   save_bitmap_fpga2(name)   -   Ausgabe der Bitmap als FPGA-Steuerdatei   *****
//
// #epb - enable direct pixel linear set, binary
//        Start ab Koordinate x,y (x = ACCU3, y = 0)
//        Erwartet n Pixel in binrer Form (n*3 bytes) (n = ACCU1)
//        char(r);char(g);char(b) = 24bit per Pixel
//        Die Pixel werden Spaltenweise (y=min bis y=max) geschrieben. Nach jeder vollen Spalte wird die nchste Reihe selektiert und so weiter...
//        ENDE nach n Pixel oder mit HIGH auf FPGA_CMD_RESET
//        !! ACHTUNG: Modi wird sofort scharf nach \CR !! Ein Folgendes \LF wird bereits als Farbe registriert!!

void save_bitmap_fpga2(char filename[])
	{
	FILE *f;
	signed short int		x;
	signed short int		y;

	f = fopen(filename,"wb");
	fprintf(f,"#clr\r");																	// Display lschen
	fprintf(f,"#ldc\r00000000\r");												// Akku 3 mit 0 vorbelegen (X = 0)
	fprintf(f,"#lda\r%08x\r",STRIPE_X * STRIPE_Y);					// Akku 1 mit Anzahl der Pixels fttern
	fprintf(f,"#epb\r");																	// Direct Pixel Mode aktivieren

	for(x = 0;x < STRIPE_X;x++)
		{
		for(y = STRIPE_Y - 1;y >= 0;y--)
			{
			fprintf(f,"%c%c%c",stripe_r[x][y],stripe_g[x][y],stripe_b[x][y]);
			}
		}
	fprintf(f,"#mrf\r");																	// manueller Refresh
	fclose(f);
	}



// *****   save_bitmap_fpga3(name)   -   Ausgabe der Bitmap als FPGA-Steuerdatei   *****
//
// #epb - enable direct pixel linear set, binary
//        Start ab Koordinate x,y (x = ACCU3, y = 0)
//        Erwartet n Pixel in binrer Form (n*3 bytes) (n = ACCU1)
//        char(r);char(g);char(b) = 24bit per Pixel
//        Die Pixel werden Spaltenweise (y=min bis y=max) geschrieben. Nach jeder vollen Spalte wird die nchste Reihe selektiert und so weiter...
//        ENDE nach n Pixel oder mit HIGH auf FPGA_CMD_RESET
//        !! ACHTUNG: Modi wird sofort scharf nach \CR !! Ein Folgendes \LF wird bereits als Farbe registriert!!

void save_bitmap_fpga3(int ser_p)
	{
	signed short int		x;
	signed short int		y;
	char tmpstr[80];


	// write(ser_p,"#clr\r",5);																	// Display lschen
	fpga_write(ser_p,"#ldc\r00000000\r",14);												// Akku 3 mit 0 vorbelegen (X = 0)
	// sprintf(tmpstr,"#lda\r%08x\r",STRIPE_X * STRIPE_Y);					// Akku 1 mit Anzahl der Pixels fttern
	// write(ser_p,tmpstr,13);					// Akku 1 mit Anzahl der Pixels fttern
	fpga_write(ser_p,"#lda\r00002000\r",14);												// Akku 3 mit 0 vorbelegen (X = 0)
	// write(ser_p,"#fff\r",5);																	// weiss aufblitzen lassen
	// sleep(1);
	fpga_write(ser_p,"#epb\r",5);																	// Direct Pixel Mode aktivieren

	for(x = 0;x < STRIPE_X;x++)
		{
		// printf("x=%d\n",x);
		// for(y = STRIPE_Y - 1;y >= 0;y--)
		for(y = 0;y < STRIPE_Y;y++)
			{
			sprintf(tmpstr,"%c%c%c",stripe_r[x][y],stripe_g[x][y],stripe_b[x][y]);
			// sprintf(tmpstr,"%c%c%c",255,0,0);
			fpga_write(ser_p,tmpstr,3);
			}
		// usleep(1000);
		}
	// write(ser_p,"#mrf\r",5);																	// manueller Refresh
	}







// **********   Demon   **********

int main(int argc,char **argv)
	{
	char ep[80];
	signed long int t;
	char ticker_string[4096];
	unsigned short int xmax;
	unsigned short int long_xmax;
	unsigned short int long_display_right = 51;
	// char video_name[90];	
	int ser_p;
	signed short int x;
	signed short int y;
	char tmpstr[40];
	signed short int wincount;
	FILE *f;
	char z;
	unsigned short int laenge;


	strcpy(log_filename,"/www/reports/stripe_demon.log");	// Logfile-Name setzen
	log_level = 3;																				// Debuglevel setzen
	sprintf(ep,"----------   stripe_demon started (debuglevel=%d)   ----------",log_level);
	log_append(1,ep);																			// Startmeldung ins Logfile schreiben

	gpio_devmem_open();																		// GPIOs oeffnen


	// demon_readconfig(sensordemon_configfile);							// Einstellungen fuer den Sensordemon lesen
	// printf("TimeInterval=%ld\r\n",zeitintervall);

	ser_p = rs232_open();																	// seriellen Port ffnen
	parallel_init();																			// parallelen Port ffnen
	read_font(0);																					// Basis-Schriftart einlesen

	goto hauptschleife;


	strcpy(ticker_string,"Vllig sinn- und nutzenfreier Blind- und Taubtext ohne echten Mehrwert. Perfekt geeignet z.B. zum Einbau in politische Reden oder Parteiprogramme. So richtig 08/15, ohne irgendwelche Gehirnwindungen zu verbiegen !                                                    ");
	// strcpy(ticker_string,"GGQ1.F,383.50,10/11/2010,4:47am,+0.43,384.70,387.00,383.50,118");
	// strcpy(ticker_string,",.yqQfdkgjfdkjgkdfjgkfgds");
	stripe_fill(255,255,255);																		// Schwarzer Hintergrund
	long_stripe_fill(0,0,0);															// Schwarzer Hintergrund
	xmax = rendere_string(0,0,ticker_string,255,255,255);
	long_xmax = rendere_long_string(long_display_right,0,ticker_string,255,255,255);
	printf("long_xmax: %d\n",long_xmax);
	// rendere_string(0,16,"roter UNSINN",255,0,0);
	// rendere_string(255,16,"grner UNSINN",0,255,0);
	// save_bitmap_bmp("/root/out.bmp");
	// save_bitmap_txt("/root/out.txt");
	// t = open_bitmap("/root/grafik.bmp");



	/*
	for(t = 0;t < xmax - STRIPE_X;t++)
		{
		printf("Rendering Frame %ld of %d...\r",t,xmax - STRIPE_X);
		fflush(stdout);
		stripe_fill(0,0,0);																		// Schwarzer Hintergrund
		// rendere_string(0,16,"roter UNSINN",255,0,0);
		// rendere_string(255,16,"grner UNSINN",0,255,0);
		rendere_string(-t,0,ticker_string,255,255,0);
		// sprintf(video_name,"/root/command%05ld.bmp",t);
		// save_bitmap_bmp(video_name);

		// sprintf(video_name,"/root/command%05ld.txt",t);
		// save_bitmap_fpga(video_name);
		// sprintf(video_name,"/root/commandbin%05ld.txt",t);
		// save_bitmap_fpga2(video_name);

		save_bitmap_fpga3(ser_p);
		// return(0);
		}
	*/

	// Frame 0 in Standard-Buffer umkopieren
	for(x = 0;x < STRIPE_X;x++)
		{
		for(y = 0;y < STRIPE_Y;y++)
			{
			stripe_r[x][y] = long_stripe_r[x][y];
			stripe_g[x][y] = long_stripe_g[x][y];
			stripe_b[x][y] = long_stripe_b[x][y];
			}
		}

	fpga_write(ser_p,"#lda\r100\r",9);
	fpga_write(ser_p,"#ins\r",5);
	fpga_write(ser_p,"#ros\r",5);
	fpga_write(ser_p,"#roz\r",5);
	save_bitmap_fpga3(ser_p);															// Frame 0 anzeigen
	fpga_write(ser_p,"#eaa\r",5);
	wincount = long_display_right;

blubb:
	for(t = 1;t < long_xmax;t++)
		{
		printf("%ld\n",t);
		wincount++;
		if(wincount >= STRIPE_X) wincount = 0;

		fpga_write(ser_p,"#epx\r",5);
		for(y = 0;y < STRIPE_Y;y++)
			{
			sprintf(tmpstr,"%04x %04x %02x%02x%02x\r",wincount,y,long_stripe_r[long_display_right + t][y],long_stripe_g[long_display_right + t][y],long_stripe_b[long_display_right + t][y]);
			fpga_write(ser_p,tmpstr,strlen(tmpstr));
			}
		fpga_write(ser_p,"#dpx\r",5);
		fpga_write(ser_p,"#rsl\r",5);					// Display um 1/4 Pixel nach links schieben
		usleep(10000);
		fpga_write(ser_p,"#rsl\r",5);					// Display um 1/4 Pixel nach links schieben
		usleep(10000);
		fpga_write(ser_p,"#rsl\r",5);					// Display um 1/4 Pixel nach links schieben
		usleep(10000);
		fpga_write(ser_p,"#rsl\r",5);					// Display um 1/4 Pixel nach links schieben
		usleep(10000);
		}
goto blubb;
	// printf("\nFertig.\n");


hauptschleife:
	// sqldb_read_string("/control/ticker/display",ticker_string);
	f = fopen("/tmp/display.txt","rb");
	laenge = 0;
	ticker_string[0] = 0;
	if(f != NULL)
		{
lies: if(feof(f)) goto gelesen;
		z = fgetc(f);
		// printf("Zeichen(%d): \"%c\"\n",z,z);
		if(z < 32) goto lies;
		if(z == 255) goto lies;
		ticker_string[laenge++] = z;
		goto lies;
		}
	else
		{
		sleep(1);
		goto hauptschleife;
		}
gelesen:
		ticker_string[laenge] = 0;
		fclose(f);

	long_stripe_fill(0,0,0);															// Schwarzer Hintergrund
	long_xmax = rendere_long_string(long_display_right,0,ticker_string,255,255,255);
	printf("X:%d; \"%s\"...\n",long_xmax,ticker_string);
	// Frame 0 in Standard-Buffer umkopieren
	for(x = 0;x < STRIPE_X;x++)
		{
		for(y = 0;y < STRIPE_Y;y++)
			{
			stripe_r[x][y] = long_stripe_r[x][y];
			stripe_g[x][y] = long_stripe_g[x][y];
			stripe_b[x][y] = long_stripe_b[x][y];
			}
		}

	fpga_write(ser_p,"#lda\r100\r",9);
	fpga_write(ser_p,"#ins\r",5);
	fpga_write(ser_p,"#ros\r",5);
	fpga_write(ser_p,"#roz\r",5);
	// save_bitmap_fpga3(ser_p);															// Frame 0 anzeigen
	fpga_write(ser_p,"#daa\r",5);
	wincount = long_display_right;

	for(t = 1;t < long_xmax;t++)
		{
		// printf("%ld\n",t);
		wincount++;
		if(wincount >= STRIPE_X) wincount = 0;

		fpga_write(ser_p,"#epx\r",5);
		for(y = 0;y < STRIPE_Y;y++)
			{
			sprintf(tmpstr,"%04x %04x %02x%02x%02x\r",wincount,y,long_stripe_r[long_display_right + t][y],long_stripe_g[long_display_right + t][y],long_stripe_b[long_display_right + t][y]);
			fpga_write(ser_p,tmpstr,strlen(tmpstr));
			}
		fpga_write(ser_p,"#dpx\r",5);
		fpga_write(ser_p,"#rsl\r",5);														// Display um 1/4 Pixel nach links schieben
		usleep(DELAY_X);
		fpga_write(ser_p,"#rsl\r",5);														// Display um 1/4 Pixel nach links schieben
		usleep(DELAY_X);
		fpga_write(ser_p,"#rsl\r",5);														// Display um 1/4 Pixel nach links schieben
		usleep(DELAY_X);
		fpga_write(ser_p,"#rsl\r",5);														// Display um 1/4 Pixel nach links schieben
		usleep(DELAY_X);
		}
	fpga_write(ser_p,"#lda\r000000\r",12);
	fpga_write(ser_p,"#col\r",5);
	goto hauptschleife;																		// ... und wieder dasselbe Spiel von vorne


// ende:
	sprintf(ep,"----------   stripe_demon terminated   ----------");
	log_append(1,ep);																			// Endmeldung ins Logfile schreiben
	return(0);
	}

