// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   update_demon.c   -   Dmon zum Steuern deruatomatischen Updates und zum Lschen bergroer Logfiles           **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2011 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************

// **********   Eintrge in der Konfigurations-Datenbank, die vom Demon ausgewertet werden   **********
//
// /control/update_demon/enabled												Update-Demon aktiv/inaktiv
// /control/update_demon/loglevel												Log-Level 0...4
// /control/update_demon/max_logfile_size								max. Gre fr Logfiles (in Bytes) fr Logfile-Truncation /www/reports/*.log



#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <termios.h>
#include <time.h>
#include <sys/time.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "at91gpio.h"
// #include "all_i2c.h"
// #include "i2c_chips.h"
#include "sqldb.h"
#include "xml_lib.h"
#include "update_demon.h"
#include "logging.h"
#include "shm_use.h"



// *****   Globale Variablen   *****

signed short int			update_demon_enabled;
signed long int				max_logfile_size;
signed long int				last_run_timestamp;								// Zeit des letzten Ablaufes
signed long int				next_run_timestamp;								// Zeit des nchsten geplangen Ablaufes



// **********   Demon-Configfile einlesen   **********

void update_demon_readconfig(void)
	{
	log_level = sqldb_read_long("/control/update_demon/loglevel");
	max_logfile_size = sqldb_read_long("/control/update_demon/max_logfile_size");
	update_demon_enabled = sqldb_read_long("/control/update_demon/enabled");
	if (max_logfile_size < 10000) max_logfile_size = 10000;
	}



// **********   Updates testen, downloaden und ausfhren   **********

signed long int perform_update(void)
	{
	signed long int rw;
	char rueckmeldung[30000];
	char ep[30000];
	signed short int sp1;
	signed short int sp2;
	signed short int sp3;
	char update_id[128];
	char request_id[128];
	signed short int t;
	signed short int i;


	sprintf(ep,"checking for updates...");
	log_append(4,ep);																	// Meldung ins Logfile schreiben

	rw = 0;

	// --- berprfen, ob Updates da sind ---
	system("rm /tmp/update.txt 2>/dev/null");
	system("/www/auc.php 1 >/tmp/update.txt 2>/tmp/update.txt");
	readfileinstring("/tmp/update.txt",rueckmeldung);
	if (strlen(rueckmeldung) == 0)
		{
		// Keine Antwort
		log_append(3,"Update-Server antwortet nicht.");
		rw = -1;
		goto ende;
		}

	sprintf(ep,"Rckwert: \"%s\"",rueckmeldung);
	log_append(4,ep);																	// Meldung ins Logfile schreiben

	if(strstr(rueckmeldung,"curl:") > 0)
		{
		// "curl: (7) couldn't connect to host"
		log_append(3,"Update-Server nicht erreichbar.");
		rw = -2;
		}

	if(strstr(rueckmeldung,"ptodate") > 0)
		{
		// "Uptodate"
		log_append(4,"Keine neuen Updates verfgbar.");
		rw = 0;
		}

	rw = 1;


	sp1 = get_zeichen_position(rueckmeldung,';',1);		// 1. Strichpunkt
	if(sp1 < 0)
		{
		rw = -101;
		goto ende;
		}
	sp2 = get_zeichen_position(rueckmeldung,';',2);		// 2. Strichpunkt
	if(sp2 < 0)
		{
		rw = -102;
		goto ende;
		}
	sp3 = get_zeichen_position(rueckmeldung,';',3);		// 3. Strichpunkt
	if(sp3 < 0)
		{
		rw = -103;
		goto ende;
		}
	// sprintf(ep,"SP1:%d, SP2:%d, SP3:%d",sp1,sp2,sp3);
	// log_append(3,ep);

	i = 0;
	for(t = sp1 + 1;t < sp2;t++) update_id[i++] = rueckmeldung[t];
	update_id[i] = 0;	

	i = 0;
	for(t = sp2 + 1;t < sp3;t++) request_id[i++] = rueckmeldung[t];
	request_id[i] = 0;	

	sprintf(ep,"Update_ID:\"%s\", Request_ID:\"%s\"",update_id,request_id);
	log_append(3,ep);
	sprintf(ep,"update_demon: installing Update_ID:\"%s\", Request_ID:\"%s\"...",update_id,request_id);
	log_append(-1,ep);																// Meldung auf der Konsole ausgeben


	// --- Update Downloaden ---
	sprintf(ep,"Versuche Download von Update_ID:\"%s\" mit Request_ID:\"%s\"...",update_id,request_id);
	log_append(3,ep);
	system("rm /tmp/update.txt 2>/dev/null");
	sprintf(ep,"/www/auc.php 2 %s %s >/tmp/update.txt 2>/tmp/update.txt",update_id,request_id);
	system(ep);
	readfileinstring("/tmp/update.txt",rueckmeldung);
	if (strlen(rueckmeldung) == 0)
		{
		// Keine Antwort
		rw = -200;
		goto ende;
		}

	sprintf(ep,"Download Rckwert: \"%s\"",rueckmeldung);
	log_append(4,ep);																	// Meldung ins Logfile schreiben


	// --- Update auspacken ---
	sprintf(ep,"Versuche, Update_ID:\"%s\" mit Request_ID:\"%s\" auszupacken...",update_id,request_id);
	log_append(3,ep);
	system("rm /tmp/update.txt 2>/dev/null");
	sprintf(ep,"/www/auc.php 8 %s >/tmp/update.txt 2>/tmp/update.txt",request_id);
	system(ep);
	readfileinstring("/tmp/update.txt",rueckmeldung);
	if (strlen(rueckmeldung) == 0)
		{
		// Keine Antwort
		rw = -300;
		goto ende;
		}

	sprintf(ep,"Untar Rckwert: \"%s\"",rueckmeldung);
	log_append(4,ep);																	// Meldung ins Logfile schreiben


	// --- Update ausfhren ---
	sprintf(ep,"Request_ID:\"%s\" soll ausgefhrt werden...",request_id);
	log_append(3,ep);
	system("rm /tmp/update.txt 2>/dev/null");
	sprintf(ep,"/www/auc.php 9 %s >/tmp/update.txt 2>/tmp/update.txt",request_id);
	system(ep);
	readfileinstring("/tmp/update.txt",rueckmeldung);
	if (strlen(rueckmeldung) == 0)
		{
		// Keine Antwort
		rw = -400;
		goto ende;
		}

	sprintf(ep,"Ausfhrung Rckwert: \"%s\"",rueckmeldung);
	log_append(4,ep);																	// Meldung ins Logfile schreiben


	// --- erfolgreiches Update besttigen ---
	sprintf(ep,"Request_ID:\"%s\" soll besttigt werden...",request_id);
	log_append(3,ep);
	system("rm /tmp/update.txt 2>/dev/null");
	sprintf(ep,"/www/auc.php 7 %s >/tmp/update.txt 2>/tmp/update.txt",request_id);
	system(ep);
	readfileinstring("/tmp/update.txt",rueckmeldung);
	if (strlen(rueckmeldung) == 0)
		{
		// Keine Antwort
		rw = -500;
		goto ende;
		}

	sprintf(ep,"Besttigung Rckwert: \"%s\"",rueckmeldung);
	log_append(4,ep);																	// Meldung ins Logfile schreiben
	rw = 1;																						// Update erfolgreich durchgefhrt !

ende:
	sprintf(ep,"return value from update: %ld",rw);
	log_append(3,ep);																	// Meldung ins Logfile schreiben
	if (rw != -101)
		{
		sprintf(ep,"update_demon: return value from update: %ld",rw);
		log_append(-1,ep);															// Meldung auf der Konsole ausgeben
		}
	return(rw);
	}



// *****   truncate_logfiles()   -   Logfiles krzen   *****

void truncate_logfiles(signed long int max_size_bytes)
	{
	char ep[1024];
	signed long int freed_bytes = 0;
	FILE *f;
	FILE *p;
	char puffer[50000];
	char filename[80];
	signed long int position;
	signed short int files = 0;
	signed long int laenge;
	signed long int RESERVE = 2000;


	sprintf(ep,"Logfile Truncator active (max. Size: %ld Bytes)",max_size_bytes);
	log_append(4,ep);																			// Startmeldung ins Logfile schreiben

	system("ls -1 /www/reports/*.log > /tmp/logfiles.txt");
	f = fopen("/tmp/logfiles.txt","r");
	if(f != NULL)
		{
schleiferl:
		if(feof(f)) goto ende;
		fscanf(f,"%s",filename);
		files++;
		p = fopen(filename,"r+");
		if (p != NULL)
			{
			fseek(p,0L,SEEK_END);
			laenge = ftell(p);
			if (laenge > max_size_bytes)
				{
				sprintf(ep,"File: \"%s\" size=%ld Bytes - truncating to %ld Bytes",filename,laenge,max_size_bytes - RESERVE);
				log_append(4,ep);
				freed_bytes += laenge - (max_size_bytes - RESERVE);
				position = laenge - max_size_bytes + RESERVE;
				fseek(p,position,SEEK_SET);
				fread(puffer,1,max_size_bytes - RESERVE,p);
				fclose(p);
				p = fopen(filename,"wb");
				if (p != NULL)
					{
					fprintf(p,"--------------------   Logfile truncated to max. Size of %ld Bytes   --------------------\n",max_size_bytes);
					fwrite(puffer,1,max_size_bytes - RESERVE,p);
					fclose(p);
					}
				}	
			else
				{
				fclose(p);
				}
			}
		goto schleiferl;
ende:
		fclose(f);
		}
	sprintf(ep,"Logfile Truncator finished (%d files analyzed, %ld Bytes freed)",files,freed_bytes);
	log_append(4,ep);																			// Startmeldung ins Logfile schreiben
	}



// *****   get_zeichen_position(string,zeichen,n) - Ermitteln der Position des n-ten Vorkommens eines Zeichens   *****

signed short int get_zeichen_position (char string[],char zeichen,signed short int n)
	{
	signed short int rw;
	signed short int t;
	signed short int i;


	if(strlen(string) < 1)
		{
		rw = -1;																						// leerer String
		goto ende;
		}

	if(n < 1)
		{
		rw = -2;																						// Position muss mindestens 1 sein
		goto ende;
		}


	i = 0;
	for(t = 0;t < strlen(string);t++)
		{
		if(string[t] == zeichen)
			{
			i++;																							// Fundstellenzhler erhhen
			if (i == n)
				{
				rw = t;																					// Fundstelle zurckmelden
				goto ende;
				}
			}
		}
	rw = -3;																							// nicht gefunden

ende:
	return(rw);
	}



// *****   Hauptschleife   *****

int main(int argc,char **argv)
	{
	char									ep[1024];												// Puffer fr Logfile-Meldungen
	struct tm							*tblock;
	signed long int				t;
	signed long int				ti = 0;


	// auf existierendes Shared Memory fr Demonstatus zugreifen
	if (use_my_demonstatus_shm(9) != 1) goto errorende;		// MY_DEMON_ID = 9: update_demon

	// Status von Demon in SHM eintragen
	my_demonstatus_data[my_demon_id].demon_id = my_demon_id;														// 0 ist Default, hier wird die eigene ID eingetragen
	sprintf(my_demonstatus_data[my_demon_id].name_string,"update_demon");								// Name
	sprintf(my_demonstatus_data[my_demon_id].version_string,"%s",UPDATE_DEMON_VERSION);	// Versionsnummer
	my_demonstatus_data[my_demon_id].last_boot = time(0);																// Timestamp(UTC) des Starts
	my_demonstatus_data[my_demon_id].last_activity = time(0);														// Timestamp(UTC) der letzten Aktion
	my_demonstatus_data[my_demon_id].ti = ti;																						// Counter
	sprintf(my_demonstatus_data[my_demon_id].debug_message,"Start");										// letzte Debug-Meldung
	my_demonstatus_data[my_demon_id].error_message[0] = 0;															// letzte Fehlermeldung
	my_demonstatus_data[my_demon_id].status_long = 0;																		// numerischer Status
	sprintf(my_demonstatus_data[my_demon_id].status_string,"startup");									// String-Status

	// Logging-System initialisieren
	strcpy (log_filename,"/www/reports/update_demon.log"); // Logfilenamen setzen
	log_level = sqldb_read_long("/control/update_demon/loglevel");
	// log_kill();																						// Logfile loeschen
	sprintf(ep,"----------   update_demon V %s started (debuglevel=%d)   ----------",UPDATE_DEMON_VERSION,log_level);
	log_append(1,ep);																			// Startmeldung ins Logfile schreiben

	sprintf(ep,"update demon V %s started.",UPDATE_DEMON_VERSION);
	log_append(-1,ep);																		// Meldung auf der Konsole ausgeben

	update_demon_readconfig();														// Einstellungen fr lokalen Demon

	printf("update demon V %s started.\n",UPDATE_DEMON_VERSION);


	last_run_timestamp = time(NULL);											// now() merken
	next_run_timestamp = last_run_timestamp + 14;					// erster Update-Versuch nach 14 Sekunden
	tblock = localtime(&next_run_timestamp);
	sprintf(ep,"Next Check for Updates: %02d.%02d.%04d %02d:%02d:%02d",tblock->tm_mday,tblock->tm_mon + 1,tblock->tm_year+1900,tblock->tm_hour,tblock->tm_min,tblock->tm_sec);
	log_append(3,ep);																			// Startmeldung ins Logfile schreiben

	update_demondebug("running");													// letzte Debug-Meldung
hauptschleife:
	update_demonstatus(ti,1,"main loop");									// Status von Demon in SHM aktualisieren
	ti++;
	sleep(1);																							// 1 Sekunde Rechenzeit freigeben
	if (time(NULL) >= next_run_timestamp)
		{
		truncate_logfiles(max_logfile_size);								// Logfiles prfen und ggfs. auf maximale Gre krzen
		update_demon_enabled = sqldb_read_long("/control/update_demon/enabled");
		if(update_demon_enabled > 0)
			{
			update_demondebug("checking");										// letzte Debug-Meldung
			sprintf(ep,"*** UPDATE-Check started ***");
			log_append(4,ep);																	// Meldung ins Logfile schreiben
murmeltier:
			update_demondebug("installing");									// letzte Debug-Meldung
			t = perform_update();															// Anzahl Updates prfen und das erste davon ausfhren
			if (t > 0) goto murmeltier;												// solange wiederholen, bis keine Updates mehr da sind
			}
		else
			{
			update_demonstatus(ti,20,"sleeping");							// Status von Demon in SHM aktualisieren
			update_demondebug("deactivated");									// letzte Debug-Meldung
			sleep(5);
			}
		last_run_timestamp = time(NULL);										// now() merken
		// next_run_timestamp = last_run_timestamp + 10;
		next_run_timestamp = last_run_timestamp + rand() % 8192 + 3600 * 24; // 24 Stunden + 0...8192 Sekunden
		tblock = localtime(&next_run_timestamp);
		sprintf(ep,"Next Check for Updates: %02d.%02d.%04d %02d:%02d:%02d",tblock->tm_mday,tblock->tm_mon + 1,tblock->tm_year+1900,tblock->tm_hour,tblock->tm_min,tblock->tm_sec);
		log_append(4,ep);																		// Meldung ins Logfile schreiben
		}
	else
		{
		// sprintf(ep,"Noch %ld Sekunden bis zum Check",next_run_timestamp - time(NULL));
		// log_append(4,ep);																		// Meldung ins Logfile schreiben
		sprintf(ep,"%ld sec.",next_run_timestamp - time(NULL));
		update_demondebug(ep);
		}

	goto hauptschleife;

	return(0);

errorende:
	return(-1);
	}
