// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   xml_lib.c   -   Tags aus XML-Dateien auslesen und diese auch hineinschreiben                                 **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2009 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include "xml_lib.h"


signed short int		iteration_depth = 0;



// *****   readfileinstring(filename,stringposition)   -   Datei in den Speicher holen   *****

signed long int readfileinstring(char filename[],char *stringposition)
	{
	FILE *f;
	signed long int zcount = 0;
	signed long int rw = -1;
	signed int x;



	f =  fopen(filename,"rb");
	if (f == NULL)
		{
		// printf("ERROR: readfileinstring() could not open file \"%s\" !\n",filename);
		rw = -2;
		goto ende;
		}

	zcount = 0;
schleife:
	x = fgetc(f);
	if (x == EOF)
		{
		rw = zcount;
		stringposition[zcount] = 0;
		goto ende;
		}
	stringposition[zcount] = x;
	zcount++;
	goto schleife;

ende:
	// printf ("%ld Bytes read.\r\n",zcount);
	if (f != NULL) fclose(f);
	return(rw);
	}



// *****   parse_xml_recursive()   -   ein XML-Tag mit rekursiver Sektionssuche lesen   *****

signed long int parse_xml_recursive (char xml[],char feldname[],char *stringposition)
	{
	signed long int rw = -1;
	signed long int t;
	char tagname[255];
	char anfangstag[255];
	char xmlneu[XML_MAX_MEMSIZE];

	
	iteration_depth++;																		// Iterationstiefe erhhen
	if (iteration_depth > 10)
		{
		printf("maximum iteration depth reached - aborting !\n");
		rw = -1;
		goto ende;
		}
	// printf("--------------------------------depth=%d, tagname=\"%s\"\n",depth,feldname);
	// printf("%s\n---------------------------------------------------\n",xml);

	if (strlen(xml) < 8)																	// Lnge des XML-Rests auf Plausibilitt prfen
		{
		stringposition[0] = 0;															// leeren String zurckgeben
		rw = -1;
		goto ende;
		}

	if (strlen(feldname) < 1)															// Lnge des Feldnamens auf Plausibilitt prfen
		{
		stringposition[0] = 0;															// leeren String zurckgeben
		rw = -1;
		goto ende;
		}

	for (t = 0; t < strlen(feldname) - 1;t++)							// Prfen, ob Sektionen vorhanden sind
		{
		if(feldname[t] == '.') goto punkt;									// Ja, es muss also eine Stufe weitergebohrt werden
		}

	// printf("Hier mte jetzt das Tag ausgelesen werden !\n");
	rw = parse_xml(xml,feldname,stringposition);					// Tag auslesen
	goto ende;

punkt:
	strcpy(anfangstag,feldname);													// Tagname umkopieren
	anfangstag[t] = 0;																		// und auf oberste Ebene begrenzen
	// printf("-- Anfangstag = \"%s\" --\n",anfangstag);
	strcpy(tagname,feldname + t + 1);											// Rest des Strings ist der neue Tagname
	parse_xml(xml,anfangstag,xmlneu);											// Rest-XML ist der Inhalt des Anfangstags
	iteration_depth = 0;																	// Iterations-Counter zurckstellen
	rw = parse_xml_recursive(xmlneu,tagname,stringposition); // hiermit eine Ebene tiefer einsteigen

ende:
	return(rw);
	}



// *****   parse_xml(xml,feldname,stringposition)   -   ein XML-Tag lesen   *****

signed long int parse_xml(char xml[],char feldname[],char *stringposition)
	{
	char *tag_anfang;
	char *tag_ende;
	signed long int rw = -1;
	char label_anfang[80] = "<";
	char label_ende[80] = "</";
	signed long int laenge_anfang;
	signed long int laenge;
	signed long int i;
	signed long int pos_anfang;



	strcat(label_anfang,feldname);
	strcat(label_anfang,">");
	strcat(label_ende,feldname);
	strcat(label_ende,">");



	tag_anfang = strstr(xml,label_anfang);
	if (tag_anfang == NULL)
		{
		rw = -2;
		stringposition[0] = 0;
		goto ende;
		}

	tag_ende = strstr(xml,label_ende);
	if (tag_ende == NULL)
		{
		rw = -3;
		stringposition[0] = 0;
		goto ende;
		}

	laenge_anfang = strlen(label_anfang);
	laenge = tag_ende -(tag_anfang + laenge_anfang);
	pos_anfang = tag_anfang - &xml[0];
	if (laenge > 0)
		{
		for (i = 0;i < laenge;i++)
			{
			stringposition[i] = xml[pos_anfang + laenge_anfang + i];
			}
		stringposition[laenge] = 0;
		rw = laenge;
		}
	else
		{
		rw = 0;
		stringposition[0] = 0;
		goto ende;
		}

ende:
	return(rw);
	}



// *****   read_xml() - XML-Dateien auslesen   *****

signed long int read_xml(char xml_filename[],char section_and_tagname[],char *stringposition)
	{
	int rw = 0;																						// Rckwert
	char xml_dateiinhalt[XML_MAX_MEMSIZE];								// Puffer fr die Datei im Speicher
	signed long int t;

	t = readfileinstring(xml_filename,xml_dateiinhalt);
	if (t < 1)
		{
		printf("ERROR: File \"%s\" konnte nicht gelesen werden oder ist leer.\n",xml_filename);
		rw = -1;
		goto ende;
		}

	iteration_depth = 0;																	// Iterations-Counter zurckstellen
	rw = parse_xml_recursive (xml_dateiinhalt,section_and_tagname,stringposition);
ende:
	return(rw);

	/*
	int rw = 0;																						// Rckwert
	char xml_dateiinhalt[XML_MAX_MEMSIZE];								// Puffer fr die Datei im Speicher
	char sektionsname[80];																// cmdline-Parameter Name des Tags
	char rawtagname[80];																	// Tagname ohne Sektionsprefix
	char sektion_dateiinhalt[XML_MAX_MEMSIZE];						// Puffer fr die Sektion
	signed long int t;
	char *punkt_anfang;
	long int pos_punkt;
	long int i;


	punkt_anfang = strstr(section_and_tagname,".");
	if (punkt_anfang == NULL)
		{
		// keine Sektion gefunden
		sektionsname[0] = 0;
		strcpy (rawtagname,section_and_tagname);
		}
	else
		{
		// Punkt gefunden, also Sektion ermitteln
		pos_punkt = punkt_anfang - &section_and_tagname[0]; // Speicherposition in Stringposition umrechnen
		section_and_tagname[pos_punkt] = 0;								// Punkt durch Stringterminator ersetzen
		strcpy(sektionsname,section_and_tagname);					// Anfang ist die Sektion
		for (i = 0;i < 80;i++)
			{
			rawtagname[i] = section_and_tagname[++pos_punkt];
			if (rawtagname[i] == 0) break;
			}
		}


	t = readfileinstring(xml_filename,xml_dateiinhalt);
	if (t < 1)
		{
		// printf("ERROR: File \"%s\" konnte nicht gelesen werden oder ist leer.\n",xml_filename);
		rw = -1;
		goto ende;
		}


	if (sektionsname[0] != 0)
		{
		parse_xml(xml_dateiinhalt,sektionsname,sektion_dateiinhalt);
		t = strlen(sektion_dateiinhalt);
		if (t != 0)
			{
			parse_xml(sektion_dateiinhalt,rawtagname,stringposition);
			}
		else
			{
			stringposition[0] = 0;														// Sektion nicht gefunden, also Result leer
			}
		}
	else
		{
		parse_xml(xml_dateiinhalt,rawtagname,stringposition);
		}
	rw = strlen(stringposition);
	goto ende;


ende:
	return(rw);
*/
	}



// *****   write_xml() - in XML-Dateien schreiben   *****

signed long int write_xml(char xml_filename[],char section_and_tagname[],char daten[])
	{
	int rw = 0;																						// Rckwert
	char xml_dateiinhalt[32768];													// Puffer fr die Datei im Speicher
	char vorspiel[32768];																	// Puffer fr die Datei im Speicher (vor XML Tag)
	char nachspiel[32768];																// Puffer fr die Datei im Speicher (nach XML Tag)
	char neu[32768];																			// Puffer fr die Datei im Speicher (neuer Kram)
	char neu2[32768];																			// Puffer fr die Datei im Speicher (neuer Kram)
	char sektionsname[80];																// cmdline-Parameter Name des Tags
	char rawtagname[80];																	// Tagname ohne Sektionsprefix
	char sektion_dateiinhalt[32768];											// Puffer fr die Sektion
	signed long int t;
	char puffer[32768];
	char *punkt_anfang;
	char *tag_anfang;
	char *tag_ende;
	long int pos_punkt;
	long int pos_anfang;
	long int pos_ende;
	long int l;
	long int i;
	FILE *f;


	punkt_anfang = strstr(section_and_tagname,".");
	if (punkt_anfang == NULL)
		{
		// keine Sektion gefunden
		sektionsname[0] = 0;
		strcpy (rawtagname,section_and_tagname);
		}
	else
		{
		// Punkt gefunden, also Sektion ermitteln
		pos_punkt = punkt_anfang - &section_and_tagname[0];	// Speicherposition in Stringposition umrechnen
		section_and_tagname[pos_punkt] = 0;									// Punkt durch Stringterminator ersetzen
		strcpy(sektionsname,section_and_tagname);						// Anfang ist die Sektion
		for (i = 0;i < 80;i++)
			{
			rawtagname[i] = section_and_tagname[++pos_punkt];
			if (rawtagname[i] == 0) break;
			}
		}


	t = readfileinstring(xml_filename,xml_dateiinhalt);
	if (t < 1)
		{
		printf("ERROR: File \"%s\" konnte nicht gelesen werden oder ist leer.\n",xml_filename);
		rw = -1;
		goto ende;
		}


	if (sektionsname[0] == 0)
		{
		// Alte Version ohne Sektionen
		rw = parse_xml(xml_dateiinhalt,rawtagname,neu);			// neu wird nur als Opferpuffer benutzt
		if (rw < 0)
			{
			// Nicht gefunden, muss am Ende angehngt werden
			strcpy(neu,xml_dateiinhalt);											// Dateiinhalt komplett in den neuen Puffer kopieren
			sprintf(puffer,"<%s>%s</%s>\n",rawtagname,daten,rawtagname); // neuen Tag mit Inhalt erzeugen
			strcat(neu,puffer);																// Tag/Inhalt an alte Datei anhngen
			}
		else
			{
			// Tag gefunden, Strings vor und nach dem Vorkommen mssen ermittelt werden,
			// um den Ersatzstring dann an gleicher Stelle einzubauen


			sprintf(puffer,"<%s>",rawtagname);								// Anfangstag erzeugen
			tag_anfang = strstr(xml_dateiinhalt,puffer);			// Speicherposition des Anfangs ermitteln
			pos_anfang = tag_anfang - &xml_dateiinhalt[0];		// Speicherposition in Stringposition umrechnen
			strncpy(vorspiel,xml_dateiinhalt,pos_anfang);			// Vorstring herauskopieren
			vorspiel[pos_anfang] = 0;													// und abschlieende 0 anhngen

			sprintf(puffer,"</%s>",rawtagname);								// Endtag erzeugen
			t = strlen(puffer);																// Lnge des Endtags ermitteln
			l = strlen(xml_dateiinhalt);											// Lnge der XML-Daten gesamt ermitteln
			tag_ende = strstr(xml_dateiinhalt,puffer);				// Speicherposition des Endes ermitteln
			pos_ende = tag_ende - &xml_dateiinhalt[0];				// Speicherposition in Stringposition umrechnen
			// strncpy(nachspiel,&xml_dateiinhalt[pos_ende + t],l - pos_ende + t - 1);		// Nachstring herauskopieren
			// nachspiel[l - pos_ende + t - 1 + 1] = 0;					// und abschlieende 0 anhngen
			strcpy(nachspiel,&xml_dateiinhalt[pos_ende + t]);	// Nachstring herauskopieren

			strcpy(neu,vorspiel);															// Sermon vor dem Tag kopieren
			sprintf(puffer,"<%s>%s</%s>",rawtagname,daten,rawtagname); // neuen Tag mit Inhalt erzeugen
			strcat(neu,puffer);																// Tag/Inhalt anhngen
			strcat(neu,nachspiel);														// Sermon nach dem Tag anhngen
			}

		// Nun knnen wir das Ergebnis in die Datei zurckschreiben
		f = fopen(xml_filename,"wb");
			if (f == NULL)
			{
			rw = -2;
			printf("ERROR: Konnte Datei nicht zum Schreiben ffnen !\n");
			goto ende;
			}
		fprintf(f,"%s",neu);
		fclose(f);
		rw = strlen(daten);																	// Rckwert ist Anzahl geschriebener Bytes
		goto ende;
		}
	else
		{
		// neue Version mit Sektionen

		rw = parse_xml(xml_dateiinhalt,sektionsname,sektion_dateiinhalt); // Feststellen, ob die Sektion existiert
		if (rw < 0)
			{
			// Sektion existiert nicht und muss am Ende angehngt werden
			sprintf(sektion_dateiinhalt,"<%s>\n<%s>%s</%s>\n</%s>\n",sektionsname,rawtagname,daten,rawtagname,sektionsname);
			strcpy(neu,xml_dateiinhalt);
			strcat(neu,sektion_dateiinhalt);
			}
		else
			{
			// Sektion besteht, es muss innerhalb der Sektion editiert werden
			// Zunchst mal feststellen, ob das Tag innerhalb der Sektion existiert
			rw = parse_xml(sektion_dateiinhalt,rawtagname,neu2);	// neu2 wird nur als Opferpuffer benutzt
			if (rw < 0)
				{
				// Nicht gefunden, muss am Ende angehngt werden
				strcpy(neu2,sektion_dateiinhalt);								// Dateiinhalt komplett in den neuen Puffer kopieren
				sprintf(puffer,"<%s>%s</%s>\n",rawtagname,daten,rawtagname); // neuen Tag mit Inhalt erzeugen
				strcat(neu2,puffer);														// Tag/Inhalt an alte Sektion anhngen
				}
			else
				{
				// Tag gefunden, Strings vor und nach dem Vorkommen mssen ermittxelt werden,
				// um den Ersatzstring dann an gleicher Stelle einzubauen

				sprintf(puffer,"<%s>",rawtagname);							// Anfangstag erzeugen
				tag_anfang = strstr(sektion_dateiinhalt,puffer); // Speicherposition des Anfangs ermitteln
				pos_anfang = tag_anfang - &sektion_dateiinhalt[0]; // Speicherposition in Stringposition umrechnen
				strncpy(vorspiel,sektion_dateiinhalt,pos_anfang); // Vorstring herauskopieren
				vorspiel[pos_anfang] = 0;												// und abschlieende 0 anhngen

				sprintf(puffer,"</%s>",rawtagname);							// Endtag erzeugen
				t = strlen(puffer);															// Lnge des Endtags ermitteln
				l = strlen(sektion_dateiinhalt);								// Lnge der Sektions-Daten gesamt ermitteln
				tag_ende = strstr(sektion_dateiinhalt,puffer);	// Speicherposition des Endes ermitteln
				pos_ende = tag_ende - &sektion_dateiinhalt[0];	// Speicherposition in Stringposition umrechnen
				strncpy(nachspiel,&sektion_dateiinhalt[pos_ende + t],l - pos_ende + t - 1); // Nachstring herauskopieren
				nachspiel[l - pos_ende + t - 1 + 1] = 0;				// und abschlieende 0 anhngen

				strcpy(neu2,vorspiel);													// Sermon vor dem Tag kopieren
				sprintf(puffer,"<%s>%s</%s>",rawtagname,daten,rawtagname); // neuen Tag mit Inhalt erzeugen
				strcat(neu2,puffer);														// Tag/Inhalt anhngen
				strcat(neu2,nachspiel);													// Sermon nach dem Tag anhngen
				}

			// Zunchst mal feststellen, ob die Sektion innerhalb der XML-Datei existiert
			rw = parse_xml(xml_dateiinhalt,sektionsname,neu);	// neu wird nur als Opferpuffer benutzt
			// Tag gefunden, Strings vor und nach dem Vorkommen mssen ermittelt werden,
			// um den Ersatzstring dann an gleicher Stelle einzubauen

			sprintf(puffer,"<%s>",sektionsname);							// Anfangstag erzeugen
			tag_anfang = strstr(xml_dateiinhalt,puffer); 			// Speicherposition des Anfangs ermitteln
			pos_anfang = tag_anfang - &xml_dateiinhalt[0];		// Speicherposition in Stringposition umrechnen
			strncpy(vorspiel,xml_dateiinhalt,pos_anfang);			// Vorstring herauskopieren
			vorspiel[pos_anfang] = 0;													// und abschlieende 0 anhngen

			sprintf(puffer,"</%s>",sektionsname);							// Endtag erzeugen
			t = strlen(puffer);																// Lnge des Endtags ermitteln
			l = strlen(xml_dateiinhalt);											// Lnge der Sektions-Daten gesamt ermitteln
			tag_ende = strstr(xml_dateiinhalt,puffer);				// Speicherposition des Endes ermitteln
			pos_ende = tag_ende - &xml_dateiinhalt[0];				// Speicherposition in Stringposition umrechnen
			strncpy(nachspiel,&xml_dateiinhalt[pos_ende + t],l - pos_ende + t - 1); // Nachstring herauskopieren
			nachspiel[l - pos_ende + t - 1 + 1] = 0;					// und abschlieende 0 anhngen

			strcpy(neu,vorspiel);															// Sermon vor dem Tag kopieren
			sprintf(puffer,"<%s>%s</%s>",sektionsname,neu2,sektionsname); // neue Sektion mit Inhalt erzeugen
			strcat(neu,puffer);																// Tag/Inhalt anhngen
			strcat(neu,nachspiel);														// Sermon nach dem Tag anhngen
			}

		// Nun knnen wir das Ergebnis in die Datei zurckschreiben
		f = fopen(xml_filename,"wb");
			if (f == NULL)
			{
			rw = -2;
			printf("ERROR: Konnte Datei nicht zum Schreiben ffnen !\n");
			goto ende;
			}
		fprintf(f,"%s",neu);
		fclose(f);
		rw = strlen(daten);
		goto ende;
		}

ende:
	return(rw);
	}



// *****   schreibe_xml_string(filepointer,feldname,datenstring,crlf)   -   einen String als XML-Tag in eine Datei schreiben   *****

void schreibe_xml_string(char xml_filename[],char section_and_tagname[],char datenstring[])
	{
	write_xml(xml_filename,section_and_tagname,datenstring);
	}



// *****   schreibe_xml_int(filepointer,feldname,datenint,crlf)   -   einen Integer-Wert als XML-Tag in eine Datei schreiben   *****

void schreibe_xml_int(char xml_filename[],char section_and_tagname[],signed long int datenint)
	{
	char a[30];

	sprintf(a,"%ld",datenint);
	write_xml(xml_filename,section_and_tagname,a);
	}



// *****   schreibe_xml_float(filepointer,feldname,datenfloat,crlf)   -   einen Float-Wert als XML-Tag in eine Datei schreiben   *****

void schreibe_xml_float(char xml_filename[],char section_and_tagname[],double datenfloat)
	{
	char a[30];

	sprintf(a,"%6.2f",datenfloat);
	write_xml(xml_filename,section_and_tagname,a);
	}



/// *****   lese_xml_string(xml,feldname,stringposition,sdefault)   -   einen String aus dem XML im Speicher einlesen   **********

void lese_xml_string(char xml[],char section_and_tagname[],char *stringposition,char sdefault[])
	{
	signed long int l;
	char puffer[4096];

	// printf(": parse_xml(\"%s\")...\n",section_and_tagname);
	iteration_depth = 0;																	// Iterations-Counter zurckstellen
	l = parse_xml_recursive(xml,section_and_tagname,puffer);
	if (l < 0)																						// Tag nicht gefunden
		{
		// printf(": nicht gefunden, RW=\"%s\"\n",sdefault);
		strcpy(stringposition,sdefault);										// Defaultstring zurckliefern
		goto ende;
		}
	if (l == 0)																						// Tag gefunden, aber leerer String
		{
		// printf(": gefunden, aber leer\n");
		stringposition[0] = 0;															// Rckgabewert = ""
		goto ende;
		}
	if (l > 0)
		{
		// printf(": gefunden: \"%s\"\n",stringposition);
		}
	strcpy(stringposition,puffer);												// gefundene Daten zurckliefern

	
ende: ;
	}



/// *****   lese_xml_int(xml,feldname,idefault)   -   einen Int aus dem XML im Speicher einlesen   **********

signed long int lese_xml_int(char xml[],char section_and_tagname[],signed long int idefault)
	{
	signed long int rw;
	signed long int l;
	char puffer[250];

	iteration_depth = 0;																	// Iterations-Counter zurckstellen
	l = parse_xml_recursive(xml,section_and_tagname,puffer);
	if (l < 0)																						// Tag nicht gefunden
		{
		rw = idefault;																			// Defaultwert zurckliefern
		goto ende;
		}
	if (l == 0)																						// Tag gefunden, aber leerer String
		{
		rw = 0;																							// Rckgabewert = 0
		goto ende;
		}

	rw = atol(puffer);																		// gefundene Daten zurckliefern

	
ende: return(rw);
	}



/// *****   lese_xml_float(xml,feldname,fdefault)   -   einen Float aus dem XML im Speicher einlesen   **********

double lese_xml_float(char xml[],char section_and_tagname[],double fdefault)
	{
	double rw;
	signed long int l;
	char puffer[250];

	iteration_depth = 0;																	// Iterations-Counter zurckstellen
	l = parse_xml_recursive(xml,section_and_tagname,puffer);
	if (l < 0)																						// Tag nicht gefunden
		{
		rw = fdefault;																			// Defaultwert zurckliefern
		goto ende;
		}
	if (l == 0)																						// Tag gefunden, aber leerer String
		{
		rw = 0;																							// Rckgabewert = 0
		goto ende;
		}

	rw = atof(puffer);																		// gefundene Daten zurckliefern

	
ende: return(rw);
	}
