// *********************************************************************************************************************
// *********************************************************************************************************************
// **                                                                                                                 **
// **   sensor_demon.c   -   Demon zum Polling angeschlossener Sensoren fr den ALL5000/ALL4500                       **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** (C) 2009 - 2011 by Allnet Deutschland GmbH (www.allnet.de)                                                      **
// **                    Michael Buchstaller     (buchi@allnet.de)                                                    **
// **                                                                                                                 **
// **-----------------------------------------------------------------------------------------------------------------**
// **                                                                                                                 **
// ** This program is free software: you can redistribute it and/or modify                                            **
// ** it under the terms of the GNU General Public License as published by                                            **
// ** the Free Software Foundation, either version 3 of the License, or                                               **
// ** (at your option) any later version.                                                                             **
// **                                                                                                                 **
// ** This program is distributed in the hope that it will be useful,                                                 **
// ** but WITHOUT ANY WARRANTY; without even the implied warranty of                                                  **
// ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                                   **
// ** GNU General Public License for more details.                                                                    **
// **                                                                                                                 **
// ** You should have received a copy of the GNU General Public License                                               **
// ** along with this program.  If not, see http://www.gnu.org/licenses                                               **
// **                                                                                                                 **
// *********************************************************************************************************************
// *********************************************************************************************************************


// ----------   Sensortypen (ALL4000-konform)   ----------
// Nr.   Bezeichnung																		Art					Chip					ID
//   0 - Kein Sensor erkannt														-						-							-
//   1 - ALL3005, ALL3010																Temperatur	LM75					7
//   2 - ALL3006, ALL3015																Temperatur	LM92					3
//   2 - ALL3007																				Temperatur	LM92					0
//   2 - ALL3007																				Temperatur	LM92					1
//   3 - ALL3018, ALL3029, ALL4018, ALL4029							Temperatur	SHT71					-
//  32 - ALL3022																				Spannung		MAX1237				-
//  33 - ALL5000intADC																	Spannung		ADS1100				0
//  40 - ALL3028, ALL3029, ALL4029											Luftdruck		DSDX015A2R		-
//  41 - ALL4028A																				Luftdruck		HDI0811ARY8P3	-
//  42 - ALL4529																				Luftdruck		BMP085				-
//  65 - ALL3018, ALL3029, ALL4018, ALL4029							Luftfeuchte	SHT71					-
//  97 - ALL3002, ALL3003, ALL4020											Ausg. 1 Bit	PCF8574				0
// 101 - ALL3024, ALL3026, ALL4026											Eing. 1 Bit	PCF8574				7
// 102 - ALL4027, ALL4042																Ausg. 8 Bit	PCF8574				6
// 103 - ALL4040																				Aus6, Ein6	2*PCF8574			4 und 3
// 129 - ALL5000 intern																	Spannung		ADS1100A0			0
// 130 - ALL4590 Powermeter															Strom				ADS1100A0			0
// 133 - ALL3025, ALL4025																Zhler			PCF8583				1
// 134 - ALL4035																				ZhlerS0Bus	PCF8583				0
// 151 - Knuerr-Stromsensor															Strom				AtMega8				1
// 153 - ALL4023																				3*Spannung	AtTiny85			3
// 155 - ALL3032                                        Helligkeit	AtTiny85			2
// 156 - ALL4033                                        Spannung AC	AtTiny85			56
// 199 - ALL4500/ALL5000 neue version										allerlei		PCF8574				0...7
// 200 - LED-Controller ALL4500/ALL5000 intern					Ausg. 2 Bit	PCF8574A			0...7
// 300 - RFID Cardreader001                             Karten-ID		RFID001				-
//
// ----------   Chip-IDs   ----------
// Nr.   Bezeichnung																		IDs								Kommentar
//   0 - nicht vorhanden																0										-
//   1 - LM75																						0...7								Temperatursensor
//   2 - LM92																						0...3								Temperatursensor
//   3 - SHT71																					0										Temperatur
//  32 - SHT71																					0										Feuchte
//  40 - DSDX015A2R																			0										Luftdruck
//  41 - HDI0811ARY8P3																	0										Luftdruck
//  42 - BMP085																					0										Luftdruck
//	64 - PCF8574																				0...7								8-Bit I/O Output (nur Bit 0  einzeln genutzt)
//	65 - PCF8574																				0...7								8-Bit I/O Output (Bits 0...3 einzeln genutzt)
//	66 - PCF8574																				0...7								8-Bit I/O Output (Bits 4...7 einzeln genutzt)
//	67 - PCF8574																				0...7								8-Bit I/O Output (Bits 0...7 einzeln genutzt)
//	68 - PCF8574																				0...7								8-Bit I/O Output (Bits 0...7 als 8-Bit-Wort genutzt)
//	74 - PCF8574																				0...7								8-Bit I/O Input (nur Bit 0  einzeln genutzt)
//	75 - PCF8574																				0...7								8-Bit I/O Input (Bits 0...3 einzeln genutzt)
//	76 - PCF8574																				0...7								8-Bit I/O Input (Bits 4...7 einzeln genutzt)
//	77 - PCF8574																				0...7								8-Bit I/O Input (Bits 0...7 einzeln genutzt)
//	78 - PCF8574																				0...7								8-Bit I/O Input (Bits 0...7 als 8-Bit-Wort genutzt)
//	96 - PCF8583																				1										Zhler und RTC
//	97 - PCF8583																				0										Zhler und RTC
// 128 - MAX1237																				0										3-fach ADC 12 bit
// 129 - ADS1100																				0...3								1-fach ADC 16 bit mit mux PCF8574.2 bits 6...8
// 130 - ADS1100																				0...2								1-fach ADC 16 bit mit mux PCF8574.5 bits 0...2
// 164 - PCF8574A																				0...7								8-Bit I/O Output (nur Bit 0  einzeln genutzt)
// 165 - PCF8574A																				0...7								8-Bit I/O Output (Bits 0...3 einzeln genutzt)
// 166 - PCF8574A																				0...7								8-Bit I/O Output (Bits 4...7 einzeln genutzt)
// 167 - PCF8574A																				0...7								8-Bit I/O Output (Bits 0...7 einzeln genutzt)
// 168 - PCF8574A																				0...7								8-Bit I/O Output (Bits 0...7 als 8-Bit-Wort genutzt)
// 174 - PCF8574A																				0...7								8-Bit I/O Input (nur Bit 0  einzeln genutzt)
// 175 - PCF8574A																				0...7								8-Bit I/O Input (Bits 0...3 einzeln genutzt)
// 176 - PCF8574A																				0...7								8-Bit I/O Input (Bits 4...7 einzeln genutzt)
// 177 - PCF8574A																				0...7								8-Bit I/O Input (Bits 0...7 einzeln genutzt)
// 178 - PCF8574A																				0...7								8-Bit I/O Input (Bits 0...7 als 8-Bit-Wort genutzt)
// 197 - PCF8574																				1										8-Bit Output (ALL5000 Relais 0...7) L:On, H:Off
// 198 - PCF8574																				2										4-BitIN/3-BitOUT (ALL5000 Digitaleingnge 0...3 auf Bit 0...3, ALL5000 ADC Multiplexer auf Bit 5...7) L:On, H:Off
// 199 - PCF8574																				0...7								8-Bit I/O IN/Output (Autodetect, Funktion unklar)
// 200 - PCF8574A																				0...7								8-Bit I/O IN/Output (Autodetect, Funktion unklar)
// 256 - AtTiny85																				57									Atmel AtTiny85 fr ALL4033 AC Voltmeter
// 257 - AtMega8																				0...255							Atmel AtMega8 Prozessor
// 258 - AtMega8535																			0...255							Atmel AtMega8535 Prozessor
// 259 - AtTiny85																				54									Atmel AtTiny85 fr ALL3032 analogen Helligkeitssensor
// 260 - AtTiny85																				52									Atmel AtTiny85 fr ALL4023 3-fach Spannungsmesser
// 300 - RFID reader																		0										RFID Leser
//
// ----------   Funktionstypen (ersetzen die alten Sensortypen)   -----
// Nr.	Verwendung																			Einheit							Kommentar
//   0	inaktiv																					-										kein Sensor angeschlossen
//   1  Temperatur																			Grad C							LM75, LM92, SHT71_T
//   2	Luftfeuchtigkeit																% RH								SHT71_H
//   3	Gleichspannung																	V DC								MAX1237, ATTiny85, ADS1100
//   4	Wechselspannung																	V AC								ATTINY85
//   5	Gleichstrom																			A DC								-
//   6	Wechselstrom																		A AC								-
//   7	Leistung																				W										-
//   8	Impulszhler																		-										PCF8583
//   9  Helligkeit (simpel)															-										0...100
//  10	Helligkeit (kalibriert)													Lux									-
//  11	Schaltausgang 1-bit/8-bit/n-bit									-										0=off/100=On
//  12	Schalteingang 1-bit/8-bit/n-bit									-										0/100
//  13	Analogausgang (simpel)													-										0...100
//  14	Analogausgang (kalibriert)											V DC								-
//  15	Textstring																			-										-
//  16	Luftdruck																				mBar								-
//
//
// ----------   Anschlsse am ARM-CPU-Board (fr ALL4500/ALL5000)   ----------
// I2C BUS 1 (extern)
// Port	Verwendung
// PB30	I2C_Master Data Input
// PB29	I2C_Master Data Output
// PB28	I2C_Master Clock Output
// PB25	Groupselect 0
// PB24	Groupselect 1
// PB23	Groupselect 2
// PB10	Portselect 0
// PB11	Portselect 1
// PB27	Portselect 2
//
// I2C BUS 0 (intern)
// Port	Verwendung
// PA20	Data Input/Output
// PA21	Clock Output
//
// Direkte I/Os:
// Port	Verwendung
// PB22	Power_Enable (schalter 12V und 3.3V fr die Sensoren ein, wenn H)
// PD15 Firmware-Reset Input (H:normal, L: totale Lobotomie beim Booten)


#include <errno.h>																			// Error number definitions
#include <termios.h>																		//POSIX terminal control definitions
#include <sys/stat.h>
#include <unistd.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <sys/shm.h>
#include "logging.h"
#include "xml_lib.h"
#include "../sqlite/sqlite-3.7.3_arm/sqlite3.h"
#include "at91gpio.h"
#include "all_i2c.h"
#include "i2c_chips.h"
#include "sqldb.h"
#include "sensor_demon.h"
#include "sensor_history.h"
#include "shm_use.h"


#define CONFIG_DATABASE_FILE "/etc/allnetenv/config.s3db"
#define VAR_DATABASE_FILE "/tmp/var.s3db"



// *****   strcompress(destination,source)   -   wie strcpy, entfernt aber noch Leerzeichen und Tabs   *****

void strcompress(char *destination,char *source)
	{
	char z;
	signed long int t = 0;
	signed long int i = 0;


looping:
	z = source[t++];
	if (z == 0) goto write;
	if (z <= 32) goto weiter;
write: destination[i++] = z;
weiter:
	if (z != 0) goto looping;
	}



// *****   lcd_show_sensors()   -   Sensorwerte im LCD-Display des ALL5000 anzeigen   *****

void lcd_show_sensors(void)
	{
	static unsigned short int sensor_nummer = 0;
	static unsigned short int delay_counter = 0;
	char kommando[90];
	char einheit[10];


	if (config_demon_lcd_enabled > 0)											// LCD aktiviert ?
		{
		if (delay_counter > 0)															// Delayzeit abgelaufen ?
			{
			delay_counter--;																	// warten, bis wir wieder dran sind
			}
		else
			{
			delay_counter = config_demon_lcd_delay;						// Delayloop wieder neu starten
			einheit[0] = 0;																		// angezeigte Einheit lschen
			if (logical_function[sensor_nummer] ==  1) sprintf(einheit," C");
			if (logical_function[sensor_nummer] ==  2) sprintf(einheit," %% RH");
			if (logical_function[sensor_nummer] ==  3) sprintf(einheit," V DC");
			if (logical_function[sensor_nummer] ==  4) sprintf(einheit," V AC");
			if (logical_function[sensor_nummer] ==  5) sprintf(einheit," A DC");
			if (logical_function[sensor_nummer] ==  6) sprintf(einheit," A AC");
			if (logical_function[sensor_nummer] ==  7) sprintf(einheit," W");
			if (logical_function[sensor_nummer] == 10) sprintf(einheit," lux");
			if (logical_function[sensor_nummer] == 14) sprintf(einheit," V DC");
			if (logical_function[sensor_nummer] == 16) sprintf(einheit," mBar");
			sprintf(kommando,"/usr/sbin/lcd_write 0 \"%s\"",logical_name[sensor_nummer]); // Sensornamen in Zeile 0 anzeigen
			system(kommando);
			sprintf(kommando,"/usr/sbin/lcd_write 1 \"%s%s\"",logical_string[sensor_nummer],einheit); // Sensorwert in Zeile 1 anzeigen
			system(kommando);
naechster:
			sensor_nummer++;
			if(sensor_nummer > logical_max_sensors)
				{
				sensor_nummer = 0;
				goto ende;
				}
			if(logical_int[sensor_nummer] <= -204800000) goto naechster;
			}
		}
ende: ;
	}



// *****   history_setzedefault()   -   History-Tabelle auf Standardwerte setzen   *****

void history_setzedefault(void)
	{
	unsigned short int t;
	unsigned short int i;
	char ep[200];

	sprintf(ep,"clearing sensor history...");
	log_append(4,ep);																		// Fehler protokollieren

	for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		for(i = 0;i < MAX_HISTORY;i++)
			{
			history_array[t][i] = -2048000.00;
			}
		}
	}



// *****   history_shift()   -   History-Tabelle nach hinten verschieben   *****

void history_shift(void)
	{
	unsigned short int t;
	signed short int i;

	for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		for(i = MAX_HISTORY - 2;i >= 0;i--)
			{
			history_array[t][i + 1] = history_array[t][i];		// Um 1 nach hinten verschieben
			// if((t == 1) && (i < 4)) printf("s:%d, f:%4.2f\n",i,f);
			}
		history_array[t][0] = -2048000.00;									// Poition 0 lschen
		}
	}



// *****   history_update()   -   History-Tabellenposition 0 mit Sensorwerten berschreiben   *****

void history_update(void)
	{
	unsigned short int t;

	for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		if (logical_sum_count <= 0)
			{
			history_array[t][0] = logical_float[t];
			}
		else
			{
			history_array[t][0] = logical_sum_float[t] / logical_sum_count;
			logical_sum_float[t] = 0;
			}
		}

	logical_sum_count = 0;
	}



// *****   history_write_data()   -   History-Tabelle sichern   *****

void history_write_data(void)
	{
	FILE *f;
	char ep[200];

	sprintf(ep,"writing sensor history data...");
	log_append(4,ep);																		// Fehler protokollieren

	f = fopen(history_savefile,"wb");
	if (f != NULL)
		{
		fwrite(&history_array[0][0],1,sizeof(history_array),f);
		fclose(f);
		}
	else
		{
		sprintf(ep,"*** ERROR: Could not open file \"%s\" for write",history_savefile);
		log_append(2,ep);																		// Fehler protokollieren
		}
	}



// *****   history_read_data()   -   History-Tabelle einlesen   *****

void history_read_data(void)
	{
	FILE *f;
	char ep[200];

	sprintf(ep,"reading sensor history data \"%s\" (%d bytes)...",history_savefile,sizeof(history_array));
	log_append(4,ep);																		// Fehler protokollieren

	f = fopen(history_savefile,"rb");
	if (f != NULL)
		{
		fread(&history_array[0][0],1,sizeof(history_array),f);
		fclose(f);
		}
	else
		{
		sprintf(ep,"*** ERROR: Could not open file \"%s\" for read",history_savefile);
		log_append(2,ep);																		// Fehler protokollieren
		}
	}



// *****   history_write_xml()   -   History-Tabelle als XML sichern   *****

void history_write_xml(void)
	{
	unsigned short int t;
	unsigned short int i;
	FILE *f;
	char ep[200];


	sprintf(ep,"writing sensor history data as XML file...");
	log_append(4,ep);																			// Fehler protokollieren

	f = fopen(history_tmpfile,"wb");
	if (f != NULL)
		{
		fprintf(f,"<data>");
		for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
			{
			if(logical_function[t] > 0)
				{
				fprintf(f,"<sensor%d>",t);
				for(i = 0;i < MAX_HISTORY;i++)
					{
					fprintf(f,"<v%d>%1.2f</v%d>",i,history_array[t][i],i);
					}
				fprintf(f,"</sensor%d>\n",t);
				}
			}
		fprintf(f,"</data>\n");
		fclose(f);


		/*
		t = rename(history_tmpfile,history_xmlfile);				// POSIX-Version des Unix "mv" Kommandos
		if (t != 0)
			{
			sprintf(ep,"*** ERROR %d: Could not move file \"%s\" to \"%s\"",t,history_tmpfile,history_xmlfile);
			log_append(2,ep);																	// Fehler protokollieren
			}
		*/
		sprintf(ep,"mv %s %s",history_tmpfile,history_xmlfile);
		system(ep);
		}
	else
		{
		sprintf(ep,"*** ERROR: Could not open file \"%s\" for write",history_tmpfile);
		log_append(2,ep);																		// Fehler protokollieren
		}
	}



// *****   history_write_svg()   -   History-Tabelle als SVG sichern   *****

void history_write_svg(signed char all_true)
	{
	char farbe_gray[20] = "707070";
	char farbe_graytext[20] = "808080";
	unsigned short int s;
	unsigned short int t;
	// unsigned short int i;
	FILE *f;
	char ep[200];
	char svg_filename[80];
	signed short int groesse_x;
	signed short int groesse_y;
	signed short int rand_links;
	signed short int rand_rechts;
	signed short int rand_oben;
	signed short int rand_unten;
	char farbe_grafikhintergrund[20];
	char farbe_fensterhintergrund[20];
	char farbe_text[20];
	char farbe_gitter[20];
	char farbe_gitter0[20];
	char farbe_graph[20];
	signed short int linientyp_graph;
	char farbe_min[20];
	char farbe_max[20];
	signed short int xint;
	float xlauf;
	float ylauf;
	float scale_y;
	float scale_zero_y;
	float scale_y_minvalue;
	float scale_y_maxvalue;
	float scale_y_steps;
	float grid_y_stepsize;
	float scale_x;
	float scale_zero_x;
	float scale_x_minvalue;
	float scale_x_maxvalue;
	float scale_x_steps;
	float grid_x_stepsize;
	float x;
	float y;
	float gesamtsekunden;
	signed short int sekundengittermodus = 0;
	float sekundengitterabstand;
	float sekundengitterpixel;
	signed long int sgt;
	signed long int scount;
	signed short int m;


	groesse_x = svg_x;
	groesse_y = svg_y;
	rand_links = 40;
	rand_rechts = 5;
	if (logical_svg_includevalues > 0)
		{
		rand_oben = 45;
		}
	else
		{
		rand_oben = 1;
		}
	rand_unten = 20;


	sprintf(ep,"writing sensor history data as SVG files...");
	log_append(4,ep);																			// Fehler protokollieren

	for(s = 0;s < MAX_LOGICAL_SENSORS;s++)
		{
		// neue konfigurierbare Farben
		sprintf(farbe_grafikhintergrund,"#%s",logical_svg_bgcolor[s]);
		sprintf(farbe_fensterhintergrund,"#%s",logical_svg_bgcolor[s]);
		sprintf(farbe_text,"#%s",logical_svg_gridcolor[s]);
		sprintf(farbe_gitter,"#%s",logical_svg_gridcolor[s]);
		sprintf(farbe_gitter0,"#%s",logical_svg_gridcolor[s]);
		sprintf(farbe_graph,"#%s",logical_svg_graphcolor[s]);
		strcpy(farbe_min,"#0000FF");
		strcpy(farbe_max,"#FF0000");
		linientyp_graph = logical_svg_graphstyle[s];

		if((logical_float[s] <= - 2048000.0) && (all_true != 1))
			{
			sprintf(farbe_grafikhintergrund,"#%s",farbe_gray);
			sprintf(farbe_fensterhintergrund,"#%s",farbe_gray);
			sprintf(farbe_text,"#%s",farbe_graytext);
			sprintf(farbe_gitter,"#%s",farbe_graytext);
			sprintf(farbe_gitter0,"#%s",farbe_graytext);
			sprintf(farbe_graph,"#%s",farbe_graytext);
			strcpy(farbe_min,"#000080");
			strcpy(farbe_max,"#800000");
			}

		scale_y_minvalue = logical_display_min[s] - 2;
		scale_y_maxvalue = logical_display_max[s] + 2;
		scale_y_steps = scale_y_maxvalue - scale_y_minvalue;
		scale_y = (groesse_y - rand_oben - rand_unten) / scale_y_steps;
		scale_zero_y = rand_oben + scale_y_maxvalue * scale_y;
		grid_y_stepsize = scale_y_steps / 10;
		grid_y_stepsize = (signed long int)grid_y_stepsize;
		if (grid_y_stepsize < 1) grid_y_stepsize = 1;

		scale_x_minvalue = 0;
		scale_x_maxvalue = MAX_HISTORY;
		scale_x_steps = scale_x_maxvalue - scale_x_minvalue;
		scale_x = (groesse_x - rand_links - rand_rechts) / scale_x_steps;
		scale_zero_x = groesse_x - rand_rechts;
		grid_x_stepsize = scale_x_steps / 15;
		grid_x_stepsize = (signed long int)grid_x_stepsize;
		if (grid_x_stepsize < 1) grid_x_stepsize = 1;

		if(logical_function[s] > 0)
			{
			// sprintf(svg_filename,"/www/img/svg/sensor%d.svg",s);
			sprintf(svg_filename,"/tmp/svg/sensor%d.svg",s);
			f = fopen(svg_tmpfile,"wb");
			if (f != NULL)
				{
				fprintf(f,"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
				fprintf(f,"<svg xmlns=\"http://www.w3.org/2000/svg\"\n");
				fprintf(f,"xmlns:xlink=\"http://www.w3.org/1999/xlink\"\n");
				fprintf(f,"xmlns:ev=\"http://www.w3.org/2001/xml-events\"\n");
				fprintf(f,"version=\"1.1\" baseProfile=\"full\"\n");
				fprintf(f,"width=\"%d\" height=\"%d\" viewBox=\"0 0 %d %d\">\n",groesse_x,groesse_y,groesse_x,groesse_y);

				// Fensterhintergrund
				fprintf(f,"<rect x=\"0\" y=\"0\" width=\"%d\" height=\"%d\" rx=\"0\" fill=\"%s\" />\n",
								groesse_x,groesse_y,farbe_fensterhintergrund);

				// Grafikhintergrund
				fprintf(f,"<rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" rx=\"0\" fill=\"%s\" />\n",
								rand_links,rand_oben,groesse_x - rand_links - rand_rechts,groesse_y - rand_oben - rand_unten,farbe_grafikhintergrund);

				// Waagerechte Gitterlinien und Einheiten
				// >= 0
				for(ylauf = 0;ylauf < scale_y_maxvalue;ylauf += grid_y_stepsize)
					{
					if ((ylauf > scale_y_minvalue) && (ylauf < scale_y_maxvalue))
						{
						fprintf(f,"<line x1=\"%d\" y1=\"%4.2f\" x2=\"%d\" y2=\"%4.2f\" style=\"stroke:%s;stroke-width:0.6;\" />\n",
										rand_links - 1,scale_zero_y - ylauf * scale_y,groesse_x - rand_rechts,scale_zero_y - ylauf * scale_y,farbe_gitter);
						fprintf(f,"<text x=\"%d\" y=\"%4.2f\" text-anchor=\"end\" style=\"font-size:8px;font-family:Arial;fill:%s\">%4.1f</text>\n",
										rand_links - 3,scale_zero_y - ylauf * scale_y + 4,farbe_gitter,ylauf);
						}
					}
				// < 0
				for(ylauf = -grid_y_stepsize;ylauf > scale_y_minvalue;ylauf -= grid_y_stepsize)
					{
					if ((ylauf > scale_y_minvalue) && (ylauf < scale_y_maxvalue))
						{
						fprintf(f,"<line x1=\"%d\" y1=\"%4.2f\" x2=\"%d\" y2=\"%4.2f\" style=\"stroke:%s;stroke-width:0.6;\" />\n",
										rand_links - 1,scale_zero_y - ylauf * scale_y,groesse_x - rand_rechts,scale_zero_y - ylauf * scale_y,farbe_gitter);
						fprintf(f,"<text x=\"%d\" y=\"%4.2f\" text-anchor=\"end\" style=\"font-size:8px;font-family:Arial;fill:%s\">%4.1f</text>\n",
										rand_links - 3,scale_zero_y - ylauf * scale_y + 4,farbe_gitter,ylauf);
						}
					}
				// Nulllinie
				if ((0 > scale_y_minvalue) && (0 < scale_y_maxvalue))
					{
					fprintf(f,"<line x1=\"%d\" y1=\"%4.2f\" x2=\"%d\" y2=\"%4.2f\" style=\"stroke:%s;stroke-width:1;\" />\n",
									rand_links - 1,scale_zero_y - 0 * scale_y,groesse_x - rand_rechts,scale_zero_y - 0 * scale_y,farbe_gitter0);
					}
				if(logical_meta_enabled[s] < 1)										// Minimum- und Maximum-Linien nur anzeigen, wenn es kein Metasensor ist
					{
					// Minimum
					if ((logical_limit_min[s] > scale_y_minvalue) && (logical_limit_min[s] < scale_y_maxvalue))
						{
						fprintf(f,"<line x1=\"%d\" y1=\"%4.2f\" x2=\"%d\" y2=\"%4.2f\" style=\"stroke:%s;stroke-width:1;\" />\n",
										rand_links - 1,scale_zero_y - logical_limit_min[s] * scale_y,groesse_x - rand_rechts,scale_zero_y - logical_limit_min[s] * scale_y,farbe_min);
						}
					// Maximum
					if ((logical_limit_max[s] > scale_y_minvalue) && (logical_limit_max[s] < scale_y_maxvalue))
						{
						fprintf(f,"<line x1=\"%d\" y1=\"%4.2f\" x2=\"%d\" y2=\"%4.2f\" style=\"stroke:%s;stroke-width:1;\" />\n",
										rand_links - 1,scale_zero_y - logical_limit_max[s] * scale_y,groesse_x - rand_rechts,scale_zero_y - logical_limit_max[s] * scale_y,farbe_max);
						}
					}

				// Senkrechte Gitterlinien und Einheiten
				// Gitterweite ermitteln
				gesamtsekunden = config_demon_historyinterval * MAX_HISTORY;
				sekundengittermodus = 0;												// Einheit: Sekunden minus
				sekundengitterabstand = 30;											// 30 Sekunden Gitterraster
				if (config_demon_historyinterval >= 3)					// alle 3 Sekunden
					{
					sekundengittermodus = 0;											// Einheit: Sekunden minus
					sekundengitterabstand = 60;										// 60 Sekunden Gitterraster
					}
				if (config_demon_historyinterval >= 10)					// 10 Sekunden und mehr
					{
					sekundengittermodus = 1;											// Einheit: Minuten minus
					sekundengitterabstand = 300;									// 5 Minuten Gitterraster
					}
				if (config_demon_historyinterval >= 30)					// 30 Sekunden umd mehr
					{
					sekundengittermodus = 1;											// Einheit: Minuten minus
					sekundengitterabstand = 600;									// 10 Minuten Gitterraster
					}
				if (config_demon_historyinterval >= 60)					// 1 Minute und mehr
					{
					sekundengittermodus = 1;											// Einheit: Minuten minus
					sekundengitterabstand = 1200;									// 20 Minuten Gitterraster
					}
				if (config_demon_historyinterval >= 120)				// 2 Minuten und mehr
					{
					sekundengittermodus = 3;											// Einheit: Stunden minus
					sekundengitterabstand = 18000;								// 5 Stunden Gitterraster
					}
				sekundengitterpixel = MAX_HISTORY / (gesamtsekunden / sekundengitterabstand);

				sgt = 0;
				scount = -1;
				for(xlauf = 0;xlauf < scale_x_maxvalue;xlauf += sekundengitterpixel)
					{
					scount++;
					fprintf(f,"<line x1=\"%4.2f\" y1=\"%d\" x2=\"%4.2f\" y2=\"%d\" style=\"stroke:%s;stroke-width:0.6;\" />\n",
									scale_zero_x - xlauf * scale_x,rand_oben,scale_zero_x - xlauf * scale_x,groesse_y - rand_unten,farbe_gitter);
					sgt++;
					if (sgt >= 2)
						{
					fprintf(f,"<line x1=\"%4.2f\" y1=\"%d\" x2=\"%4.2f\" y2=\"%d\" style=\"stroke:%s;stroke-width:1;\" />\n",
									scale_zero_x - xlauf * scale_x,groesse_y - rand_unten,scale_zero_x - xlauf * scale_x,groesse_y - rand_unten + 2,farbe_gitter);
						sgt = 0;
						// xx[t].setColor("#000000");	// schwarz
						// xx[t].setStroke(1);
						// xx[t].setFont("arial","9px",Font.NORMAL); 
						if (sekundengittermodus == 0)
							{
							// xx[t].drawString(sg * sekundengitterabstand+"s",divx_max - sg * sekundengitterpixel - 5,divy_min + 3);
							fprintf(f,"<text x=\"%f\" y=\"%d\" style=\"font-size:8px;font-family:Arial;fill:%s\">%4.0fs</text>\n",
								scale_zero_x - xlauf * scale_x - 5,groesse_y - rand_unten + 9,farbe_gitter,scount * sekundengitterabstand);
							}
						if (sekundengittermodus == 1)
							{
							// xx[t].drawString(sg * sekundengitterabstand/60+"m",divx_max - sg * sekundengitterpixel - 5,divy_min + 3);
							fprintf(f,"<text x=\"%f\" y=\"%d\" style=\"font-size:8px;font-family:Arial;fill:%s\">%4.0fm</text>\n",
								scale_zero_x - xlauf * scale_x - 5,groesse_y - rand_unten + 9,farbe_gitter,scount * sekundengitterabstand / 60);
							}
						if (sekundengittermodus == 2)
							{
							// xx[t].drawString(sg * sekundengitterabstand/3600+"h",divx_max - sg * sekundengitterpixel - 5,divy_min + 3);
							fprintf(f,"<text x=\"%f\" y=\"%d\" style=\"font-size:8px;font-family:Arial;fill:%s\">%4.0fh</text>\n",
								scale_zero_x - xlauf * scale_x - 5,groesse_y - rand_unten + 9,farbe_gitter,scount * sekundengitterabstand / 3600);
							}
						if (sekundengittermodus == 3)
							{
							// xx[t].drawString(sg * sekundengitterabstand/3600+"h",divx_max - sg * sekundengitterpixel - 5,divy_min + 3);
							fprintf(f,"<text x=\"%f\" y=\"%d\" style=\"font-size:8px;font-family:Arial;fill:%s\">%4.0fh</text>\n",
								scale_zero_x - xlauf * scale_x - 5,groesse_y - rand_unten + 9,farbe_gitter,scount * sekundengitterabstand / 3600);
							}
						
						}
					}


				if(logical_meta_enabled[s] < 1)
					{
					// printf("Sensor %d normal\n",s);
					// Sensor-Verlaufsgrafik fr den Standard-Sensor (mit Polyline)
					x = scale_zero_x - 0 * scale_x;
					y = scale_zero_y - history_array[s][0] * scale_y;
					if (y < rand_oben) y = rand_oben;
					if (y > groesse_y - rand_unten) y = groesse_y - rand_unten;
					fprintf(f,"<g style=\"stroke:%s;stroke-width:1.75;fill:none\">\n",farbe_graph);
					fprintf(f,"<polyline points=\"%4.1f,%4.1f",x,y);
					for(xint = 1;xint < MAX_HISTORY;xint++)
						{
						x = scale_zero_x - (float)xint * scale_x;
						y = scale_zero_y - history_array[s][xint] * scale_y;
						if (y < rand_oben) y = rand_oben;
						if (y > groesse_y - rand_unten) y = groesse_y - rand_unten;
						fprintf(f," %4.1f,%4.1f",x,y);
						}

					if (linientyp_graph == 0)												// 0: durchgehende Linie
						{
						fprintf(f,"\" stroke-dasharray=\"\" />\n");
						}
					if (linientyp_graph == 1)												// 1: kurze Striche
						{
						fprintf(f,"\" stroke-dasharray=\"2,4\" />\n");
						}
					if (linientyp_graph == 2)												// 2: lange Striche
						{
						fprintf(f,"\" stroke-dasharray=\"20,4\" />\n");
						}
					if (linientyp_graph == 3)												// 3: lange und kurze Striche
						{
						fprintf(f,"\" stroke-dasharray=\"20,4,2,4\" />\n");
						}
					fprintf(f,"</g>\n");

						if (logical_svg_includevalues > 0)
							{
						// roter Hintergrund, wenn bertemperatur
						if (logical_float[s] >= logical_limit_max[s])
							{
							fprintf(f,"<rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" rx=\"0\" fill=\"%s\" />\n",
											2,2,groesse_x - 4,rand_oben - 4,farbe_max);
							}
						// blauer Hintergrund, wenn Untertemperatur
						if (logical_float[s] <= logical_limit_min[s])
							{
							fprintf(f,"<rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" rx=\"0\" fill=\"%s\" />\n",
											2,2,groesse_x - 4,rand_oben - 4,farbe_min);
							}
						// aktueller Sensor-Wert fr den Standard-Sensor
						fprintf(f,"<text x=\"%d\" y=\"29\" style=\"font-size:20px;font-family:Arial;fill:%s\">%s: %s%s</text>\n",
										rand_links,farbe_graph,logical_name[s],logical_string[s],scale_unit[logical_function[s]]);
								}
					}
				else
					{
					// printf("Sensor %d META\n",s);
					// Metasensor, bis zu 4 Grafiken und Mewerte werden angezeigt
					for(m = 0;m < 4;m++)
						{
						sprintf(farbe_graph,"#%s",logical_meta_graphcolor[s][m]);
						linientyp_graph = logical_meta_graphstyle[s][m];
						// Verlaufsgrafik
						x = scale_zero_x - 0 * scale_x;
						y = scale_zero_y - history_array[logical_meta_source[s][m]][0] * scale_y;
						if (y < rand_oben) y = rand_oben;
						if (y > groesse_y - rand_unten) y = groesse_y - rand_unten;
						fprintf(f,"<g style=\"stroke:%s;stroke-width:1.75;fill:none\">\n",farbe_graph);
						fprintf(f,"<polyline points=\"%4.1f,%4.1f",x,y);
						for(xint = 1;xint < MAX_HISTORY;xint++)
							{
							x = scale_zero_x - (float)xint * scale_x;
							y = scale_zero_y - history_array[logical_meta_source[s][m]][xint] * scale_y;
							if (y < rand_oben) y = rand_oben;
							if (y > groesse_y - rand_unten) y = groesse_y - rand_unten;
							fprintf(f," %4.1f,%4.1f",x,y);
							}

						if (linientyp_graph == 0)												// 0: durchgehende Linie
							{
							fprintf(f,"\" stroke-dasharray=\"\" />\n");
							}
						if (linientyp_graph == 1)												// 1: kurze Striche
							{
							fprintf(f,"\" stroke-dasharray=\"2,4\" />\n");
							}
						if (linientyp_graph == 2)												// 2: lange Striche
							{
							fprintf(f,"\" stroke-dasharray=\"20,4\" />\n");
							}
						if (linientyp_graph == 3)												// 3: lange und kurze Striche
							{
							fprintf(f,"\" stroke-dasharray=\"20,4,2,4\" />\n");
							}
						fprintf(f,"</g>\n");

						if (logical_svg_includevalues > 0)
							{
							// roter Hintergrund, wenn bertemperatur
							if (logical_float[logical_meta_source[s][m]] >= logical_limit_max[logical_meta_source[s][m]])
								{
								fprintf(f,"<rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" rx=\"0\" fill=\"%s\" />\n",
												2,1 + m * 11,groesse_x - 4,10,farbe_max);
								}
							// blauer Hintergrund, wenn Untertemperatur
							if (logical_float[logical_meta_source[s][m]] <= logical_limit_min[logical_meta_source[s][m]])
								{
								fprintf(f,"<rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" rx=\"0\" fill=\"%s\" />\n",
												2,1 + m * 11,groesse_x - 4,10,farbe_min);
								}
							// aktueller Sensor-Wert fr den Metasensor
							fprintf(f,"<text x=\"%d\" y=\"%d\" style=\"font-size:8px;font-family:Arial;fill:%s\">%s: %s%s</text>\n",
											rand_links,9 + 11 * m - 1,farbe_graph,
											logical_name[logical_meta_source[s][m]],
											logical_string[logical_meta_source[s][m]],
											scale_unit[logical_function[logical_meta_source[s][m]]]);
							}

						}
					}

				fprintf(f,"</svg>\n");
				fclose(f);
				t = rename(svg_tmpfile,svg_filename);						// POSIX-Version des Unix "mv" Kommandos
				if (t != 0)
					{
					sprintf(ep,"*** ERROR: Could not move file \"%s\" to \"%s\"",svg_tmpfile,svg_filename);
					log_append(2,ep);															// Fehler protokollieren
					}
				}
			else
				{
				sprintf(ep,"*** ERROR: Could not open file \"%s\" for write",svg_tmpfile);
				log_append(2,ep);																// Fehler protokollieren
				}
			}
		}
	}



// *****   set_function_units   -   Einheiten zu Funktionstypen festlegen

void set_function_units(void)
	{
	unsigned short int t;

	for(t = 0;t < 256;t++)
		{
		function_unit[t][0] = 0;
		}

	strcpy(function_unit[ 1],"deg.C");
	strcpy(function_unit[ 2],"%");
	strcpy(function_unit[ 3],"V DC");
	strcpy(function_unit[ 4],"V AC");
	strcpy(function_unit[ 5],"A DC");
	strcpy(function_unit[ 6],"A AC");
	strcpy(function_unit[ 7],"W");
	strcpy(function_unit[ 9],"Lux");
	strcpy(function_unit[14],"V DC");
	strcpy(function_unit[16],"mBar");
	}



/*
// *****   do_sql   -   Ausfhren eines Querys   *****

void do_sql(sqlite3 *ptr,char kommando[],unsigned char wait_for_write)
	{
	signed long int t;
	char *zErrMsg = 0;
	unsigned short int max_retries = 30;
	unsigned short int retries = 0;
	char kommando2[512];


	if(wait_for_write > 0)
		{
		strcpy(kommando2,"PRAGMA count_changes = 0; ");
		strcat(kommando2,kommando);
		}
	else
		{
		strcpy(kommando2,"PRAGMA count_changes = 0; PRAGMA synchronous = 0; ");
		strcat(kommando2,kommando);
		}
again:
	t = sqlite3_exec(ptr,kommando2,NULL,NULL,&zErrMsg);

	if (t != SQLITE_OK)
		{
		if((t == SQLITE_BUSY) || (t == SQLITE_LOCKED))
			{
			usleep(33333);																		// 33 ms warten
			retries++;
			// printf("Retry %d...\n",retries);
			if(retries <= max_retries) goto again;						// wieder probieren
			// printf("Nach %d Versuchen: ",retries);						// max. 1 Sekunde
			}
    fprintf(stderr,"(SQL: \"%s\"\n",kommando);
    fprintf(stderr,"SQL error #%ld: %s\n",t,zErrMsg);
		sqlite3_free(zErrMsg);
		}
	sqlite3_free(zErrMsg);
	}
*/



// *****   sql_erase_sensorvalues   -   Lschen und Neuanlegen der Sensorwert-Tabelle   *****

void sql_erase_sensorvalues(sqlite3 *ptr)
	{
	char sql[255];
	unsigned long int t;

	// unlink(VAR_DATABASE_FILE);														// evtl. vorhandene alte DB lschen
	do_sqlyy(ptr,"DROP TABLE [sensors];",0);
	do_sqlyy(ptr,"CREATE TABLE [sensors] ([id] INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,[sensor_id] INTEGER NULL,[sensor_name] NVARCHAR(100) NULL,[sensor_type] INTEGER NULL,[sensor_str] NVARCHAR(100) NULL,[sensor_float] FLOAT NULL,[sensor_int] INTEGER NULL);",0);
	// do_sqlyy(ptr,"DELETE FROM sensors;",0);
	for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		sql_sensorvalue_old_float[t] = -2048000.0;
		sprintf(sql,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"%ld\",\"%s\",\"%d\",\"-204800000\",\"%4.2f\",\"-2048000.00\");",t,logical_name[t],logical_function[t],sql_sensorvalue_old_float[t]);
		// printf("SQL: \"%s\"\n",sql);
		do_sqlyy(ptr,sql,0);
		}
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-300\",\"Overall Sensor Time ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-290\",\"Sensor Poll read ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-289\",\"Sensor Poll read sensors\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-280\",\"Sensor Poll write ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-279\",\"Sensor Poll write sensors\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-270\",\"Remote Poll ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-260\",\"Schaltmatrix ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-250\",\"Timers ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-240\",\"matrix_actor_copy ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-230\",\"Alerts ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-220\",\"sensor_copyvalues ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-210\",\"actor_copy_from_shm ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-200\",\"logic_copy_to_shm ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-190\",\"actor_copyvalues ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-180\",\"LCD ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-170\",\"write data file ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-160\",\"update_sensorvalues ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-150\",\"update_actorvalues ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-110\",\"Sensor Data Overhead 1 ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-100\",\"Sensor Data Overhead 2 ms\",\"-3\",\"-204800000\",\"-2048888.0\",\"-2048000.00\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_id,sensor_name,sensor_type,sensor_int,sensor_float,sensor_str) VALUES (\"-1\",\"TI\",\"-2\",\"0\",\"0.0\",\"0\")",0);
	do_sqlyy(ptr,"INSERT INTO sensors (sensor_name,sensor_id,sensor_type,sensor_str,sensor_float,sensor_int) VALUES (\"Date, Time and Timestamp\",2048000,2048000,\"(reboot)\",0,0);",0);


	do_sqlyy(ptr,"DROP TABLE [actors];",0);
	do_sqlyy(ptr,"CREATE TABLE [actors] ([id] INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,[actor_id] INTEGER NULL,[actor_name] NVARCHAR(100) NULL,[actor_type] INTEGER NULL,[actor_write] INTEGER NULL,[actor_read] INTEGER NULL);",0);
	for(t = 0;t < MAX_LOGICAL_ACTORS;t++)
		{
		sql_actorvalue_old_read[t] = -204800000;
		sql_actorvalue_old_write[t] = -204800000;
		sprintf(sql,"INSERT INTO actors (actor_id,actor_name,actor_type,actor_read,actor_write) VALUES (\"%ld\",\"%s\",\"%d\",\"-204800000\",\"-204800000\");",t,actor_name[t],actor_type[t]);
		do_sqlyy(ptr,sql,0);
		}
	do_sqlyy(ptr,"INSERT INTO actors (actor_id,actor_name,actor_type,actor_read,actor_write) VALUES (\"-1\",\"TI\",\"-2\",\"-1\",\"-1\")",0);
	}



// *****   sql_update_sensorvalues   -   Aktualisieren der Sensorwerte   *****

void sql_update_sensorvalues(sqlite3 *ptr)
	{
	char sql[255];
	unsigned short int t;
	static unsigned long int z = 0;
	time_t timer;
	struct tm *tblock;


	timer = time(NULL);
	tblock = localtime(&timer);
	z++;
	for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		if(sql_sensorvalue_old_float[t] != logical_float[t])
			{
			sql_sensorvalue_old_float[t] = logical_float[t];
			sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\",sensor_float=\"%4.2f\",sensor_str=\"%s\" WHERE sensor_id=\"%d\";",logical_int[t],logical_float[t],logical_string[t],t);
			do_sqlyy(ptr,sql,0);
			}
		}

	// Datum und Uhrzeit
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%4.2f\",sensor_str=\"%02d.%02d.%04d %02d:%02d:%02d\" WHERE sensor_id=\"2048000\";",
					timer,(double)z,tblock->tm_mday,tblock->tm_mon + 1,tblock->tm_year+1900,tblock->tm_hour,tblock->tm_min,tblock->tm_sec);
	do_sqlyy(ptr,sql,0);
	}



// *****   sql_update_timevalues   -   Aktualisieren der Statistikwerte   *****

void sql_update_timevalues(sqlite3 *ptr)
	{
	char sql[255];


	// TI Counter
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\", sensor_str=\"%ld\" WHERE sensor_id=\"-1\";",ti,ti,ti);
	do_sqlyy(ptr,sql,0);

	// Sensorpolling Read
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-290\";",
					zeit_pollr,zeit_pollr,zeit_pollr);
	do_sqlyy(ptr,sql,0);

	// Sensorpolling Read Sensors
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-289\";",
					anzahl_pollr,anzahl_pollr,anzahl_pollr);
	do_sqlyy(ptr,sql,0);

	// Sensorpolling Write
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-280\";",
					zeit_pollw,zeit_pollw,zeit_pollw);
	do_sqlyy(ptr,sql,0);

	// Sensorpolling Write Sensors
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-279\";",
					anzahl_pollw,anzahl_pollw,anzahl_pollw);
	do_sqlyy(ptr,sql,0);

	// zeit_remote_poll
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-270\";",
					zeit_remote_poll,zeit_remote_poll,zeit_remote_poll);
	do_sqlyy(ptr,sql,0);

	// zeit_matrix
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-260\";",
					zeit_matrix,zeit_matrix,zeit_matrix);
	do_sqlyy(ptr,sql,0);

	// zeit_timer
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-250\";",
					zeit_timer,zeit_timer,zeit_timer);
	do_sqlyy(ptr,sql,0);

	// zeit_matrix_actor_copy
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-240\";",
					zeit_matrix_actor_copy,zeit_matrix_actor_copy,zeit_matrix_actor_copy);
	do_sqlyy(ptr,sql,0);

	// zeit_alert
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-230\";",
					zeit_alert,zeit_alert,zeit_alert);
	do_sqlyy(ptr,sql,0);

	// zeit_sensor_copyvalues
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-220\";",
					zeit_sensor_copyvalues,zeit_sensor_copyvalues,zeit_sensor_copyvalues);
	do_sqlyy(ptr,sql,0);

	// zeit_actor_copy_from_shm
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-210\";",
					zeit_actor_copy_from_shm,zeit_actor_copy_from_shm,zeit_actor_copy_from_shm);
	do_sqlyy(ptr,sql,0);

	// zeit_logic_copy_to_shm
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-200\";",
					zeit_logic_copy_to_shm,zeit_logic_copy_to_shm,zeit_logic_copy_to_shm);
	do_sqlyy(ptr,sql,0);

	// zeit_actor_copyvalues
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-190\";",
					zeit_actor_copyvalues,zeit_actor_copyvalues,zeit_actor_copyvalues);
	do_sqlyy(ptr,sql,0);

	// zeit_lcd
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-180\";",
					zeit_lcd,zeit_lcd,zeit_lcd);
	do_sqlyy(ptr,sql,0);

	// zeit_schreibe_datenfile
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-170\";",
					zeit_schreibe_datenfile,zeit_schreibe_datenfile,zeit_schreibe_datenfile);
	do_sqlyy(ptr,sql,0);

	// zeit_update_sensorvalues
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-160\";",
					zeit_update_sensorvalues,zeit_update_sensorvalues,zeit_update_sensorvalues);
	do_sqlyy(ptr,sql,0);

	// zeit_update_actorvalues
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-150\";",
					zeit_update_actorvalues,zeit_update_actorvalues,zeit_update_actorvalues);
	do_sqlyy(ptr,sql,0);

	// Verwaltungsoverhead 1
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-110\";",
					zeit_verwaltung1,zeit_verwaltung1,zeit_verwaltung1);
	do_sqlyy(ptr,sql,0);

	// Verwaltungsoverhead 2
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-100\";",
					zeit_verwaltung2,zeit_verwaltung2,zeit_verwaltung2);
	do_sqlyy(ptr,sql,0);

	// Gesamtzeit
	sprintf(sql,"UPDATE sensors SET sensor_int=\"%ld\", sensor_float=\"%ld\",sensor_str=\"%ld\" WHERE sensor_id=\"-300\";",
					zeit_gesamt,zeit_gesamt,zeit_gesamt);
	do_sqlyy(ptr,sql,0);
	}



// *****   sql_update_actorvalues   -   Aktualisieren der Aktorwerte   *****

void sql_update_actorvalues(sqlite3 *ptr)
	{
	char sql[255];
	unsigned short int t;
	static unsigned long int z = 0;


	z++;

	// READ
	for(t = 0;t < MAX_LOGICAL_ACTORS;t++)
		{
		if(sql_actorvalue_old_read[t] != shm_actor_r_data[t].value_long)
			{
			sql_actorvalue_old_read[t] = shm_actor_r_data[t].value_long;
			sprintf(sql,"UPDATE actors SET actor_read=\"%ld\" WHERE actor_id=\"%d\";",shm_actor_r_data[t].value_long,t);
			do_sqlyy(ptr,sql,0);
			}
		}

	// WRITE
	for(t = 0;t < MAX_LOGICAL_ACTORS;t++)
		{
		if(sql_actorvalue_old_write[t] != shm_actor_w_data[t].value_long)
			{
			sql_actorvalue_old_write[t] = shm_actor_w_data[t].value_long;
			sprintf(sql,"UPDATE actors SET actor_write=\"%ld\" WHERE actor_id=\"%d\";",shm_actor_w_data[t].value_long,t);
			do_sqlyy(ptr,sql,0);
			}
		}

	// TI Counter
	sprintf(sql,"UPDATE actors SET actor_read=\"%ld\" WHERE actor_id=\"-1\";",z);
	do_sqlyy(ptr,sql,0);
	}



// *****   matrix_actor_copy()   -   Umkopieren genderter Werte vom Zwischenpuffer ins SHM   *****

void matrix_actor_copy(void)
	{
	unsigned long int t;

	for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		if(matrix_actor_flag[t] > 0)
			{
			// printf("COPY Actor %ld: %ld\n",t,matrix_actor_value[t]);
			shm_actor_w_data[t].value_long = matrix_actor_value[t];
			shm_actor_w_data[t].value_float = matrix_actor_value[t];
			sprintf(shm_actor_w_data[t].value_string,"%ld",matrix_actor_value[t]);
			matrix_actor_flag[t] = 0;
			}
		}
	}



// *****   logic_copy_to_shm()   -   logische Sensoren in Shared-Memory umkopieren   *****
// darf nicht in sensor_demon.h stehen, weil dann die Reihenfolge der #includes nicht stimmt...

void logic_copy_to_shm(void)
	{
	unsigned long int t;

	for (t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		shm_sensordata[t].value_float = logical_float[t];
		shm_sensordata[t].value_long = logical_int[t];
		strcpy(shm_sensordata[t].value_string,logical_string[t]);
		}
	(*shm_sensorcount)++;																		// Counter im SharedMemory erhhen
	}



// *****   logic_analyze_changes()   -   Vernderungen detektieren und merken   *****

void logic_analyze_changes(void)
	{
	unsigned long int t;
	static double alter_sensorwert[MAX_LOGICAL_SENSORS];


	for (t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		logical_change[t] = logical_float[t] - alter_sensorwert[t]; // Differenz bilden
		alter_sensorwert[t] = logical_float[t];							// jetzigen Wert als alten Wert merken

		// if(logical_change[t] != 0)
		//	{
		//	printf("logical Sensor #%ld: change %6.2f (neuer Wert: %6.2f)\n",t,logical_change[t],logical_float[t]);
		//	}
		}
	}



// *****   actor_copy_bootdefault_to_shm()   *****
// darf nicht in sensor_demon.h stehen, weil dann die Reihenfolge der #includes nicht stimmt...

void actor_copy_bootdefault_to_shm(void)
	{
	unsigned long int t;

	for (t = 0;t < MAX_LOGICAL_ACTORS;t++)
		{
		shm_actor_w_data[t].value_long = actormapping_bootdefault[t];
		}
	}



// *****   actor_copy_from_shm()   -   logische Aktoren aus Shared-Memory in Sensordaten umkopieren und in SQLDB schreiben   *****
// darf nicht in sensor_demon.h stehen, weil dann die Reihenfolge der #includes nicht stimmt...

void actor_copy_from_shm(void)
	{
	unsigned long int t;
	signed long int e;
	// char ep[300];

	for (t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		if (actor_enabled[t] > 0)
			{
			e = actormapping_mapto_sensor[t];
			if (e >= 0)
				{
				// printf("Mappe I/O Status von Aktor %ld auf Sensor %ld\n",t,e);
				if (logical_int[e] != shm_actor_r_data[t].value_long)
					{
					logical_int[e] = shm_actor_r_data[t].value_long;
					logical_float[e] = logical_int[e];
					sprintf(logical_string[e],"%ld",logical_int[e]);
/*
					// Schreiben in DB
					if (logical_int[e] != logical_old_int[e])
						{
						logical_old_int[e] = logical_int[e];
						sprintf(ep,"/control/sensors_logical/sensor%ld/value_int",e);
						sqldb_write_long2(sql_config_handle,ep,logical_int[e]);
						}
					if (logical_float[e] != logical_old_float[e])
						{
						logical_old_float[e] = logical_float[e];
						sprintf(ep,"/control/sensors_logical/sensor%ld/value_float",e);
						sqldb_write_float2(sql_config_handle,ep,logical_float[e]);
						}
					if (strcmp(logical_string[e],logical_old_string[e]) != 0)
						{
						strcpy(logical_old_string[e],logical_string[e]);
						sprintf(ep,"/control/sensors_logical/sensor%ld/value_string",e);
						sqldb_write_string2(sql_config_handle,ep,logical_string[e]);
						}
*/
					}
				}
			e = actormapping_wattmapto_sensor[t];
			if (e >= 0)
				{
				// printf("Mappe Watt Status von Aktor %ld auf Sensor %ld\n",t,e);
				if (logical_int[e] != shm_actor_r_data[t].value_long)
					{
					logical_float[e] = shm_actor_r_data[t].value_float;
					logical_int[e] = logical_float[e] * 100;
					sprintf(logical_string[e],"%4.2f",logical_float[e]);
/*
					// Schreiben in DB
					if (logical_int[e] != logical_old_int[e])
						{
						logical_old_int[e] = logical_int[e];
						sprintf(ep,"/control/sensors_logical/sensor%ld/value_int",e);
						sqldb_write_long2(sql_config_handle,ep,logical_int[e]);
						}
					if (logical_float[e] != logical_old_float[e])
						{
						logical_old_float[e] = logical_float[e];
						sprintf(ep,"/control/sensors_logical/sensor%ld/value_float",e);
						sqldb_write_float2(sql_config_handle,ep,logical_float[e]);
						}
					if (strcmp(logical_string[e],logical_old_string[e]) != 0)
						{
						strcpy(logical_old_string[e],logical_string[e]);
						sprintf(ep,"/control/sensors_logical/sensor%ld/value_string",e);
						sqldb_write_string2(sql_config_handle,ep,logical_string[e]);
						}
*/
					}
				}
			}
		}
	}



// **********   actor_copyvalues()   -   Schaltdaten aus SharedMemory Sensor-Outputs umkopieren   **********

void actor_copyvalues(void)
	{
	unsigned short int t;
	unsigned short int i;
	signed short int wert;
	char ep[300];

	for(t = 0;t < MAX_SENSORS;t++)
		{
		if(physical_enabled[t] > 0)
			{
			if(physical_outchannels[t] > 0)
				{
				for(i = 0;i < physical_outchannels[t];i++)
					{
					if(physical_mapfrom_actor[i][t] >= 0)
						{
						wert = 0;
						if(shm_actor_w_data[physical_mapfrom_actor[i][t]].value_long > 0)
							{
							wert = 1;
							}
						physical_outvalue_int[i][t] = wert;							// Den Wert als Bitmuster speichern
						physical_outvalue_float[i][t] = wert;
						sprintf(physical_outvalue_string[i][t],"%d",wert);
						sprintf(ep,"Sensor %d ch%d Outvalue: %d (mapped from Actor %d)",t,i,wert,physical_mapfrom_actor[i][t]);
						log_append(4,ep);
						// READ-Wert auf WRITE-Wert setzen (dirty Trick fr interne Sensoren)
						shm_actor_r_data[physical_mapfrom_actor[i][t]].value_long = shm_actor_w_data[physical_mapfrom_actor[i][t]].value_long;
						}
					}
				}
			else
				{
				}
			}
		}

	}



// *****   timer_setzedefault()   -   Timertabelle auf Standardwerte setzen   *****

void timer_setzedefault(void)
	{
	unsigned short int t;

	for(t = 0;t < MAX_TIMERS;t++)
		{
		timer_enabled[t] = 0;																// Timer Enabled 0/1
		sprintf(timer_name[t],"Timer %d",t);								// Timer Benennung
		timer_start_monday[t] = 0;													// Schaltzeitpunkt Wochentag
		timer_start_tuesday[t] = 0;													// Schaltzeitpunkt Wochentag
		timer_start_wednesday[t] = 0;												// Schaltzeitpunkt Wochentag
		timer_start_thursday[t] = 0;												// Schaltzeitpunkt Wochentag
		timer_start_friday[t] = 0;													// Schaltzeitpunkt Wochentag
		timer_start_saturday[t] = 0;												// Schaltzeitpunkt Wochentag
		timer_start_sunday[t] = 0;													// Schaltzeitpunkt Wochentag
		timer_start_hour[t] = 0;														// Uhrzeit Stunde
		timer_start_minute[t] = 0;													// Uhrzeit Minute
		timer_start_second[t] = 0;													// Uhrzeit Sekunde
		timer_start_onoff[t] = 1;														// On/Off 0/1
		timer_start_actor[t] = -1;													// Ziel-Aktor
		timer_stop_monday[t] = 0;														// Schaltzeitpunkt Wochentag
		timer_stop_tuesday[t] = 0;													// Schaltzeitpunkt Wochentag
		timer_stop_wednesday[t] = 0;												// Schaltzeitpunkt Wochentag
		timer_stop_thursday[t] = 0;													// Schaltzeitpunkt Wochentag
		timer_stop_friday[t] = 0;														// Schaltzeitpunkt Wochentag
		timer_stop_saturday[t] = 0;													// Schaltzeitpunkt Wochentag
		timer_stop_sunday[t] = 0;														// Schaltzeitpunkt Wochentag
		timer_stop_hour[t] = 0;															// Uhrzeit Stunde
		timer_stop_minute[t] = 0;														// Uhrzeit Minute
		timer_stop_second[t] = 0;														// Uhrzeit Sekunde
		timer_stop_onoff[t] = 0;														// On/Off 0/1
		timer_stop_actor[t] = -1;														// Ziel-Aktor
		}
	}



// **********   Timer-Konfiguration einlesen   **********

void timer_readconfig(void)
	{
	char ep[1024];
	signed long int t;
	signed long int i;
	signed long int lcount;

	i = sqldb_read_long2(sql_config_handle,"/timers/max_timer");
	if(i > MAX_TIMERS)
		{
		sprintf(ep,"Configured value /timers/max_timer is %ld; larger than MAX_TIMERS, using MAX_TIMERS=%d instead.",i,MAX_TIMERS);
		log_append(2,ep);																			// Fehlermeldung ins Logfile schreiben
		timer_max_timers = MAX_TIMERS;
		}

	timer_setzedefault();
	lcount = 0;
	timer_reload_counter = sqldb_read_long2(sql_config_handle,"/timers/reload_timer");

	for(t = 0;t < i;t++)
		{
		sprintf(ep,"/timers/timer%ld/enabled",t);
		timer_enabled[t] = sqldb_read_long2(sql_config_handle,ep);
		if(timer_enabled[t] > 0)
			{
			lcount++;

			sprintf(ep,"/timers/timer%ld/name",t);
			sqldb_read_string2(sql_config_handle,ep,timer_name[t]);

			sprintf(ep,"/timers/timer%ld/start/mo",t);
			timer_start_monday[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/start/tu",t);
			timer_start_tuesday[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/start/we",t);
			timer_start_wednesday[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/start/th",t);
			timer_start_thursday[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/start/fr",t);
			timer_start_friday[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/start/sa",t);
			timer_start_saturday[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/start/su",t);
			timer_start_sunday[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/start/hour",t);
			timer_start_hour[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/start/min",t);
			timer_start_minute[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/start/sec",t);
			timer_start_second[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/start/action",t);
			timer_start_onoff[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/start/actor",t);
			timer_start_actor[t] = sqldb_read_long2(sql_config_handle,ep);

			sprintf(ep,"/timers/timer%ld/stop/mo",t);
			timer_stop_monday[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/stop/tu",t);
			timer_stop_tuesday[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/stop/we",t);
			timer_stop_wednesday[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/stop/th",t);
			timer_stop_thursday[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/stop/fr",t);
			timer_stop_friday[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/stop/sa",t);
			timer_stop_saturday[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/stop/su",t);
			timer_stop_sunday[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/stop/hour",t);
			timer_stop_hour[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/stop/min",t);
			timer_stop_minute[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/stop/sec",t);
			timer_stop_second[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/stop/action",t);
			timer_stop_onoff[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/timers/timer%ld/stop/actor",t);
			timer_stop_actor[t] = sqldb_read_long2(sql_config_handle,ep);
			}
		else
			{
			}
		}

	sprintf(ep,"Configuration data for %ld Timers read.",lcount);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	}



// *****   timer_execute   -   Timers prfen und abarbeiten   *****

void timer_execute(void)
	{
	char ep[256];
	signed long int t;
	signed long int now_sec;
	signed char now_tag;
	signed long int timer_sec;
	time_t timer;
	struct tm *tblock;


	timer = time(NULL);
	tblock = localtime(&timer);
	now_sec = tblock->tm_hour * 3600 + tblock->tm_min * 60 + tblock->tm_sec;
	now_tag = tblock->tm_wday; // 0:Sonntag, 1:Montag, ..., 6:Samstag

	// printf("TX (now=%ldsec, Tag:%d, %02d:%02d:%02d)\n",now_sec,now_tag,tblock->tm_hour,tblock->tm_min,tblock->tm_sec);
	for(t = 0;t < MAX_TIMERS;t++)
		{
		if(timer_enabled[t] > 0)
			{
			if ( ((timer_start_monday[t] > 0) && (now_tag == 1))
				|| ((timer_start_tuesday[t] > 0) && (now_tag == 2))
				|| ((timer_start_wednesday[t] > 0) && (now_tag == 3))
				|| ((timer_start_thursday[t] > 0) && (now_tag == 4))
				|| ((timer_start_friday[t] > 0) && (now_tag == 5))
				|| ((timer_start_saturday[t] > 0) && (now_tag == 6))
				|| ((timer_start_sunday[t] > 0) && (now_tag == 0)) )
				{
				timer_sec = timer_start_hour[t] * 3600 + timer_start_minute[t] * 60 + timer_start_second[t];
				// printf("T%ld (start=%ldsec, %02d:%02d:%02d)\n",t,timer_sec,timer_start_hour[t],timer_start_minute[t],timer_start_second[t]);
				if ((now_sec >= timer_sec) && (now_sec <= timer_sec + TIMER_GRACE_PERIOD))
					{
					// printf("TimerStart%ld: Actor%d:%d\n",t,timer_start_actor[t],timer_start_onoff[t]);
					if(timer_start_onoff[t] > 0)
						{
						matrix_actor_value[timer_start_actor[t]] = 1;
						matrix_actor_flag[timer_start_actor[t]] = 1;
						sprintf(ep,"TimerStart%ld: tActor %d ON",t,timer_start_actor[t]);
						log_append(4,ep);															// Meldung ins Logfile schreiben
						}
					else
						{
						matrix_actor_value[timer_start_actor[t]] = 0;
						matrix_actor_flag[timer_start_actor[t]] = 1;
						sprintf(ep,"TimerStart%ld: tActor %d OFF",t,timer_start_actor[t]);
						log_append(4,ep);															// Meldung ins Logfile schreiben
						}
					}
				}

			if ( ((timer_stop_monday[t] > 0) && (now_tag == 1))
				|| ((timer_stop_tuesday[t] > 0) && (now_tag == 2))
				|| ((timer_stop_wednesday[t] > 0) && (now_tag == 3))
				|| ((timer_stop_thursday[t] > 0) && (now_tag == 4))
				|| ((timer_stop_friday[t] > 0) && (now_tag == 5))
				|| ((timer_stop_saturday[t] > 0) && (now_tag == 6))
				|| ((timer_stop_sunday[t] > 0) && (now_tag == 0)) )
				{
				timer_sec = timer_stop_hour[t] * 3600 + timer_stop_minute[t] * 60 + timer_stop_second[t];
				if ((now_sec >= timer_sec) && (now_sec <= timer_sec + TIMER_GRACE_PERIOD))
					{
					// printf("TimerStop%ld: Actor%d:%d\n",t,timer_start_actor[t],timer_start_onoff[t]);
					if(timer_stop_onoff[t] > 0)
						{
						matrix_actor_value[timer_stop_actor[t]] = 1;
						matrix_actor_flag[timer_stop_actor[t]] = 1;
						sprintf(ep,"TimerStop%ld: tActor %d ON",t,timer_stop_actor[t]);
						log_append(4,ep);															// Meldung ins Logfile schreiben
						}
					else
						{
						matrix_actor_value[timer_stop_actor[t]] = 0;
						matrix_actor_flag[timer_stop_actor[t]] = 1;
						sprintf(ep,"TimerStop%ld: tActor %d OFF",t,timer_stop_actor[t]);
						log_append(4,ep);															// Meldung ins Logfile schreiben
						}
					}
				}

			}
		}
	}



// *****   alert_setzedefault()   -   Alerttabelle auf Standardwerte setzen   *****

void alert_setzedefault(void)
	{
	unsigned short int t;

	for(t = 0;t < MAX_ALERTS;t++)
		{
		alert_enabled[t] = 0;																// Enabled 0/1
		alert_name[t][0] = 0;																// Benennung
		alert_description[t][0] = 0;												// Blatext
		alert_sensor[t] = -1;																// Sensornummer, die zu berwachen ist
		alert_treshold[t] = 0.0;														// Grenzwert
		alert_comparison[t] = 0;														// Vergleichsoperator
		alert_email_to[t] = -1;															// Email-Adressnummer
		alert_email_subject[t][0] = 0;											// Email-Subject
		alert_sms_to[t] = -1;																// SMS-Adressnummer
		alert_sms_subject[t][0] = 0;												// SMS-Subject
		alert_actor[t] = -1;																// zu schaltender Aktor
		alert_onoff[t] = 0;																	// Schaltwunsch 0/1
		alert_status[t] = -1;																// Alarm-Status (0:kein Alarm, 1:Alarm, -1:undefiniert)
		alert_okcount[t] = 0;																// OK-Zhler (fr kein Alarm)
		alert_errcount[t] = 0;															// Error-Zhler (fr Alarm)
		alert_script[t][0] = 0;															// Kommando lschen
		alert_include_values[t] = 0;												// Mewerte mitschicken 0/1
		}
	}



// **********   Alert-Konfiguration einlesen   **********

void alert_readconfig(void)
	{
	char ep[1024];
	signed long int t;
	signed long int i;
	signed long int lcount;

	i = sqldb_read_long2(sql_config_handle,"/alerts/max_alert");
	if(i > MAX_ALERTS)
		{
		sprintf(ep,"Configured value /alerts/max_alert is %ld; larger than MAX_ALERTS, using MAX_ALERTS=%d instead.",i,MAX_ALERTS);
		log_append(2,ep);																			// Fehlermeldung ins Logfile schreiben
		alert_max_alerts = MAX_ALERTS;
		}

	alert_setzedefault();
	lcount = 0;
	alert_reload_counter = sqldb_read_long2(sql_config_handle,"/alerts/reload_alert");

	for(t = 0;t < i;t++)
		{
		sprintf(ep,"/alerts/alert%ld/enabled",t);
		alert_enabled[t] = sqldb_read_long2(sql_config_handle,ep);
		if(alert_enabled[t] > 0)
			{
			lcount++;
			sprintf(ep,"/alerts/alert%ld/name",t);
			sqldb_read_string2(sql_config_handle,ep,alert_name[t]);
			sprintf(ep,"/alerts/alert%ld/sensor",t);
			alert_sensor[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/alerts/alert%ld/constant",t);
			alert_treshold[t] = sqldb_read_float2(sql_config_handle,ep);
			sprintf(ep,"/alerts/alert%ld/operator",t);
			alert_comparison[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/alerts/alert%ld/recipient",t);
			alert_email_to[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/alerts/alert%ld/subject",t);
			sqldb_read_string2(sql_config_handle,ep,alert_email_subject[t]);
			sprintf(ep,"/alerts/alert%ld/smsrecipient",t);
			alert_sms_to[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/alerts/alert%ld/smssubject",t);
			sqldb_read_string2(sql_config_handle,ep,alert_sms_subject[t]);
			sprintf(ep,"/alerts/alert%ld/actor",t);
			alert_actor[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/alerts/alert%ld/actor_onoff",t);
			alert_onoff[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/alerts/alert%ld/script",t);
			sqldb_read_string2(sql_config_handle,ep,alert_script[t]);
			sprintf(ep,"/alerts/alert%ld/include_values",t);
			alert_include_values[t] = sqldb_read_long2(sql_config_handle,ep);
			}
		}

	sprintf(ep,"Configuration data for %ld Alerts read.",lcount);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	}



// *****   alert_execute   -   Alerts prfen und abarbeiten   *****

void alert_execute(void)
	{
	char ep[256];
	signed long int t;
	signed char wahr;
	double wert_links;
	double wert_rechts;
	char op[9];
	FILE *f;
	char alert_body[65536];																// Body der Alert-Email
	char smtp_username[80];
	char smtp_password[80];
	char smtp_server[80];
	char smtp_sender[80];
	char alert_recipient[80];
	char email_absender[80];
	signed short int smtp_encoding = 0;
	char sms_device[80];
	char kommando[2048];


	for(t = 0;t < MAX_ALERTS;t++)
		{
		if(alert_enabled[t] > 0)
			{
			// lvalue holen
			if(alert_sensor[t] >= 0)
				{
				wert_links = logical_float[alert_sensor[t]];
				}
			else
				{
				wert_links = -2048000.00;
				}			
			// rvalue holen
			wert_rechts = alert_treshold[t];

			// Vergleichsoperation durchfhren
			wahr = 0;
			if(alert_comparison[t] == 0)	{ strcpy(op, "<"); if(wert_links <  wert_rechts) wahr = 1; }
			if(alert_comparison[t] == 1)	{ strcpy(op,"<="); if(wert_links <= wert_rechts) wahr = 1; }
			if(alert_comparison[t] == 2)	{ strcpy(op, "="); if(wert_links == wert_rechts) wahr = 1; }
			if(alert_comparison[t] == 3)	{ strcpy(op,">="); if(wert_links >= wert_rechts) wahr = 1; }
			if(alert_comparison[t] == 4)	{ strcpy(op, ">"); if(wert_links >  wert_rechts) wahr = 1; }
			if(alert_comparison[t] == 5)	{ strcpy(op,"<>"); if(wert_links != wert_rechts) wahr = 1; }
			// printf("Alert %ld: lvalue:%4.2f %s rvalue:%4.2f, Wahr:%d, OK:%d, Err:%d, Status:%d\n",t,wert_links,op,wert_rechts,wahr,alert_okcount[t],alert_errcount[t],alert_status[t]);
			sprintf(ep,"Alert %ld: lvalue:%4.2f %s rvalue:%4.2f, Wahr:%d",t,wert_links,op,wert_rechts,wahr);
			log_append(4,ep);																	// Meldung ins Logfile schreiben
			if(wahr == 0)
				{
				if(alert_okcount[t] < ALERT_TRESHOLD_COUNT)
					{
					alert_okcount[t]++;														// OK-Counter hochzhlen
					alert_errcount[t] = 0;												// Error-Counter lschen
					if(alert_okcount[t] >= ALERT_TRESHOLD_COUNT)
						{
						alert_status[t] = 0;
						// printf("no Alert %ld !!!\n",t);
						}
					}
				}
			else
				{
				if(alert_errcount[t] < ALERT_TRESHOLD_COUNT)
					{
					alert_errcount[t]++;													// Error-Counter hochzhlen
					alert_okcount[t] = 0;													// OK-Counter lschen
					if(alert_errcount[t] >= ALERT_TRESHOLD_COUNT)
						{
						alert_status[t] = 1;
						printf("ALERT %ld !!!\n",t);
						sprintf(ep,"Alert %ld: lvalue:%4.2f %s rvalue:%4.2f",t,wert_links,op,wert_rechts);
						log_append(3,ep);														// Meldung ins Logfile schreiben

						// Email senden
						if (alert_email_to[t] >= 0)
							{
							sprintf(ep,"/alerts/alert%ld/body",t);
							sqldb_read_string2(sql_config_handle,ep,alert_body);
							sqldb_read_string2(sql_config_handle,"/sys/network/mail/user",smtp_username);
							sqldb_read_string2(sql_config_handle,"/sys/network/mail/pass",smtp_password);
							sqldb_read_string2(sql_config_handle,"/sys/network/mail/smtp",smtp_server);
							smtp_encoding = sqldb_read_long2(sql_config_handle,"/sys/network/mail/encoding");
							sqldb_read_string2(sql_config_handle,"/sys/network/mail/sender",smtp_sender);
							sprintf(ep,"/sys/network/mail/recipient%d",alert_email_to[t]);
							sqldb_read_string2(sql_config_handle,ep,alert_recipient);
							sqldb_read_string2(sql_config_handle,"/control/devicename",email_absender);

							f = fopen("/tmp/mail.txt","wb");
							fprintf(f,"%s\n",alert_body);
							if (alert_include_values[t] > 0)
								{
								insert_sensor_values(f);
								}
							fprintf(f,"\n-%s\n",email_absender);
							fclose(f);
							versende_email(smtp_encoding,smtp_username,smtp_password,smtp_server,smtp_sender,alert_recipient,alert_email_subject[t],"/tmp/mail.txt");
							}

						// SMS senden
						if (alert_sms_to[t] >= 0)
							{
							sqldb_read_string2(sql_config_handle,"/control/umts/sms_device",sms_device);
							sprintf(ep,"/sys/network/mail/smsrecipient%d",alert_sms_to[t]);
							sqldb_read_string2(sql_config_handle,ep,alert_recipient);
							// allumts  -s -t +491713718950 -b "test von ALL4500 2010.11.29" -i /dev/ttyUSB2
							sprintf(kommando,"allumts  -s -t %s -b \"%s\" -i %s",
											alert_recipient,alert_sms_subject[t],sms_device);
							printf("%s\n",kommando);
							system(kommando);
							}

						// Schalten
						if (alert_actor[t] >= 0)
							{
							if(alert_onoff[t] >= 0)
								{
								matrix_actor_value[alert_actor[t]] = 1;
								matrix_actor_flag[alert_actor[t]] = 1;
								sprintf(ep,"aActor %d ON",alert_actor[t]);
								log_append(3,ep);															// Meldung ins Logfile schreiben
								// printf("aActor %d ON\n",alert_actor[t]);
								}
							else
								{
								matrix_actor_value[alert_actor[t]] = 0;
								matrix_actor_flag[alert_actor[t]] = 1;
								sprintf(ep,"aActor %d OFF",alert_actor[t]);
								log_append(3,ep);															// Meldung ins Logfile schreiben
								// printf("aActor %d OFF\n",alert_actor[t]);
								}
							}

						// Scriptbefehl ausfhren
						if (strlen(alert_script[t]) > 0)
							{
							sprintf(ep,"aScript %d: \"%s\"",alert_actor[t],alert_script[t]);
							log_append(3,ep);																// Meldung ins Logfile schreiben
							system(alert_script[t]);
							}

						}
					}
				}
			}
		}
	}



// *****   Email senden   *****

signed short int versende_email(signed short int smtp_encoding,char smtp_username[],char smtp_password[],char smtp_server[],char from[],char to[],char subject[],char body[])
	{
	signed short int rw = 0;
	signed short int umts_email = 0;
	signed long int i;
	char kommando[2048];
	struct stat st;
	char ep[1024];
	char resultfile[80] = "/tmp/mailresult.txt";
	char resultstring[65536];
	signed short int tries;
	signed short int mail_retries = 5;
	FILE *f;


	tries = 0;
versende:
	umts_email = sqldb_read_long2(sql_config_handle,"/sys/network/umts/lan");
	sprintf(ep,"umts_email=%d",umts_email);
	mail_log_append(4,ep);
	if(umts_email > 0)
		{
		system("ifconfig ppp0 >/tmp/umts.txt 2>/dev/null");
		stat("/tmp/umts.txt",&st);
		i = st.st_size;
		if(i > 40)
			{
			// UMTS-Verbindung besteht bereits und kann verwendet werden
			mail_log_append(4,"** UMTS-Verbindung bereits vorhanden **");
			}
		else
			{
			mail_log_append(4,"UMTS-Verbindung wird aufgebaut...");
			system("/etc/scripts/umtsdial.sh start >/dev/null 2>/dev/null");
			sleep(15);
			}
		}
	else
		{
		mail_log_append(4,"Email wird ber LAN/WLAN versandt.");
		}

	if(strlen(smtp_username) > 0)
		{
		// Username und Passwort vorhanden
		sprintf(ep,"SMTP-Verbindung zum Server \"%s\" mit Username \"%s\" und Passwort \"%s\"",smtp_server,smtp_username,smtp_password);
		mail_log_append(4,ep);
		sprintf(ep,"Subject:\"%s\"",subject);
		mail_log_append(3,ep);
		sprintf(ep,"From:\"%s\", To:\"%s\"",from,to);
		mail_log_append(3,ep);
		if (smtp_encoding == 0)
			{
			// Multipart MIME
			sprintf(kommando,"makemime -a \"Subject: %s\" /tmp/mail.txt > /tmp/mailtmp.tyt; sendmail -w8 -au%s -ap`php -r 'echo base64_decode(\"%s\");'` -S %s -f %s -t %s </tmp/mailtmp.tyt 2>%s",
							subject,smtp_username,smtp_password,smtp_server,from,to,resultfile);
			}
		else
			{
			// Plaintext
			f = fopen("/tmp/mail.hdr","wb");
			if (f != NULL)
				{
				// fprintf("to: %s\n",to);
				fprintf(f,"subject: %s\n\n",subject);
				fclose(f);
				}
			system("cat /tmp/mail.hdr /tmp/mail.txt > /tmp/mailtmp.tyt");
			sprintf(kommando,"sendmail -w8 -au%s -ap`php -r 'echo base64_decode(\"%s\");'` -S %s -f %s -t %s </tmp/mailtmp.tyt 2>%s",
							smtp_username,smtp_password,smtp_server,from,to,resultfile);
			}
		}
	else
		{
		// kein Username
		sprintf(ep,"SMTP-Verbindung zum Server \"%s\" ohne Username und Passwort",smtp_server);
		mail_log_append(4,ep);
		sprintf(ep,"Subject:\"%s\"",subject);
		mail_log_append(3,ep);
		sprintf(ep,"From:\"%s\", To:\"%s\"",from,to);
		mail_log_append(3,ep);
		if (smtp_encoding == 0)
			{
			// Multipart MIME
			sprintf(kommando,"makemime -a \"Subject: %s\" /tmp/mail.txt > /tmp/mailtmp.tyt; sendmail -w8 -S %s -f %s -t %s </tmp/mailtmp.tyt 2>%s",
							subject,smtp_server,from,to,resultfile);
			}
		else
			{
			// Plaintext
			f = fopen("/tmp/mail.hdr","wb");
			if (f != NULL)
				{
				// fprintf("to: %s\n",to);
				fprintf(f,"subject: %s\n\n",subject);
				fclose(f);
				}
			system("cat /tmp/mail.hdr /tmp/mail.txt > /tmp/mailtmp.tyt");
			sprintf(kommando,"sendmail -w8 -S %s -f %s -t %s </tmp/mailtmp.tyt 2>%s",
							smtp_server,from,to,resultfile);
			}
		}

	// printf("%s\n",kommando);
	system(kommando);
	mail_log_append(3,"fertig.");

	i = readfileinstring(resultfile,resultstring);
	if(i > 0)
		{
		sprintf(ep,"ERROR (%ld Bytes)**: \"%s\"",i,resultstring);
		mail_log_append(3,ep);
		tries++;
		if (tries < mail_retries)
			{
			sprintf(ep,"Retry # %d of %d...",tries,mail_retries);
			mail_log_append(3,ep);
			goto versende;
			}
		rw = 0;
		}
	else
		{
		sprintf(ep,"OK");
		rw = 1;
		}
	mail_log_append(4,ep);
	// kill(resultfile);
	return(rw);
	}



// *****   mail_log_append - Daten in Logfile schreiben   *****

void mail_log_append(unsigned char level,char meldung[])
	{
	FILE *f;
	struct tm *ptr;
	time_t lt;
	signed short int mail_log_level = 4;


	mail_log_level = sqldb_read_long2(sql_config_handle,"/sys/network/mail/smtplog");

	if (level <= mail_log_level)													// nur protokollieren, wenn Level hoch genug (= ber Schwelle)
		{
		f = fopen("/www/reports/smtp.log","ab");					// File fr append ffnen
		if (f != NULL)
			{
			lt = time(NULL);
			ptr = localtime(&lt);
			fprintf(f,"%04d-%02d-%02d,%02d:%02d:%02d[%d]%s\n",ptr->tm_year + 1900,ptr->tm_mon + 1,ptr->tm_mday,ptr->tm_hour,ptr->tm_min,ptr->tm_sec,level,meldung);
			fflush(f);
			fclose(f);
			}
		}
	}



// *****   sensor_setzedefault()   -   Sensortabelle auf Standardwerte setzen   *****

void sensor_setzedefault(void)
	{
	unsigned short int t;
	unsigned short int i;

	for(t = 0;t < MAX_SENSORS;t++)
		{
		strcpy(physical_name[t],"");
		strcpy(physical_comment[t],"");
		physical_chipid[t] = 0;
		strcpy(physical_chipname[t],"");
		physical_chipadresse[t] = 0;
		physical_channels[t] = 0;
		physical_outchannels[t] = 0;
		physical_bus[t] = 0;
		physical_group[t] = 0;
		physical_port[t] = 0;
		physical_speed[t] = 100;
		physical_all4000_typ[t] = 0;
		physical_timestamp[t] = time(NULL);
		for(i = 0;i < 8;i++)
			{
			physical_value_int[i][t] = -204800001;
			physical_value_float[i][t] = -2048000.01;
			strcpy(physical_value_string[i][t],"-2048000.01");

			physical_mapto_sensor[i][t] = -1;
			physical_mapto_function[i][t] = 0;
			physical_mapto_math_enabled[i][t] = 0;
			physical_mapto_math_add1[i][t] = 0;
			physical_mapto_math_multiply1[i][t] = 1;
			physical_mapto_math_add2[i][t] = 0;
			physical_mapto_math_multiply2[i][t] = 1;
			physical_outcmd[i][t] = -1;
			physical_outvalue_int[i][t] = -204800001;
			physical_outvalue_float[i][t] = -2048000.01;
			strcpy(physical_outvalue_string[i][t],"-2048000.01");
			physical_mapfrom_actor[i][t] = -1;
			}
		}
	}



// *****   logic_setzedefault()   -   Sensortabelle auf Standardwerte setzen   *****

void logic_setzedefault(void)
	{
	unsigned short int t;
	unsigned short int i;


	for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		strcpy(logical_name[t],"(unused)");
		strcpy(logical_comment[t],"");
		logical_function[t] = 0;
		logical_chipid[t] = 0;
		logical_limit_min[t] = 10;
		logical_limit_max[t] = 85;
		logical_display_min[t] = -20;
		logical_display_max[t] = 125;
		logical_int[t] = -204800001;
		logical_float[t] = -2048000.01;
		strcpy(logical_string[t],"-2048000.01");
		logical_old_int[t] = -204800001;
		logical_old_float[t] = -2048000.01;
		strcpy(logical_old_string[t],"-2048000.01");
		logical_minimum[t] = 2048000.02;
		logical_maximum[t] = -2048000.02;
		logical_minimum_counter[t] = 0;
		logical_maximum_counter[t] = 0;
		logical_timestamp[t] = 0;
		strcpy(logical_svg_bgcolor[t],"000000");
		strcpy(logical_svg_gridcolor[t],"404040");
		strcpy(logical_svg_graphcolor[t],"FFFF00");
		logical_svg_graphstyle[t] = 0;
		logical_meta_enabled[t] = 0;
		for (i = 0;i < 4;i++)
			{
			logical_meta_source[t][i] = -1;
			strcpy(logical_meta_graphcolor[t][i],"FFFF00");
			logical_meta_graphstyle[t][i] = 0;
			}
		logical_math_enabled[t] = 0;
		for (i = 0;i < 4;i++)
			{
			logical_math_operation[t][i] = -1;
			logical_math_source[t][i] = -1;
			logical_math_constant[t][i] = 0;
			}
		}

	alarmcount = 0;
	}



// *****   logical_getfirstfree()   -   erste freie Nummer fr einen logischen Sensor ermitteln   *****

signed short int logical_getfirstfree(void)
	{
	signed short int t;
	signed short int rw;

	for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		if(logical_function[t] <= 0)
			{
			rw = t;
			goto ende;
			}
		}
	rw = MAX_LOGICAL_SENSORS - 1; // -1
ende: 
	// printf("logical_getfirstfree() = %d\n",rw);
	return(rw);
	}



// *****   actor_getfirstfree()   -   erste freie Nummer fr einen logischen Aktor ermitteln   *****

signed short int actor_getfirstfree(void)
	{
	signed short int t;
	signed short int rw;

	for(t = 0;t < MAX_LOGICAL_ACTORS;t++)
		{
		if(actor_enabled[t] <= 0)
			{
			rw = t;
			goto ende;
			}
		}
	rw = MAX_LOGICAL_ACTORS - 1; // -1
ende: 
	// printf("actor_getfirstfree() = %d\n",rw);
	return(rw);
	}



// *****   physical_getfirstfree()   -   erste freie Nummer fr einen physikalischen Sensor ermitteln   *****

signed short int physical_getfirstfree(void)
	{
	signed short int t;
	signed short int rw;

	for(t = 0;t < MAX_SENSORS;t++)
		{
		if(physical_chipid[t] <= 0)
			{
			rw = t;
			goto ende;
			}
		}
	rw = MAX_SENSORS; // -1
ende: return(rw);
	}



// *****   schreibe_datenfile_xml   -   Sensordaten in XML-File schreiben   *****

void schreibe_datenfile_xml(void)
	{
	FILE *f;
	signed short int t;
	char path_tmpfile[80] = "/tmp/sensordata.tmp";
	// char path_all4000_xml[80] = "/www/reports/xml4000.xml";
	// char path_xml[80] = "/www/reports/xml.xml";
	char path_all4000_xml[80] = "/tmp/xml/xml4000.xml";
	char path_xml[80] = "/tmp/xml/sensordata.xml";
	char ep[200];
	struct timeval tv;																		// Uhrzeit-Kram
	time_t curtime;																				// Uhrzeit-Kram
	struct tm *myTm;																			// Uhrzeit-Kram
	unsigned short int jahr;
	unsigned char monat;
	unsigned char tag;
	unsigned char stunde;
	unsigned char minute;
	unsigned char sekunde;
	signed char error;



	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln 
  curtime = tv.tv_sec;																	// Lores-Zeit in s rausziehen
	myTm = localtime(&curtime);														// ...und berfhren in tm-Struktur
	stunde = myTm->tm_hour;																// Stnde
	minute = myTm->tm_min;																// Minute
	sekunde = myTm->tm_sec;																// Sekunde
	jahr = myTm->tm_year + 1900;													// Jahr
	monat = myTm->tm_mon + 1;															// Monat
	tag = myTm->tm_mday;																	// Tag


	// ALL4000-kompatibles File
	f = fopen(path_tmpfile,"wb");
	if(f != NULL)
		{
		fprintf(f,"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\" ?>\n");
		fprintf(f,"<data>\n");
		fprintf(f,"<devicename>%s</devicename>\n",device_name);
		for(t = 0;t < 32;t++)																// 32 wurde willkrlich gewhlt, um die Datei nicht zu gro werden zu lassen
			{
			fprintf(f,"<n%d>%s</n%d>",t,logical_name[t],t);		// Sensorbenennung
			fprintf(f,"<t%d>%4.2f</t%d>",t,logical_float[t],t); // Sensorwert
			fprintf(f,"<min%d>%4.2f</min%d>",t,logical_minimum[t],t); // Sensorwert Minimum
			fprintf(f,"<max%d>%4.2f</max%d>",t,logical_maximum[t],t); // Sensorwert Maximum
			fprintf(f,"<l%d>%4.2f</l%d>",t,logical_limit_min[t],t); // unteres Limit
			fprintf(f,"<h%d>%4.2f</h%d>",t,logical_limit_max[t],t); // oberes Limit
			fprintf(f,"<s%d>%d</s%d>",t,logical_all4000_typ[t],t); // ALL4000 Sensortyp
			error = 0;
			if(logical_float[t] < logical_limit_min[t]) error = -1;
			if(logical_float[t] > logical_limit_max[t]) error = 1;
			fprintf(f,"<alarm%d>%d</alarm%d>\n",t,error,t); // Error: 0:kein Fehler, -1:Unterschreitung, 1:berschreitung
			}
		fprintf(f,"<date>%02d.%02d.%04d</date>\n",tag,monat,jahr);
		fprintf(f,"<time>%02d:%02d:%02d</time>\n",stunde,minute,sekunde);
		fprintf(f,"<dev>%s</dev>\n",device_type);
		fprintf(f,"<sys>%ld</sys>\n",ti);
		fprintf(f,"<alarmcount>%d</alarmcount>\n",alarmcount);

		fprintf(f,"<ad>%d</ad>",demon_all4000_ad);
		fprintf(f,"<i>%d</i>",demon_all4000_muell);
		fprintf(f,"<f>%d</f>",demon_all4000_f);
		fprintf(f,"<sensorx>%d</sensorx>",demon_all4000_sensorx);
		fprintf(f,"<sensory>%d</sensory>",demon_all4000_sensory);
		fprintf(f,"<px>%d</px>",demon_all4000_px);
		fprintf(f,"<py>%d</py>\n",demon_all4000_py);

		fprintf(f,"</data>\n");
		fclose(f);
		t = rename(path_tmpfile,path_all4000_xml);					// POSIX-Version des Unix "mv" Kommandos
		if (t != 0)
			{
			sprintf(ep,"*** ERROR: Could not move file \"%s\" to \"%s\"",path_tmpfile,path_all4000_xml);
			log_append(2,ep);																	// Fehler protokollieren
			}
		}
	else
		{
		sprintf(ep,"*** ERROR: Could not open file \"%s\" for write",path_tmpfile);
		log_append(2,ep);																		// Fehler protokollieren
		}

	// Neue Version
	f = fopen(path_tmpfile,"wb");
	if(f != NULL)
		{
		fprintf(f,"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\" ?>\n");
		fprintf(f,"<data>\n");
		for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
			{
			if(logical_function[t] > 0)
				{
				fprintf(f," <sensor%d>\n",t);										// Sensornummer
				fprintf(f,"  <name>%s</name>\n",logical_name[t]); // Sensorbenennung
				fprintf(f,"  <value_float>%1.2f</value_float>\n",logical_float[t]); // Sensorwert float
				fprintf(f,"  <value_int>%ld</value_int>\n",logical_int[t]); // Sensorwert int
				fprintf(f,"  <value_string>%s</value_string>\n",logical_string[t]); // Sensorwert string
				fprintf(f,"  <minimum>%1.2f</minimum>\n",logical_minimum[t]); // Sensorwert Minimum
				fprintf(f,"  <maximum>%1.2f</maximum>\n",logical_maximum[t]); // Sensorwert Maximum
				fprintf(f,"  <limit_low>%1.2f</limit_low>\n",logical_limit_min[t]); // unteres Limit
				fprintf(f,"  <limit_high>%1.2f</limit_high>\n",logical_limit_max[t]); // oberes Limit
				fprintf(f,"  <function>%d</function>\n",logical_function[t]); // Sensorfunktion
				fprintf(f,"  <all4000_typ>%d</all4000_typ>\n",logical_all4000_typ[t]); // ALL4000 Sensortyp
				error = 0;
				if(logical_float[t] < logical_limit_min[t]) error = -1;
				if(logical_float[t] > logical_limit_max[t]) error = 1;
				fprintf(f,"  <alarm%d>%d</alarm%d>\n",t,error,t); // Error: 0:kein Fehler, -1:Unterschreitung, 1:berschreitung
				fprintf(f," </sensor%d>\n",t);									// Sensornummer
				}
			}
		fprintf(f," <system>\n");
		fprintf(f,"  <date>%02d.%02d.%04d</date>\n",tag,monat,jahr);
		fprintf(f,"  <time>%02d:%02d:%02d</time>\n",stunde,minute,sekunde);
		fprintf(f,"  <devicename>%s</devicename>\n",device_name);
		fprintf(f,"  <devicetype>%s</devicetype>\n",device_type);
		fprintf(f,"  <sys>%ld</sys>\n",ti);
		fprintf(f,"  <alarmcount>%d</alarmcount>\n",alarmcount);
		fprintf(f," </system>\n");
		fprintf(f,"</data>\n");
		fclose(f);
		t = rename(path_tmpfile,path_xml);									// POSIX-Version des Unix "mv" Kommandos
		if (t != 0)
			{
			sprintf(ep,"*** ERROR: Could not move file \"%s\" to \"%s\"",path_tmpfile,path_xml);
			log_append(2,ep);																	// Fehler protokollieren
			}
		}
	else
		{
		sprintf(ep,"*** ERROR: Could not open file \"%s\" for write",path_tmpfile);
		log_append(2,ep);																		// Fehler protokollieren
		}

	}



// **********   Demon-Configfile einlesen   **********

void demon_readconfig(void)
	{
	char ep[1024];
	unsigned short int t;
	unsigned short int i;
	unsigned short int remote_pagecount = 0;
	unsigned short int remote_sensorcount = 0;

	sqldb_write_long2(sql_config_handle,"/control/sensordemon/flag_globalsettings",0); // Merkflag lschen

	sensordemon_loglevel = sqldb_read_long2(sql_config_handle,"/control/sensordemon/log_level");
	log_level = sensordemon_loglevel;
	config_demon_delay = sqldb_read_long2(sql_config_handle,"/control/sensordemon/delay_between_measurements");
	config_sekundenlock = sqldb_read_long2(sql_config_handle,"/control/sensordemon/sekundenlock");
	config_demon_ps_enabled = sqldb_read_long2(sql_config_handle,"/control/sensordemon/powersave/enabled");
	config_demon_ps_offtime = sqldb_read_long2(sql_config_handle,"/control/sensordemon/powersave/offtime");
	config_demon_ps_boottime = sqldb_read_long2(sql_config_handle,"/control/sensordemon/powersave/boottime");
	config_demon_historyinterval = sqldb_read_long2(sql_config_handle,"/control/sensordemon/historyinterval");
	sqldb_read_string2(sql_config_handle,"/control/devicename",device_name);
	sqldb_read_string2(sql_config_handle,"/control/devicetype",device_type);
	ortsueblicher_luftdruck = sqldb_read_float2(sql_config_handle,"/control/sensor_units/local_air_pressure");
	svg_x = sqldb_read_long2(sql_config_handle,"/control/sensordemon/svg_width");
	svg_y = sqldb_read_long2(sql_config_handle,"/control/sensordemon/svg_height");
	logical_svg_includevalues = sqldb_read_long2(sql_config_handle,"/control/sensordemon/svg_include_values");

	sensor_remote_anzahl = sqldb_read_long2(sql_config_handle,"/control/sensors_remote/max_pages");
	for(t = 0;t < sensor_remote_anzahl;t++)
		{
		sprintf(ep,"/control/sensors_remote/page%d/enabled",t);
		sensor_remote_enabled[t] = sqldb_read_long2(sql_config_handle,ep);
		if(sensor_remote_enabled[t] > 0) remote_pagecount++;
		sprintf(ep,"/control/sensors_remote/page%d/local_filename",t);
		sqldb_read_string2(sql_config_handle,ep,sensor_remote_local_filename[t]);
		sprintf(ep,"/control/sensors_remote/page%d/max_sensors",t);
		sensor_remote_max_sensors[t] = sqldb_read_long2(sql_config_handle,ep);
		sprintf(ep,"/control/sensors_remote/page%d/max_age",t);
		sensor_remote_max_age[t] = sqldb_read_long2(sql_config_handle,ep);
		for(i = 0;i < sensor_remote_max_sensors[t];i++)
			{
			sprintf(ep,"/control/sensors_remote/page%d/sensor%d/xmltag",t,i);
			sqldb_read_string2(sql_config_handle,ep,sensor_remote_xmltag[t][i]);
			sprintf(ep,"/control/sensors_remote/page%d/sensor%d/target",t,i);
			sensor_remote_target[t][i] = sqldb_read_long2(sql_config_handle,ep);
			if(sensor_remote_target[t][i] >= 0) remote_sensorcount++;
			sensor_remote_timestamp[t][i] = 0;
			strcpy(sensor_remote_value_string[t][i],"-2048000.0");
			}
		}

	sprintf(ep,"Global Configuration data read. Loglevel:%d, Delay:%ld, PS_enabled:%ld, PS_offtime:%ld, PS_boottime:%ld, air_pressure:%4.2fmBar, History:%d",
					sensordemon_loglevel,config_demon_delay,config_demon_ps_enabled,config_demon_ps_offtime,config_demon_ps_boottime,ortsueblicher_luftdruck,config_demon_historyinterval);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	sprintf(ep,"SVG_X:%d, SVG_Y:%d, Remote:%d, rCount:%d, rXML:%d",svg_x,svg_y,sensor_remote_anzahl,remote_pagecount,remote_sensorcount);
	log_append(3,ep);																			// Meldung ins Logfile schreiben

	demon_all4000_ad = 1;																	// ALL4000 AD
	sprintf(ep,"/control/sensors_display/page0/update_interval_graphical");
	demon_all4000_muell = sqldb_read_long2(sql_config_handle,ep);								// ALL4000 Zeitintervall fr Grafik
	demon_all4000_f = 0;																	// ALL4000 Fahrenheit
	sprintf(ep,"/control/sensors_display/page0/sensorsx");
	demon_all4000_sensorx = sqldb_read_long2(sql_config_handle,ep);					// ALL4000 Sensors Y
	sprintf(ep,"/control/sensors_display/page0/sensorsy");
	demon_all4000_sensory = sqldb_read_long2(sql_config_handle,ep);					// ALL4000 Sensors X
	sprintf(ep,"/control/sensors_display/page0/width_pixels");
	demon_all4000_px = sqldb_read_long2(sql_config_handle,ep);								// ALL4000 Pixels X
	sprintf(ep,"/control/sensors_display/page0/height_pixels");
	demon_all4000_py = sqldb_read_long2(sql_config_handle,ep);								// ALL4000 Pixels Y
	config_demon_lcd_enabled = sqldb_read_long2(sql_config_handle,"/control/sensordemon/lcd_enabled");
	config_demon_lcd_delay = sqldb_read_long2(sql_config_handle,"/control/sensordemon/lcd_delay");

	strcpy(scale_unit[ 0],"");
	strcpy(scale_unit[ 1]," deg.C");
	strcpy(scale_unit[ 2]," % RH");
	strcpy(scale_unit[ 3]," V DC");
	strcpy(scale_unit[ 4]," V AC");
	strcpy(scale_unit[ 5]," A DC");
	strcpy(scale_unit[ 6]," A AC");
	strcpy(scale_unit[ 7]," W");
	strcpy(scale_unit[ 8],"");
	strcpy(scale_unit[ 9],"");
	strcpy(scale_unit[10]," lux");
	strcpy(scale_unit[11],"");
	strcpy(scale_unit[12],"");
	strcpy(scale_unit[13],"");
	strcpy(scale_unit[14]," V DC");
	strcpy(scale_unit[15],"");
	strcpy(scale_unit[16]," mBar");
	max_units = 17;
	}



// **********   alle Aktoren lschen   **********

void actor_setzedefault(void)
	{
	unsigned short int t;
	char ep[40];

	for(t = 0;t < MAX_LOGICAL_ACTORS;t++)
		{
		actor_enabled[t] = 0;
		actor_type[t] = 0;
		sprintf(ep,"Actor %d",t);
		strcpy(actor_name[t],ep);
		actormapping_mapto_sensor[t] = -1;
		actormapping_wattmapto_sensor[t] = -1;
		actormapping_bootdefault[t] = 0;
		}
	}



// **********   Datenbank-Konfiguration einlesen   **********

void db_readconfig(void)
	{
	db_enabled = sqldb_read_long2(sql_config_handle,"/control/database/activated");
	db_fields = sqldb_read_long2(sql_config_handle,"/control/database/fields");
	db_interval = sqldb_read_long2(sql_config_handle,"/control/database/interval");
	sqldb_read_string2(sql_config_handle,"/control/database/name",db_name);
	sqldb_read_string2(sql_config_handle,"/control/database/path",db_path);
	db_type = sqldb_read_long2(sql_config_handle,"/control/database/type");

	db_merker = sqldb_read_long2(sql_config_handle,"/control/database/reload");
	db_timestamp = 0;																			// letzter DB-write
	db_next = time(0);																		// nchster DB-Write
	}



// **********   Sensordaten in die Datenbank schreiben   **********

void db_write_sensordata(void)
	{
	FILE *f;
	// char ep[4096];
	signed long int t;
	char filename[150];
	struct timeval tv;																		// Uhrzeit-Kram
	time_t curtime;																				// Uhrzeit-Kram
	struct tm *myTm;																			// Uhrzeit-Kram
	unsigned short int jahr;
	unsigned char monat;
	unsigned char tag;
	unsigned char stunde;
	unsigned char minute;
	unsigned char sekunde;
	char sql[256000];
	char kommando[257000];
	char uaa[1000];


	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln 
  curtime = tv.tv_sec;																	// Lores-Zeit in s rausziehen
	myTm = localtime(&curtime);														// ...und berfhren in tm-Struktur
	stunde = myTm->tm_hour;																// Stnde
	minute = myTm->tm_min;																// Minute
	sekunde = myTm->tm_sec;																// Sekunde
	jahr = myTm->tm_year + 1900;													// Jahr
	monat = myTm->tm_mon + 1;															// Monat
	tag = myTm->tm_mday;																	// Tag

	if(db_enabled > 0)
		{
		// printf("SQL DB Write\n");
		sprintf(filename,"%s%s",db_path,db_name);
		if(db_type == 0)
			{
			// --- Textdatei ---
			f = fopen(filename,"rb");
			if(f == NULL)
				{
				f = fopen(filename,"wb");
				fprintf(f,"DATE;TIME");
				for(t = 0;t < db_fields;t++)
					{
					fprintf(f,";SENSOR%ld",t);
					}
				fprintf(f,"\n");
				}
			fclose(f);
			f = fopen(filename,"a");
			if(f != NULL)
				{
				fprintf(f,"%04d-%02d-%02d;%02d:%02d:%02d",jahr,monat,tag,stunde,minute,sekunde);
				for(t = 0;t < db_fields;t++)
					{
					fprintf(f,";%4.2f",logical_float[t]);
					}
				fprintf(f,"\n");
				fclose(f);
				}
			}
		if(db_type == 1)
			{
			// --- SQL ---
			sprintf(sql,"INSERT INTO measurements(");
				for(t = 0;t < db_fields;t++)
					{
					if(t > 0) strcat(sql,",");
					sprintf(uaa,"sensor%ld",t);
					strcat(sql,uaa);
					}

			strcat(sql,") VALUES (");
				for(t = 0;t < db_fields;t++)
					{
					if(t > 0) strcat(sql,",");
					sprintf(uaa,"'%4.2f'",logical_float[t]);
					strcat(sql,uaa);
					}
			strcat(sql,");");

			sprintf(kommando,"sqlite3 %s \"%s\"",filename,sql);
			// printf("%s\n",kommando);
			system(kommando);
			}
		}
	}



// **********   Actor-Configfile einlesen   **********

signed short int actor_readconfig(void)
	{
	char ep[1024];
	unsigned short int t;
	unsigned short int i = 0;

	actor_setzedefault();
	actor_reload_counter = sqldb_read_long2(sql_config_handle,"/control/actors/reload_counter");
	actor_anzahl = sqldb_read_long2(sql_config_handle,"/control/actors/max_actors");
	for(t = 0;t < actor_anzahl;t++)
		{
		sprintf(ep,"/control/actors/actor%d/enabled",t);
		actor_enabled[t] = sqldb_read_long2(sql_config_handle,ep);
		if(actor_enabled[t] > 0)
			{
			i++;
			sprintf(ep,"/control/actors/actor%d/name",t);
			sqldb_read_string2(sql_config_handle,ep,actor_name[t]);
			sprintf(ep,"/control/actors/actor%d/type",t);
			actor_type[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/actors/actor%d/bootdefault",t);
			actormapping_bootdefault[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/actors/actor%d/mapto_sensor",t);
			actormapping_mapto_sensor[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/actors/actor%d/wattmapto_sensor",t);
			actormapping_wattmapto_sensor[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"Read Mapping Data for Actor#%d; will be mapped to logical Sensor %ld (Watts to Sensor %ld), Bootefault:%ld",
							t,actormapping_mapto_sensor[t],actormapping_wattmapto_sensor[t],actormapping_bootdefault[t]);
			log_append(4,ep);																	// Meldung ins Logfile schreiben
			}
		}

	sprintf(ep,"Actor configuration for %d Actors read.",i);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	return(i);
	}



// **********   einzelnen Actor schreiben   **********

void actor_write_single(unsigned long int t)
	{
	char ep[40];

	sprintf(ep,"/control/actors/actor%ld/enabled",t);
	sqldb_write_long2(sql_config_handle,ep,actor_enabled[t]);
	sprintf(ep,"/control/actors/actor%ld/name",t);
	sqldb_write_string2(sql_config_handle,ep,actor_name[t]);
	sprintf(ep,"/control/actors/actor%ld/type",t);
	sqldb_write_long2(sql_config_handle,ep,actor_type[t]);
	sprintf(ep,"/control/actors/actormapping%ld/enabled",t);
	sqldb_write_long2(sql_config_handle,ep,actor_enabled[t]);
	sprintf(ep,"/control/actors/actor%ld/bootdefault",t);
	sqldb_write_long2(sql_config_handle,ep,actormapping_bootdefault[t]);
	sprintf(ep,"/control/actors/actor%ld/mapto_sensor",t);
	sqldb_write_long2(sql_config_handle,ep,actormapping_mapto_sensor[t]);
	sprintf(ep,"/control/actors/actor%ld/wattmapto_sensor",t);
	sqldb_write_long2(sql_config_handle,ep,actormapping_wattmapto_sensor[t]);
	}



// **********   Remotesensor-Configfile einlesen   **********

void remote_readconfig(void)
	{
	char ep[1024];
	unsigned short int t;
	unsigned short int i;
	unsigned short int remote_pagecount = 0;
	unsigned short int remote_sensorcount = 0;


	sensor_remote_reload_counter = sqldb_read_long2(sql_config_handle,"/control/sensors_remote/reload_counter");
	sensor_remote_anzahl = sqldb_read_long2(sql_config_handle,"/control/sensors_remote/max_pages");
	for(t = 0;t < sensor_remote_anzahl;t++)
		{
		sprintf(ep,"/control/sensors_remote/page%d/enabled",t);
		sensor_remote_enabled[t] = sqldb_read_long2(sql_config_handle,ep);
		if(sensor_remote_enabled[t] > 0) remote_pagecount++;
		sprintf(ep,"/control/sensors_remote/page%d/local_filename",t);
		sqldb_read_string2(sql_config_handle,ep,sensor_remote_local_filename[t]);
		sprintf(ep,"/control/sensors_remote/page%d/max_sensors",t);
		sensor_remote_max_sensors[t] = sqldb_read_long2(sql_config_handle,ep);
		sprintf(ep,"/control/sensors_remote/page%d/max_age",t);
		sensor_remote_max_age[t] = sqldb_read_long2(sql_config_handle,ep);
		for(i = 0;i < sensor_remote_max_sensors[t];i++)
			{
			sprintf(ep,"/control/sensors_remote/page%d/sensor%d/xmltag",t,i);
			sqldb_read_string2(sql_config_handle,ep,sensor_remote_xmltag[t][i]);
			sprintf(ep,"/control/sensors_remote/page%d/sensor%d/target",t,i);
			sensor_remote_target[t][i] = sqldb_read_long2(sql_config_handle,ep);
			if(sensor_remote_target[t][i] >= 0) remote_sensorcount++;
			sensor_remote_timestamp[t][i] = 0;
			strcpy(sensor_remote_value_string[t][i],"-2048000.0");
			}
		}

	sprintf(ep,"Remote:%d, rCount:%d, rXML:%d",sensor_remote_anzahl,remote_pagecount,remote_sensorcount);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	}



// **********   physikalische Sensor-Konfiguration einlesen   **********

unsigned long int physical_readconfig(void)
	{
	char ep[1024];
	signed long int t;
	signed long int i;
	signed long int scount;
	signed long int mux_count;


	sensor_setzedefault();																// Alle physikalischen Sensoren auf Standardwerte zuruecksetzen
	sqldb_write_long2(sql_config_handle,"/control/sensors_i2c/flag_sensorsettings",0); // Merkflag lschen

	physical_max_sensors = sqldb_read_long2(sql_config_handle,"/control/sensors_i2c/max_sensors");
	if(physical_max_sensors > MAX_SENSORS)
		{
		sprintf(ep,"Configured value /control/sensors_i2c/max_sensors is %ld; larger than MAX_SENSORS, using MAX_SENSORS=%d instead.",physical_max_sensors,MAX_SENSORS);
		log_append(2,ep);																			// Fehlermeldung ins Logfile schreiben
		physical_max_sensors = MAX_SENSORS;
		}

	// printf("physical_max_sensors=\"%ld\":\n",physical_max_sensors);
	scount = 0;
	mux_count = 0;
	for(t = 0;t < physical_max_sensors;t++)
		{
		sprintf(ep,"/control/sensors_i2c/sensor%ld/enabled",t);
		physical_enabled[t] = sqldb_read_long2(sql_config_handle,ep);
		if(physical_enabled[t] > 0)
			{
			scount++;
			sprintf(ep,"/control/sensors_i2c/sensor%ld/name",t);
			sqldb_read_string2(sql_config_handle,ep,physical_name[t]);
			sprintf(ep,"/control/sensors_i2c/sensor%ld/comment",t);
			sqldb_read_string2(sql_config_handle,ep,physical_comment[t]);
			sprintf(ep,"/control/sensors_i2c/sensor%ld/chipid",t);
			physical_chipid[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_i2c/sensor%ld/chipname",t);
			sqldb_read_string2(sql_config_handle,ep,physical_chipname[t]);
			// printf("physical_sensorname[%ld]=\"%s\"\n",t,physical_name[t]);
			// printf("physical_chipname[%ld]=\"%s\"\n",t,physical_chipname[t]);
			sprintf(ep,"/control/sensors_i2c/sensor%ld/chipaddress",t);
			physical_chipadresse[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_i2c/sensor%ld/channels",t);
			physical_channels[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_i2c/sensor%ld/outchannels",t);
			physical_outchannels[t] = sqldb_read_long2(sql_config_handle,ep);
			// printf("physical_channels[%ld]=\"%d\", ",t,physical_channels[t]);
			// printf("physical_outchannels[%ld]=\"%d\"\n",t,physical_outchannels[t]);
			sprintf(ep,"/control/sensors_i2c/sensor%ld/bus",t);
			physical_bus[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_i2c/sensor%ld/group",t);
			physical_group[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_i2c/sensor%ld/port",t);
			physical_port[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_i2c/sensor%ld/speed",t);
			physical_speed[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_i2c/sensor%ld/priority",t);
			physical_priority[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_i2c/sensor%ld/all4000_typ",t);
			physical_all4000_typ[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_i2c/sensor%ld/mux_enabled",t);
			physical_mux_enabled[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_i2c/sensor%ld/mux_port",t);
			physical_mux_port[t] = sqldb_read_long2(sql_config_handle,ep);
			if(physical_mux_enabled[t] > 0) mux_count++;			// gemultiplexte Ports extra zhlen
			sprintf(ep,"Read Config Data for Sensor #%ld: ChipID:%d(%s), Address:%d, Channels:%d, OutChannels:%d, Bus:%d, Group:%d, Port:%d, Speed:%d, ALL4000typ:%d, mux_enabled:%d(Port:%d)",
							t,physical_chipid[t],physical_chipname[t],physical_chipadresse[t],physical_channels[t],physical_outchannels[t],
							physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],physical_all4000_typ[t],
							physical_mux_enabled[t],physical_mux_port[t]);
			log_append(4,ep);																	// Meldung ins Logfile schreiben
			// printf("PRC: %s\n",ep);
			for(i = 0;i < physical_channels[t];i++)
				{
				sprintf(ep,"/control/sensors_i2c/sensor%ld/channel%ld/mapto_sensor",t,i);
				physical_mapto_sensor[i][t] = sqldb_read_long2(sql_config_handle,ep);
				sprintf(ep,"/control/sensors_i2c/sensor%ld/channel%ld/function",t,i);
				physical_mapto_function[i][t] = sqldb_read_long2(sql_config_handle,ep);
				sprintf(ep,"/control/sensors_i2c/sensor%ld/channel%ld/mapto_math_enabled",t,i);
				physical_mapto_math_enabled[i][t] = sqldb_read_long2(sql_config_handle,ep);
				sprintf(ep,"/control/sensors_i2c/sensor%ld/channel%ld/mapto_math_add1",t,i);
				physical_mapto_math_add1[i][t] = sqldb_read_float2(sql_config_handle,ep);
				sprintf(ep,"/control/sensors_i2c/sensor%ld/channel%ld/mapto_math_multiply1",t,i);
				physical_mapto_math_multiply1[i][t] = sqldb_read_float2(sql_config_handle,ep);
				sprintf(ep,"/control/sensors_i2c/sensor%ld/channel%ld/mapto_math_add2",t,i);
				physical_mapto_math_add2[i][t] = sqldb_read_float2(sql_config_handle,ep);
				sprintf(ep,"/control/sensors_i2c/sensor%ld/channel%ld/mapto_math_multiply2",t,i);
				physical_mapto_math_multiply2[i][t] = sqldb_read_float2(sql_config_handle,ep);
				sprintf(ep,"/control/sensors_i2c/sensor%ld/channel%ld/out_value_command",t,i);
				physical_outcmd[i][t] = sqldb_read_long2(sql_config_handle,ep);
				sprintf(ep,"                    Channel #%ld: mapto:%d, math_enabled:%d, add1:%4.2f, mul1:%4.2f, add2:%4.2f, mul2:%4.2f",
								i,physical_mapto_sensor[i][t],physical_mapto_math_enabled[i][t],physical_mapto_math_add1[i][t],
								physical_mapto_math_multiply1[i][t],physical_mapto_math_add2[i][t],physical_mapto_math_multiply2[i][t]);
				log_append(4,ep);																// Meldung ins Logfile schreiben
				}

			for(i = 0;i < physical_outchannels[t];i++)
				{
				sprintf(ep,"/control/sensors_i2c/sensor%ld/channel%ld/out_value_command",t,i);
				physical_outcmd[i][t] = sqldb_read_long2(sql_config_handle,ep);
				sprintf(ep,"/control/sensors_i2c/sensor%ld/channel%ld/out_value_int",t,i);
				physical_outvalue_int[i][t] = sqldb_read_long2(sql_config_handle,ep);
				sprintf(ep,"/control/sensors_i2c/sensor%ld/channel%ld/out_value_float",t,i);
				physical_outvalue_float[i][t] = sqldb_read_long2(sql_config_handle,ep);
				sprintf(ep,"/control/sensors_i2c/sensor%ld/channel%ld/out_value_string",t,i);
				sqldb_read_string2(sql_config_handle,ep,physical_outvalue_string[i][t]);
				sprintf(ep,"/control/sensors_i2c/sensor%ld/channel%ld/mapfrom_actor",t,i);
				physical_mapfrom_actor[i][t] = sqldb_read_long2(sql_config_handle,ep);
				sprintf(ep,"                    OutChannel #%ld: mapfrom:%d, outcmd:%d",
								i,physical_mapfrom_actor[i][t],physical_outcmd[i][t]);
				log_append(4,ep);																// Meldung ins Logfile schreiben
				}

			}
		else
			{
			physical_name[t][0] = 0;													// Name
			physical_comment[t][0] = 0;												// Beschreibung
			physical_chipid[t] = 0;														// Chip-ID
			physical_chipname[t][0] = 0;											// Chipname
			physical_chipadresse[t] = 0;											// Adresse
			physical_channels[t] = 1;													// Anzahl Ports
			physical_outchannels[t] = 0;											// Anzahl Ports
			physical_bus[t] = 0;															// an welchem Bus hngt dieser Sensor ?
			physical_group[t] = 0;														// an welcher Gruppe hngt der Sensor ?
			physical_port[t] = 0;															// an welchem Port dieser Gruppe ist der Sensor ?
			physical_speed[t] = 0;														// Delay-Parameter
			physical_all4000_typ[t] = 0;											// ALL4000 Typ
			physical_mux_enabled[t] = 0;											// ist der Port gemultiplext ?
			physical_mux_port[t] = 0;													// Multiplexer-Portnummer
			}
		}

	sprintf(ep,"Configuration data for %ld physical Sensors read (%ld multiplexed).",scount,mux_count);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	return(scount);
	}



// **********   physikalische Sensor-Konfiguration fr einen Sensor schreiben   **********

void physical_write_single(signed short int nummer)
	{
	char ep[200];
	signed long int i;

	// printf("physical_write_single[%d) \"%s\"\n",nummer,physical_name[nummer]);

	physical_max_sensors = sqldb_read_long2(sql_config_handle,"/control/sensors_i2c/max_sensors");
	if(nummer >= physical_max_sensors)
		{
		physical_max_sensors = nummer + 1;
		sqldb_write_long2(sql_config_handle,"/control/sensors_i2c/max_sensors",physical_max_sensors);
		}

	sprintf(ep,"/control/sensors_i2c/sensor%d/enabled",nummer);
	sqldb_write_long2(sql_config_handle,ep,physical_enabled[nummer]);
	sprintf(ep,"/control/sensors_i2c/sensor%d/name",nummer);
	sqldb_write_string2(sql_config_handle,ep,physical_name[nummer]);
	sprintf(ep,"/control/sensors_i2c/sensor%d/comment",nummer);
	sqldb_write_string2(sql_config_handle,ep,physical_comment[nummer]);
	sprintf(ep,"/control/sensors_i2c/sensor%d/chipid",nummer);
	sqldb_write_long2(sql_config_handle,ep,physical_chipid[nummer]);
	sprintf(ep,"/control/sensors_i2c/sensor%d/chipname",nummer);
	sqldb_write_string2(sql_config_handle,ep,physical_chipname[nummer]);
	sprintf(ep,"/control/sensors_i2c/sensor%d/chipaddress",nummer);
	sqldb_write_long2(sql_config_handle,ep,physical_chipadresse[nummer]);
	sprintf(ep,"/control/sensors_i2c/sensor%d/channels",nummer);
	sqldb_write_long2(sql_config_handle,ep,physical_channels[nummer]);
	sprintf(ep,"/control/sensors_i2c/sensor%d/outchannels",nummer);
	sqldb_write_long2(sql_config_handle,ep,physical_outchannels[nummer]);
	sprintf(ep,"/control/sensors_i2c/sensor%d/bus",nummer);
	sqldb_write_long2(sql_config_handle,ep,physical_bus[nummer]);
	sprintf(ep,"/control/sensors_i2c/sensor%d/group",nummer);
	sqldb_write_long2(sql_config_handle,ep,physical_group[nummer]);
	sprintf(ep,"/control/sensors_i2c/sensor%d/port",nummer);
	sqldb_write_long2(sql_config_handle,ep,physical_port[nummer]);
	sprintf(ep,"/control/sensors_i2c/sensor%d/speed",nummer);
	sqldb_write_long2(sql_config_handle,ep,physical_speed[nummer]);
	sprintf(ep,"/control/sensors_i2c/sensor%d/priority",nummer);
	sqldb_write_long2(sql_config_handle,ep,physical_priority[nummer]);
	sprintf(ep,"/control/sensors_i2c/sensor%d/all4000_typ",nummer);
	sqldb_write_long2(sql_config_handle,ep,physical_all4000_typ[nummer]);
	sprintf(ep,"/control/sensors_i2c/sensor%d/mux_enabled",nummer);
	sqldb_write_long2(sql_config_handle,ep,physical_mux_enabled[nummer]);
	sprintf(ep,"/control/sensors_i2c/sensor%d/mux_port",nummer);
	sqldb_write_long2(sql_config_handle,ep,physical_mux_port[nummer]);

	for(i = 0;i < 8;i++)
		{
		// printf(" mapch[%ld] to logical %d\n",i,physical_mapto_sensor[i][nummer]);
		sprintf(ep,"/control/sensors_i2c/sensor%d/channel%ld/mapto_sensor",nummer,i);
		sqldb_write_long2(sql_config_handle,ep,physical_mapto_sensor[i][nummer]);
		sprintf(ep,"/control/sensors_i2c/sensor%d/channel%ld/function",nummer,i);
		sqldb_write_long2(sql_config_handle,ep,physical_mapto_function[i][nummer]);
		sprintf(ep,"/control/sensors_i2c/sensor%d/channel%ld/mapto_math_enabled",nummer,i);
		sqldb_write_long2(sql_config_handle,ep,physical_mapto_math_enabled[i][nummer]);
		sprintf(ep,"/control/sensors_i2c/sensor%d/channel%ld/mapto_math_add1",nummer,i);
		sqldb_write_float2(sql_config_handle,ep,physical_mapto_math_add1[i][nummer]);
		sprintf(ep,"/control/sensors_i2c/sensor%d/channel%ld/mapto_math_multiply1",nummer,i);
		sqldb_write_float2(sql_config_handle,ep,physical_mapto_math_multiply1[i][nummer]);
		sprintf(ep,"/control/sensors_i2c/sensor%d/channel%ld/mapto_math_add2",nummer,i);
		sqldb_write_float2(sql_config_handle,ep,physical_mapto_math_add2[i][nummer]);
		sprintf(ep,"/control/sensors_i2c/sensor%d/channel%ld/mapto_math_multiply2",nummer,i);
		sqldb_write_float2(sql_config_handle,ep,physical_mapto_math_multiply2[i][nummer]);
		sprintf(ep,"/control/sensors_i2c/sensor%d/channel%ld/out_value_command",nummer,i);
		sqldb_write_long2(sql_config_handle,ep,physical_outcmd[i][nummer]);
		sprintf(ep,"/control/sensors_i2c/sensor%d/channel%ld/out_value_int",nummer,i);
		sqldb_write_long2(sql_config_handle,ep,physical_outvalue_int[i][nummer]);
		sprintf(ep,"/control/sensors_i2c/sensor%d/channel%ld/out_value_float",nummer,i);
		sqldb_write_long2(sql_config_handle,ep,physical_outvalue_float[i][nummer]);
		sprintf(ep,"/control/sensors_i2c/sensor%d/channel%ld/out_value_string",nummer,i);
		sqldb_write_string2(sql_config_handle,ep,physical_outvalue_string[i][nummer]);
		sprintf(ep,"/control/sensors_i2c/sensor%d/channel%ld/mapfrom_actor",nummer,i);
		sqldb_write_long2(sql_config_handle,ep,physical_mapfrom_actor[i][nummer]);
		}

	sprintf(ep,"Configuration data for physical Sensor %d written.",nummer);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	}






// **********   logische Sensor-Konfiguration einlesen   **********

unsigned long int logic_readconfig(void)
	{
	char ep[1024];
	signed long int t;
	unsigned long int lcount;
	signed short int i;


	logic_setzedefault();																	// Alle logischen Sensoren auf Standardwerte zuruecksetzen
	sqldb_write_long2(sql_config_handle,"/control/sensors_i2c/flag_logicalsettings",0); // Merkflag lschen

	logical_max_sensors = sqldb_read_long2(sql_config_handle,"/control/sensors_logical/max_sensors");
	if(logical_max_sensors > MAX_LOGICAL_SENSORS)
		{
		sprintf(ep,"Configured value /control/sensors_logical/max_sensors is %ld; larger than MAX_LOGICAL_SENSORS, using MAX_LOGICAL_SENSORS=%d instead.",logical_max_sensors,MAX_LOGICAL_SENSORS);
		log_append(2,ep);																			// Fehlermeldung ins Logfile schreiben
		logical_max_sensors = MAX_LOGICAL_SENSORS;
		}

	for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		logical_function[t] = 0;
		}

	lcount = 0;

	for(t = 0;t < logical_max_sensors;t++)
		{
		sprintf(ep,"/control/sensors_logical/sensor%ld/function",t);
		logical_function[t] = sqldb_read_long2(sql_config_handle,ep);
		if(logical_function[t] > 0)
			{
			lcount++;
			logical_chipid[t] = 0;
			sprintf(ep,"/control/sensors_logical/sensor%ld/name",t);
			sqldb_read_string2(sql_config_handle,ep,logical_name[t]);
			sprintf(ep,"/control/sensors_logical/sensor%ld/comment",t);
			sqldb_read_string2(sql_config_handle,ep,logical_comment[t]);
			sprintf(ep,"/control/sensors_logical/sensor%ld/function",t);
			logical_function[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_logical/sensor%ld/all4000_typ",t);
			logical_all4000_typ[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_logical/sensor%ld/value_int",t);
			logical_int[t] = sqldb_read_long2(sql_config_handle,ep);
			logical_old_int[t] = logical_int[t];
			sprintf(ep,"/control/sensors_logical/sensor%ld/value_float",t);
			logical_float[t] = sqldb_read_float2(sql_config_handle,ep);
			logical_old_float[t] = logical_float[t];
			sprintf(ep,"/control/sensors_logical/sensor%ld/value_string",t);
			sqldb_read_string2(sql_config_handle,ep,logical_string[t]);
			strcpy(logical_old_string[t],logical_string[t]);
			sprintf(ep,"/control/sensors_logical/sensor%ld/limit_min",t);
			logical_limit_min[t] = sqldb_read_float2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_logical/sensor%ld/limit_max",t);
			logical_limit_max[t] = sqldb_read_float2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_logical/sensor%ld/display_min",t);
			logical_display_min[t] = sqldb_read_float2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_logical/sensor%ld/display_max",t);
			logical_display_max[t] = sqldb_read_float2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_logical/sensor%ld/minimum",t);
			logical_minimum[t] = sqldb_read_float2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_logical/sensor%ld/maximum",t);
			logical_maximum[t] = sqldb_read_float2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_logical/sensor%ld/meta/background",t);
			sqldb_read_string2(sql_config_handle,ep,logical_svg_bgcolor[t]);
			sprintf(ep,"/control/sensors_logical/sensor%ld/meta/grid",t);
			sqldb_read_string2(sql_config_handle,ep,logical_svg_gridcolor[t]);
			sprintf(ep,"/control/sensors_logical/sensor%ld/graph",t);
			sqldb_read_string2(sql_config_handle,ep,logical_svg_graphcolor[t]);
			sprintf(ep,"/control/sensors_logical/sensor%ld/line",t);
			logical_svg_graphstyle[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/sensors_logical/sensor%ld/meta/enabled",t);
			logical_meta_enabled[t] = sqldb_read_long2(sql_config_handle,ep);
			for (i = 0;i < 4;i++)
				{
				sprintf(ep,"/control/sensors_logical/sensor%ld/meta/%d",t,i);
				logical_meta_source[t][i] = sqldb_read_long2(sql_config_handle,ep);
				sprintf(ep,"/control/sensors_logical/sensor%ld/meta/%d/line",t,i);
				logical_meta_graphstyle[t][i] = sqldb_read_long2(sql_config_handle,ep);
				sprintf(ep,"/control/sensors_logical/sensor%ld/meta/%d/graph",t,i);
				sqldb_read_string2(sql_config_handle,ep,logical_meta_graphcolor[t][i]);
				}
			// falls die RGB-Werte ungltig sind, die Defaults verwenden
			if (strlen(logical_svg_bgcolor[t]) != 6) strcpy(logical_svg_bgcolor[t],"000000");
			if (strlen(logical_svg_gridcolor[t]) != 6) strcpy(logical_svg_gridcolor[t],"404040");
			if (strlen(logical_svg_graphcolor[t]) != 6) strcpy(logical_svg_graphcolor[t],"FFFF00");
			sprintf(ep,"/control/sensors_logical/sensor%ld/math/enabled",t);
			logical_math_enabled[t] = sqldb_read_long2(sql_config_handle,ep);
			for (i = 0;i < 4;i++)
				{
				sprintf(ep,"/control/sensors_logical/sensor%ld/math/%d/operation",t,i);
				logical_math_operation[t][i] = sqldb_read_long2(sql_config_handle,ep);
				sprintf(ep,"/control/sensors_logical/sensor%ld/math/%d/source",t,i);
				logical_math_source[t][i] = sqldb_read_long2(sql_config_handle,ep);
				sprintf(ep,"/control/sensors_logical/sensor%ld/math/%d/constant",t,i);
				logical_math_constant[t][i] = sqldb_read_float2(sql_config_handle,ep);
				}

			sprintf(ep,"Read Config Data for LogicS.#%ld: Name:%s, Function:%d, intV:%ld, floatV:%4.2f, StringV:%s, minLimit:%4.2f, maxLimit:%4.2f, minDisplay:%4.2f, maxDisplay:%4.2f, Minimum:%4.2f, Maximum:%4.2f",
							t,logical_name[t],logical_function[t],logical_int[t],logical_float[t],logical_string[t],
							logical_limit_min[t],logical_limit_max[t],logical_display_min[t],logical_display_max[t],
							logical_minimum[t],logical_maximum[t]);
			log_append(4,ep);																	// Meldung ins Logfile schreiben
			}
		}

	sprintf(ep,"Configuration data for %ld logical Sensors read.",lcount);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	return(lcount);
	}



// *****   Statusmail-Einstellungen lschen   *****

void statusmail_setzedefault(void)
	{
	signed short int t;

	for (t = 0;t < MAX_MAIL_RECIPIENTS;t++)
		{
		statusmail_boot_enabled[t] = 0;
		statusmail_continous_enabled[t] = 0;
		statusmail_continous_interval[t] = 60;
		statusmail_daily_enabled[t] = 0;
		statusmail_daily_hour[t] = 0;
		statusmail_daily_minute[t] = 0;
		statusmail_recipient[t][0] = 0;
		statusmail_subject[t][0] = 0;
		statusmail_interval_counter[t] = 0;
		}
	statusmail_max_recipients = 0;
	statusmail_counter = sqldb_read_long2(sql_config_handle,"/statusmail/reload");
	}



// **********   Statusmail-Konfiguration einlesen   **********

void statusmail_readconfig(void)
	{
	char ep[1024];
	signed long int t;


	statusmail_setzedefault();														// Alle Statusmails auf Standardwerte zuruecksetzen
	statusmail_counter = sqldb_read_long2(sql_config_handle,"/statusmail/reload"); // Merkflag lschen

	statusmail_max_recipients = sqldb_read_long2(sql_config_handle,"/sys/network/mail/max_recipient");

	for(t = 0;t < statusmail_max_recipients;t++)
		{
		sprintf(ep,"/statusmail%ld/boot/enabled",t);
		statusmail_boot_enabled[t] = sqldb_read_long2(sql_config_handle,ep);
		sprintf(ep,"/statusmail%ld/continuous/enabled",t);
		statusmail_continous_enabled[t] = sqldb_read_long2(sql_config_handle,ep);
		sprintf(ep,"/statusmail%ld/continuous/intervall",t);
		statusmail_continous_interval[t] = sqldb_read_long2(sql_config_handle,ep);
		sprintf(ep,"/statusmail%ld/daily/enabled",t);
		statusmail_daily_enabled[t] = sqldb_read_long2(sql_config_handle,ep);
		sprintf(ep,"/statusmail%ld/daily/hour",t);
		statusmail_daily_hour[t] = sqldb_read_long2(sql_config_handle,ep);
		sprintf(ep,"/statusmail%ld/daily/minute",t);
		statusmail_daily_minute[t] = sqldb_read_long2(sql_config_handle,ep);
		sprintf(ep,"/sys/network/mail/recipient%ld",t);
		sqldb_read_string2(sql_config_handle,ep,statusmail_recipient[t]);
		sprintf(ep,"/statusmail%ld/subject",t);
		sqldb_read_string2(sql_config_handle,ep,statusmail_subject[t]);
		}

	sprintf(ep,"Configuration data for %d Statusmails read.",statusmail_max_recipients);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	}



// *****   Statusmails (Bootzeit) versenden   *****

void statusmail_boot(void)
	{
	signed long int t;
	char smtp_username[80];
	char smtp_password[80];
	char smtp_server[80];
	char smtp_sender[80];
	signed short int smtp_encoding;
	FILE *f;
	char geraetename[80];
	char headline[255];


	sqldb_read_string2(sql_config_handle,"/sys/network/mail/user",smtp_username);
	sqldb_read_string2(sql_config_handle,"/sys/network/mail/pass",smtp_password);
	sqldb_read_string2(sql_config_handle,"/sys/network/mail/smtp",smtp_server);
	smtp_encoding = sqldb_read_long2(sql_config_handle,"/sys/network/mail/encoding");
	sqldb_read_string2(sql_config_handle,"/sys/network/mail/sender",smtp_sender);
	sqldb_read_string2(sql_config_handle,"/control/devicename",geraetename);

	for(t = 0;t < statusmail_max_recipients;t++)
		{
		if (statusmail_boot_enabled[t] > 0)
			{
			f = fopen("/tmp/mail.txt","wb");
			fprintf(f,"Device has been (re-)started.\n");
			fprintf(f,"\n-%s\n",smtp_sender);
			fclose(f);
			sprintf(headline,"%s Boot notification",geraetename);
			versende_email(smtp_encoding,smtp_username,smtp_password,smtp_server,smtp_sender,statusmail_recipient[t],headline,"/tmp/mail.txt");
			}
		}
	}



// *****   periodische Statusmails versenden, falls ntig   *****

void statusmail_work(void)
	{
	struct timeval tv;																		// Uhrzeit-Kram
	time_t curtime;																				// Uhrzeit-Kram
	struct tm *myTm;																			// Uhrzeit-Kram
	unsigned char stunde;
	unsigned char minute;
	static unsigned char old_stunde = 0;
	static unsigned char old_minute = 0;
	signed short int jahr;
	signed short int monat;
	signed short int tag;
	char smtp_username[80];
	char smtp_password[80];
	char smtp_server[80];
	char smtp_sender[80];
	signed short int smtp_encoding;
	FILE *f;
	char geraetename[80];
	char headline[255];
	signed short int t;


	sqldb_read_string2(sql_config_handle,"/sys/network/mail/user",smtp_username);
	sqldb_read_string2(sql_config_handle,"/sys/network/mail/pass",smtp_password);
	sqldb_read_string2(sql_config_handle,"/sys/network/mail/smtp",smtp_server);
	sqldb_read_string2(sql_config_handle,"/sys/network/mail/sender",smtp_sender);
	smtp_encoding = sqldb_read_long2(sql_config_handle,"/sys/network/mail/encoding");
	sqldb_read_string2(sql_config_handle,"/control/devicename",geraetename);


	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln 
  curtime = tv.tv_sec;																	// Lores-Zeit in s rausziehen
	myTm = localtime(&curtime);														// ...und berfhren in tm-Struktur
	stunde = myTm->tm_hour;																// Stnde
	minute = myTm->tm_min;																// Minute
	jahr = myTm->tm_year + 1900;													// Jahr
	monat = myTm->tm_mon + 1;															// Monat
	tag = myTm->tm_mday;																	// Tag

	if(minute != old_minute)
		{
		old_stunde = stunde;
		old_minute = minute;

		// Prfung auf Uhrzeit-Mails (daily: 1-mal pro Tag zu einer bestimmten Uhrzeit)
		for(t = 0;t < statusmail_max_recipients;t++)
			{
			if (statusmail_daily_enabled[t] > 0)
				{
				if ((statusmail_daily_hour[t] == stunde) && (statusmail_daily_minute[t] == minute))
					{
					f = fopen("/tmp/mail.txt","wb");
					fprintf(f,"Sensor data from %s at %04d-%02d-%02d %02d:%02d:\n",geraetename,jahr,monat,tag,stunde,minute);
					insert_sensor_values(f);
					fprintf(f,"\n-%s\n",smtp_sender);
					fclose(f);
					sprintf(headline,"%s",statusmail_subject[t]);
					versende_email(smtp_encoding,smtp_username,smtp_password,smtp_server,smtp_sender,statusmail_recipient[t],headline,"/tmp/mail.txt");
					}
				}
			}


		// Prfung auf periodische Mails
		for(t = 0;t < statusmail_max_recipients;t++)
			{
			if (statusmail_continous_enabled[t] > 0)
				{
				statusmail_interval_counter[t]++;								// Erhhen des Periodenzhlers
				// printf("enabled=%d,ic=%d, intervall:%d, t=%d\n",statusmail_continous_enabled[t],statusmail_interval_counter[t],statusmail_continous_interval[t],t);
				if (statusmail_interval_counter[t] >= statusmail_continous_interval[t])
					{
					// printf("** Statusmail (intervall) #%d **\n",t);
					statusmail_interval_counter[t] = 0;
					f = fopen("/tmp/mail.txt","wb");
					fprintf(f,"Sensor data from %s at %04d-%02d-%02d %02d:%02d:\n",geraetename,jahr,monat,tag,stunde,minute);
					insert_sensor_values(f);
					fprintf(f,"\n-%s\n",smtp_sender);
					fclose(f);
					sprintf(headline,"%s",statusmail_subject[t]);
					versende_email(smtp_encoding,smtp_username,smtp_password,smtp_server,smtp_sender,statusmail_recipient[t],headline,"/tmp/mail.txt");
					}
				}
			}

		}
	}



// *****   Sensorwerte in ein Email einbauen   *****

void insert_sensor_values(FILE *f)
	{
	signed short int i;

	for (i = 0;i < MAX_LOGICAL_SENSORS;i++)
		{
		if (logical_float[i] > -2048000.0)
			{
			fprintf(f,"%s: %s%s\n",logical_name[i],logical_string[i],scale_unit[logical_function[i]]);
			}
		}
	}


// **********   logische Sensor-Konfiguration fr einen Sensor schreiben   **********

void logical_write_single(signed short int nummer)
	{
	char ep[200];

	sprintf(ep,"/control/sensors_logical/sensor%d/name",nummer);
	sqldb_write_string2(sql_config_handle,ep,logical_name[nummer]);
	sprintf(ep,"/control/sensors_logical/sensor%d/comment",nummer);
	sqldb_write_string2(sql_config_handle,ep,logical_comment[nummer]);
	sprintf(ep,"/control/sensors_logical/sensor%d/function",nummer);
	sqldb_write_long2(sql_config_handle,ep,logical_function[nummer]);
	sprintf(ep,"/control/sensors_logical/sensor%d/all4000_typ",nummer);
	sqldb_write_long2(sql_config_handle,ep,logical_all4000_typ[nummer]);
	sprintf(ep,"/control/sensors_logical/sensor%d/value_int",nummer);
	sqldb_write_long2(sql_config_handle,ep,logical_int[nummer]);
	sprintf(ep,"/control/sensors_logical/sensor%d/value_float",nummer);
	sqldb_write_float2(sql_config_handle,ep,logical_float[nummer]);
	sprintf(ep,"/control/sensors_logical/sensor%d/value_string",nummer);
	sqldb_write_string2(sql_config_handle,ep,logical_string[nummer]);
	sprintf(ep,"/control/sensors_logical/sensor%d/limit_min",nummer);
	sqldb_write_float2(sql_config_handle,ep,logical_limit_min[nummer]);
	sprintf(ep,"/control/sensors_logical/sensor%d/limit_max",nummer);
	sqldb_write_float2(sql_config_handle,ep,logical_limit_max[nummer]);
	sprintf(ep,"/control/sensors_logical/sensor%d/display_min",nummer);
	sqldb_write_float2(sql_config_handle,ep,logical_display_min[nummer]);
	sprintf(ep,"/control/sensors_logical/sensor%d/display_max",nummer);
	sqldb_write_float2(sql_config_handle,ep,logical_display_max[nummer]);
	sprintf(ep,"/control/sensors_logical/sensor%d/minimum",nummer);
	sqldb_write_float2(sql_config_handle,ep,logical_minimum[nummer]);
	sprintf(ep,"/control/sensors_logical/sensor%d/maximum",nummer);
	sqldb_write_float2(sql_config_handle,ep,logical_maximum[nummer]);

	sprintf(ep,"Configuration data for logical Sensor %d written.",nummer);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	}



// **********   remote_poll()   -   Remote-Sensoren (XML Files) abfragen   **********

void remote_poll(void)
	{
	unsigned short int t;
	char ep[400];
	unsigned long int timestamp;
	FILE *f;
	unsigned short int i;
	signed long int now;
	signed long int age;
	char xmlpuffer[128000];


	timestamp = time(NULL);

	for(t = 0;t < sensor_remote_anzahl;t++)
		{
		if (sensor_remote_enabled[t] > 0)
			{
			f = fopen(sensor_remote_local_filename[t],"rb");
			if(f != NULL)
				{
				// File vorhanden, also durchrdeln
				fclose(f);
				sprintf(ep,"sensor_demon remote_poll: File \"%s\" gefunden.",sensor_remote_local_filename[t]);
				log_append(4,ep);																// Meldung ins Logfile schreiben
				i = readfileinstring(sensor_remote_local_filename[t],xmlpuffer);
				sprintf(ep,"File %d eingelesen (%d Bytes), checke nun auf %d Remotesensoren...",t,i,sensor_remote_max_sensors[t]);
				log_append(4,ep);																// Meldung ins Logfile schreiben
				for(i = 0;i < sensor_remote_max_sensors[t];i++)
					{
					if(sensor_remote_target[t][i] >= 0)
						{
						sprintf(ep,"  (%d): Tag=\"%s\"",i,sensor_remote_xmltag[t][i]);
						log_append(4,ep);														// Meldung ins Logfile schreiben
						lese_xml_string(xmlpuffer,sensor_remote_xmltag[t][i],ep,"-2048000.2");
						strcompress(sensor_remote_value_string[t][i],ep); // Leerzeichen entfernen
						sensor_remote_timestamp[t][i] = time(NULL);
						sprintf(ep,"  rSensor[%d][%d]=\"%s\"  timestamp:%ld",t,i,sensor_remote_value_string[t][i],sensor_remote_timestamp[t][i]);
						log_append(4,ep);														// Meldung ins Logfile schreiben
						}
					} 
				unlink(sensor_remote_local_filename[t]);				// File nach Bearbeitung lschen
				}
			else
				{
				// File nicht vorhanden, also Sensor-Timestamps prfen
				sprintf(ep,"sensor_demon remote_poll: File \"%s\" nicht gefunden.",sensor_remote_local_filename[t]);
				log_append(4,ep);																// Meldung ins Logfile schreiben
				for(i = 0;i < sensor_remote_max_sensors[t];i++)
					{
					if(sensor_remote_target[t][i] >= 0)
						{
						now = time(NULL);
						age = difftime(now,sensor_remote_timestamp[t][i]);
						sprintf(ep,"  rSensor[%d][%i]=\"%s\"  age:%lds",t,i,sensor_remote_value_string[t][i],age);
						log_append(4,ep);														// Meldung ins Logfile schreiben
						if(age > sensor_remote_max_age[t])
							{
							sprintf(ep,"  rSensor[%d][%i]=\"%s\"  age:%lds",t,i,sensor_remote_value_string[t][i],age);
							log_append(4,ep);													// Meldung ins Logfile schreiben
							strcpy(sensor_remote_value_string[t][i],"-2048000.3");
							}
						}
					} 
				}
			}
		}	
	}



// **********   sensor_do_poll(nummer,r=0/w=1)   -   einen einzelnen Sensor abfragen   *****

signed long int sensor_do_poll(signed short int t,signed short int writeflag)
	{
	signed long int rw = 0;
	// char ep[200];
	double tmp_float;
	signed long int tmp_int;
	unsigned char tmp_byte;
	unsigned short int i;
	unsigned char zp[8] = {1,2,4,8,16,32,64,128};
	char pufferchen[1024];
	time_t now;
	signed long int age;
	unsigned char bytewert;
	time_t timer;
	struct tm *tblock;


	if (writeflag == 0)
		{
		// ----------   LESE-Sensoren   ----------
			// -----   ALL4504 Portmultiplexer setzen, falls erforderlich   -----
			if (physical_mux_enabled[t] > 0)
				{
				// Es ist immer ein PCF8574A auf ID 4 vorhanden, den mssen wir setzen !
				tmp_float = simplified_pcf8574a_write(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],4,physical_mux_port[t]);
				// printf("R:PCF8547AWrite(Bus%d,Group%d,Port%d,Speed%d,Adr%d,Byte%d)=%1.2f\n",physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],4,physical_mux_port[t],tmp_float);
				if (tmp_float == 0.0)
					{
					// 0.0: OK
					}
				else
					{
					// 1.0: MUX hat nicht geantwortet, hier sollte eine Fehlermeldung hin
					}
				}
			// -----   Clocktick-Sensor (Stunde)   -----
			if(physical_chipid[t] == 4000)
				{
				timer = time(NULL);
				tblock = localtime(&timer);
				tmp_float = tblock->tm_hour;										// Sensor-Rckwert ist die aktuelle Stunde
				physical_value_int[0][t] = (signed long int)(tmp_float * 100);
				physical_value_float[0][t] = tmp_float;
				sprintf(physical_value_string[0][t],"%4.2f",tmp_float);
				rw = 1;
				goto fertig;
				}
			// -----   Clocktick-Sensor (Minute)   -----
			if(physical_chipid[t] == 4001)
				{
				timer = time(NULL);
				tblock = localtime(&timer);
				tmp_float = tblock->tm_min;											// Sensor-Rckwert ist die aktuelle Minute
				physical_value_int[0][t] = (signed long int)(tmp_float * 100);
				physical_value_float[0][t] = tmp_float;
				sprintf(physical_value_string[0][t],"%4.2f",tmp_float);
				rw = 1;
				goto fertig;
				}
			// -----   Clocktick-Sensor (Sekunde)   -----
			if(physical_chipid[t] == 4002)
				{
				timer = time(NULL);
				tblock = localtime(&timer);
				tmp_float = tblock->tm_sec;											// Sensor-Rckwert ist die aktuelle Minute
				physical_value_int[0][t] = (signed long int)(tmp_float * 100);
				physical_value_float[0][t] = tmp_float;
				sprintf(physical_value_string[0][t],"%4.2f",tmp_float);
				rw = 1;
				goto fertig;
				}
			// -----   Clocktick-Sensor (Stunde.Minute)   -----
			if(physical_chipid[t] == 4003)
				{
				timer = time(NULL);
				tblock = localtime(&timer);
				tmp_float = (double)(tblock->tm_hour * 100 + tblock->tm_min) / 100; // Sensor-Rckwert ist die aktuelle Stunde:Minute als Nackommastellen
				physical_value_int[0][t] = (signed long int)(tmp_float * 100);
				physical_value_float[0][t] = tmp_float;
				sprintf(physical_value_string[0][t],"%4.2f",tmp_float);
				rw = 1;
				goto fertig;
				}
			// -----   Clocktick-Sensor (Stunde.MinuteSekunde)   -----
			if(physical_chipid[t] == 4004)
				{
				timer = time(NULL);
				tblock = localtime(&timer);
				tmp_float = (double)(tblock->tm_hour * 10000 + tblock->tm_min * 100 + tblock->tm_sec) / 10000; // Sensor-Rckwert ist die aktuelle Stunde:Minute:Sekunde als Nackommastellen
				physical_value_int[0][t] = (signed long int)(tmp_float * 100);
				physical_value_float[0][t] = tmp_float;
				sprintf(physical_value_string[0][t],"%4.2f",tmp_float);
				rw = 1;
				goto fertig;
				}

			// -----   LM75 Temperatursensor   -----
			if(physical_chipid[t] == 1)
				{
				tmp_float = simplified_lm75_read_temp(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],physical_chipadresse[t]);
				if (tmp_float > -2048000.0)
					{
					physical_value_int[0][t] = (signed long int)(tmp_float * 100);
					physical_value_float[0][t] = tmp_float;
					sprintf(physical_value_string[0][t],"%4.2f",tmp_float);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   LM92 Temperatursensor   -----
			if(physical_chipid[t] == 2)
				{
				tmp_float = simplified_lm92_read_temp(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],physical_chipadresse[t]);
				if (tmp_float > -2048000.0)
					{
					physical_value_int[0][t] = (signed long int)(tmp_float * 100);
					physical_value_float[0][t] = tmp_float;
					sprintf(physical_value_string[0][t],"%4.2f",tmp_float);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   SHT71 Temperatursensor   -----
			if(physical_chipid[t] == 3)
				{
				tmp_float = simplified_sht71_read_temp(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t]);
				if (tmp_float > -2048000.0)
					{
					physical_value_int[0][t] = (signed long int)(tmp_float * 100);
					physical_value_float[0][t] = tmp_float;
					sprintf(physical_value_string[0][t],"%4.2f",tmp_float);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   SHT71 Feuchtesensor   -----
			if(physical_chipid[t] == 32)
				{
				tmp_float = simplified_sht71_read_humidity(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t]);
				if (tmp_float > -2048000.0)
					{
					physical_value_int[0][t] = (signed long int)(tmp_float * 100);
					physical_value_float[0][t] = tmp_float;
					sprintf(physical_value_string[0][t],"%4.2f",tmp_float);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   SHT71 Temperatursensor (noWait)   -----
			if(physical_chipid[t] == 4)
				{
				tmp_float = simplified_sht71_read_temp_nw(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t]);
				if (tmp_float > -2048000.0)
					{
					physical_value_int[0][t] = (signed long int)(tmp_float * 100);
					physical_value_float[0][t] = tmp_float;
					sprintf(physical_value_string[0][t],"%4.2f",tmp_float);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   SHT71 Feuchtesensor (noWait)   -----
			if(physical_chipid[t] == 33)
				{
				tmp_float = simplified_sht71_read_humidity_nw(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t]);
				if (tmp_float > -2048000.0)
					{
					physical_value_int[0][t] = (signed long int)(tmp_float * 100);
					physical_value_float[0][t] = tmp_float;
					sprintf(physical_value_string[0][t],"%4.2f",tmp_float);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   DSDX015A2R Luftdrucksensor   -----
			if(physical_chipid[t] == 40)
				{
				tmp_float = simplified_dsdx015a2r_read_pressure(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t]);
				if (tmp_float > -2048000.0)
					{
					physical_value_float[0][t] = tmp_float + 1023.25 - ortsueblicher_luftdruck;
					physical_value_int[0][t] = (signed long int)(physical_value_float[0][t] * 100);
					sprintf(physical_value_string[0][t],"%4.2f",physical_value_float[0][t]);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   HDI Luftdrucksensor   -----
			if(physical_chipid[t] == 41)
				{
				tmp_float = simplified_hdi0811_read_pressure(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t]);
				if (tmp_float > -2048000.0)
					{
					physical_value_float[0][t] = tmp_float + 1023.25 - ortsueblicher_luftdruck;
					physical_value_int[0][t] = (signed long int)(physical_value_float[0][t] * 100);
					sprintf(physical_value_string[0][t],"%4.2f",physical_value_float[0][t]);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   BMP085 Luftdrucksensor   -----
			if(physical_chipid[t] == 42)
				{
				tmp_float = simplified_bmp085_read_pressure(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t]);
				// printf("tmp_floats(BMP085): %1.2f\n",tmp_float);
				if (tmp_float > -2048000.0)
					{
					physical_value_float[0][t] = tmp_float + 1023.25 - ortsueblicher_luftdruck;
					physical_value_int[0][t] = (signed long int)(physical_value_float[0][t] * 100);
					sprintf(physical_value_string[0][t],"%4.2f",physical_value_float[0][t]);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   ALL3026 8-fach Eingang als 1 einzelner Eingang (Bit 0) L:100, H:0   -----
			if(physical_chipid[t] == 74)
				{
				tmp_float = simplified_pcf8574_read(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],physical_chipadresse[t]);
				// printf("all3052READ: %4.2f\n",tmp_float);
				if (tmp_float > -2048000.0)
					{
					tmp_byte = (unsigned char)tmp_float;
					tmp_int = 100;
					if((tmp_byte & zp[0]) > 0) tmp_int = 0;
					physical_value_int[0][t] = tmp_int;
					physical_value_float[0][t] = tmp_int;
					sprintf(physical_value_string[0][t],"%ld",tmp_int);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   ALL4042 8-fach Eingang als 4 einzelne Eingnge (Bit 0...3)   -----
			if(physical_chipid[t] == 75)
				{
				tmp_float = simplified_pcf8574_read(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],physical_chipadresse[t]);
				if (tmp_float > -2048000.0)
					{
					tmp_byte = (unsigned char)tmp_float;
					for(i = 0;i < 4;i++)
						{
						tmp_int = 0;
						if((tmp_byte & zp[i]) > 0) tmp_int = 1;
						physical_value_int[i][t] = tmp_int;
						physical_value_float[i][t] = (double)tmp_int;
						sprintf(physical_value_string[i][t],"%ld",tmp_int);
						}
					}
				else
					{
					for(i = 0;i < 4;i++)
						{
						physical_value_int[i][t] = -204800000;
						physical_value_float[i][t] = -2048000.00;
						strcpy(physical_value_string[i][t],"-2048000.00");
						}
					}
				rw = 1;
				goto fertig;
				}

			// -----   ALL4042 8-fach Eingang als 4 einzelne Eingnge (Bit 4...7)   -----
			if(physical_chipid[t] == 76)
				{
				tmp_float = simplified_pcf8574_read(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],physical_chipadresse[t]);
				if (tmp_float > -2048000.0)
					{
					tmp_byte = (unsigned char)tmp_float;
					for(i = 0;i < 4;i++)
						{
						tmp_int = 0;
						if((tmp_byte & zp[i + 4]) > 0) tmp_int = 1;
						physical_value_int[i][t] = tmp_int;
						physical_value_float[i][t] = (double)tmp_int;
						sprintf(physical_value_string[i][t],"%ld",tmp_int);
						}
					}
				else
					{
					for(i = 0;i < 4;i++)
						{
						physical_value_int[i][t] = -204800000;
						physical_value_float[i][t] = -2048000.00;
						strcpy(physical_value_string[i][t],"-2048000.00");
						}
					}
				rw = 1;
				goto fertig;
				}

			// -----   ALL4042 8-fach Eingang als 8 einzelne Eingnge   -----
			if(physical_chipid[t] == 77)
				{
				tmp_float = simplified_pcf8574_read(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],physical_chipadresse[t]);
				if (tmp_float > -2048000.0)
					{
					tmp_byte = (unsigned char)tmp_float;
					for(i = 0;i < 8;i++)
						{
						tmp_int = 0;
						if((tmp_byte & zp[i]) > 0) tmp_int = 1;
						physical_value_int[i][t] = tmp_int;
						physical_value_float[i][t] = (double)tmp_int;
						sprintf(physical_value_string[i][t],"%ld",tmp_int);
						}
					}
				else
					{
					for(i = 0;i < 8;i++)
						{
						physical_value_int[i][t] = -204800000;
						physical_value_float[i][t] = -2048000.00;
						strcpy(physical_value_string[i][t],"-2048000.00");
						}
					}
				rw = 1;
				goto fertig;
				}

			// -----   ALL4042 8-fach Eingang als 8-bit Binrwert   -----
			if(physical_chipid[t] == 78)
				{
				tmp_float = simplified_pcf8574_read(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],physical_chipadresse[t]);
				if (tmp_float > -2048000.0)
					{
					// physical_value_int[0][t] = (signed long int)(tmp_float * 100);
					physical_value_int[0][t] = (signed long int)tmp_float;
					physical_value_float[0][t] = tmp_float;
					sprintf(physical_value_string[0][t],"%ld",(signed long int)tmp_float);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   ADS1100 ADC (onboard ALL5000 mit PCF8574 auf ID2 als Kanalwhler)   -----
			if(physical_chipid[t] == 129)
				{
				for(i = 0;i < physical_channels[t];i++)
					{
					simplified_pcf8574_write(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],2,31 + i * 32); // Bits 0...4 auf H setzen
					// usleep(1); // 1 us warten
					tmp_float = simplified_ads1100_read(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],physical_chipadresse[t],14);
					// printf("ADC%d:%4.2f\n",i,tmp_float);
					if (tmp_float > 327.67) tmp_float = 0;
					if (tmp_float > -2048000.0)
						{
						tmp_float *= 0.4;														// Korrekturfaktor fr 3.84V Referenzspannung
						physical_value_int[i][t] = (signed long int)(tmp_float * 100);
						physical_value_float[i][t] = tmp_float;
						sprintf(physical_value_string[i][t],"%4.2f",tmp_float);
						}
					else
						{
						physical_value_int[i][t] = -204800000;
						physical_value_float[i][t] = -2048000.00;
						strcpy(physical_value_string[i][t],"-2048000.00");
						}
					}
				rw = 1;
				goto fertig;
				}

			// -----   ADS1100 ADC (ALL4590 I2C Powermeter mit PCF8574 auf ID5 als Kanalwhler)   -----
			if(physical_chipid[t] == 130)
				{
				for(i = 0;i < physical_channels[t];i++)
					{
					simplified_pcf8574_write(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],5,i); // Bits 0...3 sind die Adressleitungen
					// usleep(1); // 1 us warten
					tmp_float = simplified_ads1100_read(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],physical_chipadresse[t],14);
					// sprintf(ep,"extADC%d:%4.2f\n",i,tmp_float);
					// log_append(4,ep);
					if (tmp_float > 327.67) tmp_float = 0;
					if (tmp_float > -2048000.0)
						{
						tmp_float *= 0.4;														// Korrekturfaktor fr 3.84V Referenzspannung
						physical_value_int[i][t] = (signed long int)(tmp_float * 100);
						physical_value_float[i][t] = tmp_float;
						sprintf(physical_value_string[i][t],"%4.2f",tmp_float);
						}
					else
						{
						physical_value_int[i][t] = -204800000;
						physical_value_float[i][t] = -2048000.00;
						strcpy(physical_value_string[i][t],"-2048000.00");
						}
					}
				rw = 1;
				goto fertig;
				}

			// -----   ALL3025 Zhler   -----
			if(physical_chipid[t] == 96)
				{
				tmp_float = simplified_all3025_read_counter(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t]);
				if (tmp_float > -2048000.0)
					{
					physical_value_int[0][t] = (signed long int)(tmp_float * 100);
					physical_value_float[0][t] = tmp_float * 100;
					sprintf(physical_value_string[0][t],"%4.2f",tmp_float * 100);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   ALL4035 Zhler   -----
			if(physical_chipid[t] == 97)
				{
				tmp_float = simplified_all4035_read_counter(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t]);
				if (tmp_float > -2048000.0)
					{
					physical_value_int[0][t] = (signed long int)(tmp_float * 100);
					physical_value_float[0][t] = tmp_float * 100;
					sprintf(physical_value_string[0][t],"%4.2f",tmp_float * 100);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   ALL4033 AC-Spannungssensor   -----
			if(physical_chipid[t] == 256)
				{
				tmp_float = simplified_all4033_read_voltage(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t]);
				if (tmp_float > -2048000.0)
					{
					physical_value_int[0][t] = (signed long int)(tmp_float * 100);
					physical_value_float[0][t] = tmp_float;
					sprintf(physical_value_string[0][t],"%4.2f",tmp_float);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   ALL3032 analoger Helligkeitssensor   -----
			if(physical_chipid[t] == 259)
				{
				tmp_float = simplified_all3032_read_brightness(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t]);
				if (tmp_float > -2048000.0)
					{
					physical_value_int[0][t] = (signed long int)(tmp_float * 100);
					physical_value_float[0][t] = tmp_float;
					sprintf(physical_value_string[0][t],"%4.2f",tmp_float);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   ALL4023 3-fach Spannungsmesser   -----
			if(physical_chipid[t] == 260)
				{
				for(i = 0;i < 3;i++)
					{
					tmp_float = simplified_all4023_read_voltage(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],i);
					if (tmp_float > -2048000.0)
						{
						physical_value_int[i][t] = (signed long int)(tmp_float * 100);
						physical_value_float[i][t] = tmp_float;
						sprintf(physical_value_string[i][t],"%4.2f",tmp_float);
						}
					else
						{
						physical_value_int[i][t] = -204800000;
						physical_value_float[i][t] = -2048000.00;
						strcpy(physical_value_string[i][t],"-2048000.00");
						}
					}
				rw = 1;
				goto fertig;
				}

			// -----   ALL3021 1-fach Spannungsmesser   -----
			if(physical_chipid[t] == 261)
				{
				tmp_float = simplified_all3021_read_voltage(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t]);
				if (tmp_float > -2048000.0)
					{
					physical_value_int[0][t] = (signed long int)(tmp_float * 100);
					physical_value_float[0][t] = tmp_float;
					sprintf(physical_value_string[0][t],"%4.2f",tmp_float);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   ALL3022 2-fach Spannungsmesser   -----
			if(physical_chipid[t] == 262)
				{
				for(i = 0;i < 2;i++)
					{
					tmp_float = simplified_all3022_read_voltage(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],i);
					if (tmp_float > -2048000.0)
						{
						physical_value_int[i][t] = (signed long int)(tmp_float * 100);
						physical_value_float[i][t] = tmp_float;
						sprintf(physical_value_string[i][t],"%4.2f",tmp_float);
						}
					else
						{
						physical_value_int[i][t] = -204800000;
						physical_value_float[i][t] = -2048000.00;
						strcpy(physical_value_string[i][t],"-2048000.00");
						}
					}
				rw = 1;
				goto fertig;
				}

			// -----   RFID Reader   -----
			if(physical_chipid[t] == 300)
				{
				tmp_float = simplified_rfid001_read_cardid(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],pufferchen);
				if (tmp_float > -2048000.0)
					{
					if(tmp_float > 0)
						{
						// Neue Karte gefunden
						physical_value_int[0][t] = 100;							// 100 zurckgeben als Hinweis, da eine Karte da ist
						physical_value_float[0][t] = 1.0;						// 1.00 als Hinweis: Karte ist da
						strcpy(physical_value_string[0][t],pufferchen); // und den Karten-ID-String bergeben
						physical_timestamp[t] = time(NULL);					// Timestamp setzen
						}
					else
						{
						now = time(NULL);
						age = difftime(now,physical_timestamp[t]);
						if(age > MAX_PHYSICAL_AGE)
							{
							physical_value_int[0][t] = 0;
							physical_value_float[0][t] = 0;
							strcpy(physical_value_string[0][t],"0000000000000000");
							}
						else
							{
							}
						}
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"0000000000000000");
					physical_timestamp[t] = time(NULL);
					}
				// printf("RWf:%5.2f, RWi:%ld, String:\"%s\"\n",physical_value_float[0][t],physical_value_int[0][t],physical_value_string[0][t]);
				rw = 1;
				goto fertig;
				}

			// -----   ALL5000 int. 4-fach Eingang als 4 einzelne Eingnge (Bit 0...3)   -----
			if(physical_chipid[t] == 198)
				{
				tmp_float = simplified_pcf8574_read(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],physical_chipadresse[t]);
				if (tmp_float > -2048000.0)
					{
					tmp_byte = (unsigned char)tmp_float;
					for(i = 0;i < 4;i++)
						{
						tmp_int = 0;
						if((tmp_byte & zp[i]) > 0) tmp_int = 1;
						physical_value_int[i][t] = tmp_int;
						physical_value_float[i][t] = (double)tmp_int;
						sprintf(physical_value_string[i][t],"%ld",tmp_int);
						}
					}
				else
					{
					for(i = 0;i < 4;i++)
						{
						physical_value_int[i][t] = -204800000;
						physical_value_float[i][t] = -2048000.00;
						strcpy(physical_value_string[i][t],"-2048000.00");
						}
					}
				rw = 1;
				goto fertig;
				}

			// -----   PCF8574A ID0 auf der Hauptplatine   -----
			if (physical_chipid[t] ==   0) goto fertig;				// Ignorieren, 0 wird nicht behandelt
			if (physical_chipid[t] == 164) goto fertig;				// Ignorieren, ist kein echter Sensor sondern nur die LED
			if (physical_chipid[t] == 167) goto fertig;				// Ignorieren, ist kein echter Sensor sondern nur die LED im ALL5000 (nach Autodetect)

		}
	else
		{
		// ----------   SCHREIB-Sensoren   ----------

			// -----   ALL4504 Portmultiplexer setzen, falls erforderlich   -----
			if (physical_mux_enabled[t] > 0)
				{
				// Es ist immer ein PCF8574A auf ID 4 vorhanden, den mssen wir setzen !
				tmp_float = simplified_pcf8574a_write(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],4,physical_mux_port[t]);
				// printf("W:PCF8547AWrite(Bus%d,Group%d,Port%d,Speed%d,Adr%d,Byte%d)=%1.2f\n",physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],4,physical_mux_port[t],tmp_float);
				if (tmp_float == 0.0)
					{
					// 0.0: OK
					}
				else
					{
					// 1.0: MUX hat nicht geantwortet, hier sollte eine Fehlermeldung hin
					}
				}
			// -----   ALL3002 1-fach Ausgang (Bit0) invertiert: L:an, H:aus   -----
			if(physical_chipid[t] == 64)
				{
				bytewert = 255;
				if(physical_outvalue_int[0][t] > 0) bytewert = 254;
				tmp_float = simplified_pcf8574_write(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],physical_chipadresse[t],bytewert);
				if (tmp_float > -2048000.0)
					{
					physical_value_int[0][t] = (signed long int)physical_outvalue_int[0][t];
					physical_value_float[0][t] = physical_outvalue_int[0][t];
					sprintf(physical_value_string[0][t],"%ld",physical_outvalue_int[0][t]);
					}
				else
					{
					physical_value_int[0][t] = -204800000;
					physical_value_float[0][t] = -2048000.00;
					strcpy(physical_value_string[0][t],"-2048000.00");
					}
				rw = 1;
				goto fertig;
				}

			// -----   ALL4027 8-fach Ausgang als invertierte 8-bit Zahl   -----
			if(physical_chipid[t] == 67)
				{
				bytewert = 255;
				for(i = 0;i < 8;i++)
					{
					if(physical_outvalue_int[i][t] > 0)
						{
						bytewert -= zp[i];
						}
					}
				tmp_float = simplified_pcf8574_write(physical_bus[t],physical_group[t],physical_port[t],physical_speed[t],physical_chipadresse[t],bytewert);
				if (tmp_float > -2048000.0)
					{
					for(i = 0;i < 8;i++)
						{
						physical_value_int[i][t] = (signed long int)physical_outvalue_int[i][t];
						physical_value_float[i][t] = physical_outvalue_int[i][t];
						sprintf(physical_value_string[i][t],"%ld",physical_outvalue_int[i][t]);
						}
					}
				else
					{
					for(i = 0;i < 8;i++)
						{
						physical_value_int[i][t] = -204800000;
						physical_value_float[i][t] = -2048000.00;
						strcpy(physical_value_string[i][t],"-2048000.00");
						}
					}
				rw = 1;
				goto fertig;
				}

		}

	// Meldung rausgenommen, da diese nach Umstellung auf R/W Funktionalitit jedesmal
	// erscheint, wenn ein Sensor nicht dem Anforderungsschema entspricht.

	// sprintf(ep,"ERROR: at Sensor #%d, found unknown chipid %d",t,physical_chipid[t]);
	// log_append(4,ep);																	// Meldung ins Logfile schreiben

fertig: return(rw);
	}



// **********   sensor_poll_read()   -   Sensoren abfragen (Lese-Sensoren)   **********

signed long int sensor_poll_read(void)
	{
	signed long int rw = 0;
	signed short int t;
	static signed short int last_lowlevel = 0;
	signed short int z;



	// zuerst alle Sensoren mit Standardprioritt abfragen
	z = 0;
	for(t = 0;t < MAX_SENSORS;t++)
		{
		if(physical_enabled[t] > 0)
			{
			if (physical_priority[t] >= 0)
				{
				// printf("Poll High %d\n",t);
				rw += sensor_do_poll(t,0);											// Sensor abfragen
				// usleep(100);																		// 0.1 ms Pause zwischen einzelnen Sensoren
				}
			else
				{
				z++;																						// Low-Priority-Sensoren zhlen
				}
			}
		}

	// und nun die Sensoren mit low priority verarbeiten
	if(z > 0)
		{
llschleife:
		last_lowlevel++;
		if(last_lowlevel >= MAX_SENSORS)
			{
			last_lowlevel = -1;
			sensor_valid_flag = 1;
			goto llschleife;
			}
		if((physical_enabled[last_lowlevel] > 0) && (physical_priority[last_lowlevel] < 0))
			{
			// printf("Poll Low  %d\n",last_lowlevel);
			rw += sensor_do_poll(last_lowlevel,0);						// Sensor abfragen
			goto llfertig;
			}
		goto llschleife;
llfertig: ;
		}
	else
		{
		sensor_valid_flag = 1;
		}

	return(rw);
	}



// **********   sensor_poll_write()   -   Sensoren abfragen (Schreib-Sensoren)   **********

signed long int sensor_poll_write(void)
	{
	signed long int rw = 0;
	signed short int t;


	for(t = 0;t < MAX_SENSORS;t++)
		{
		if(physical_enabled[t] > 0)
			{
			rw += sensor_do_poll(t,1);												// Sensor abfragen
			// usleep(100);																			// 0.1 ms Pause zwischen einzelnen Sensoren
			}
		}

	return(rw);
	}



// **********   sensor_copyvalues()   -   Sensordaten in virtuelle Sensoren umkopieren   **********

void sensor_copyvalues(void)
	{
	unsigned short int t;
	unsigned short int i;
	unsigned short int e;
	double f;
	char n[88];
	unsigned char treshold = 10;													// So viele Messungen mssen berschritten sein, bevor Minima/Maxima upgedated werden


	// --- remote (XML-over-HTTP) Sensoren umkopieren ---
	for(t = 0;t < sensor_remote_anzahl;t++)
		{
		for(i = 0;i < sensor_remote_max_sensors[t];i++)
			{
			if(sensor_remote_target[t][i] >= 0)
				{
				e = sensor_remote_target[t][i];
				// printf("kopiere remote[%d][%d] --> logical[%d] (\"%s\")",t,i,e,sensor_remote_value_string[t][i]);
				strcpy(logical_string[e],sensor_remote_value_string[t][i]);
				logical_float[e] = atof(logical_string[e]);
				logical_int[e] = logical_float[e] * 100;
				// printf(", float:%4.2f, int:%ld)\n",logical_float[e],logical_int[e]);
				}
			}
		}


	// --- lokale (I2C) Sensoren umkopieren ---
	for(t = 0;t < MAX_SENSORS;t++)
		{
		if(physical_enabled[t] > 0)
			{
			for(i = 0;i < physical_channels[t];i++)
				{
				if(physical_mapto_sensor[i][t] >= 0)
					{
					e = physical_mapto_sensor[i][t];
					if (physical_mapto_math_enabled[i][t] > 0)
						{
						f = ((physical_value_float[i][t] + physical_mapto_math_add1[i][t]) * physical_mapto_math_multiply1[i][t] + physical_mapto_math_add2[i][t]) * physical_mapto_math_multiply2[i][t];
						logical_float[e] = f;
						logical_int[e] = (signed long int)(f * 100);
						sprintf(logical_string[e],"%4.2f",f);
						}
					else
						{
						logical_int[e] = physical_value_int[i][t];
						logical_float[e] = physical_value_float[i][t];
						strcpy(logical_string[e],physical_value_string[i][t]);
						}
					}
				}
			}
		}

	// wenn ein Sensor ein Metasensor ist, den Wert Meta0 in den Sensorwert umkopieren
	for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		if (logical_meta_enabled[t] > 0)
			{
			i = logical_meta_source[t][0];
			if (i >= 0)
				{
				logical_int[t] = logical_int[i];
				logical_float[t] = logical_float[i];
				strcpy(logical_string[t],logical_string[i]);
				}
			else
				{
				logical_int[t] = -204800000;
				logical_float[t] = -2048000.0;
				strcpy(logical_string[t],"-2048000.00");
				}
			}
		}


	// Mathematische Operationen ausfhren
	sensor_math();

	// Minimal- und Maximalwerte updaten
	for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		if(logical_function[t] > 0)
			{
			if (logical_float[t] > logical_maximum[t])
				{
				logical_maximum_counter[t]++;
				if (logical_maximum_counter[t] > treshold)
					{
					logical_maximum[t] = logical_float[t];
					logical_maximum_counter[t] = 0;
					sprintf(n,"/control/sensors_logical/sensor%d/maximum",t);
					sqldb_write_float2(sql_config_handle,n,logical_maximum[t]);
					}
				}
			else
				{
				logical_maximum_counter[t] = 0;
				}
			if (logical_float[t] < logical_maximum[t])
				{
				logical_minimum_counter[t]++;
				if (logical_minimum_counter[t] > treshold)
					{
					logical_minimum[t] = logical_float[t];
					logical_minimum_counter[t] = 0;
					sprintf(n,"/control/sensors_logical/sensor%d/minimum",t);
					sqldb_write_float2(sql_config_handle,n,logical_minimum[t]);
					}
				}
			else
				{
				logical_minimum_counter[t] = 0;
				}
			}
		}

	// Alarme zhlen
	alarmcount = 0;
	for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		if(logical_function[t] > 0)
			{
			if (logical_float[t] > logical_limit_max[t]) alarmcount++;
			if (logical_float[t] < logical_limit_min[t]) alarmcount++;
			}
		}

	// Summenbildung fr die Durchschnittswerte
	for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		logical_sum_float[t] += logical_float[t];
		}
	logical_sum_count++;

	}



// **********   sensor_math()   -   mathematische Operationen auf logische Sensoren anwenden   **********

void sensor_math(void)
	{
	unsigned short int t;
	unsigned short int i;
	double f;
	static char mreport_old[65536] = "";									// Buffer for the old report
	char mreport[65536];																	// Buffer for the report
	char ep[256];
	char wert[128];
	FILE *fp;


	mreport[0] = 0;																				// String lschen

	for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		if(logical_math_enabled[t] > 0)
			{
			for(i = 0;i < 4;i++)
				{
				if(logical_math_operation[t][i] >= 0)
					{
					sprintf(ep,"Sensor %d math operation %d: ",t,i);
					strcat(mreport,ep);
					if (logical_math_source[t][i] < 0)
						{
						f = logical_math_constant[t][i];						// Datenquelle ist eine Konstante
						sprintf(wert,"%9.2f",f);
						}
					else
						{
						f = logical_float[logical_math_source[t][i]]; // Datenquelle ist ein anderer logischer Sensor
						sprintf(wert,"(sensor %d)",logical_math_source[t][i]);
						}
					if (logical_math_operation[t][i] == 0)				// 0: einfaches Kopieren des Werts
						{
						logical_float[t] = f;
						sprintf(ep,"(sensor %d) = %s\n",t,wert);
						strcat(mreport,ep);
						}
					if (logical_math_operation[t][i] == 1)				// 1: Wert addieren
						{
						logical_float[t] += f;
						sprintf(ep,"(sensor %d) = (sensor %d) + %s\n",t,t,wert);
						strcat(mreport,ep);
						}
					if (logical_math_operation[t][i] == 2)				// 2: Wert subtrahieren
						{
						logical_float[t] -= f;
						sprintf(ep,"(sensor %d) = (sensor %d) - %s\n",t,t,wert);
						strcat(mreport,ep);
						}
					if (logical_math_operation[t][i] == 3)				// 3: mit dem Wert multiplizieren
						{
						logical_float[t] *= f;
						sprintf(ep,"(sensor %d) = (sensor %d) * %s\n",t,t,wert);
						strcat(mreport,ep);
						}
					if (logical_math_operation[t][i] == 4)				// 4: durch den Wert dividieren
						{
						if(f != 0)
							{
							logical_float[t] /= f;
							sprintf(ep,"(sensor %d) = (sensor %d) / %s\n",t,t,wert);
							strcat(mreport,ep);
							}
						else
							{
							logical_float[t] = -2048000.0;						// Division by Zero ! das gibt nen Fehlerwert
							sprintf(ep,"(sensor %d) = (sensor %d) / %s ERROR: Division by Zero !\n",t,t,wert);
							strcat(mreport,ep);
							}
						}
					logical_int[t] = logical_float[t] * 100;			// float auch in int umrechnen
					sprintf(logical_string[t],"%9.2f",logical_float[t]); // und in den String schreiben
					}
				}
			}
		}
	// if(strlen(mreport) > 0) printf("%s",mreport);
	if (strcmp(mreport,mreport_old) != 0)
		{
		strcpy(mreport_old,mreport);
		fp = fopen("/www/reports/sensor_math.txt","w");
		fprintf(fp,"%s",mreport);
		fclose(fp);
		}
	}



// *****   scan_report() - Ergebnis in STRDB und in Arrays schreiben   *****

void scan_report(FILE *f,signed short int nummer,char chip_name[],signed short int bus_nummer,signed short int bus_group,
								 signed short int bus_port,signed short int chip_id,signed short int chip_address,signed short int bus_speed,
								 signed long int i,signed short int function,signed short int channels,signed short int outchannels,char guess_name[],signed short int all4000_typ)
	{
	char ep[255];

	fprintf(f,"*** FOUND %d ***  %s at Bus%d, Group%d, Port%d, ID%d, address%d, (return Value=%ld, guess:%s, function:%d, ALL4000typ:%d), %d inchannels, %d outchannels\n",nummer,chip_name,bus_nummer,bus_group,bus_port,chip_id,chip_address,i,guess_name,function,all4000_typ,channels,outchannels);

	sprintf(ep,"/control/sensors_scanned/sensor%d/chipname",nummer);
	sqldb_write_string2(sql_config_handle,ep,chip_name);
	sprintf(ep,"/control/sensors_scanned/sensor%d/bus",nummer);
	sqldb_write_long2(sql_config_handle,ep,bus_nummer);
	sprintf(ep,"/control/sensors_scanned/sensor%d/group",nummer);
	sqldb_write_long2(sql_config_handle,ep,bus_group);
	sprintf(ep,"/control/sensors_scanned/sensor%d/port",nummer);
	sqldb_write_long2(sql_config_handle,ep,bus_port);
	sprintf(ep,"/control/sensors_scanned/sensor%d/chipid",nummer);
	sqldb_write_long2(sql_config_handle,ep,chip_id);
	sprintf(ep,"/control/sensors_scanned/sensor%d/chipaddress",nummer);
	sqldb_write_long2(sql_config_handle,ep,chip_address);
	sprintf(ep,"/control/sensors_scanned/sensor%d/speed",nummer);
	sqldb_write_long2(sql_config_handle,ep,bus_speed);
	sprintf(ep,"/control/sensors_scanned/sensor%d/function",nummer);
	sqldb_write_long2(sql_config_handle,ep,function);
	sprintf(ep,"/control/sensors_scanned/sensor%d/channels",nummer);
	sqldb_write_long2(sql_config_handle,ep,channels);
	sprintf(ep,"/control/sensors_scanned/sensor%d/outchannels",nummer);
	sqldb_write_long2(sql_config_handle,ep,outchannels);
	sprintf(ep,"/control/sensors_scanned/sensor%d/guess_name",nummer);
	sqldb_write_string2(sql_config_handle,ep,guess_name);
	sprintf(ep,"/control/sensors_scanned/sensor%d/all4000_typ",nummer);
	sqldb_write_long2(sql_config_handle,ep,all4000_typ);

	strcpy(scanned_chipname[nummer],chip_name);						// Chip-Bezeichnung
	scanned_bus[nummer] = bus_nummer;											// an welchem Bus hngt dieser Sensor ?
	scanned_group[nummer] = bus_group;										// an welcher Gruppe hngt der Sensor ?
	scanned_port[nummer] = bus_port;											// an welchem Port dieser Gruppe ist der Sensor ?
	scanned_chipid[nummer] = chip_id;											// Chip-ID
	scanned_chipaddress[nummer] = chip_address;						// Adresse
	scanned_speed[nummer] = bus_speed;										// Delay-Parameter
	// scanned_use_flag[nummer] = -1;												// bereits gemappt
	scanned_function[nummer] = function;									// Funktion
	scanned_channels[nummer] = channels;									// Kanle (In)
	scanned_outchannels[nummer] = outchannels;						// Kanle (Out)
	strcpy(scanned_guess_name[nummer],guess_name);				// Sensor-Benennung
	scanned_all4000_typ[nummer] = all4000_typ;						// ALL4000 Sensortyp
	}



// *****   scan_port() - Testen, welche Chips gefunden wurden   *****

signed long int scan_port(unsigned char bus_nummer,unsigned char bus_group,unsigned char bus_port,unsigned long int bus_delay,FILE *f,signed long int begin_target)
	{
	signed long int rw = 0;
	signed short int t;
	signed long int i;
	signed int chip_id;
	char chip_name[80];
	// char ep[200];
	unsigned char lm92_map[8];
	signed short int funktion = 0;
	signed short int channels = 0;
	signed short int outchannels = 0;
	char guess_name[25];
	unsigned short int all4000_typ = 0;
	char opfer[1024];
	char flag_pcf8574_5;

	// printf("scan_port(%d,%d,%d)\n",bus_nummer,bus_group,bus_port);
	bus_speed = bus_delay;																// Geschwindigkeit einstellen
	selektiere_gruppe(bus_nummer,bus_group);							// Gruppe auswhlen
	selektiere_port(bus_nummer,bus_port);									// Port auswhlen
	i2c_ruhe(bus_nummer);																	// Ruhe am Bus (Initialisierung auf Standardwerte)


	// LM92_Map lschen
	for(t = 0;t < 8;t++)
		{
		lm92_map[t] = 0;
		}

	// -----   ADS1100   -----
	flag_pcf8574_5 = 0;
	t = 5;
	i = all_i2c_chip_pcf8574_read(bus_nummer,t);
	if (i > 0)
		{
		chip_id = 164; // wird momentan ignoriert
		funktion = 0;
		channels = 0;
		outchannels = 0;
		all4000_typ = 0;
		strcpy(chip_name,"PCF8574");
		strcpy(guess_name,"ALL4590_MUX");
		flag_pcf8574_5 = 1;
		scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
		rw++;
		}
	i2c_send_stop(bus_nummer);
	// printf("flag_pcf8574_5=%d\n",flag_pcf8574_5);

	chip_id = 129;
	funktion = 3;
	strcpy(chip_name,"ADS1100");
	for(t = 0;t <= 7;t++)
		{
		if(lm92_map[t] == 0)
			{
			channels = 4;
			outchannels = 0;
			if(t == 0) 															{ all4000_typ = 33;	strcpy(guess_name,"ALL5000intADC.0"); }
			if((t == 0) && (flag_pcf8574_5 > 0))	 	{ all4000_typ =130;	strcpy(guess_name,"ALL4590_ADC"); channels = 3; chip_id = 130; funktion = 6;}
			if(t == 1) 	{ all4000_typ = 33;	strcpy(guess_name,"ALL5000intADC.1"); }
			if(t == 2) 	{ all4000_typ = 33;	strcpy(guess_name,"ALL5000intADC.2"); }
			if(t == 3) 	{ all4000_typ = 33;	strcpy(guess_name,"ALL5000intADC.3"); }
			if(t == 4) 	{ all4000_typ = 33;	strcpy(guess_name,"ALL5000intADC.4"); }
			if(t == 5) 	{ all4000_typ = 33;	strcpy(guess_name,"ALL5000intADC.5"); }
			if(t == 6) 	{ all4000_typ = 33;	strcpy(guess_name,"ALL5000intADC.6"); }
			if(t == 7) 	{ all4000_typ = 33;	strcpy(guess_name,"ALL5000intADC.7"); }
			// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,t);
			i = all_i2c_chip_ads1100_read(bus_nummer,t,14);
			if (i > -204800000)
				{
				lm92_map[t] = 1;
				scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
				rw++;
				i2c_send_stop(bus_nummer);											// Datentransfer sauber beenden
				}
			else
				{
				i2c_send_stop(bus_nummer);											// Datentransfer sauber beenden
				}
			}
		}	
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden

	// -----   LM92   -----
	chip_id = 2;
	funktion = 1;
	strcpy(chip_name,"LM92");
	channels = 1;
	outchannels = 0;
	for(t = 0;t <= 3;t++)
		{
		if(lm92_map[t] == 0)
			{
			if(t == 0) 	{ all4000_typ = 2;	strcpy(guess_name,"ALL3007.0"); }
			if(t == 1) 	{ all4000_typ = 2;	strcpy(guess_name,"ALL3007.1"); }
			if(t == 2) 	{ all4000_typ = 2;	strcpy(guess_name,"ALL3007.2"); }
			if(t == 3) 	{ all4000_typ = 2;	strcpy(guess_name,"ALL3006");      }
			// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,t);
			i = all_i2c_chip_lm92_read_temp(bus_nummer,t);
			if (i > -204800000)
				{
				lm92_map[t] = 1;
				scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
				rw++;
				i2c_send_stop(bus_nummer);											// Datentransfer sauber beenden
				}
			else
				{
				i2c_send_stop(bus_nummer);											// Datentransfer sauber beenden
				}
			}
		}	
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden

	// -----   LM75   -----
	chip_id = 1;
	funktion = 1;
	strcpy(chip_name,"LM75");
	channels = 1;
	outchannels = 0;
	for(t = 0;t <= 7;t++)
		{
		if(lm92_map[t] == 1) goto skip_lm75;
		if(t == 0) 	{ all4000_typ = 1;	strcpy(guess_name,"ALL3005.0");	}
		if(t == 1) 	{ all4000_typ = 1;	strcpy(guess_name,"ALL3005.1");	}
		if(t == 2) 	{ all4000_typ = 1;	strcpy(guess_name,"ALL3005.2");	}
		if(t == 3) 	{ all4000_typ = 1;	strcpy(guess_name,"ALL3005.3");	}
		if(t == 4) 	{ all4000_typ = 1;	strcpy(guess_name,"ALL3005.4");	}
		if(t == 5) 	{ all4000_typ = 1;	strcpy(guess_name,"ALL3005.5");	}
		if(t == 6) 	{ all4000_typ = 1;	strcpy(guess_name,"ALL3005.6");	}
		if(t == 7) 	{ all4000_typ = 1;	strcpy(guess_name,"ALL3005");				}

		// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,t);
		i = all_i2c_chip_lm75_read_temp(bus_nummer,t);
		if (i > -204800000)
			{
			scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
			rw++;
			}
		else
			{
			i2c_send_stop(bus_nummer);												// Datentransfer sauber beenden
			}
skip_lm75: ;
		}	
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden


	// -----   SHT71 Temperatur   -----
	chip_id = 3;
	funktion = 1;
	strcpy(guess_name,"ALL3018T");
	all4000_typ = 3;
	strcpy(chip_name,"SHT71_T");
	channels = 1;
	outchannels = 0;
	t = 0; // ID ist immer 0, da sie nicht gendert werden kann
	// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,t);
	i = all_i2c_chip_sht71_read_temp(bus_nummer);
	if (i > -204800000)
		{
		scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
		rw++;
		}
	else
		{
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
		}
	i2c_send_stop(bus_nummer);														// Datentransfer sauber beenden

	// -----   SHT71 Luftfeuchtigkeit   -----
	chip_id = 32;
	funktion = 2;
	all4000_typ = 65;	
	strcpy(guess_name,"ALL3018H");
	strcpy(chip_name,"SHT71_H");
	channels = 1;
	outchannels = 0;
	t = 0; // ID ist immer 0, da sie nicht gendert werden kann
	// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,t);
	i = all_i2c_chip_sht71_read_humidity(bus_nummer);
	if (i > -204800000)
		{
		scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
		rw++;
		}
	else
		{
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
		}
	i2c_send_stop(bus_nummer);														// Datentransfer sauber beenden

	// -----   DSDX015A2R Luftdruck   -----
	chip_id = 40;
	funktion = 16;
	all4000_typ = 40;	
	strcpy(guess_name,"ALL3028");
	strcpy(chip_name,"DSDX015A2R");
	channels = 1;
	outchannels = 0;
	t = 0; // ID ist immer 0, da sie nicht gendert werden kann
	// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,t);
	i = all_i2c_chip_dsdx015a2r_read_pressure(bus_nummer);
	if (i > -204800000)
		{
		scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
		rw++;
		}
	else
		{
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
		}
	i2c_send_stop(bus_nummer);														// Datentransfer sauber beenden

	// -----   ALL4023 3-fach Spannung   -----
	chip_id = 260;
	funktion = 3;
	all4000_typ = 153;	
	strcpy(guess_name,"ALL4023");
	strcpy(chip_name,"ATTINY85");
	channels = 3;
	outchannels = 0;
	t = 0; // ID ist immer 0, da sie nicht gendert werden kann
	// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,t);
	i = all_i2c_chip_all4023_read_voltage(bus_nummer,0);
	if (i > -204800000)
		{
		scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
		rw++;
		}
	else
		{
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
		}
	i2c_send_stop(bus_nummer);														// Datentransfer sauber beenden


	// -----   ALL3022 2-fach Spannung   -----
	chip_id = 262;
	funktion = 3;
	all4000_typ = 32;	
	strcpy(guess_name,"ALL3022");
	strcpy(chip_name,"MAX1237");
	channels = 2;
	outchannels = 0;
	t = 0; // ID ist immer 0, da sie nicht gendert werden kann
	// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,t);
	i = all_i2c_chip_all3022_read_voltage(bus_nummer,1);
	if (i > -204800000)
		{
		scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
		rw++;
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
		goto no_3021;
		}
	else
		{
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
		}

	// -----   ALL3021 1-fach Spannung   -----
	chip_id = 261;
	funktion = 3;
	all4000_typ = 32;	
	strcpy(guess_name,"ALL3021");
	strcpy(chip_name,"MAX1237");
	channels = 1;
	outchannels = 0;
	t = 0; // ID ist immer 0, da sie nicht gendert werden kann
	// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,t);
	i = all_i2c_chip_all3022_read_voltage(bus_nummer,0);
	if (i > -204800000)
		{
		scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
		rw++;
		}
	else
		{
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
		}
	i2c_send_stop(bus_nummer);														// Datentransfer sauber beenden
no_3021: ;

	// -----   ALL4529 Luftdruck   -----
	chip_id = 42;
	funktion = 16;
	all4000_typ = 42;	
	strcpy(guess_name,"ALL4529");
	strcpy(chip_name,"BMP085");
	channels = 1;
	outchannels = 0;
	t = 0; // ID ist immer 0, da sie nicht gendert werden kann
	// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,t);
	i = all_i2c_chip_bmp085_read_pressure(bus_nummer);
	if (i > -204800000)
		{
		scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
		rw++;
		}
	else
		{
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
		}
	i2c_send_stop(bus_nummer);														// Datentransfer sauber beenden

	// -----   ALL3032 Helligkeit   -----
	chip_id = 259;
	funktion = 9;
	all4000_typ = 155;	
	strcpy(guess_name,"ALL3032");
	strcpy(chip_name,"ATTINY85");
	channels = 1;
	outchannels = 0;
	t = 0; // ID ist immer 0, da sie nicht gendert werden kann
	// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,t);
	i = all_i2c_chip_all3032_read_brightness(bus_nummer);
	if (i > -204800000)
		{
		scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
		rw++;
		}
	else
		{
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
		}
	i2c_send_stop(bus_nummer);														// Datentransfer sauber beenden

	// -----   ALL3025 Zhler   -----
	chip_id = 96;
	funktion = 8;
	all4000_typ = 133;	
	strcpy(guess_name,"ALL3025");
	strcpy(chip_name,"PCF8583");
	channels = 1;
	outchannels = 0;
	t = 1; // ID ist immer 1, da sie nicht gendert werden kann
	// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,t);
	i = all_i2c_chip_all3025_read_counter(bus_nummer);
	if (i > -204800000)
		{
		scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
		rw++;
		}
	else
		{
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
		}
	i2c_send_stop(bus_nummer);														// Datentransfer sauber beenden

	// -----   ALL4035 Zhler   -----
	chip_id = 97;
	funktion = 8;
	all4000_typ = 134;	
	strcpy(guess_name,"ALL4035");
	strcpy(chip_name,"PCF8583");
	channels = 1;
	outchannels = 0;
	t = 0; // ID ist immer 0, da sie nicht gendert werden kann
	// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,t);
	i = all_i2c_chip_all4035_read_counter(bus_nummer);
	if (i > -204800000)
		{
		scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
		rw++;
		}
	else
		{
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
		}
	i2c_send_stop(bus_nummer);														// Datentransfer sauber beenden


	// -----   ALL4042 8-port Eingang als einzelne Bits   -----
	chip_id = 77;
	funktion = 12;
	all4000_typ = 102;
	strcpy(guess_name,"ALL4042");
	strcpy(chip_name,"PCF8574R");
	channels = 8;
	outchannels = 0;

	// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,6);
	i = all_i2c_chip_pcf8574_read(bus_nummer,6);
	if (i >= 0)
		{
		scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,6,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
		rw++;
		}
	else
		{
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
		}
	i2c_send_stop(bus_nummer);														// Datentransfer sauber beenden


	// -----   ALL4027 8-port Ausgang als einzelne Bits   -----
	chip_id = 67;
	funktion = 11;
	all4000_typ = 102;
	strcpy(guess_name,"ALL4027");
	strcpy(chip_name,"PCF8574W");
	channels = 0;
	outchannels = 8;

	// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,6);
	i = all_i2c_chip_pcf8574_read(bus_nummer,6);
	if (i >= 0)
		{
		scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,6,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
		rw++;
		}
	else
		{
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
		}
	i2c_send_stop(bus_nummer);														// Datentransfer sauber beenden

	// -----   ALL3026/ALL4026 1-port Eingang   -----
	chip_id = 74;
	funktion = 12;
	all4000_typ = 101;
	strcpy(guess_name,"ALL4026");
	strcpy(chip_name,"PCF8574R");
	channels = 1;
	outchannels = 0;

	// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,6);
	i = all_i2c_chip_pcf8574_read(bus_nummer,7);
	if (i >= 0)
		{
		scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,7,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
		rw++;
		}
	else
		{
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
		}
	i2c_send_stop(bus_nummer);														// Datentransfer sauber beenden


	// -----   ALL4020 1-port Ausgang   -----
	chip_id = 64;
	funktion = 11;
	all4000_typ = 97;
	strcpy(guess_name,"ALL4020");
	strcpy(chip_name,"PCF8574W");
	channels = 0;
	outchannels = 1;

	// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,6);
	i = all_i2c_chip_pcf8574_read(bus_nummer,0);
	if (i >= 0)
		{
		scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,0,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
		rw++;
		}
	else
		{
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
		}
	i2c_send_stop(bus_nummer);														// Datentransfer sauber beenden

	/*
	// -----   andere, unspezifizierte PCF8574   -----
	chip_id = 199;
	strcpy(chip_name,"PCF8574");
	for(t = 0;t <= 7;t++)
		{
		if(t == 0) { funktion = 11;	all4000_typ =  97;	chip_id =  64;	strcpy(guess_name,"ALL3002/ALL3003");		channels = 1;	outchannels = 0; }
		if(t == 1) { funktion = 11;	all4000_typ =   0;	chip_id = 199;	strcpy(guess_name,"-");									channels = 8;	outchannels = 0; }
		if(t == 2) { funktion = 11;	all4000_typ =   0;	chip_id = 199;	strcpy(guess_name,"-");									channels = 8;	outchannels = 0; }
		if(t == 3) { funktion = 11;	all4000_typ = 103;	chip_id =  67;	strcpy(guess_name,"ALL4040W");					channels = 0;	outchannels = 8; }
		if(t == 4) { funktion = 12;	all4000_typ = 103;	chip_id =  77;	strcpy(guess_name,"ALL4040R");					channels = 6;	outchannels = 0; }
		if(t == 5) { funktion = 11;	all4000_typ =   0;	chip_id = 199;	strcpy(guess_name,"-");									channels = 8;	outchannels = 0; }
		if(t == 6) { funktion = 12;	all4000_typ = 102;	chip_id =  77;	strcpy(guess_name,"ALL4042/ALL3027");		channels = 1;	outchannels = 8; }
		if(t == 7) { funktion = 12;	all4000_typ = 101;	chip_id =  74;	strcpy(guess_name,"ALL3026/ALL3024");		channels = 1;	outchannels = 0; }

		// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,t);
		i = all_i2c_chip_pcf8574_read(bus_nummer,t);
		if (i >= 0)
			{
			scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
			rw++;
			}
		else
			{
			i2c_send_stop(bus_nummer);												// Datentransfer sauber beenden
			}
		}	
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
	*/

	// -----   PCF8574A   -----
	chip_id = 200;
	strcpy(chip_name,"PCF8574A");
	for(t = 0;t <= 7;t++)
		{
		if(t == 0) { funktion = 11;	all4000_typ = 300;	chip_id = 167;	strcpy(guess_name,"ALL4500.0");	channels = 8;	outchannels = 0; }
		if(t == 1) { funktion = 11;	all4000_typ = 300;	chip_id = 167;	strcpy(guess_name,"ALL4500.1");	channels = 8;	outchannels = 0; }
		if(t == 2) { funktion = 12;	all4000_typ = 301;	chip_id = 177;	strcpy(guess_name,"ALL4500.2");	channels = 8;	outchannels = 0; }
		if(t == 3) { funktion = 11;	all4000_typ = 300;	chip_id = 167;	strcpy(guess_name,"ALL4500.3");	channels = 8;	outchannels = 0; }
		if(t == 4) { funktion = 11;	all4000_typ = 300;	chip_id = 167;	strcpy(guess_name,"ALL4500.4");	channels = 8;	outchannels = 0; }
		if(t == 5) { funktion = 11;	all4000_typ = 300;	chip_id = 167;	strcpy(guess_name,"ALL4500.5");	channels = 8;	outchannels = 0; }
		if(t == 6) { funktion = 11;	all4000_typ = 300;	chip_id = 167;	strcpy(guess_name,"ALL4500.6");	channels = 8;	outchannels = 0; }
		if(t == 7) { funktion = 11;	all4000_typ = 300;	chip_id = 167;	strcpy(guess_name,"ALL4500.7");	channels = 8;	outchannels = 0; }

		// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,t);
		i = all_i2c_chip_pcf8574a_read(bus_nummer,t);
		if (i >= 0)
			{
			scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
			rw++;
			}
		else
			{
			i2c_send_stop(bus_nummer);												// Datentransfer sauber beenden
			}
		}	
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden

	i2c_send_stop(bus_nummer);														// Datentransfer sauber beenden

	// -----   RFID001   -----
	chip_id = 300;
	funktion = 15;
	all4000_typ = 300;	
	strcpy(guess_name,"RFID001");
	strcpy(chip_name,"RFID001");
	channels = 1;
	outchannels = 0;
	t = 0; // ID ist immer 0, da sie nicht gendert werden kann
	// fprintf(f,"scanning Bus%d, Group%d, Port%d for %s at ID%d\n",bus_nummer,bus_group,bus_port,chip_name,t);
	i = all_i2c_chip_rfid001_read_cardid(bus_nummer,opfer);
	if (i > -204800000)
		{
		scan_report(f,begin_target + rw,chip_name,bus_nummer,bus_group,bus_port,chip_id,t,bus_speed,i,funktion,channels,outchannels,guess_name,all4000_typ);
		rw++;
		}
	else
		{
		i2c_send_stop(bus_nummer);													// Datentransfer sauber beenden
		}
	i2c_send_stop(bus_nummer);														// Datentransfer sauber beenden



	i2c_ruhe(bus_nummer);																	// Ruhe am Bus (Initialisierung auf Standardwerte)
	fflush(f);
	return(rw);
	}



// *****   portscan_add_internalsensors()   -   interne Sensoren gleich als Preset in die DB schreiben   *****

void portscan_add_internalsensors(void)
	{
	signed short int int_speed = 25;

	// printf("hwmodel: %d\n",hwmodel);
	if (hwmodel == 4500)
		{
		// printf("ALL4500...\n");

		// interner Temperatursensor LM92 auf ID 3
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/enabled",					1);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor0/name",							"int. Temperature");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor0/comment",					"LM92 at ID3, mounted on the carrier PCB");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/bus",							0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/group",						0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/port",							0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/speed",						int_speed);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/chipid",						2);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/chipaddress",			3);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor0/chipname",					"LM92");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channels",					1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/all4000typ",				1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/function",1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/value_int",-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/value_float",-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/value_string","-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/mapto_sensor",0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/maptp_math_enabled",0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/mapto_math_add1",0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/mapto_math_multiply1",1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/mapto_math_add2",0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/mapto_math_multiply2",1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/outchannels",			0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/out_value_command",-1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/out_value_int",-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/out_value_float",-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/out_value_string","-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/mapfrom_actor",-1);
		// dazu passender logischer Sensor
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor0/name",														"int. Temperature");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor0/comment",												"mounted inside the ALL4500 main unit");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor0/function",												1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor0/all4000typ",											1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor0/value_int",											-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor0/value_float",										-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor0/value_string",										"-2048000.00");
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor0/limit_min",											10.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor0/limit_max",											50.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor0/display_min",										5.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor0/display_max",										60.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor0/minimum",												2048000.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor0/maximum",												-2048000.0);
		// eintragen auf Anzeigeseite 0
		sqldb_write_long2		(sql_config_handle,"/control/sensors_display/page0/sensor0",													0);

		// PCF8574A auf ID0 - LED-Driver fr ALL4500 interne LEDs
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/enabled",														1);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor1/name",																"int. PCF8574A#0");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor1/comment",														"bit 0: BUS Led driver");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/bus",																0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/group",															0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/port",																0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/speed",															int_speed);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/chipid",															164);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/chipaddress",												0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor1/chipname",														"PCF8574A");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channels",														0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/all4000typ",													0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/function",									0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/value_int",									-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/value_float",								-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/value_string",							"-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/mapto_sensor",							-1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/maptp_math_enabled",				0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/mapto_math_add1",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/mapto_math_multiply1",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/mapto_math_add2",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/mapto_math_multiply2",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/outchannels",												0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/mapfrom_actor",							-1);

		// PCF8574 auf ID 3 (LEDs 0...7)
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/enabled",														1);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/name",																"int. PCF8574#3");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/comment",														"bit 0...7: LEDs 0...7");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/bus",																0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/group",															0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/port",																0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/speed",															int_speed);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/chipid",															67);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/chipaddress",												3);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/chipname",														"PCF8574");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channels",														0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/all4000typ",													0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/function",									0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/value_int",									-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/value_float",								-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/value_string",							"-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/mapto_sensor",							-1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/maptp_math_enabled",				0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/mapto_math_add1",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/mapto_math_multiply1",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/mapto_math_add2",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/mapto_math_multiply2",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/outchannels",												8);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/mapfrom_actor",							0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel1/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel1/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel1/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel1/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel1/mapfrom_actor",							1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel2/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel2/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel2/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel2/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel2/mapfrom_actor",							2);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel3/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel3/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel3/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel3/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel3/mapfrom_actor",							3);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel4/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel4/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel4/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel4/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel4/mapfrom_actor",							4);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel5/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel5/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel5/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel5/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel5/mapfrom_actor",							5);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel6/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel6/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel6/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel6/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel6/mapfrom_actor",							6);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel7/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel7/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel7/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel7/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel7/mapfrom_actor",							7);
		// PCF8574 auf ID 4 (LEDs 8...15)
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/enabled",														1);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/name",																"int. PCF8574#4");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/comment",														"bit 0...7: LEDs 8...15");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/bus",																0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/group",															0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/port",																0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/speed",															int_speed);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/chipid",															67);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/chipaddress",												4);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/chipname",														"PCF8574");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channels",														0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/all4000typ",													0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/function",									0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/value_int",									-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/value_float",								-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/value_string",							"-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/mapto_sensor",							-1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/maptp_math_enabled",				0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/mapto_math_add1",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/mapto_math_multiply1",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/mapto_math_add2",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/mapto_math_multiply2",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/outchannels",												8);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/mapfrom_actor",							8);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel1/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel1/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel1/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel1/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel1/mapfrom_actor",							9);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel2/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel2/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel2/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel2/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel2/mapfrom_actor",							10);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel3/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel3/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel3/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel3/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel3/mapfrom_actor",							11);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel4/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel4/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel4/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel4/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel4/mapfrom_actor",							12);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel5/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel5/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel5/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel5/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel5/mapfrom_actor",							13);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel6/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel6/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel6/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel6/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel6/mapfrom_actor",							14);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel7/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel7/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel7/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel7/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel7/mapfrom_actor",							15);
		// die passenden Aktoren dazu
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor0/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor0/name",																			"LED 0");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor0/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor0/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor0/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor0/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor0/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor1/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor1/name",																			"LED 1");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor1/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor1/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor1/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor1/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor1/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor2/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor2/name",																			"LED 2");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor2/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor2/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor2/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor2/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor2/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor3/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor3/name",																			"LED 3");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor3/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor3/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor3/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor3/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor3/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor4/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor4/name",																			"LED 4");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor4/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor4/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor4/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor4/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor4/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor5/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor5/name",																			"LED 5");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor5/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor5/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor5/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor5/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor5/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor6/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor6/name",																			"LED 6");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor6/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor6/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor6/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor6/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor6/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor7/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor7/name",																			"LED 7");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor7/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor7/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor7/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor7/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor7/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor8/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor8/name",																			"LED 8");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor8/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor8/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor8/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor8/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor8/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor9/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor9/name",																			"LED 9");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor9/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor9/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor9/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor9/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor9/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor10/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor10/name",																		"LED 10");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor10/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor10/type",																		1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor10/mapto_sensor",														-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor10/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor10/wattmapto_sensor",												-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor11/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor11/name",																		"LED 11");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor11/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor11/type",																		1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor11/mapto_sensor",														-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor11/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor11/wattmapto_sensor",												-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor12/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor12/name",																		"LED 12");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor12/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor12/type",																		1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor12/mapto_sensor",														-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor12/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor12/wattmapto_sensor",												-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor13/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor13/name",																		"LED 13");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor13/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor13/type",																		1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor13/mapto_sensor",														-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor13/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor13/wattmapto_sensor",												-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor14/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor14/name",																		"LED 14");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor14/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor14/type",																		1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor14/mapto_sensor",														-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor14/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor14/wattmapto_sensor",												-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor15/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor15/name",																		"LED 15");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor15/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor15/type",																		1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor15/mapto_sensor",														-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor15/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor15/wattmapto_sensor",												-1);
		// eintragen auf Aktorenseite 0
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor0",														0);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor1",														1);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor2",														2);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor3",														3);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor4",														4);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor5",														5);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor6",														6);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor7",														7);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor8",														8);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor9",														9);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor10",														10);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor11",														11);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor12",														12);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor13",														13);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor14",														14);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor15",														15);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/enabled",														1);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/appearance",												0);
		sqldb_write_string2	(sql_config_handle,"/control/actors_display/page0/link_title",												"show_actors.php?nr=0");
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/max_actors",												16);
		sqldb_write_string2	(sql_config_handle,"/control/actors_display/page0/page_headline",											"LEDs");

		// Verwaltungs-Infos
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/max_sensors",																4);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/max_sensors",														1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_display/page0/max_sensors",											1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/max_actors",																			16);
		portscan_max_physical_sensors = 4;
		portscan_max_logical_sensors = 1;
		portscan_max_logical_actors = 16;
		}

	if (hwmodel == 5000)
		{
		// printf("ALL5000...\n");

		// interner Temperatursensor LM92 auf ID 3
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/enabled",					1);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor0/name",							"int. Temperature");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor0/comment",					"LM92 at ID3, mounted on the front PCB");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/bus",							0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/group",						0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/port",							0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/speed",						int_speed);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/chipid",						2);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/chipaddress",			3);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor0/chipname",					"LM92");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channels",					1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/all4000typ",				1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/function",1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/value_int",-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/value_float",-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/value_string","-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/mapto_sensor",0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/maptp_math_enabled",0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/mapto_math_add1",0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/mapto_math_multiply1",1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/mapto_math_add2",0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/mapto_math_multiply2",1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/outchannels",			0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/out_value_command",-1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/out_value_int",-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/out_value_float",-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/out_value_string","-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor0/channel0/mapfrom_actor",-1);
		// dazu passender logischer Sensor
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor0/name",														"int. Temperature");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor0/comment",												"mounted inside the ALL5000 main unit");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor0/function",												1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor0/all4000typ",											1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor0/value_int",											-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor0/value_float",										-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor0/value_string",										"-2048000.00");
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor0/limit_min",											10.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor0/limit_max",											50.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor0/display_min",										5.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor0/display_max",										60.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor0/minimum",												2048000.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor0/maximum",												-2048000.0);
		// eintragen auf Anzeigeseite 0
		sqldb_write_long2		(sql_config_handle,"/control/sensors_display/page0/sensor0",													0);
	

		// interner A/D-Wandler ADS1100 auf ID 0
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/enabled",														1);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor1/name",																"int. ADC");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor1/comment",														"for the 4 analog Voltage inputs at the rear of the unit");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/bus",																0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/group",															0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/port",																0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/speed",															int_speed);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/chipid",															129);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/chipaddress",												0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor1/chipname",														"ADS1100");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channels",														4);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/all4000typ",													33);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/function",									3);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/value_int",									-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/value_float",								-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/value_string",							"-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/mapto_sensor",							1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/maptp_math_enabled",				0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/mapto_math_add1",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/mapto_math_multiply1",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/mapto_math_add2",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/mapto_math_multiply2",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel1/function",									3);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel1/value_int",									-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel1/value_float",								-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel1/value_string",							"-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel1/mapto_sensor",							2);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel1/maptp_math_enabled",				0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel1/mapto_math_add1",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel1/mapto_math_multiply1",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel1/mapto_math_add2",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel1/mapto_math_multiply2",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel2/function",									3);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel2/value_int",									-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel2/value_float",								-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel2/value_string",							"-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel2/mapto_sensor",							3);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel2/maptp_math_enabled",				0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel2/mapto_math_add1",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel2/mapto_math_multiply1",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel2/mapto_math_add2",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel2/mapto_math_multiply2",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel3/function",									3);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel3/value_int",									-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel3/value_float",								-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel3/value_string",							"-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel3/mapto_sensor",							4);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel3/maptp_math_enabled",				0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel3/mapto_math_add1",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel3/mapto_math_multiply1",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel3/mapto_math_add2",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel3/mapto_math_multiply2",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/outchannels",												0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/out_value_command",					-1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/out_value_int",							-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/out_value_float",						-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/out_value_string",					"-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor1/channel0/mapfrom_actor",							-1);
		// dazu passende logische Sensoren
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor1/name",														"ADC 0");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor1/comment",												"");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor1/function",												3);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor1/all4000typ",											33);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor1/value_int",											-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor1/value_float",										-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor1/value_string",										"-2048000.00");
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor1/limit_min",											-0.5);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor1/limit_max",											50.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor1/display_min",										-1.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor1/display_max",										60.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor1/minimum",												2048000.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor1/maximum",												-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor2/name",														"ADC 1");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor2/comment",												"");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor2/function",												3);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor2/all4000typ",											33);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor2/value_int",											-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor2/value_float",										-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor2/value_string",										"-2048000.00");
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor2/limit_min",											-0.5);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor2/limit_max",											50.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor2/display_min",										-1.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor2/display_max",										60.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor2/minimum",												2048000.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor2/maximum",												-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor3/name",														"ADC 2");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor3/comment",												"");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor3/function",												3);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor3/all4000typ",											33);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor3/value_int",											-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor3/value_float",										-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor3/value_string",										"-2048000.00");
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor3/limit_min",											-0.5);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor3/limit_max",											50.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor3/display_min",										-1.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor3/display_max",										60.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor3/minimum",												2048000.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor3/maximum",												-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor4/name",														"ADC 3");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor4/comment",												"");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor4/function",												3);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor4/all4000typ",											33);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor4/value_int",											-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor4/value_float",										-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor4/value_string",										"-2048000.00");
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor4/limit_min",											-0.5);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor4/limit_max",											50.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor4/display_min",										-1.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor4/display_max",										60.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor4/minimum",												2048000.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor4/maximum",												-2048000.0);
		// eintragen auf Anzeigeseite 0
		sqldb_write_long2		(sql_config_handle,"/control/sensors_display/page0/sensor1",													1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_display/page0/sensor2",													2);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_display/page0/sensor3",													3);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_display/page0/sensor4",													4);

		// PCF8574 auf ID 2 (untere 4 Bits: Digitaleingnge, obere Bits 5...7 Multiplexer-Steurerung fr ADC)
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/enabled",														1);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/name",																"int. PCF8574#2");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/comment",														"lower 4 bits are the digital inputs");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/bus",																0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/group",															0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/port",																0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/speed",															int_speed);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/chipid",															198);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/chipaddress",												2);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/chipname",														"PCF8574");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channels",														4);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/all4000typ",													0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/function",									12);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/value_int",									-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/value_float",								-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/value_string",							"-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/mapto_sensor",							5);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/maptp_math_enabled",				0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/mapto_math_add1",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/mapto_math_multiply1",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/mapto_math_add2",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/mapto_math_multiply2",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel1/function",									12);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel1/value_int",									-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel1/value_float",								-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel1/value_string",							"-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel1/mapto_sensor",							6);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel1/maptp_math_enabled",				0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel1/mapto_math_add1",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel1/mapto_math_multiply1",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel1/mapto_math_add2",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel1/mapto_math_multiply2",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel2/function",									12);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel2/value_int",									-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel2/value_float",								-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel2/value_string",							"-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel2/mapto_sensor",							7);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel2/maptp_math_enabled",				0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel2/mapto_math_add1",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel2/mapto_math_multiply1",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel2/mapto_math_add2",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel2/mapto_math_multiply2",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel3/function",									12);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel3/value_int",									-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel3/value_float",								-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel3/value_string",							"-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel3/mapto_sensor",							8);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel3/maptp_math_enabled",				0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel3/mapto_math_add1",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel3/mapto_math_multiply1",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel3/mapto_math_add2",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel3/mapto_math_multiply2",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/outchannels",												0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/out_value_command",					-1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/out_value_int",							-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/out_value_float",						-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/out_value_string",					"-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor2/channel0/mapfrom_actor",							-1);
		// dazu passende logische Sensoren
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor5/name",														"Digital In 0");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor5/comment",												"");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor5/function",												12);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor5/all4000typ",											101);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor5/value_int",											-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor5/value_float",										-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor5/value_string",										"-2048000.00");
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor5/limit_min",											-0.5);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor5/limit_max",											1.5);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor5/display_min",										-1.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor5/display_max",										2.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor5/minimum",												2048000.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor5/maximum",												-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor6/name",														"Digital In 1");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor6/comment",												"");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor6/function",												12);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor6/all4000typ",											101);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor6/value_int",											-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor6/value_float",										-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor6/value_string",										"-2048000.00");
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor6/limit_min",											-0.5);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor6/limit_max",											1.5);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor6/display_min",										-1.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor6/display_max",										2.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor6/minimum",												2048000.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor6/maximum",												-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor7/name",														"Digital In 2");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor7/comment",												"");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor7/function",												12);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor7/all4000typ",											101);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor7/value_int",											-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor7/value_float",										-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor7/value_string",										"-2048000.00");
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor7/limit_min",											-0.5);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor7/limit_max",											1.5);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor7/display_min",										-1.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor7/display_max",										2.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor7/minimum",												2048000.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor7/maximum",												-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor8/name",														"Digital In 3");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor8/comment",												"");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor8/function",												12);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor8/all4000typ",											101);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/sensor8/value_int",											-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor8/value_float",										-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_logical/sensor8/value_string",										"-2048000.00");
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor8/limit_min",											-0.5);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor8/limit_max",											1.5);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor8/display_min",										-1.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor8/display_max",										2.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor8/minimum",												2048000.0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_logical/sensor8/maximum",												-2048000.0);
		// eintragen auf Anzeigeseite 0
		sqldb_write_long2		(sql_config_handle,"/control/sensors_display/page0/sensor5",													5);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_display/page0/sensor6",													6);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_display/page0/sensor7",													7);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_display/page0/sensor8",													8);

		// PCF8574 auf ID 1 (untere 4 Bits: 230V-Relais, obere 4 Bits Reedrelais)
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/enabled",														1);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/name",																"int. PCF8574#1");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/comment",														"bit 0...3: 230V Relays, bit 4...7: reed Relays");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/bus",																0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/group",															0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/port",																0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/speed",															int_speed);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/chipid",															67);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/chipaddress",												1);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/chipname",														"PCF8574");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channels",														0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/all4000typ",													0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/function",									0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/value_int",									-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/value_float",								-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/value_string",							"-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/mapto_sensor",							-1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/maptp_math_enabled",				0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/mapto_math_add1",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/mapto_math_multiply1",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/mapto_math_add2",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/mapto_math_multiply2",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/outchannels",												8);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel0/mapfrom_actor",							0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel1/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel1/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel1/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel1/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel1/mapfrom_actor",							1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel2/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel2/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel2/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel2/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel2/mapfrom_actor",							2);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel3/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel3/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel3/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel3/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel3/mapfrom_actor",							3);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel4/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel4/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel4/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel4/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel4/mapfrom_actor",							4);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel5/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel5/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel5/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel5/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel5/mapfrom_actor",							5);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel6/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel6/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel6/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel6/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel6/mapfrom_actor",							6);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel7/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel7/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel7/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor3/channel7/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor3/channel7/mapfrom_actor",							7);
		// die passenden Aktoren dazu
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor0/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor0/name",																			"Power Relay 0");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor0/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor0/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor0/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor0/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor0/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor1/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor1/name",																			"Power Relay 1");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor1/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor1/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor1/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor1/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor1/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor2/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor2/name",																			"Power Relay 2");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor2/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor2/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor2/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor2/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor2/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor3/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor3/name",																			"Power Relay 3");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor3/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor3/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor3/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor3/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor3/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor4/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor4/name",																			"Reed Relay 0");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor4/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor4/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor4/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor4/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor4/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor5/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor5/name",																			"Reed Relay 1");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor5/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor5/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor5/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor5/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor5/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor6/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor6/name",																			"Reed Relay 2");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor6/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor6/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor6/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor6/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor6/wattmapto_sensor",													-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor7/enabled",																	1);
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor7/name",																			"Reed Relay 3");
		sqldb_write_string2	(sql_config_handle,"/control/actors/actor7/comment",																	"");
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor7/type",																			1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor7/mapto_sensor",															-1);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor7/bootdefault",															0);
		sqldb_write_long2		(sql_config_handle,"/control/actors/actor7/wattmapto_sensor",													-1);
		// eintragen auf Aktorenseite 0
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor0",														0);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor1",														1);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor2",														2);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor3",														3);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor4",														4);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor5",														5);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/actor6",														6);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/enabled",														1);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/appearance",												0);
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/sensorsx",													3);
		sqldb_write_string2	(sql_config_handle,"/control/actors_display/page0/link_title",												"show_actors.php?nr=0");
		sqldb_write_long2		(sql_config_handle,"/control/actors_display/page0/max_actors",												8);
		sqldb_write_string2	(sql_config_handle,"/control/actors_display/page0/page_headline",											"Actors");
		// Reloadcounter erhhen
		sqldb_write_long2		(sql_config_handle,"/control/actors/reload_counter",sqldb_read_long2(sql_config_handle,"/control/actors/reload_counter") + 1);

		// PCF8574A auf ID0 - LED-Driver fr ALL5000 Gehuse
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/enabled",														1);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor4/name",																"int. PCF8574A#0");
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor4/comment",														"bit 0: BUS Led driver");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/bus",																0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/group",															0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/port",																0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/speed",															int_speed);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/chipid",															164);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/chipaddress",												0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor4/chipname",														"PCF8574A");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/channels",														0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/all4000typ",													0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/channel0/function",									0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/channel0/value_int",									-204800000);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor4/channel0/value_float",								-2048000.0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor4/channel0/value_string",							"-2048000.00");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/channel0/mapto_sensor",							-1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/channel0/maptp_math_enabled",				0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/channel0/mapto_math_add1",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor4/channel0/mapto_math_multiply1",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/channel0/mapto_math_add2",						0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor4/channel0/mapto_math_multiply2",			1);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/outchannels",												0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/channel0/out_value_command",					0);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/channel0/out_value_int",							0);
		sqldb_write_float2	(sql_config_handle,"/control/sensors_i2c/sensor4/channel0/out_value_float",						0);
		sqldb_write_string2	(sql_config_handle,"/control/sensors_i2c/sensor4/channel0/out_value_string",					"0");
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/sensor4/channel0/mapfrom_actor",							-1);

		// Verwaltungs-Infos
		sqldb_write_long2		(sql_config_handle,"/control/sensors_i2c/max_sensors",																5);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_logical/max_sensors",														9);
		sqldb_write_long2		(sql_config_handle,"/control/sensors_display/page0/max_sensors",											9);
		sqldb_write_long2		(sql_config_handle,"/control/actors/max_actors",																			8);
		portscan_max_physical_sensors = 5;
		portscan_max_logical_sensors = 9;
		portscan_max_logical_actors = 8;
		}

	}



// *****   do_portscan()   -   Portscan ausfhren   *****

void do_portscan(FILE *f)
	{
	signed short int		port;
	signed short int		group;
	signed short int		bus;
	signed short int		speed;
	signed long int			x;
	signed long int			found_counter = 0;
	char								ep[390];
	signed short int		t;
	signed short int		i;
	signed short int		u;
	signed short int		y;
	char								n[80];


	portscan_max_physical_sensors = 0;
	portscan_max_logical_sensors = 0;
	portscan_max_logical_actors = 0;

	sqldb_write_string2(sql_config_handle,"/control/portscan/progress_text",""); // Fortschritt: Text
	sqldb_write_long2(sql_config_handle,"/control/portscan/progress_bus",-1); // Fortschritt: Bus
	sqldb_write_long2(sql_config_handle,"/control/portscan/progress_group",-1); // Fortschritt: Gruppe
	sqldb_write_long2(sql_config_handle,"/control/portscan/progress_port",-1); // Fortschritt: Port

	do_sqlyy(sql_config_handle,"DELETE FROM config WHERE tag LIKE '/control/sensors_scanned/%'",0); // Scan-Daten lschen
	sqldb_write_long2(sql_config_handle,"/control/sensors_scanned/found_sensors",0); // gefundene Anzahl auf 0 setzen
	for(t = 0;t < MAX_SENSORS;t++)
		{
		scanned_chipid[t] = 0;
		}

	// --- Scan Bus 0 ---
	fprintf(f,"*****   Scanning internal Bus (Bus0 )   *****\n");
	sqldb_write_long2(sql_config_handle,"/control/portscan/progress_bus",0); // Fortschritt: Bus
	x = sqldb_read_long2(sql_config_handle,"/control/portscan/scan_bus0");		// mu Bus0 gescannt werden ?
	if(x <= 0)
		{
		fprintf(f,"(No scan of Bus 0 according to user settings)\n");
		}
	else
		{
		bus = 0;
		group = 0;
		port = 0;
		speed = sqldb_read_long2(sql_config_handle,"/control/portscan/scan_bus0_speed");
		i2c_port_group_out(bus);														// Datenrichtungsregister fr den Bus setzen
		x = scan_port(bus,group,port,speed,f,found_counter);// Alle mglichen Chips abfragen
		found_counter += x;																	// Counter um die Anzahl gefundener Chips erhhen
		}

	// --- Scan Bus 1 ---
	fprintf(f,"*****   Scanning external Bus (Bus1 )   *****\n");
	bus = 1;
	speed = sqldb_read_long2(sql_config_handle,"/control/portscan/scan_bus1_speed");
	sqldb_write_long2(sql_config_handle,"/control/portscan/progress_bus",1); // Fortschritt: Bus
	x = sqldb_read_long2(sql_config_handle,"/control/portscan/scan_bus1");		// mu Bus1 gescannt werden ?
	if(x <= 0)
		{
		fprintf(f,"(No scan of Bus 1 according to user settings)\n");
		}
	else
		{
		i2c_port_group_out(bus);														// Datenrichtungsregister fr den Bus setzen
		for(group = 0;group <= 7;group++)
			{
			sqldb_write_long2(sql_config_handle,"/control/portscan/progress_group",group); // Fortschritt: Gruppe
			fprintf(f,"Group %d:\n",group);
			sprintf(ep,"/control/portscan/scan_bus1/group%d",group);
			x = sqldb_read_long2(sql_config_handle,ep);													// mu diese Gruppe gescannt werden ?
			if(x <= 0)
				{
				fprintf(f,"(No scan of Group %d according to user settings)\n",group);
				}
			else
				{
				for(port = 0;port <= 7;port++)
					{
					sqldb_write_long2(sql_config_handle,"/control/portscan/progress_port",port); // Fortschritt: Port
					x = scan_port(bus,group,port,speed,f,found_counter);
					found_counter += x;															// Counter um die Anzahl gefundener Chips erhhen
					}
				}
			}
		}

	// --- Scan Bus 2 ---
	fprintf(f,"*****   Scanning seconday internal Bus (Bus2 )   *****\n");
	sqldb_write_long2(sql_config_handle,"/control/portscan/progress_bus",2); // Fortschritt: Bus
	x = sqldb_read_long2(sql_config_handle,"/control/portscan/scan_bus2");		// mu Bus2 gescannt werden ?
	if(x <= 0)
		{
		fprintf(f,"(No scan of Bus 2 according to user settings)\n");
		}
	else
		{
		bus = 2;
		group = 0;
		port = 0;
		speed = sqldb_read_long2(sql_config_handle,"/control/portscan/scan_bus2_speed");
		i2c_port_group_out(bus);														// Datenrichtungsregister fr den Bus setzen
		x = scan_port(bus,group,port,speed,f,found_counter);// Alle mglichen Chips abfragen
		found_counter += x;																	// Counter um die Anzahl gefundener Chips erhhen
		}

	sqldb_write_long2(sql_config_handle,"/control/sensors_scanned/found_sensors",found_counter); // gefundene Anzahl in STRDB schreiben
	fprintf(f,"\n*****   Found %ld I2C Targets !   *****\n",found_counter);
	sqldb_write_long2(sql_config_handle,"/control/portscan/progress_bus",-1); // Fortschritt: Bus
	sqldb_write_long2(sql_config_handle,"/control/portscan/progress_group",-1); // Fortschritt: Gruppe
	sqldb_write_long2(sql_config_handle,"/control/portscan/progress_port",-1); // Fortschritt: Port

	// --- full Lobotomy ---
	x = sqldb_read_long2(sql_config_handle,"/control/portscan/lobotomy_before_start"); // mu lobotomiert werden ?
	if(x <= 0)
		{
		fprintf(f,"(No full erase according to user settings)\n");
		}
	else
		{
		sqldb_write_string2(sql_config_handle,"/control/portscan/progress_text","Erasing Data...<BR>(logical Sensors)"); // Fortschritt: Text
		fprintf(f,"***** Full Erasure of Sensor Data, Link information and display configuration started   *****\n");

		fprintf(f,"Deleting logical Actors...\n");
		x = sqldb_read_long2(sql_config_handle,"/control/actors/max_actors");
		if(x < 10)
			{
			x = 10;
			}
		for(t = 0;t < x;t++)
			{
			sprintf(ep,"/control/actors/actor%d/name",t);
			sprintf(n,"Actor %d",t);
			sqldb_write_string2(sql_config_handle,ep,n);
			sprintf(ep,"/control/actors/actor%d/comment",t);
			sqldb_write_string2(sql_config_handle,ep,"");
			sprintf(ep,"/control/actors/actor%d/type",t);
			sqldb_write_long2(sql_config_handle,ep,1);
			sprintf(ep,"/control/actors/actor%d/enabled",t);
			sqldb_write_long2(sql_config_handle,ep,0);
			sprintf(ep,"/control/actors/actor%d/bootdefault",t);
			sqldb_write_long2(sql_config_handle,ep,0);
			sprintf(ep,"/control/actors/actor%d/mapto_sensor",t);
			sqldb_write_long2(sql_config_handle,ep,-1);
			sprintf(ep,"/control/actors/actor%d/wattmapto_sensor",t);
			sqldb_write_long2(sql_config_handle,ep,-1);
			}
		sqldb_write_long2(sql_config_handle,"/control/actors/max_actors",0);

		sqldb_write_string2(sql_config_handle,"/control/portscan/progress_text","Erasing Data...<BR>(Actor Display Pages)"); // Fortschritt: Text
		fprintf(f,"Deleting Actor Display Pages...\n");
		x = sqldb_read_long2(sql_config_handle,"/control/actors_display/max_pages");
		for(t = 0; t < x;t++)
			{
			sprintf(ep,"/control/actors_display/page%d/max_actors",t);
			y = sqldb_read_long2(sql_config_handle,ep);
			if((t == 0) && (y < found_counter + 5))
				{
				y = found_counter + 5;
				}
			sqldb_write_long2(sql_config_handle,ep,0);
			sprintf(ep,"/control/actors_display/page%d/enabled",t);
			sqldb_write_long2(sql_config_handle,ep,0);
		  if(t == 0) sqldb_write_long2(sql_config_handle,ep,1);
			sprintf(ep,"/control/actors_display/page%d/appearance",t);
			sqldb_write_long2(sql_config_handle,ep,0);
			sprintf(ep,"/control/actors_display/page%d/link_title",t);
			sprintf(n,"show_actors.php?nr=%d",t);
			sqldb_write_string2(sql_config_handle,ep,n);
			sprintf(ep,"/control/actors_display/page%d/page_headline",t);
			sprintf(n,"Actors %d",t);
			sqldb_write_string2(sql_config_handle,ep,n);
			sprintf(ep,"/control/sensors_display/page%d/actorsx",t);
			sqldb_write_long2(sql_config_handle,ep,3);
			sprintf(ep,"/control/sensors_display/page%d/actorsy",t);
			sqldb_write_long2(sql_config_handle,ep,2);
			for(i = 0;i < y;i++)
				{
				sprintf(ep,"/control/actors_display/page%d/actor%d",t,i);
				sqldb_write_long2(sql_config_handle,ep,-1);
				}
			}

		fprintf(f,"Un-linking Powerline and Plugwise Slaves...\n");
		do_sqlyy(sql_config_handle,"UPDATE config SET value='-1' WHERE tag LIKE '/control/powerline/slave%/mapfrom_actor'",0);
		do_sqlyy(sql_config_handle,"UPDATE config SET value='-1' WHERE tag LIKE '/control/plugwise/slave%/mapfrom_actor'",0);

		fprintf(f,"Deleting logical Sensors...\n");
		do_sqlyy(sql_config_handle,"DELETE FROM config WHERE tag LIKE '/control/sensors_logical/%'",0);
		sqldb_write_long2(sql_config_handle,"/control/sensors_logical/max_sensors",0);

		sqldb_write_string2(sql_config_handle,"/control/portscan/progress_text","Erasing Data...<BR>(Sensor Display Pages)"); // Fortschritt: Text
		fprintf(f,"Deleting Sensor Display Pages...\n");
		x = sqldb_read_long2(sql_config_handle,"/control/sensors_display/max_pages");
		for(t = 0; t < x;t++)
			{
			sprintf(ep,"/control/sensors_display/page%d/max_sensors",t);
			y = sqldb_read_long2(sql_config_handle,ep);
			if((t == 0) && (y < found_counter + 5))
				{
				y = found_counter + 5;
				sqldb_write_long2(sql_config_handle,ep,y);
				}
			sprintf(ep,"/control/sensors_display/page%d/enabled",t);
			sqldb_write_long2(sql_config_handle,ep,0);
		  if(t == 0) sqldb_write_long2(sql_config_handle,ep,1);
			sprintf(ep,"/control/sensors_display/page%d/appearance",t);
			sqldb_write_long2(sql_config_handle,ep,0);
			sprintf(ep,"/control/sensors_display/page%d/link_title",t);
			sprintf(n,"show_sensors.php?nr=%d",t);
			sqldb_write_string2(sql_config_handle,ep,n);
			sprintf(ep,"/control/sensors_display/page%d/page_headline",t);
			sprintf(n,"Sensors %d",t);
			sqldb_write_string2(sql_config_handle,ep,n);
			sprintf(ep,"/control/sensors_display/page%d/sensorsx",t);
			sqldb_write_long2(sql_config_handle,ep,3);
			sprintf(ep,"/control/sensors_display/page%d/sensorsy",t);
			sqldb_write_long2(sql_config_handle,ep,2);
			sprintf(ep,"/control/sensors_display/page%d/width_pixels",t);
			sqldb_write_long2(sql_config_handle,ep,250);
			sprintf(ep,"/control/sensors_display/page%d/height_pixels",t);
			sqldb_write_long2(sql_config_handle,ep,250);
			sprintf(ep,"/control/sensors_display/page%d/update_interval_numerical",t);
			sqldb_write_long2(sql_config_handle,ep,2);
			sprintf(ep,"/control/sensors_display/page%d/update_interval_graphical",t);
			sqldb_write_long2(sql_config_handle,ep,30);
			for(i = 0;i < y;i++)
				{
				sprintf(ep,"/control/sensors_display/page%d/sensor%d",t,i);
				sqldb_write_long2(sql_config_handle,ep,-1);
				}
			}

		sqldb_write_string2(sql_config_handle,"/control/portscan/progress_text","Erasing Data...<BR>(physical Sensors)"); // Fortschritt: Text
		fprintf(f,"Deleting physical Sensors...\n");
		do_sqlyy(sql_config_handle,"DELETE FROM config WHERE tag LIKE '/control/sensors_i2c/%'",0);
		sqldb_write_long2(sql_config_handle,"/control/sensors_i2c/max_sensors",0);

		// --- Gerteinterne Sensoren gleich vorbesetzen ---
		sqldb_write_string2(sql_config_handle,"/control/portscan/progress_text","Erasing Data...<BR>(auto-add internal Sensors)"); // Fortschritt: Text
		// printf("*** autom. Add von internen Sensoren ***\n");
		fprintf(f,"*** autom. Add von internen Sensoren ***\n");
		portscan_add_internalsensors();
		}

	


	// Sensordaten in den Speicher einlesen
	sqldb_write_string2(sql_config_handle,"/control/portscan/progress_text","Reloading logical Sensor Configuration Data..."); // Fortschritt: Text
	fprintf(f,"Re-Reading logic data... ");
	x = logic_readconfig();
	fprintf(f,"%ld logical sensors found.\n",x);

	sqldb_write_string2(sql_config_handle,"/control/portscan/progress_text","Reloading physical Sensor Configuration Data..."); // Fortschritt: Text
	fprintf(f,"Re-Reading physical data... ");
	x = physical_readconfig();
	fprintf(f,"%ld physical sensors found.\n",x);

	fprintf(f,"Re-Reading actor data... ");
	x = actor_readconfig();																// aktualisierte logische Aktorenliste in den Speicher holen
	fprintf(f,"%ld logical actors found.\n",x);



	// --- Auto-Add Physikalische Sensoren ---
	x = sqldb_read_long2(sql_config_handle,"/control/portscan/autoadd_physical");
	if(x <= 0)
		{
		fprintf(f,"(No auto-add of physical Sensors)\n");
		}
	else
		{
		sqldb_write_string2(sql_config_handle,"/control/portscan/progress_text","Auto-Adding physical Sensors..."); // Fortschritt: Text
		fprintf(f,"***** Automatic Adding of %ld physical Sensors   *****\n",found_counter);
		for(t = 0;t < found_counter;t++)
			{
			fprintf(f,"Scan Result %d (%s at ID %d; Bus%d, Group%d, Port%d): ",t,scanned_chipname[t],scanned_chipaddress[t],scanned_bus[t],scanned_group[t],scanned_port[t]);
			for(i = 0;i < MAX_SENSORS;i++)
				{
				if((strcmp(physical_chipname[i],scanned_chipname[t]) == 0) && (physical_chipadresse[i] == scanned_chipaddress[t]) &&
				   (physical_bus[i] == scanned_bus[t]) && (physical_group[i] == scanned_group[t]) && (physical_port[i] == scanned_port[t]))
					{
					fprintf(f,"Already configured as physical Sensor %d (%s)\n",i,physical_name[i]);
					goto physical_noadd;
					}
				}
			i = physical_getfirstfree();											// neue leere Nummer ziehen
			fprintf(f,"physical:\"%s\", scanned:\"%s\"\n",physical_chipname[i],scanned_chipname[t]);
			fprintf(f,"Added as physical Sensor %d.\n",i);
			sprintf(physical_name[i],"%s_%d",scanned_chipname[t],scanned_chipaddress[t]);
			sprintf(physical_comment[i],"Bus%d, Group%d, Port%d",scanned_bus[t],scanned_group[t],scanned_port[t]);
			// sprintf(physical_comment[i],"Auto-Added by Port Scan of Bus%d, Group%d, Port%d",scanned_bus[t],scanned_group[t],scanned_port[t]);
			physical_enabled[i] = 1;
			physical_chipid[i] = scanned_chipid[t];
			strcpy(physical_chipname[i],scanned_chipname[t]);
			physical_chipadresse[i] = scanned_chipaddress[t];
			physical_bus[i] = scanned_bus[t];
			physical_group[i] = scanned_group[t];
			physical_port[i] = scanned_port[t];
			physical_speed[i] = scanned_speed[t];
			physical_priority[i] = 0;
			physical_all4000_typ[i] = scanned_all4000_typ[t];
			physical_channels[i] = scanned_channels[t]; // 1
			physical_outchannels[i] = scanned_outchannels[t];
			// if (scanned_all4000_typ[t] == 102) physical_channels[i] = 8; // ALL4042

			for(u = 0;u < physical_channels[i];u++)
				{
				physical_mapto_sensor[u][i] = -1;
				physical_mapto_function[u][i] = 0;
				if (scanned_all4000_typ[t] ==   1) physical_mapto_function[u][i] =   1; // ALL3005 Temperatur
				if (scanned_all4000_typ[t] ==   2) physical_mapto_function[u][i] =   1; // ALL3006 Temperatur
				if (scanned_all4000_typ[t] ==   3) physical_mapto_function[u][i] =   1; // ALL3018 Temperatur
				if (scanned_all4000_typ[t] ==  32) physical_mapto_function[u][i] =   3; // DC Voltage
				if (scanned_all4000_typ[t] ==  33) physical_mapto_function[u][i] = 129; // ALL5000int. ADC 4-Channels
				if (scanned_all4000_typ[t] ==  40) physical_mapto_function[u][i] =  16; // ALL3028 Luftdruck
				if (scanned_all4000_typ[t] ==  42) physical_mapto_function[u][i] =  16; // ALL4529 Luftdruck
				if (scanned_all4000_typ[t] ==  65) physical_mapto_function[u][i] =   2; // ALL3018 Feuchte
				if (scanned_all4000_typ[t] == 101) physical_mapto_function[u][i] =  12; // ALL4026 1-fach Digital Input
				if (scanned_all4000_typ[t] == 102) physical_mapto_function[u][i] =  12; // ALL4042 8-fach Digital Input
				if (scanned_all4000_typ[t] == 130) physical_mapto_function[u][i] =   6; // ALL4590 Strom
				if (scanned_all4000_typ[t] == 133) physical_mapto_function[u][i] =   8; // ALL3025 Zhler
				if (scanned_all4000_typ[t] == 134) physical_mapto_function[u][i] =   8; // ALL4035 Zhler
				if (scanned_all4000_typ[t] == 153) physical_mapto_function[u][i] =   3; // ALL4023 Spannung
				if (scanned_all4000_typ[t] == 155) physical_mapto_function[u][i] =   9; // ALL3032 Helligkeit
				if (scanned_all4000_typ[t] == 300) physical_mapto_function[u][i] =  15; // RFID001
				}

			for(u = 0;u < physical_outchannels[i];u++)
				{
				physical_mapfrom_actor[u][i] = -1;
				// if (scanned_all4000_typ[t] ==  97) physical_mapto_function[u][i] =  12; // ALL4026 1-fach Digital Input
				}

			physical_write_single(i);
			portscan_max_physical_sensors++;									// Zhler fr gefundene Sensoren erhhen
physical_noadd: ;
			}
		}

	// --- Auto-Add Logische Sensoren ---
	x = sqldb_read_long2(sql_config_handle,"/control/portscan/autoadd_logical");
	if(x <= 0)
		{
		fprintf(f,"(No auto-add of logical Sensors)\n");
		}
	else
		{
		sqldb_write_string2(sql_config_handle,"/control/portscan/progress_text","Auto-Adding logical Sensors..."); // Fortschritt: Text
		fprintf(f,"***** Automatic Adding and mapping of logical Sensors   *****\n");
		physical_readconfig();															// aktualisierte physikalische Sensorliste in den Speicher holen
		for(t = 0;t < MAX_SENSORS;t++)
			{
			if(physical_enabled[t] > 0)
				{
				if(physical_mapto_sensor[0][t] >= 0)
					{
					fprintf(f,"physical Sensor # %d Port 0 already mapped to logical Sensor # %d, skipping the rest.\n",t,physical_mapto_sensor[0][t]);
					}
				else
					{
					// printf("physical_enabled[%d]=%d, inchannels=%d, outchannels=%d\n",t,physical_enabled[t],physical_channels[t],physical_outchannels[t]);
					for(i = 0;i < physical_channels[t];i++)
						{
						u = logical_getfirstfree();									// neue leere Nummer ziehen
						strcpy(logical_name[u],"(unused)");
						logical_function[u] = 0;
						logical_chipid[u] = 0;
						logical_limit_min[u] = 10;
						logical_limit_max[u] = 85;
						logical_display_min[u] = -20;
						logical_display_max[u] = 125;
						logical_int[u] = -204800001;
						logical_float[u] = -2048000.01;
						strcpy(logical_string[u],"-2048000.01");
						logical_old_int[u] = -204800001;
						logical_old_float[u] = -2048000.01;
						strcpy(logical_old_string[u],"-2048000.01");
						fprintf(f,"Mapping physical Sensor # %d Port %d to logical Sensor # %d\n",t,i,u);
						sprintf(logical_name[u],"Sensor %d",u);
						sprintf(logical_comment[u],"Auto-added and auto-mapped from logical Sensor %d Port %d",t,i);
						logical_function[u] = physical_mapto_function[i][t];
						logical_all4000_typ[u] = physical_all4000_typ[t];

						// lustiges Namenraten anhand der ALL4000-ID
						if (physical_all4000_typ[t] ==   1) strcat(logical_name[u]," (ALL3005/3010)");
						if (physical_all4000_typ[t] ==   2) strcat(logical_name[u]," (ALL3006/3007/3015)");
						if (physical_all4000_typ[t] ==   3) strcat(logical_name[u]," (ALL3018T)");
						if (physical_all4000_typ[t] ==  32) strcat(logical_name[u]," (ALL3021/3022)");
						if (physical_all4000_typ[t] ==  33) strcat(logical_name[u]," (ALL5000int)");
						if (physical_all4000_typ[t] ==  40) strcat(logical_name[u]," (ALL3028)");
						if (physical_all4000_typ[t] ==  42) strcat(logical_name[u]," (ALL4529)");
						if (physical_all4000_typ[t] ==  65) strcat(logical_name[u]," (ALL3018H)");
						if (physical_all4000_typ[t] == 102) strcat(logical_name[u]," (ALL4042)");
						if (physical_all4000_typ[t] == 130) strcat(logical_name[u]," (ALL4590)");
						if (physical_all4000_typ[t] == 133) strcat(logical_name[u]," (ALL3025/4025)");
						if (physical_all4000_typ[t] == 134) strcat(logical_name[u]," (ALL4035)");
						if (physical_all4000_typ[t] == 153) strcat(logical_name[u]," (ALL4023)");
						if (physical_all4000_typ[t] == 155) strcat(logical_name[u]," (ALL3032)");
						if (physical_all4000_typ[t] == 300) strcat(logical_name[u]," (RFID001)");

						if (physical_channels[t] > 1)
							{
							sprintf(ep,"[ch.%d]",i);
							strcat(logical_name[u],ep);
							}

						// printf("lws(%d) function:%d\n",u,logical_function[u]);
						logical_write_single(u);										// logischen Sensor schreiben
						physical_mapto_sensor[i][t] = u;						// Physikalischen Sensor auf diesen Mappen
						physical_write_single(t);										// phsyikalischen Sensor schreiben
						fprintf(f,"Wrote Mapping to log.Sensor %d into physical Sensor %d, Port %d.\n",u,t,i);
						portscan_max_logical_sensors++;							// Zhler fr gefundene Sensoren erhhen
						}
					}
				}
			}
		}

	// --- Auto-Add Display (logische Sensoren) ---
	x = sqldb_read_long2(sql_config_handle,"/control/portscan/autoadd_display");
	if(x < 0)
		{
		fprintf(f,"(No auto-add of logical Sensors to Display Pages)\n");
		}
	else
		{
		sqldb_write_string2(sql_config_handle,"/control/portscan/progress_text","Auto-Adding logical Sensors to Display Page..."); // Fortschritt: Text
		fprintf(f,"***** Automatic Adding of logical Sensors to Display Page %ld   *****\n",x);
		for(t = 0;t < MAX_SENSORS;t++)
			{
			display_mapflag[t] = 0;
			display_sensor[t] = -1;
			}
		y = sqldb_read_long2(sql_config_handle,"/control/sensors_display/max_pages");
		for(i = 0;i < y;i++)
			{
			sprintf(ep,"/control/sensors_display/page%d/max_sensors",i);
			display_max_sensors = sqldb_read_long2(sql_config_handle,ep);
			for(t = 0;t < display_max_sensors;t++)
				{
				sprintf(ep,"/control/sensors_display/page%d/sensor%d",i,t);
				u = sqldb_read_long2(sql_config_handle,ep);
				if (u >= 0) display_mapflag[u] = 1;
				}		
			}		

		sprintf(ep,"/control/sensors_display/page%ld/max_sensors",x);
		display_max_sensors = sqldb_read_long2(sql_config_handle,ep);
		y = -1;
		for(t = 0;t < display_max_sensors;t++)
			{
			sprintf(ep,"/control/sensors_display/page%ld/sensor%d",x,t);
			u = sqldb_read_long2(sql_config_handle,ep);
			if (u >= 0) y = u;
			}

		for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
			{
			if(logical_function[t] > 0)
				{
				if(display_mapflag[t] == 0)
					{
					display_mapflag[t] = 1;
					y++;
					sprintf(ep,"/control/sensors_display/page%ld/sensor%d",x,y);
					sqldb_write_long2(sql_config_handle,ep,t);
					fprintf(f,"logical Sensor # %d mapped to display Sensor %d at page %ld.\n",t,y,x);
					}
				}
			}
		if(y + 1 > display_max_sensors)
			{
			sprintf(ep,"/control/sensors_display/page%ld/max_sensors",x);
			sqldb_write_long2(sql_config_handle,ep,y + 1);
			}
		sqldb_write_long2(sql_config_handle,"/control/sensors_display/page0/sensorsy",(y / 3) + 1);
		}


	// --- Auto-Add Logische Aktoren ---
	// x = sqldb_read_long2(sql_config_handle,"/control/portscan/autoadd_actors");
	x = 1;
	if(x <= 0)
		{
		fprintf(f,"(No auto-add of logical Actors)\n");
		}
	else
		{
		sqldb_write_string2(sql_config_handle,"/control/portscan/progress_text","Auto-Adding logical Actors..."); // Fortschritt: Text
		fprintf(f,"***** Automatic Adding and mapping of logical Actors   *****\n");
		fprintf(f,"Re-Reading actor data... ");
		x = actor_readconfig();																// aktualisierte logische Aktorenliste in den Speicher holen
		fprintf(f,"%ld logical actors found.\n",x);
		for(t = 0;t < MAX_SENSORS;t++)
			{
			if(physical_enabled[t] > 0)
				{
				if(physical_mapfrom_actor[0][t] >= 0)
					{
					fprintf(f,"physical Sensor # %d Port 0 already mapped from logical Actor # %d, skipping the rest.\n",t,physical_mapfrom_actor[0][t]);
					}
				else
					{
					for(i = 0;i < physical_outchannels[t];i++)
						{
						u = actor_getfirstfree();									// neue leere Nummer ziehen
						actor_enabled[u] = 1;
						actor_type[u] = 1;
						actormapping_mapto_sensor[u] = -1;
						actormapping_wattmapto_sensor[u] = -1;
						actormapping_bootdefault[u] = -1;
						fprintf(f,"Mapping physical Sensor # %d Outport %d to logical Actor # %d\n",t,i,u);
						sprintf(actor_name[u],"Actor %d",u);

						// lustiges Namenraten anhand der ALL4000-ID
						if (physical_all4000_typ[t] ==  97) strcat(actor_name[u]," (ALL3002)");
						if (physical_all4000_typ[t] == 102) strcat(actor_name[u]," (ALL4027)");

						if (physical_outchannels[t] > 1)
							{
							sprintf(ep,"[ch.%d]",i);
							strcat(actor_name[u],ep);
							}

						// printf("lwa(%d) function:%d\n",u,actor_type[u]);
						actor_write_single(u);											// logischen Aktor schreiben
						physical_mapfrom_actor[i][t] = u; 
						physical_write_single(t);										// phsyikalischen Sensor schreiben
						fprintf(f,"Wrote Mapping to log.Actor %d into physical Sensor %d, Port %d.\n",u,t,i);
						portscan_max_logical_actors++;							// Zhler fr gefundene Aktoren erhhen
						}
					}
				}
			}
		}

	// --- Auto-Add Display (logische Aktoren) ---
	x = sqldb_read_long2(sql_config_handle,"/control/portscan/autoadd_display");
	if(x < 0)
		{
		fprintf(f,"(No auto-add of logical Actors to Display Pages)\n");
		}
	else
		{
		sqldb_write_string2(sql_config_handle,"/control/portscan/progress_text","Auto-Adding logical Actors to Display Page..."); // Fortschritt: Text
		fprintf(f,"***** Automatic Adding of logical Actors to Display Page %ld   *****\n",x);

		for(t = 0;t < portscan_max_logical_actors;t++)
			{
			sprintf(ep,"/control/actors_display/page0/actor%d",t);
			sqldb_write_long2(sql_config_handle,ep,t);
			}
		}

	// Maximalanzahlen schreiben
	sqldb_write_long2(sql_config_handle,"/control/sensors_i2c/max_sensors",portscan_max_physical_sensors);
	sqldb_write_long2(sql_config_handle,"/control/sensors_logical/max_sensors",portscan_max_logical_sensors);
	sqldb_write_long2(sql_config_handle,"/control/actors/max_actors",portscan_max_logical_actors);
	sqldb_write_long2(sql_config_handle,"/control/sensors_display/page0/max_sensors",portscan_max_logical_sensors);
	sqldb_write_long2(sql_config_handle,"/control/actors_display/page0/max_actors",portscan_max_logical_actors);

	physical_max_sensors = portscan_max_physical_sensors;
	logical_max_sensors = portscan_max_logical_sensors;
	// logical_max_actors = portscan_max_logical_actors;

	// Sensordaten in den Speicher einlesen
	sqldb_write_string2(sql_config_handle,"/control/portscan/progress_text","Reloading physical Sensor Configuration Data..."); // Fortschritt: Text
	fprintf(f,"Re-Reading physical data... ");
	x = physical_readconfig();
	fprintf(f,"%ld physical sensors found.\n",x);

	sqldb_write_string2(sql_config_handle,"/control/portscan/progress_text","Reloading logical Sensor Configuration Data..."); // Fortschritt: Text
	fprintf(f,"Re-Reading logic data... ");
	x = logic_readconfig();
	fprintf(f,"%ld logical sensors found.\n",x);
	sqldb_write_long2(sql_config_handle,"/control/sensors_i2c/flag_logicalsettings",42);

	// Aktordaten in den Speicher einlesen
	fprintf(f,"Re-Reading actor data... ");
	x = actor_readconfig();																// aktualisierte logische Aktorenliste in den Speicher holen
	fprintf(f,"%ld logical actors found.\n",x);
	}



// *****   matrix_setzedefault()   -   Schaltmatrix-Einstellungen auf Standardwerte setzen   *****

void matrix_setzedefault(void)
	{
	unsigned short int t;

	for(t = 0;t < MAX_MATRIX_RULES;t++)
		{
		matrix_rule_enabled[t] = 0;													// Regel aktiv
		matrix_rule_parameterset[t] = 0;										// Prfe logische Sensorwerte (1: nderungen)
		matrix_rule_left_number[t] = -1;										// Index auf Sensornummer (-1: keiner)
		matrix_rule_operator_type[t] = -1;									// Vergleichsoperator
		matrix_rule_right_number[t] = -1;										// Index auf Sensornummer (-1: keiner, -2: Konstante)
		matrix_rule_right_parameter[t] = 0.00;							// Konstante fr Vergleich
		matrix_rule_action_on[t] = -1;											// Aktor-Index fr Einschalten (-1: Keiner)
		matrix_rule_action_off[t] = -1;											// Aktor-Index fr Ausschalten (-1: Keiner)
		matrix_rule_action_toggle[t] = -1;									// Aktor-Index fr Toggeln (-1: Keiner)
		matrix_rule_action_script[t][0] = '0';							// Auszufhrendes Shellscript
		}

	for(t = 0;t < MAX_LOGICAL_ACTORS;t++)
		{
		matrix_actor_value[t] = -1;
		matrix_actor_flag[t] = -1;
		}
	}



// **********   Schaltmatrix-Konfiguration einlesen   **********

void matrix_read_config(void)
	{
	char ep[1024];
	signed long int t;
	signed long int i;
	signed long int lcount;

	i = sqldb_read_long2(sql_config_handle,"/control/matrix/max_rules");
	if(i > MAX_MATRIX_RULES)
		{
		sprintf(ep,"Configured value /control/matrix/max_rules is %ld; larger than MAX_MATRIX_RULES, using MAX_MATRIX_RULES=%d instead.",i,MAX_MATRIX_RULES);
		log_append(2,ep);																			// Fehlermeldung ins Logfile schreiben
		logical_max_sensors = MAX_MATRIX_RULES;
		}

	matrix_setzedefault();
	lcount = 0;
	matrix_merker = sqldb_read_long2(sql_config_handle,"/control/matrix/reload_counter");

	for(t = 0;t < i;t++)
		{
		sprintf(ep,"/control/matrix/rule%ld/enabled",t);
		matrix_rule_enabled[t] = sqldb_read_long2(sql_config_handle,ep);
		if(matrix_rule_enabled[t] > 0)
			{
			lcount++;

			sprintf(ep,"/control/matrix/rule%ld/parameterset",t);
			matrix_rule_parameterset[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/matrix/rule%ld/left_number",t);
			matrix_rule_left_number[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/matrix/rule%ld/operator_type",t);
			matrix_rule_operator_type[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/matrix/rule%ld/right_number",t);
			matrix_rule_right_number[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/matrix/rule%ld/right_parameter",t);
			matrix_rule_right_parameter[t] = sqldb_read_float2(sql_config_handle,ep);
			sprintf(ep,"/control/matrix/rule%ld/action_number_on",t);
			matrix_rule_action_on[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/matrix/rule%ld/action_number_off",t);
			matrix_rule_action_off[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/matrix/rule%ld/action_number_toggle",t);
			matrix_rule_action_toggle[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/matrix/rule%ld/action_script",t);
			sqldb_read_string2(sql_config_handle,ep,matrix_rule_action_script[t]);

			sprintf(ep,"Read Matrix Rule#%ld: IF sensor[%d] <=> sensor[%d] THEN ON[%d], OFF[%d], Script:\"%s\"",
							t,matrix_rule_left_number[t],matrix_rule_right_number[t],matrix_rule_action_on[t],matrix_rule_action_off[t],matrix_rule_action_script[t]);
			log_append(4,ep);																	// Meldung ins Logfile schreiben
			}
		else
			{
			}
		}

	sprintf(ep,"Configuration data for %ld Switch Matrix Rules read.",lcount);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	}



// *****   matrix_execute   -   Schaltmatrix-Rules prfen und abarbeiten   *****

void matrix_execute(void)
	{
	char ep[1024];
	signed long int t;
	double wert_links = -2048000.00;
	double wert_rechts = -2048000.00;
	unsigned short int wahr;
	char op[9];

	// printf("MX\n");
	for(t = 0;t < MAX_MATRIX_RULES;t++)
		{
		if(matrix_rule_enabled[t] > 0)
			{
			// lvalue holen
			if(matrix_rule_left_number[t] >= 0)
				{
				if (matrix_rule_parameterset[t] == 0) wert_links = logical_float[matrix_rule_left_number[t]];
				if (matrix_rule_parameterset[t] == 1) wert_links = logical_change[matrix_rule_left_number[t]];
				}
			else
				{
				wert_links = -2048000.00;
				}			

			// rvalue holen
			if(matrix_rule_right_number[t] >= 0)
				{
				if (matrix_rule_parameterset[t] == 0) wert_rechts = logical_float[matrix_rule_right_number[t]];
				if (matrix_rule_parameterset[t] == 1) wert_rechts = logical_change[matrix_rule_right_number[t]];
				}
			else
				{
				wert_rechts = -2048000.00;
				if(matrix_rule_right_number[t] == -2) wert_rechts = matrix_rule_right_parameter[t];
				}

			// Vergleichsoperation durchfhren
			wahr = 0;
			if(matrix_rule_operator_type[t] == 0)	{ strcpy(op, "<"); if(wert_links <  wert_rechts) wahr = 1; }
			if(matrix_rule_operator_type[t] == 1)	{ strcpy(op,"<="); if(wert_links <= wert_rechts) wahr = 1; }
			if(matrix_rule_operator_type[t] == 2)	{ strcpy(op, "="); if(wert_links == wert_rechts) wahr = 1; }
			if(matrix_rule_operator_type[t] == 3)	{ strcpy(op,">="); if(wert_links >= wert_rechts) wahr = 1; }
			if(matrix_rule_operator_type[t] == 4)	{ strcpy(op, ">"); if(wert_links >  wert_rechts) wahr = 1; }
			if(matrix_rule_operator_type[t] == 5)	{ strcpy(op,"<>"); if(wert_links != wert_rechts) wahr = 1; }
			// printf("Rule %ld: lvalue:%4.2f %s rvalue:%4.2f, Wahr:%d\n",t,wert_links,op,wert_rechts,wahr);
			sprintf(ep,"Matrix Rule %ld: lvalue:%4.2f %s rvalue:%4.2f, Wahr:%d",t,wert_links,op,wert_rechts,wahr);
			log_append(4,ep);																	// Meldung ins Logfile schreiben

			// Aktion ausfhren
			if(wahr > 0)
				{
				if(matrix_rule_action_on[t] >= 0)
					{
					matrix_actor_value[matrix_rule_action_on[t]] = 1;
					matrix_actor_flag[matrix_rule_action_on[t]] = 1;
					sprintf(ep,"mActor %d ON",matrix_rule_action_on[t]);
					log_append(4,ep);															// Meldung ins Logfile schreiben
					}
				if(matrix_rule_action_off[t] >= 0)
					{
					matrix_actor_value[matrix_rule_action_off[t]] = 0;
					matrix_actor_flag[matrix_rule_action_off[t]] = 1;
					sprintf(ep,"mActor %d OFF",matrix_rule_action_off[t]);
					log_append(4,ep);															// Meldung ins Logfile schreiben
					}
				if(matrix_rule_action_toggle[t] >= 0)
					{
					if (matrix_actor_value[matrix_rule_action_toggle[t]] != 0)
						{
						matrix_actor_value[matrix_rule_action_toggle[t]] = 0;
						matrix_actor_flag[matrix_rule_action_toggle[t]] = 1;
						sprintf(ep,"mActor %d Toggle to OFF",matrix_rule_action_toggle[t]);
						log_append(4,ep);														// Meldung ins Logfile schreiben
						}
					else
						{
						matrix_actor_value[matrix_rule_action_toggle[t]] = 1;
						matrix_actor_flag[matrix_rule_action_toggle[t]] = 1;
						sprintf(ep,"mActor %d Toggle to ON",matrix_rule_action_toggle[t]);
						log_append(4,ep);														// Meldung ins Logfile schreiben
						}
					}
				if(matrix_rule_action_script[t][0] != 0)
					{
					system(matrix_rule_action_script[t]);
					sprintf(ep,"mExec \"%s\"",matrix_rule_action_script[t]);
					log_append(4,ep);															// Meldung ins Logfile schreiben
					}
				}

			}
		}
	}



// **********   alle Mobotix-Upload-Einstellungen lschen   **********

void mobotix_upload_setzedefault(void)
	{
	unsigned short int t;
	unsigned short int i;


	for(t = 0;t < MAX_MOBOTIX_UPLOADS;t++)
		{
		mobotix_upload_enabled[t] = 0;
		mobotix_upload_ip[t][0] = 0;
		mobotix_upload_username[t][0] = 0;
		mobotix_upload_password[t][0] = 0;
		sprintf(mobotix_upload_name[t],"Camera %d",t);
		mobotix_upload_text_top[t][0] = 0;
		mobotix_upload_text_bottom[t][0] = 0;
		mobotix_upload_insert_timestamp[t] = 0;
		mobotix_upload_time_interval[t] = 30;
		mobotix_upload_timestamp[t] = 0;
		for(i = 0;i < 32;i++)
			{
			mobotix_upload_sensor[t][i] = -1;
			}
		mobotix_upload_sensors[t] = 0;
		}
	}



// **********   Mobotix-Upload-Konfiguration einlesen   **********

void mobotix_upload_readconfig(void)
	{
	char ep[1024];
	signed long int t;
	signed long int i;
	signed long int lcount;


	i = sqldb_read_long2(sql_config_handle,"/control/mobotix_upload/max_pages");
	if(i > MAX_MOBOTIX_UPLOADS)
		{
		sprintf(ep,"Configured value /control/mobotix_upload/max_pages is %ld; larger than MAX_MOBOTIX_UPLOADS, using MAX_MOBOTIX_UPLOADS=%d instead.",i,MAX_MOBOTIX_UPLOADS);
		log_append(2,ep);																			// Fehlermeldung ins Logfile schreiben
		mobotix_upload_anzahl = MAX_MOBOTIX_UPLOADS;
		}
	else
		{
		mobotix_upload_anzahl = i;
		}

	mobotix_upload_setzedefault();
	lcount = 0;
	mobotix_upload_reload_counter = sqldb_read_long2(sql_config_handle,"/control/mobotix_upload/reloadcounter");

	for(t = 0;t < mobotix_upload_anzahl;t++)
		{
		sprintf(ep,"/control/mobotix_upload/page%ld/enabled",t);
		mobotix_upload_enabled[t] = sqldb_read_long2(sql_config_handle,ep);
		if(mobotix_upload_enabled[t] > 0)
			{
			lcount++;

			sprintf(ep,"/control/mobotix_upload/page%ld/name",t);
			sqldb_read_string2(sql_config_handle,ep,mobotix_upload_name[t]);
			sprintf(ep,"/control/mobotix_upload/page%ld/ip",t);
			sqldb_read_string2(sql_config_handle,ep,mobotix_upload_ip[t]);
			sprintf(ep,"/control/mobotix_upload/page%ld/username",t);
			sqldb_read_string2(sql_config_handle,ep,mobotix_upload_username[t]);
			sprintf(ep,"/control/mobotix_upload/page%ld/password",t);
			sqldb_read_string2(sql_config_handle,ep,mobotix_upload_password[t]);
			sprintf(ep,"/control/mobotix_upload/page%ld/textline_top",t);
			sqldb_read_string2(sql_config_handle,ep,mobotix_upload_text_top[t]);
			sprintf(ep,"/control/mobotix_upload/page%ld/textline_bottom",t);
			sqldb_read_string2(sql_config_handle,ep,mobotix_upload_text_bottom[t]);
			sprintf(ep,"/control/mobotix_upload/page%ld/time_interval",t);
			mobotix_upload_time_interval[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/mobotix_upload/page%ld/insert_timestamp",t);
			mobotix_upload_insert_timestamp[t] = sqldb_read_long2(sql_config_handle,ep);
			sprintf(ep,"/control/mobotix_upload/page%ld/max_sensors",t);
			mobotix_upload_sensors[t] = sqldb_read_long2(sql_config_handle,ep);

			for(i = 0;i < mobotix_upload_sensors[t];i++)
				{
				sprintf(ep,"/control/mobotix_upload/page%ld/sensor%ld",t,i);
				mobotix_upload_sensor[t][i] = sqldb_read_long2(sql_config_handle,ep);
				}
			}
		else
			{
			}
		}

	sprintf(ep,"Configuration data for %ld Mobotix Uploads read.",lcount);
	log_append(3,ep);																			// Meldung ins Logfile schreiben
	}



// **********   Mobotix-Upload durchfhren   **********

void mobotix_upload(void)
	{
	char ep[1024];
	char kommando[4096];
	signed long int t;
	signed long int i;
	signed long int now;
	signed long int zeitdifferenz;
	signed short int tag;
	signed short int monat;
	signed short int jahr;
	signed short int stunde;
	signed short int minute;
	signed short int sekunde;
	time_t timer;
	struct tm *tblock;
	char login[256];


	if(mobotix_upload_anzahl > 0)
		{
		timer = time(NULL);
		tblock = localtime(&timer);
		stunde = tblock->tm_hour;
		minute = tblock->tm_min;
		sekunde = tblock->tm_sec;
		tag = tblock->tm_mday;
		monat = tblock->tm_mon + 1;
		jahr = tblock->tm_year + 1900;
		for(t = 0;t < mobotix_upload_anzahl;t++)
			{
			if (mobotix_upload_enabled[t] > 0)
				{
				now = time(0);																	// aktuellen Timestamp ermitteln
				zeitdifferenz = now - mobotix_upload_timestamp[t];
				if (zeitdifferenz < 0) zeitdifferenz = -zeitdifferenz;
				if (zeitdifferenz >= mobotix_upload_time_interval[t])
					{
					// printf("Mobotix_Upload[%ld]...\n",t);
					mobotix_upload_timestamp[t] = now;
					login[0] = 0;																	// Login-Credentials lschen
					if((strlen(mobotix_upload_username[t]) > 0) && (strlen(mobotix_upload_password[t]) > 0))
						{
						sprintf(login,"-u %s:%s ",mobotix_upload_username[t],mobotix_upload_password[t]);
						}
					sprintf(kommando,"curl %s-s -o /dev/null \"http://%s/control/control?set&section=text&print=",login,mobotix_upload_ip[t]);
					if (strlen(mobotix_upload_text_top[t]) > 0)
						{
						sprintf(ep,"%s%%0A",mobotix_upload_text_top[t]);
						strcat(kommando,ep);
						}
					for(i = 0;i < mobotix_upload_sensors[t];i++)
						{
						if (mobotix_upload_sensor[t][i] >= 0)
							{
							sprintf(ep,"%s: %s%%0A",logical_name[mobotix_upload_sensor[t][i]],logical_string[mobotix_upload_sensor[t][i]]);
							strcat(kommando,ep);
							}
						}
					if (strlen(mobotix_upload_text_bottom[t]) > 0)
						{
						sprintf(ep,"%s%%0A",mobotix_upload_text_bottom[t]);
						strcat(kommando,ep);
						}
					if (mobotix_upload_insert_timestamp[t] > 0)
						{
						sprintf(ep,"%02d.%02d.%04d, %02d:%02d:%02d%%0A",tag,monat,jahr,stunde,minute,sekunde);
						strcat(kommando,ep);
						}
					for (i = 25 + strlen(login);i < strlen(kommando);i++)					// nicht am Anfang beginnen !
						{
						if (kommando[i] == 32) kommando[i] = '+';		// Leerzeichen durch Pluszeichen ersetzen
						}
					// strcat(kommando,"\"-O /dev/null -q");
					strcat(kommando,"\" &");
					// printf("cmd:\"%s\"\n",kommando);
					system(kommando);
					}
				}
			}
		}
	}



// **********   Demon   **********

int main(int argc,char **argv)
	{
	char ep[1024];
	signed long int kommando;
	FILE *f;
	struct timeval tv;																		// Uhrzeit-Kram
	int shID;																							// Shared-Memory ID
	unsigned long int startsekunde = 0;										// Sekunde fr Sekundenlock
	unsigned long int zeit1 = 0;													// Millisekunden-Merker
	unsigned long int zeit2 = 0;													// Millisekunden-Merker
	signed long int t;
	unsigned char z;


	system("mkdir /tmp/svg 2>/dev/null");
	system("mkdir /tmp/xml 2>/dev/null");
	sqlite3_open(CONFIG_DATABASE_FILE,&sql_config_handle);// SQLITE3-Datenbank ffnen
	sqlite3_open(VAR_DATABASE_FILE,&sql_var_handle);			// SQLITE3-Datenbank ffnen

	hwmodel = sqldb_read_long2(sql_config_handle,"/sys/hardware/numeric_model");		// auf welcher Hardware sind wir ? 4500 oder 5000 ?

	// auf existierendes Shared Memory fr Sensordaten zugreifen
	shID = shmget(SHM_ID_SENSORDATA,SHM_SENSORDATA_SIZE,0666);
	if (shID >= 0)
		{
		shm_sensordata = shmat(shID, 0, 0);
		if (shm_sensordata == (struct s_shm_sensordata *)-1)
			{
			printf("Problem beim Zugriff auf SENSORDATA shared memory !\n");
			goto ende;
			}
		else
			{
			// printf("SHM_Sensordata OK.\n");
			}
		}
	else
		{
		// shmget lief schief
		printf("shmget fr SHM_Sensordata war erfolglos ! (shm_create nicht aufgerufen ?)\n");
		goto ende;
    }

	// auf existierendes Shared Memory fr Sensorcount zugreifen
	shID = shmget(SHM_ID_SENSORCOUNT,SHM_SENSORCOUNT_SIZE,0666);
	if (shID >= 0)
		{
		shm_sensorcount = shmat(shID, 0, 0);
		if (shm_sensorcount == (unsigned long int *)-1)
			{
			printf("Problem beim Zugriff auf SENSORCOUNT shared memory !\n");
			goto ende;
			}
		else
			{
			// printf("SHM_Sensorcount OK.\n");
			}
		}
	else
		{
		// shmget lief schief
		printf("shmget fr SHM_Sensorcount war erfolglos ! (shm_create nicht aufgerufen ?)\n");
		goto ende;
    }
	*shm_sensorcount = 0;



	// auf existierendes Shared Memory fr Aktordaten Read zugreifen
	shID = shmget(SHM_ID_ACTOR_R_DATA,SHM_ACTOR_R_DATA_SIZE,0666);
	if (shID >= 0)
		{
		shm_actor_r_data = shmat(shID,0,0);
		if (shm_actor_r_data == (struct s_shm_actordata *)-1)
			{
			printf("Problem beim Zugriff auf ACTOR_R_DATA shared memory !\n");
			goto ende;
			}
		else
			{
			// printf("SHM_Actor_r_data OK.\n");
			}
		}
	else
		{
		// shmget lief schief
		printf("shmget fr SHM_Actor_r_data war erfolglos ! (shm_create nicht aufgerufen ?)\n");
		goto ende;
    }

	// auf existierendes Shared Memory fr Aktorcount Read zugreifen
	shID = shmget(SHM_ID_ACTOR_R_COUNT,SHM_ACTOR_R_COUNT_SIZE,0666);
	if (shID >= 0)
		{
		shm_actor_r_count = shmat(shID, 0, 0);
		if (shm_actor_r_count == (unsigned long int *)-1)
			{
			printf("Problem beim Zugriff auf ACTOR_R_COUNT shared memory !\n");
			goto ende;
			}
		else
			{
			// printf("SHM_Actor_r_count OK.\n");
			}
		}
	else
		{
		// shmget lief schief
		printf("shmget fr SHM_Actor_r_count war erfolglos ! (shm_create nicht aufgerufen ?)\n");
		goto ende;
    }
	*shm_actor_r_count = 0;



	// auf existierendes Shared Memory fr Aktordaten Write zugreifen
	shID = shmget(SHM_ID_ACTOR_W_DATA,SHM_ACTOR_W_DATA_SIZE,0666);
	if (shID >= 0)
		{
		shm_actor_w_data = shmat(shID,0,0);
		if (shm_actor_w_data == (struct s_shm_actordata *)-1)
			{
			printf("Problem beim Zugriff auf ACTOR_W_DATA shared memory !\n");
			goto ende;
			}
		else
			{
			// printf("SHM_Actor_w_data OK.\n");
			}
		}
	else
		{
		// shmget lief schief
		printf("shmget fr SHM_Actor_w_data war erfolglos ! (shm_create nicht aufgerufen ?)\n");
		goto ende;
    }

	// auf existierendes Shared Memory fr Aktorcount Write zugreifen
	shID = shmget(SHM_ID_ACTOR_W_COUNT,SHM_ACTOR_W_COUNT_SIZE,0666);
	if (shID >= 0)
		{
		shm_actor_w_count = shmat(shID, 0, 0);
		if (shm_actor_w_count == (unsigned long int *)-1)
			{
			printf("Problem beim Zugriff auf ACTOR_W_COUNT shared memory !\n");
			goto ende;
			}
		else
			{
			// printf("SHM_Actor_w_count OK.\n");
			}
		}
	else
		{
		// shmget lief schief
		printf("shmget fr SHM_Actor_w_count war erfolglos ! (shm_create nicht aufgerufen ?)\n");
		goto ende;
    }
	*shm_actor_w_count = 0;




	logical_sum_count = 0;																// Summenzhler lschen
	for(t = 0;t < MAX_LOGICAL_SENSORS;t++)
		{
		logical_sum_float[t] = 0;
		}

	gpio_devmem_open();																		// GPIOs oeffnen
	gpio_port_init_output(PORT_B,22);										  // Power 12V/3.3V (H: On)
	gpio_port_write(PORT_B,22,0);													// Strom fr Sensoren ausschalten
	// printf("Sensor Power off...\n");
	sqldb_write_long2(sql_config_handle,"/control/sensors_i2c/flag_command",0); // ggfs vorhandene alte Kommandos lschen
	strcpy(log_filename,sensordemon_logfile);							// Pfade fuer Logfile setzen
	// log_kill();																						// Logfile loeschen
	demon_readconfig();																		// Einstellungen fuer den Sensordemon lesen
	remote_readconfig();																	// Einstellungen fuer Remotesensoren lesen

	log_level = sensordemon_loglevel;											// Debuglevel setzen
	sprintf(ep,"----------   sensor_demon V %s started (debuglevel=%d)   ----------",SENSORDEMON_VERSION,log_level);
	log_append(1,ep);																			// Startmeldung ins Logfile schreiben
	printf("sensor_demon V %s started (debuglevel=%d)\n",SENSORDEMON_VERSION,log_level);

	sensor_setzedefault();																// Alle physikalischen Sensoren auf Standardwerte zuruecksetzen
	logic_setzedefault();																	// Alle logischen Sensoren auf Standardwerte zuruecksetzen
	mobotix_upload_setzedefault();												// Alle Mobotix-Uploads lschen
	logic_analyze_changes();															// Vernderungen prfen
	logic_analyze_changes();															// Vernderungen prfen 2. mal, so dass diese alle 0 sind
	logic_copy_to_shm();																	// logische Sensorwerte in Shared-Memory kopieren
	timer_setzedefault();																	// Alle Timer lschen
	physical_readconfig();																// physikalische Sensor-Konfiguration lesen
	logic_readconfig();																		// logische Sensor-Konfiguration lesen
	actor_readconfig();																		// Aktoren-Konfiguration lesen
	actor_copy_bootdefault_to_shm();											// Bootdefaults in SHM schreiben
	matrix_read_config();																	// Schaltmatrix einlesen
	timer_readconfig();																		// Timer-Konfiguration lesen
	alert_readconfig();																		// Alert-Konfiguration lesen
	db_readconfig();																			// Datenbank-Einstellungen lesen
	statusmail_readconfig();															// Alert-Konfiguration lesen
	mobotix_upload_readconfig();													// Mobotix-Upload-Konfiguration lesen

	history_setzedefault();																// Sensor-History lschen
	history_read_data();																	// Sensor-History einlesen
	history_write_xml();
	history_write_svg(1);
	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln
	last_history = tv.tv_sec;															// Timestamp auf JETZT setzen
	next_history = last_history + config_demon_historyinterval; // Timestamp fr nchsten Historypunkt ermitteln

	sql_erase_sensorvalues(sql_var_handle);								// Sensordaten-Tabelle lschen und neu aufbauen

	physical_measurementtype_rotator = 0;									// Me-Typ (0: Temp, 1: Feuchte)
	physical_measurementtype_counter = 0;									// Zhler
	physical_measurementtype_cycles = 4;									// Anzahl Mezyklen fr einen Status

	sleep(3);
	// printf("Sensor Power on...\n");
	gpio_port_write(PORT_B,22,1);													// Strom fr Sensoren einschalten
	sleep(3);
	i2c_ruhe(0);																					// Ruhe am Bus (Initialisierung auf Standardwerte)
	i2c_port_group_out(1);																// Datenrichtungsregister fr den Bus setzen

	statusmail_boot();																		// Wenn welche aktiviert wurden, hier die Bootmails versenden
	db_timestamp = 0;																			// letzter DB-write
	db_next = time(0);																		// nchster DB-Write

hauptschleife:
	if(config_sekundenlock > 0)
		{
		gettimeofday(&tv,NULL);															// Hires-Zeit ermitteln
		startsekunde = tv.tv_sec;														// momentane Sekunde merken
		}

	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln
	zeit1 = tv.tv_sec * 1000 + tv.tv_usec / 1000;					// Timestamp in ms ermitteln

	ti++;																									// Sys-Clock hochzhlen

	// Logische Sensordaten reloaden, falls ntig
	if (sqldb_read_long2(sql_config_handle,"/control/sensors_i2c/flag_logicalsettings") > 0)
		{
		log_append(1,"Re-loading logical sensor data.");
		logic_readconfig();
		sql_erase_sensorvalues(sql_var_handle);							// Sensordaten-Tabelle lschen und neu aufbauen
		goto hauptschleife;
		}

	// Aktormappingdaten reloaden, falls ntig
	if (sqldb_read_long2(sql_config_handle,"/control/actors/reload_counter") != actor_reload_counter)
		{
		log_append(1,"Re-loading actor mapping data.");
		actor_readconfig();
		goto hauptschleife;
		}

	// DB-Daten reloaden, falls ntig
	if (sqldb_read_long2(sql_config_handle,"/control/database/reload") != db_merker)
		{
		db_readconfig();
		sprintf(ep,"Reloaded database data. enabled=%d",db_enabled);
		log_append(1,ep);
		if (db_enabled > 0)
			{
			sprintf(ep,"db_fields=%d, db_interval=%d, db_type=%d",db_fields,db_interval,db_type);
			log_append(2,ep);
			sprintf(ep,"db_path=\"%s\", db_name=\"%s\"",db_path,db_name);
			log_append(2,ep);
			}
		goto hauptschleife;
		}

	// Statusmail-Daten reloaden, falls ntig
	if (sqldb_read_long2(sql_config_handle,"/statusmail/reload") != statusmail_counter)
		{
		log_append(1,"Re-loading Statusmail data.");
		statusmail_readconfig();
		goto hauptschleife;
		}

	// Physikalische Sensordaten reloaden, falls ntig
	if (sqldb_read_long2(sql_config_handle,"/control/sensors_i2c/flag_sensorsettings") > 0)
		{
		log_append(1,"Re-loading physical sensor data.");
		physical_readconfig();
		goto hauptschleife;
		}

	// Globale Einstellungen reloaden, falls ntig
	if (sqldb_read_long2(sql_config_handle,"/control/sensordemon/flag_globalsettings") > 0)
		{
		log_append(1,"Re-loading global demon configuration data.");
		demon_readconfig();
		goto hauptschleife;
		}

	// Schaltmatrix-Einstellungen reloaden, falls ntig
	if (sqldb_read_long2(sql_config_handle,"/control/matrix/reload_counter") != matrix_merker)
		{
		log_append(1,"Re-loading Switch Matrix configuration data.");
		matrix_read_config();
		goto hauptschleife;
		}

	// Timer-Einstellungen reloaden, falls ntig
	if (sqldb_read_long2(sql_config_handle,"/timers/reload_timer") != timer_reload_counter)
		{
		log_append(1,"Re-loading Timer configuration data.");
		timer_readconfig();
		goto hauptschleife;
		}

	// Alert-Einstellungen reloaden, falls ntig
	if (sqldb_read_long2(sql_config_handle,"/alerts/reload_alert") != alert_reload_counter)
		{
		log_append(1,"Re-loading Alert configuration data.");
		alert_readconfig();
		goto hauptschleife;
		}

	// Remotesensor-Einstellungen reloaden, falls ntig
	if (sqldb_read_long2(sql_config_handle,"/control/sensors_remote/reload_counter") != sensor_remote_reload_counter)
		{
		log_append(1,"Re-loading Remote Sensor (XML) configuration data.");
		remote_readconfig();
		goto hauptschleife;
		}

	// Mobotix-Upload-Einstellungen reloaden, falls ntig
	if (sqldb_read_long2(sql_config_handle,"/control/mobotix_upload/reloadcounter") != mobotix_upload_reload_counter)
		{
		log_append(1,"Re-loading Mobotix Upload configuration data.");
		mobotix_upload_readconfig();
		goto hauptschleife;
		}

	// Nachschauen, ob von der Oberflche ein Kommando angekommen ist
	kommando = sqldb_read_long2(sql_config_handle,"/control/sensors_i2c/flag_command");
	if (kommando > 0)
		{
		if(kommando == 1)
			{
			// Portscan
			f = fopen("/www/reports/portscan.log","wb");
			if (f != NULL)
				{
				sqldb_write_long2(sql_config_handle,"/control/portscan/progress_active",1); // Anzeigen, da Portscan durchgefhrt wird
				do_portscan(f);
				sqldb_write_long2(sql_config_handle,"/control/portscan/progress_active",-1); // Anzeigen, da Portscan abgeschlossen ist
				fprintf(f,"Port Scan finished.\n");
				fclose(f);
				}
			goto fertig;
			}

fertig:
		sqldb_write_long2(sql_config_handle,"/control/sensors_i2c/flag_command",0); // Flag wieder lschen
		goto hauptschleife;
		}


	if(hwmodel == 5000)
		{
		simplified_all5000_led_set(1,1);										// LED 1 einschalten
		}

	if(hwmodel == 4500)
		{
		z = simplified_pcf8574_read(0,0,0,5,3);
		// printf("beginn z=%d, ",z);
		z &= 254;																						// Bit 0 lschen
		simplified_pcf8574_write(0,0,0,5,3,z);							// ALL4500 LED 0 einschalten
		// printf("write z=%d\n",z);
		// simplified_pcf8574_write(0,0,0,5,4,testcounter++);
		}



	physical_measurementtype_counter++;										// Zhler erhhen
	if (physical_measurementtype_counter >= physical_measurementtype_cycles)
		{
		physical_measurementtype_counter = 0;
		physical_measurementtype_rotator++;									// Me-Typ (0: Temp, 1: Feuchte)
		if (physical_measurementtype_rotator > 1) physical_measurementtype_rotator = 0;
		}

	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln
	zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;					// Timestamp in ms ermitteln
	zeit_start = zeit2;																		// Startzeit merken
	zeit_verwaltung1 = zeit2 - zeit1;											// Zeit fr 1. Verwaltungsakt messen
	zeit1 = zeit2;

	// printf("---------- TI=%ld ----------\n",ti);
	// printf("sensor_poll\n");
	anzahl_pollr = sensor_poll_read();										// Sensoren abfragen

	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln
	zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;					// Timestamp in ms ermitteln
	zeit_pollr = zeit2 - zeit1;														// Zeit fr Sensor-Polling (read) messen
	zeit1 = zeit2;

	// printf("remote_poll\n");
	remote_poll();																				// Remote-Sensoren abfragen und in virtuelle Sensoren umkopieren

	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln
	zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;					// Timestamp in ms ermitteln
	zeit_remote_poll = zeit2 - zeit1;											// Zeit fr Remote-Polling messen
	zeit1 = zeit2;

	// printf("sensor_copyvalues\n");
	sensor_copyvalues();																	// Mewerte in virtuelle Sensoren umkopieren und in SHM schreiben

	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln
	zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;					// Timestamp in ms ermitteln
	zeit_sensor_copyvalues = zeit2 - zeit1;								// Zeit fr Sensor_Copyvalues messen
	zeit1 = zeit2;

	// printf("actor_copy_from_shm\n");
	actor_copy_from_shm();																// Aktor-Lesewerte in virtuelle Sensoren kopieren und in SHM schreiben

	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln
	zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;					// Timestamp in ms ermitteln
	zeit_actor_copy_from_shm = zeit2 - zeit1;							// Zeit fr messen
	zeit1 = zeit2;

	// printf("logic_copy_to_shm\n");
	logic_analyze_changes();															// Vernderungen prfen
	logic_copy_to_shm();																	// die virtuellen Werte im SharedMemory publizieren

	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln
	zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;					// Timestamp in ms ermitteln
	zeit_logic_copy_to_shm = zeit2 - zeit1;								// Zeit messen
	zeit1 = zeit2;

	if(sensor_valid_flag > 0)
		{
		// printf("matrix_execute\n");
		matrix_execute();																		// Schaltmatrix durchrdeln

		gettimeofday(&tv,NULL);															// Hires-Zeit ermitteln
		zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;				// Timestamp in ms ermitteln
		zeit_matrix = zeit2 - zeit1;												// Zeit fr Schaltmatrix messen
		zeit1 = zeit2;

		// printf("timer_execute\n");
		timer_execute();																		// Timers durchrdeln

		gettimeofday(&tv,NULL);															// Hires-Zeit ermitteln
		zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;				// Timestamp in ms ermitteln
		zeit_timer = zeit2 - zeit1;													// Zeit fr Timers messen
		zeit1 = zeit2;

		// printf("matrix_actor_copy\n");
		matrix_actor_copy();

		gettimeofday(&tv,NULL);															// Hires-Zeit ermitteln
		zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;				// Timestamp in ms ermitteln
		zeit_matrix_actor_copy = zeit2 - zeit1;							// Zeit fr Matrix_actor_copy messen
		zeit1 = zeit2;
		// printf("alert_execute\n");
		alert_execute();																		// Alerts durchrdeln

		gettimeofday(&tv,NULL);															// Hires-Zeit ermitteln
		zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;				// Timestamp in ms ermitteln
		zeit_alert = zeit2 - zeit1;													// Zeit fr Alerts messen
		zeit1 = zeit2;
		}
	else
		{
		zeit_matrix = 0;
		zeit_timer = 0;
		zeit_matrix_actor_copy = 0;
		zeit_alert = 0;
		}


	// printf("actor_copyvalues\n");
	actor_copyvalues();																		// Schaltbefehle auf SHM-Aktortabelle in Outvalue-Arrays umkopieren

	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln
	zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;					// Timestamp in ms ermitteln
	zeit_actor_copyvalues = zeit2 - zeit1;								// Zeit messen
	zeit1 = zeit2;

	lcd_show_sensors();																		// Sensorwerte im internen Display anzeigen

	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln
	zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;					// Timestamp in ms ermitteln
	zeit_lcd = zeit2 - zeit1;															// Zeit fr LCD-Anzeige messen
	zeit1 = zeit2;

	anzahl_pollw = sensor_poll_write();										// I2C Sensoren abfragen (schreiben)

	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln
	zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;					// Timestamp in ms ermitteln
	zeit_pollw = zeit2 - zeit1;														// Zeit fr Sensorpolling (write) messen
	zeit1 = zeit2;

	// printf("schreibe_datenfile_xml\n");
	schreibe_datenfile_xml();															// XML-Datei erzeugen

	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln
	zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;					// Timestamp in ms ermitteln
	zeit_schreibe_datenfile = zeit2 - zeit1;							// Zeit fr messen
	zeit1 = zeit2;

	// printf("update_sensorvalues\n");
	sql_update_sensorvalues(sql_var_handle);							// Sensordaten in Sensordaten-Tabelle aktualisieren
	// printf("OK\n");

	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln
	zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;					// Timestamp in ms ermitteln
	zeit_update_sensorvalues = zeit2 - zeit1;							// Zeit messen
	zeit1 = zeit2;

	sql_update_actorvalues(sql_var_handle);								// Aktordaten in Aktordaten-Tabelle aktualisieren

	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln
	zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;					// Timestamp in ms ermitteln
	zeit_update_actorvalues = zeit2 - zeit1;							// Zeit messen
	zeit1 = zeit2;

	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln
	// printf("now:%ld\n",tv.tv_sec);
	if(tv.tv_sec >= next_history)													// History-Zeitpunkt erreicht ?
		{
		// printf("*** Schreibe History *** (now:%ld, next_history: %ld\n",tv.tv_sec,next_history);
		sprintf(ep,"Timestamp %ld >= next timestamp (%ld)",tv.tv_sec,next_history);
		log_append(4,ep);																		// Meldung ins Logfile schreiben
		last_history = next_history;												// diesen History-Timestamp merken (nicht die Uhr, um Ungenauigkeiten zu stoppen)
		next_history = last_history + config_demon_historyinterval; // Timestamp fr nchsten Historypunkt weitersetzen
		// last_history = tv.tv_sec;														// diesen History-Timestamp merken
		history_shift();
		history_update();
		history_write_data();
		history_write_xml();
		history_write_svg(0);
		}

	mobotix_upload();																			// Sensordaten in Mobotix-Kameras schreiben


	gettimeofday(&tv,NULL);																// Hires-Zeit ermitteln
	zeit2 = tv.tv_sec * 1000 + tv.tv_usec / 1000;					// Timestamp in ms ermitteln
	zeit_verwaltung2 = zeit2 - zeit1;											// Zeit fr 2. Verwaltungsakt messen
	zeit_gesamt = zeit2 -zeit_start;
	sql_update_timevalues(sql_var_handle);								// Zeitstatistiken in Sensordaten-Tabelle aktualisieren

	if(hwmodel == 5000)
		{
		simplified_all5000_led_set(1,0);										// LED 1 ausschalten
		}

	if(hwmodel == 4500)
		{
		z = simplified_pcf8574_read(0,0,0,5,3);
		// printf("ende z=%d, ",z);
		z |= 1;																							// Bit 0 setzen
		simplified_pcf8574_write(0,0,0,5,3,z);							// ALL4500 LED 0 ausschalten
		// printf("write z=%d\n",z);
		}


	db_timestamp = time(0);																// aktuelle Zeit ermitteln
	// printf("now: %ld, next: %ld (%ld s)\n",db_timestamp,db_next,db_next - db_timestamp);
	if(db_timestamp >= db_next)
		{
		// printf("DB WRITE !\n");
		db_next = db_timestamp + 60 * db_interval;					// Neue Zeit ausrechnen
		db_write_sensordata();															// Sensordaten in DB schreiben
		}

	statusmail_work();																		// Statusmails abarbeiten

	if(config_demon_ps_enabled > 0)
		{
		gpio_port_write(PORT_B,22,0);												// Strom fr Sensoren abschalten
		sleep(config_demon_ps_offtime);											// jetzt ohne Strom abwarten...
		gpio_port_write(PORT_B,22,1);												// Strom fr Sensoren wieder einschalten
		sleep(config_demon_ps_boottime);										// Warten, bis die Sensoren aufgewacht sind
		}
	else
		{
		usleep(config_demon_delay * 1000);									// ein bichen Zeit vertrdeln
		}

	if(config_sekundenlock > 0)
		{
wartesekunde:
		usleep(50000);																			// 50 ms Gnadenfrist fr andere Programme
		gettimeofday(&tv,NULL);															// Hires-Zeit ermitteln
		if (startsekunde == tv.tv_sec) goto wartesekunde;		// warten bis sich die Sekunde ndert
		}

	goto hauptschleife;																		// ... und wieder dasselbe Spiel von vorne

ende:
	sprintf(ep,"----------   sensor_demon terminated   ----------");
	log_append(1,ep);																			// Endmeldung ins Logfile schreiben
	gpio_port_write(PORT_B,22,0);													// Strom fr Sensoren abschalten
	sqlite3_close(sql_var_handle);												// Datenbank schlieen
	sqlite3_close(sql_config_handle);											// Datenbank schlieen
	return(0);
	}

